/*
 * Mocha
 * Copyright (c) 2005 IT Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file AppSndECOM.h
 *
 * This file defines the data structures for Application Sound informations.
 *
 * @author	mobum
 */
/*:Associate with "Application Sound Manager" */

#ifndef	_APP_SND_ECOM_H_
#define _APP_SND_ECOM_H_

#include "SysEcom.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * ECOM Interface
 */
ECOM_INTERFACE(EAppSndMgr)
{
	DECLARE_EBASE()

	/* KeyTone sound */
	// Mute Key Sound Callback
	BOOL	(*RegisterMuteKeyCallback)	( AppSndKeyAppClassType iClass, AppSndMuteKeyCallback callback, void* pParam);
	BOOL	(*UnregisterMuteKeyCallback)( AppSndKeyAppClassType iClass);

	// KeySound class
	BOOL	(*LoadKeySoundClass)		( void );
	BOOL	(*SetKeySoundClass)			( AppSndKeyClassType iClassType, AppSndKeySoundClassType iSoundClassType, const char* const pszFile, int nLine );
	BOOL	(*SetKeySoundClassByKeyType)( AppSndKeyType iKeyType, AppSndKeySoundClassType iSoundClassType, const char* const pszFile, int nLine );

	// Mute
	BOOL	(*MuteKeySound)				( DID did, AppSndKeyClassType iClassType, const char* const pszFile, int nLine );
	BOOL	(*UnmuteKeySound)			( DID did, AppSndKeyClassType iClassType, const char* const pszFile, int nLine );
	BOOL	(*GetMutedKeySound)			( AppSndKeyClassType iClassType, AppSndMuteType* const pMuteType );
	AppSndKeyClassType	(*GetMutedKeyClass)	( void );
	// Mute/Unmue by Key code
	BOOL	(*MuteKeySoundByKeyType)	( DID did, AppSndKeyType iKeyType, const char* const pszFile, int nLine );
	BOOL	(*UnmuteKeySoundByKeyType)	( DID did, AppSndKeyType iKeyType, const char* const pszFile, int nLine );
	BOOL	(*IsMutedKeySoundByKeyType)	( AppSndKeyType iKeyType );
	// Play style
	BOOL	(*SetKeyPlayStyle)			( AppSndKeyClassType iClassType, AppSndKeyPlayStyleType iPlayStyle, const char* const pszFile, int nLine );
	BOOL	(*GetKeyPlayStyle)			( AppSndKeyClassType iClassType, AppSndKeyPlayStyleType* const pPlayStyle );
	// Play style by Key code
	BOOL	(*SetKeyPlayStyleByKeyType)	( AppSndKeyType iKeyType, AppSndKeyPlayStyleType iPlayStyle, const char* const pszFile, int nLine );
	AppSndKeyPlayStyleType	(*GetKeyPlayStyleByKeyType)	( AppSndKeyType iKeyType );
	// Play key sound
	BOOL	(*PlayKeySound)				( AppSndKeyType iKeyType, BOOL bVibration );
	BOOL	(*PlayKeySoundEx)			( int nKeyCode, BOOL bVibration );
	// Vibration
	BOOL	(*SetKeyVibType)			( AppSndKeyType iKeyType, AppSndVibType iVibType );
	AppSndVibType	(*GetKeyVibType)	( AppSndKeyType iKeyType );

	/* Device */
	// Enable/Disable
	void	(*EnableDevSound)				( void );
	void	(*DisableDevSound)				( void );

	/* Flip sound */
	// Mute
	BOOL	(*MuteFlipSound)				( const char* const pszFile, int nLine );
	BOOL	(*UnmuteFlipSound)				( const char* const pszFile, int nLine );
	BOOL	(*GetMutedFlipSound)			( AppSndMuteType* const pMuteType );

	/* Effect */
	// Play/Stop
	BOOL	(*PlayEffectSound)				( AppSndEffectType iEffectType, AppSndCallback callback, void* pParam, BOOL bVibration );
	BOOL	(*PlayEffectSoundEx)			( AppSndEffectType iEffectType, AmSoundType iSndType, AppSndCallback callback, void* pParam, BOOL bVibration );
	BOOL	(*StopEffectSound)				( AppSndEffectType iEffectType );
	// Set
	BOOL	(*SetEffectUserInfo)			( AppSndEffectType iEffectType, const AppSndEffectUserInfo* const pInputInfo );
	BOOL	(*SetEffectDtmfId)				( AppSndEffectType iEffectType, AvDtmfSoundIDInfo iDtmfId );
	BOOL	(*SetEffectOutSndPath)			( AppSndEffectType iEffectType, SndOutDevPath iSndOutDevPath );
	BOOL	(*SetEffectRepeatSound)			( AppSndEffectType iEffectType, BOOL bRepeat );
	BOOL	(*SetEffectVolLevel)			( AppSndEffectType iEffectType, const AvAudioVolume* const pVolLevel );
	BOOL	(*SetEffectFilePath)			( AppSndEffectType iEffectType, const char* const pszFilePath );
	BOOL	(*SetEffectStepSound)			( AppSndEffectType iEffectType, int nStepCount, const AppSndEffectStepInfo* const pStepInfo );
	BOOL	(*SetEffectIntervalSound)		( AppSndEffectType iEffectType, int nIntervalCount, const AppSndEffectIntervalInfo* const pIntervalInfo );

	// Get
	HAvPlayer	(*GetEffectHandle)			( AppSndEffectType iEffectType );
	// Mute/Unmute
	BOOL	(*MuteEffectSound)				( DID did, AppSndEffectType iEffectType, const char* const pszFile, int nLine );
	BOOL	(*UnmuteEffectSound)			( DID did, AppSndEffectType iEffectType, const char* const pszFile, int nLine );
	BOOL	(*MuteEffectSoundEx)			( DID did, AppSndEffectClassType iClassType, const char* const pszFile, int nLine );
	BOOL	(*UnmuteEffectSoundEx)			( DID did, AppSndEffectClassType iClassType, const char* const pszFile, int nLine );
	BOOL	(*IsMutedEffectSound)			( AppSndEffectType iEffectType );
	// Mute/Unmute AvPlayer
	BOOL	(*MuteEffectPlayer)				( AppSndEffectType iEffectType );
	BOOL	(*UnmuteEffectPlayer)			( AppSndEffectType iEffectType );

	// Enable/Disable
	BOOL	(*DisableEffectStepSound)		( AppSndEffectType iEffectType );

	// For test
	BOOL	(*SetEffectHandle)				( AppSndEffectType iEffectType, HAvPlayer hPlayer );
	// Vibration
	BOOL	(*SetEffectVibType)				( AppSndEffectType iEffectType, AppSndVibType iVibType );
	AppSndVibType	(*GetEffectVibType)		( AppSndEffectType iEffectType );
};

/*
 * Interface Accessor
 */
// Native/DLL Call
#if (defined(_SHP_SIMUL) && defined(_SHP_CALL_DLL_APPSNDMGR)) || (defined(_SHP_DLL) && ! defined(_SHP_DLL_APPSNDMGR) && ! defined(_SHP_CALL_DLL_APPSNDMGR))

#	define AppSndDispatch						(__AppSndGetVtbl()->Dispatch)

/* KeyTone sound */
// Mute Key Sound Callback
#	define AppSndRegisterMuteKeyCallback		(__AppSndGetVtbl()->RegisterMuteKeyCallback)
#	define AppSndUnregisterMuteKeyCallback		(__AppSndGetVtbl()->UnregisterMuteKeyCallback)
// KeySound class
#	define AppSndLoadKeySoundClass				(__AppSndGetVtbl()->LoadKeySoundClass)
// Mute
#	define __AppSndMuteKeySound					(__AppSndGetVtbl()->MuteKeySound)
#	define __AppSndUnmuteKeySound				(__AppSndGetVtbl()->UnmuteKeySound)
#	define AppSndGetMutedKeySound				(__AppSndGetVtbl()->GetMutedKeySound)
#	define AppSndGetMutedKeyClass				(__AppSndGetVtbl()->GetMutedKeyClass)
// Mute/Unmue by Key code
#	define __AppSndMuteKeySoundByKeyType		(__AppSndGetVtbl()->MuteKeySoundByKeyType)
#	define __AppSndUnmuteKeySoundByKeyType		(__AppSndGetVtbl()->UnmuteKeySoundByKeyType)
#	define AppSndIsMutedKeySoundByKeyType		(__AppSndGetVtbl()->IsMutedKeySoundByKeyType)
// Play style
#	define __AppSndSetKeyPlayStyle				(__AppSndGetVtbl()->SetKeyPlayStyle)
#	define AppSndGetKeyPlayStyle				(__AppSndGetVtbl()->GetKeyPlayStyle)
// Play style by Key code
#	define __AppSndSetKeyPlayStyleByKeyType		(__AppSndGetVtbl()->SetKeyPlayStyleByKeyType)
#	define AppSndGetKeyPlayStyleByKeyType		(__AppSndGetVtbl()->GetKeyPlayStyleByKeyType)
// Play key sound
#	define __AppSndPlayKeySound					(__AppSndGetVtbl()->PlayKeySound)
#	define __AppSndPlayKeySoundEx				(__AppSndGetVtbl()->PlayKeySoundEx)
// Vibration
#	define AppSndSetKeyVibType					(__AppSndGetVtbl()->SetKeyVibType)
#	define AppSndGetKeyVibType					(__AppSndGetVtbl()->GetKeyVibType)
	
/* Device */
// Enable/Disable
#	define AppSndEnableDevSound					(__AppSndGetVtbl()->EnableDevSound)
#	define AppSndDisableDevSound				(__AppSndGetVtbl()->DisableDevSound)

/* Flip sound */
// Mute
#	define __AppSndMuteFlipSound				(__AppSndGetVtbl()->MuteFlipSound)
#	define __AppSndUnmuteFlipSound				(__AppSndGetVtbl()->UnmuteFlipSound)
#	define __AppSndGetMutedFlipSound			(__AppSndGetVtbl()->GetMutedFlipSound)

/* Effect */
// Play Effect sound
#	define __AppSndPlayEffectSound				(__AppSndGetVtbl()->PlayEffectSound)
#	define __AppSndPlayEffectSoundEx			(__AppSndGetVtbl()->PlayEffectSoundEx)
// Stop
#	define AppSndStopEffectSound				(__AppSndGetVtbl()->StopEffectSound)
// Set
#	define AppSndSetEffectUserInfo				(__AppSndGetVtbl()->SetEffectUserInfo)
#	define AppSndSetEffectDtmfId				(__AppSndGetVtbl()->SetEffectDtmfId)
#	define AppSndSetEffectOutSndPath			(__AppSndGetVtbl()->SetEffectOutSndPath)
#	define AppSndSetEffectRepeatSound			(__AppSndGetVtbl()->SetEffectRepeatSound)
#	define AppSndSetEffectVolLevel				(__AppSndGetVtbl()->SetEffectVolLevel)
#	define AppSndSetEffectFilePath				(__AppSndGetVtbl()->SetEffectFilePath)
#	define AppSndSetEffectStepSound				(__AppSndGetVtbl()->SetEffectStepSound)
#	define AppSndSetEffectIntervalSound			(__AppSndGetVtbl()->SetEffectIntervalSound)
// Get
#	define AppSndGetEffectHandle				(__AppSndGetVtbl()->GetEffectHandle)
// Mute/Unmute
#	define __AppSndMuteEffectSound				(__AppSndGetVtbl()->MuteEffectSound)
#	define __AppSndUnmuteEffectSound			(__AppSndGetVtbl()->UnmuteEffectSound)
#	define __AppSndMuteEffectSoundEx			(__AppSndGetVtbl()->MuteEffectSoundEx)
#	define __AppSndUnmuteEffectSoundEx			(__AppSndGetVtbl()->UnmuteEffectSoundEx)
#	define AppSndIsMutedEffectSound				(__AppSndGetVtbl()->IsMutedEffectSound)
// Mute/Unmute AvPlayer
#	define AppSndMuteEffectPlayer				(__AppSndGetVtbl()->MuteEffectPlayer)
#	define AppSndUnmuteEffectPlayer				(__AppSndGetVtbl()->UnmuteEffectPlayer)
// Enable/Disable
#	define AppSndDisableEffectStepSound			(__AppSndGetVtbl()->DisableEffectStepSound)
// For test
#	define AppSndSetEffectHandle				(__AppSndGetVtbl()->SetEffectHandle)
// Vibration
#	define AppSndSetEffectVibType				(__AppSndGetVtbl()->SetEffectVibType)
#	define AppSndGetEffectVibType				(__AppSndGetVtbl()->GetEffectVibType)

#endif // (defined(_SHP_SIMUL) && defined(_SHP_CALL_DLL_APPSNDMGR)) || (defined(_SHP_DLL) && ! defined(_SHP_DLL_APPSNDMGR) && ! defined(_SHP_CALL_DLL_APPSNDMGR))


/**
 @struct TAppSndMgr

 @brief ECOM Type
 */  
typedef struct
{
	DECLARE_ECOM_VTBL(EAppSndMgr)
	UINT32					refCount;	///< reference count

} TAppSndMgr;

extern TAppSndMgr	appSndMgr;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif	// _APP_SND_ECOM_H_
