/*
 * Mocha 
 * Copyright (c) 2006 IT Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file AppSndEffect.h
 *
 * This file implements the APIs of Application Sound Manager.
 *
 * @author	mobum
 */
/*:Associate with "Application Sound Manager" */

#ifndef _APP_SND_EFFECT_H_
#define _APP_SND_EFFECT_H_

#if	(defined( _SHP_DRM_V2) || defined(_SHP_DRM_V21) )
#	include "Drm2Svc.h"
#else
#	include "DrmSvc.h"
#endif

/**
 @brief Enum for AppSndEffectClassType
 */
typedef enum
{
	APP_SND_EFFECT_CLASS_NONE		= 0x0000,	///< none
} AppSndEffectClassType;

/**
 @brief AppSndEffectStepInfo
 */
typedef struct
{
	ULONG			time; 		///< play time
} AppSndEffectStepInfo;

/**
 @brief AppSndEffectIntervalInfo
 */
typedef struct
{
	ULONG			time; 		///< interval time
} AppSndEffectIntervalInfo;

/**
 @struct AppSndEffectUserInfo

 @brief effect information
 */  
typedef struct
{
	AppSndDtmfType			iDtmfType;						///< Dtmf type
} AppSndEffectUserInfo;

/**
 @struct AppSndEffectInfo

 @brief Effect sound information
 */
typedef struct
{
	AppSndEffectType	iEffectType;				///< Effect type
} AppSndEffectInfo;


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * Public Function Prototypes
 */
 
// Local calls
#if defined(_SHP_DLL_APPSNDMGR) || ( ! defined(_SHP_DLL) && ! defined(_SHP_CALL_DLL_APPSNDMGR))

// Play Effect sound
BOOL	__AppSndPlayEffectSound			( AppSndEffectType iEffectType, AppSndCallback callback, void* pParam, BOOL bVibration );
BOOL	__AppSndPlayEffectSoundEx		( AppSndEffectType iEffectType, AmSoundType iSndType, AppSndCallback callback, void* pParam, BOOL bVibration );
BOOL	AppSndStopEffectSound			( AppSndEffectType iEffectType );

// Set
BOOL	AppSndSetEffectUserInfo			( AppSndEffectType iEffectType, const AppSndEffectUserInfo* const pUserInfo );
BOOL	AppSndSetEffectDtmfId			( AppSndEffectType iEffectType, AvDtmfSoundIDInfo iDtmfId );
BOOL	AppSndSetEffectOutSndPath		( AppSndEffectType iEffectType, SndOutDevPath iSndOutDevPath );
BOOL	AppSndSetEffectRepeatSound		( AppSndEffectType iEffectType, BOOL bRepeat );
BOOL	AppSndSetEffectRepeatSoundEx	( AppSndEffectType iEffectType, AppSndRepeatType iRepeatType );
BOOL	AppSndSetEffectVolLevel			( AppSndEffectType iEffectType, const AvAudioVolume* const pVolLevel );
BOOL	AppSndSetEffectFilePath			( AppSndEffectType iEffectType, const char* const pszFilePath );
BOOL	AppSndSetEffectStepSound		( AppSndEffectType iEffectType, int nStepCount, const AppSndEffectStepInfo* const pStepInfo );
BOOL	AppSndSetEffectIntervalSound	( AppSndEffectType iEffectType, int nIntervalCount, const AppSndEffectIntervalInfo* const pIntervalInfo );

// Get
HAvPlayer	AppSndGetEffectHandle		( AppSndEffectType iEffectType );

// Mute/Unmute
BOOL	__AppSndMuteEffectSound			( DID did, AppSndEffectType iEffectType, const char* const pszFile, int nLine );
BOOL	__AppSndUnmuteEffectSound		( DID did, AppSndEffectType iEffectType, const char* const pszFile, int nLine );
BOOL	__AppSndMuteEffectSoundEx		( DID did, AppSndEffectClassType iClassType, const char* const pszFile, int nLine );
BOOL	__AppSndUnmuteEffectSoundEx		( DID did, AppSndEffectClassType iClassType, const char* const pszFile, int nLine );
BOOL	AppSndIsMutedEffectSound		( AppSndEffectType iEffectType );

// Mute/Unmute AvPlayer
BOOL	AppSndMuteEffectPlayer			( AppSndEffectType iEffectType );
BOOL	AppSndUnmuteEffectPlayer		( AppSndEffectType iEffectType );

// Enable/Disable
BOOL	AppSndDisableEffectStepSound	( AppSndEffectType iEffectType );

// For test
BOOL	AppSndSetEffectHandle			( AppSndEffectType iEffectType, HAvPlayer hPlayer );

// Vibration
BOOL	AppSndSetEffectVibType			( AppSndEffectType iEffectType, AppSndVibType iVibType );
AppSndVibType	AppSndGetEffectVibType	( AppSndEffectType iEffectType );

BOOL	AppSndSetEffectVibIntensity		( AppSndEffectType iEffectType, int nIntensity );
BOOL	AppSndSetEffectVibCount			( AppSndEffectType iEffectType, int nPlayCount );

#endif // defined(_SHP_DLL_APPSNDMGR) || ( ! defined(_SHP_DLL) && ! defined(_SHP_CALL_DLL_APPSNDMGR))


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif // _APP_SND_EFFECT_H_
