/*
 * Mocha
 * Copyright (c) 2000-2003 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */

/*:Associate with "Event Manager" */

#ifndef _EM_ECOM_H_
#define _EM_ECOM_H_

#include "DllClientStub.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * ECOM Interface
 */
ECOM_INTERFACE(EEventMgr)
{
	DECLARE_EBASE()

	/* Event Handler */
	BOOL			(*_CallEHFunction)(WmHandleEvent handleEvent, WmEventHandlerInfo* pEHInfo, WmState* pState, WmEvent* pEvent);
	HEventHandler	(*__CreateEventHandler)(char* szName, WmState* pState, WmEventMapEntry* eventMap, HEHGroup hEHGroup, ULONG flags, DID paramDID, BOOL bAsync, WmEventProcessCallback callback);
	BOOL			(*StartEventHandler)(HEventHandler hEventHandler);
	BOOL			(*StartEventHandlerEx)(HEventHandler hEventHandler, WmEventProcessCallback callback);
	BOOL			(*StopEventHandler)(HEventHandler hEventHandler);
	BOOL			(*StopEventHandlerEx)(HEventHandler hEventHandler, WmEventProcessCallback callback);
	BOOL			(*DeleteEventHandler)(HEventHandler hEventHandler);
	BOOL			(*DeleteEventHandlerEx)(HEventHandler hEventHandler, WmEventProcessCallback callback);
	BOOL			(*ExitEventHandler)(int retCode);
	BOOL			(*ExitEventHandlerEx)(int retCode, WmEventProcessCallback callback);

	HEventHandler	(*GetEventHandlerByName)(char* szName);
	WmEventHandlerState (*GetEventHandlerState)(HEventHandler hEventHandler);
	WmEventHandlerInfo*	(*GetEventHandlerInfo)(HEventHandler hEventHandler);
	HEventHandler	(*GetCurrentEventHandler)(void);
	void			(*SetCurrentEventHandler)(HEventHandler hEventHandler);
	BOOL			(*IsEventHandlerValid)(HEventHandler hEventHandler);


	/* Event Handler Group */
	HEHGroup		(*CreateEHGroup)(char* szName, WmEventHandler* pMainEventHandler);
	BOOL			(*DeleteEHGroup)(HEHGroup hEHGroup);
	HEHGroup		(*GetEHGroupByName)(char* szName);
	HEHGroup		(*GetEHGroupByEventHandler)(HEventHandler hEventHandler);
	BOOL			(*SetMainEventHandlerObject)(HEHGroup hEHGroup, WmEventHandler* pMainEventHandler);
	WmEventHandler*	(*GetMainEventHandlerObject)(HEHGroup hEHGroup);
	HEventHandler	(*GetMainEventHandler)(HEHGroup hEHGroup);
	BOOL			(*IsEHGroupValid)(HEHGroup hEHGroup);
	BOOL			(*JoinEventHandlerInGroup)(HEHGroup hEHGroup, HEventHandler hEventHandler);
	BOOL			(*WithdrawEventHandlerFromGroup)(HEHGroup hEHGroup, HEventHandler hEventHandler);
	BOOL			(*EnumerateEventHandlersInGroup)(HEHGroup hEHGroup, HEventHandler* pEHBuf, int bufLen, int* pEHCount);
	HEHGroup		(*GetCurrentEHGroup)(void);

	/* Event Broker */
	BOOL			(*SetEventBroker)(WmEventBrokerPriority priority, WmEventBroker broker);
	WmEventBroker	(*GetEventBroker)(WmEventBrokerPriority priority);
	//Riang New Filter
	BOOL			(*RegisterEventFilterBroker)(char *szName, WmEventBroker broker, const char* szFileName,  int nLine);
	BOOL			(*UnRegisterEventFilterBroker)(char *szName);
	BOOL			(*CheckEventFilterBroker)(void);

	/* Event Passing & Processing */
	UINT16			(*GetCurrentEventID)(void);
	void			(*SetCurrentEventID)(UINT16 id);
	UINT16			(*GetNextEventID)(void);
	BOOL			(*SendEvent)(const WmEvent* pEvent);
	BOOL			(*SendUrgentEvent)(const WmEvent* pEvent);
	BOOL			(*ReceiveEvent)(ULONG timeout, WmEvent* pEvent, UINT* pRemainingEventCount);
	BOOL			(*WaitEvent)(WmEventCheckCallback checkRoutine, WmEvent* pEvent);
	BOOL			(*ProcessEvent)(WmEvent* pEvent);
	void			(*MainLoop)(void);

	/* Event Printer */
	BOOL			(*SetEventPrinter)(WmEventGroup eventGroup, WmEventPrinter eventPrinter);

	/* Timer */
	HTimer			(*CreateTimer)(char* szTmName, const char* szCallerFileName, int nLine);
	BOOL			(*StartTimer)(HTimer hTimer, ULONG timeout, WmTimerCallback timerCallback, ULONG param);
	BOOL			(*DeleteTimer)(HTimer hTimer, const char* szCallerFileName, int nLine);
	BOOL			(*CancelTimer)(HTimer hTimer);
	BOOL			(*IsTimerActive)(HTimer hTimer);
	HTimer			(*CreateTimerEx)(char* szTmName, WmTimerBindCallback bindCallback, const char* szCallerFileName, int nLine);
	HTimer			(*BindTimer)(char* szTmName, WmTimerCallback timerCallback);
	BOOL			(*UnbindTimer)(HTimer hTimer);
	BOOL			(*VerifyTimer)(HTimer hTimer);
};

/*
 * Interface Accessor
 */
// Native/DLL Call
#if (defined(_SHP_SIMUL) && defined(_SHP_CALL_DLL_EVENTMGR)) || (defined(_SHP_DLL) && ! defined(_SHP_DLL_EVENTMGR) && ! defined(_SHP_CALL_DLL_EVENTMGR))
#	define EmDispatch				(__EmGetVtbl()->Dispatch)
#	define WmCreateEventHandler(szName, pState, eventMap, hEHGroup, flags) \
	(__EmGetVtbl()->__CreateEventHandler(szName, pState, eventMap, hEHGroup, flags, dllBaseDID, FALSE, NULL))
#	define WmStartEventHandler				(__EmGetVtbl()->StartEventHandler)
#	define WmStartEventHandlerEx			(__EmGetVtbl()->StartEventHandlerEx)
#	define WmStopEventHandler				(__EmGetVtbl()->StopEventHandler)
#	define WmStopEventHandlerEx				(__EmGetVtbl()->StopEventHandlerEx)
#	define WmDeleteEventHandler				(__EmGetVtbl()->DeleteEventHandler)
#	define WmDeleteEventHandlerEx			(__EmGetVtbl()->DeleteEventHandlerEx)
#	define WmExitEventHandler				(__EmGetVtbl()->ExitEventHandler)
#	define WmExitEventHandlerEx				(__EmGetVtbl()->ExitEventHandlerEx)
#	define WmGetEventHandlerByName			(__EmGetVtbl()->GetEventHandlerByName)
#	define WmGetEventHandlerState			(__EmGetVtbl()->GetEventHandlerState)
#	define WmGetEventHandlerInfo			(__EmGetVtbl()->GetEventHandlerInfo)
#	define WmGetCurrentEventHandler			(__EmGetVtbl()->GetCurrentEventHandler)
#	define WmSetCurrentEventHandler			(__EmGetVtbl()->SetCurrentEventHandler)
#	define WmIsEventHandlerValid			(__EmGetVtbl()->IsEventHandlerValid)
#	define WmGetCurrentEventID				(__EmGetVtbl()->GetCurrentEventID)

// Event Handler Group
#	define WmCreateEHGroup					(__EmGetVtbl()->CreateEHGroup)
#	define WmDeleteEHGroup					(__EmGetVtbl()->DeleteEHGroup)
#	define WmGetEHGroupByName				(__EmGetVtbl()->GetEHGroupByName)
#	define WmGetEHGroupByEventHandler		(__EmGetVtbl()->GetEHGroupByEventHandler)
#	define WmSetMainEventHandlerObject		(__EmGetVtbl()->SetMainEventHandlerObject)
#	define WmGetMainEventHandlerObject		(__EmGetVtbl()->GetMainEventHandlerObject)
#	define WmGetMainEventHandler			(__EmGetVtbl()->GetMainEventHandler)
#	define WmIsEHGroupValid					(__EmGetVtbl()->IsEHGroupValid)
#	define WmJoinEventHandlerInGroup		(__EmGetVtbl()->JoinEventHandlerInGroup)
#	define WmWithdrawEventHandlerFromGroup	(__EmGetVtbl()->WithdrawEventHandlerFromGroup)
#	define WmEnumerateEventHandlersInGroup	(__EmGetVtbl()->EnumerateEventHandlersInGroup)
#	define WmGetCurrentEHGroup				(__EmGetVtbl()->GetCurrentEHGroup)

// Event Broker
#	define WmSetEventBroker					(__EmGetVtbl()->SetEventBroker)
#	define WmGetEventBroker					(__EmGetVtbl()->GetEventBroker)

// Riang New Filter
// Event Filter Broker
#	define WmRegisterEventFilterBroker(szFilterName,broker)			(__EmGetVtbl()->RegisterEventFilterBroker(szFilterName, broker,__FILE__, __LINE__))
#	define WmUnRegisterEventFilterBroker			(__EmGetVtbl()->UnRegisterEventFilterBroker)
#	define WmCheckEventFilterBroker				(__EmGetVtbl()->CheckEventFilterBroker)


// Event Passing & Processing
#	define WmSendEvent						(__EmGetVtbl()->SendEvent)
#	define WmSendUrgentEvent				(__EmGetVtbl()->SendUrgentEvent)
#	define WmReceiveEvent					(__EmGetVtbl()->ReceiveEvent)
#	define WmWaitEvent						(__EmGetVtbl()->WaitEvent)
#	define WmProcessEvent					(__EmGetVtbl()->ProcessEvent)
#	define WmMainLoop						(__EmGetVtbl()->MainLoop)

// Event Printer
#	define WmSetEventPrinter				(__EmGetVtbl()->SetEventPrinter)

// Timer
#	define WmCreateTimer(szTmName)		(__EmGetVtbl()->CreateTimer(szTmName, __FILE__, __LINE__))
#	define WmCreateTimerEx(szTmName, bindCallback)		(__EmGetVtbl()->CreateTimerEx(szTmName, bindCallback, __FILE__, __LINE__))
#	define WmStartTimer						(__EmGetVtbl()->StartTimer)
#	define WmDeleteTimer(hTimer)					(__EmGetVtbl()->DeleteTimer(hTimer, __FILE__, __LINE__))
#	define WmCancelTimer					(__EmGetVtbl()->CancelTimer)
#	define WmIsTimerActive					(__EmGetVtbl()->IsTimerActive)
#	define WmVerifyTimer					(__EmGetVtbl()->VerifyTimer)
#endif // _SHP_DLL


/*
 * ECOM Type
 */
typedef struct
{
	DECLARE_ECOM_VTBL(EEventMgr)
	UINT32				refCount;

} TEventMgr;

extern TEventMgr eventMgr;

#ifdef __cplusplus
}
#endif  /* __cplusplus */

#endif // _EM_ECOM_H_
