/*
 * Mocha
 * Copyright (c) 2000-2003 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */

/*:Associate with "File Manager" */

#ifndef	_FM_ECOM_H_
#define _FM_ECOM_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * ECOM Interface
 */
ECOM_INTERFACE(EFileMgr)
{
	DECLARE_EBASE()

	// File
	HFile	(*CreateFile)(const char* szPathName, ULONG mode);
	HFile	(*OpenFile)(const char* szPathName, ULONG mode);
	BOOL	(*CloseFile)(HFile hFile);
	BOOL	(*ReadFile)(HFile hFile, void* pBuffer, ULONG bufLen, ULONG* pReadLen);
	BOOL	(*WriteFile)(HFile hFile, void* pBuffer, ULONG bufLen);
	BOOL	(*FlushFile)(HFile hFile);
	BOOL	(*RemoveFile)(const char* szPathName);
	BOOL	(*MoveFile)(const char* szOldPathName, const char* szNewPathName);
	BOOL	(*SeekFile)(HFile hFile, long position, long offset);
	long	(*TellFile)(HFile hFile);
	BOOL	(*RewindFile)(HFile hFile);
	BOOL	(*GetFileAttributes)(const char* szPathName, FmFileAttribute* pFileAttr);
	BOOL	(*FGetFileAttributes)(HObj hObj, FmEntryType type,  FmFileAttribute* pFileAttr);	
	BOOL	(*SetFileAttributes)(const char* szPathName, ULONG dwFlag); 
	BOOL	(*SetFileTime)(const char* szPathName, TmDateTime dateTime);
	BOOL	(*TruncateFile)(HFile hFile, ULONG length);
	BOOL	(*CopyFile)(const char* szOrigFileName, const char* szNewFileName, BOOL bFailIfExists);

	// Directory
	BOOL	(*CreateDir)(const char* szDirName);
	HDir	(*OpenDir)(const char* szDirName);
	BOOL	(*CloseDir)(HDir hDir);
	BOOL	(*RemoveDir)(const char* szDirName);
	BOOL	(*RemoveDirEx)(const char* szDirName);
	BOOL	(*ReadDir)(HDir hDir, FmDirEntry* pEntry);

	// Volume
	BOOL	(*MountVolume)(const char* szMountPath, const char* szVolumeRoot,
							FmFsType fsType, FmPartitionType partitionType, int* pVol);
	BOOL	(*UnmountVolume)(int iVol ,FmPartitionType partition);
	BOOL	(*GetVolumeStat)(int iVol, FmVolumeStat* pVolStat);
	BOOL	(*FormatVolume)(FmFsType fsType, FmPartitionType partitionType);
	BOOL	(*GetVolumeLabel)(const char* szPathName, char* szVolumeLabel);
	BOOL	(*SetVolumeLabel)(const char* szPathName, const char* szNewVolumeLabel);
	
	// Util
	void	(*GetFileExtEx)(const char* szPathName, char* pszExt, int nBufLen);
	void	(*GetFileNameEx)(const char* szPathName, char* pszName, int nBufLen);
	BOOL	(*IsOpenedFile)(const char* szPathName);

	//opend entry query
	HOpenQuery	(*OpenEntryQuery)(void);
	BOOL	(*ReadOpenedEntry)(HOpenQuery hOpenQuery, FmOpenEntry* pOpenEntryPath);
	BOOL	(*CloseEntryQuery)(HOpenQuery hOpenQuery);


	// Quota
	BOOL (*GetQuotaStat)(const char* szPathName, FmEntryType entryType, FmQuotaStat * pQuotaStat);
};

/*
 * Interface Accessor
 */
// Native/DLL Call
#if (defined(_SHP_SIMUL) && defined(_SHP_CALL_DLL_FILEMGR)) || (defined(_SHP_DLL) && ! defined(_SHP_DLL_FILEMGR) && ! defined(_SHP_CALL_DLL_FILEMGR))
// File
#	define FmDispatch 				(__FmGetVtbl()->Dispatch)
#	define FmCreateFile				(__FmGetVtbl()->CreateFile)
#	define FmOpenFile				(__FmGetVtbl()->OpenFile)
#	define FmCloseFile				(__FmGetVtbl()->CloseFile)
#	define FmReadFile				(__FmGetVtbl()->ReadFile)
#	define FmWriteFile				(__FmGetVtbl()->WriteFile)
#	define FmFlushFile				(__FmGetVtbl()->FlushFile)
#	define FmRemoveFile				(__FmGetVtbl()->RemoveFile)
#	define FmMoveFile				(__FmGetVtbl()->MoveFile)
#	define FmSeekFile				(__FmGetVtbl()->SeekFile)
#	define FmTellFile					(__FmGetVtbl()->TellFile)
#	define FmRewindFile				(__FmGetVtbl()->RewindFile)
#	define FmGetFileAttributes		(__FmGetVtbl()->GetFileAttributes)
#	define FmFGetFileAttributes		(__FmGetVtbl()->FGetFileAttributes)
#	define FmSetFileAttributes		(__FmGetVtbl()->SetFileAttributes)
#	define FmSetFileTime				(__FmGetVtbl()->SetFileTime)
#	define FmGetFileExt(szPathName, pszExt)	\
			(__FmGetVtbl()->GetFileExtEx(szPathName, pszExt, sizeof(pszExt)))
#	define FmCopyFile				(__FmGetVtbl()->CopyFile)

// Directory
#	define FmCreateDir				(__FmGetVtbl()->CreateDir)
#	define FmOpenDir					(__FmGetVtbl()->OpenDir)
#	define FmCloseDir				(__FmGetVtbl()->CloseDir)
#	define FmRemoveDir				(__FmGetVtbl()->RemoveDir)
#	define FmRemoveDirEx			(__FmGetVtbl()->RemoveDirEx)
#	define FmReadDir					(__FmGetVtbl()->ReadDir)
#	define FmGetCurrentDir			(__FmGetVtbl()->GetCurrentDir)
#	define FmChangeDir				(__FmGetVtbl()->ChangeDir)

//opend entry query
#	define FmOpenEntryQuery			(__FmGetVtbl()->OpenEntryQuery);
#	define FmReadOpenedEntry		(__FmGetVtbl()->ReadOpenedEntry);
#	define FmCloseEntryQuery		(__FmGetVtbl()->CloseEntryQuery);

// Quota
#	define FmGetQuotaStat			(__FmGetVtbl()->GetQuotaStat)

// Util
#	define FmGetFileName(szPathName, pszName) \
				(__FmGetVtbl()->GetFileNameEx(szPathName, pszName, sizeof(pszName)))
#	define FmTruncateFile			(__FmGetVtbl()->TruncateFile)
#	define FmIsOpendFile			(__FmGetVtbl()->IsOpenedFile)

#endif // _SHP_DLL

/*
 * Virtual Function Table type. Structure for Mount System
 */
typedef struct
{
	BOOL	bUsed;

	BOOL	(*Initialize)(void);
	HFile	(*CreateFile)(const char* szPathName, ULONG mode, FmPartitionType partition);
	HFile	(*OpenFile)(const char* szPathName, ULONG mode, FmPartitionType partition);
	BOOL	(*CloseFile)(HFile hFile, FmPartitionType partition);
	BOOL	(*ReadFile)(HFile hFile, void* pBuffer, ULONG bufLen, ULONG* pReadLen, FmPartitionType partition);
	BOOL	(*WriteFile)(HFile hFile, void* pBuffer, ULONG bufLen, FmPartitionType partition);
	BOOL	(*FlushFile)(HFile hFile, FmPartitionType partition);
	BOOL	(*RemoveFile)(const char* szPathName, FmPartitionType partition);
	BOOL	(*MoveFile)(const char* szOldPathName, const char* szNewPathName, FmPartitionType partition);
	BOOL	(*SeekFile)(HFile hFile, long position, long offset, FmPartitionType partition);
	long	(*TellFile)(HFile hFile, FmPartitionType partition);
	BOOL	(*RewindFile)(HFile hFile, FmPartitionType partition);
	BOOL	(*GetFileAttributes)(const char* szPathName, FmFileAttribute* pFileAttr, FmPartitionType partition);
	BOOL	(*FGetFileAttributes)(const char* szPathName, HObj hObj, FmFileAttribute* pFileAttr, FmPartitionType partition);	
	BOOL	(*SetFileAttributes)(const char* szPathName, ULONG dwFlag, FmPartitionType partition);
	BOOL	(*SetFileTime)(const char* szPathName, TmDateTime dateTime);
	BOOL	(*TruncateFile)(HFile hFile, ULONG length, FmPartitionType partition);

	// Directory
	BOOL	(*CreateDir)(const char* szDirName, FmPartitionType partition);
	HDir	(*OpenDir)(const char* szDirName, FmPartitionType partition);
	BOOL	(*CloseDir)(HDir hDir, FmPartitionType partition);
	BOOL	(*RemoveDir)(const char* szDirName, FmPartitionType partition);
	BOOL	(*ReadDir)(HDir hDir, FmDirEntry* pEntry, FmPartitionType partition);
	BOOL	(*RewindDir)(HDir hDir, FmPartitionType partition);

	//opend entry query
	HOpenQuery	(*OpenEntryQuery)(void);
	BOOL	(*ReadOpenedEntry)(HOpenQuery hOpenQuery, FmOpenEntry* pOpenEntryPath);
	BOOL	(*CloseEntryQuery)(HOpenQuery hOpenQuery);

	//Quota
	BOOL (*GetQuotaStat)(const char* szPathName, FmEntryType entryType, FmQuotaStat * pQuotaStat);
	INT32	(*IoCtl)(const char *psDeviceName, UINT32  dwCommand, void *pBuff, FmFsType fsType);
} FmVFuncs;

typedef struct 
{
	BOOL bUsed;
	FmFileAttribute fileAttr;
	BOOL bDirty;
} FmAttributeCache;

/*
 * ECOM Type
 */
typedef struct
{
	DECLARE_ECOM_VTBL(EFileMgr)
	UINT32			refCount;

} TFileMgr;

extern TFileMgr fileMgr;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif	// _FM_ECOM_H_
