/*
 * Mocha
 * Copyright (c) 2002 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */

/*:Associate with "Time Manager" */

#ifndef	_TIME_ECOM_H_
#define _TIME_ECOM_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * ECOM Interface
 */
ECOM_INTERFACE(ETimeMgr)
{
	DECLARE_EBASE()

	void	(*ResetElapsedTime)(void);
	ULONG	(*GetElapsedTime)(void);
	void	(*SetTime)(UINT32 seconds);
	UINT32	(*GetTime)(void);
	BOOL	(*GetUptime)(TmMilliSec* pMilliSec);
	BOOL	(*GetMilliSeconds)(TmMilliSec* pMilliSec);

	BOOL	(*GetTimeZoneInfo)(TmTimeZone* pTZ);
	BOOL	(*SetTimeZoneInfo)(const TmTimeZone* pTZ);
	int		(*GetTimeZoneOffset)(void);
	BOOL	(*SetTimeZoneOffset)(int offsetMinute);
	int		(*GetDayLightSavingOffset)(void);
	BOOL	(*SetDayLightSavingOffset)(int offsetMinute);
	UINT32	(*GetLocalTime)(void);
	BOOL	(*SetLocalTime)(const TmDateTime* pDateTime);
	UINT32	(*ConvertUTC2Local)(UINT32 seconds, INT32 timezoneOffset, INT32 daylightSavingOffset);
	UINT32	(*ConvertLocal2UTC)(UINT32 seconds, INT32 timezoneOffset, INT32 daylightSavingOffset);

	BOOL	(*SecondsToDateTime)(UINT32 seconds, TmDateTime* pDateTime);
	UINT32	(*DateTimeToSeconds)(const TmDateTime* pDateTime);
	UINT32	(*DateToDays)(const TmDate* pDate);
	BOOL	(*DaysToDate)(UINT32 days, TmDate* pDate);
	int		(*GetDayOfWeek)(const TmDate* pDate);
	BOOL	(*AdjustDate)(TmDate* pDate, long adjustment);
	BOOL	(*AdjustTime)(TmDateTime* pDateTime, long adjustment);
	BOOL	(*ProcessAlarm)(UINT32 second);
};

/*
 * Interface Accessor
 */
// Native/DLL Call
#if (defined(_SHP_SIMUL) && defined(_SHP_CALL_DLL_TIMEMGR)) || (defined(_SHP_DLL) && ! defined(_SHP_DLL_TIMEMGR) && ! defined(_SHP_CALL_DLL_TIMEMGR))
#	define TmDispatch				(__TmGetVtbl()->Dispatch)

#	define TmResetElapsedTime		(__TmGetVtbl()->ResetElapsedTime)
#	define TmGetElapsedTime			(__TmGetVtbl()->GetElapsedTime)
#	define TmGetTime				(__TmGetVtbl()->GetTime)
#	define TmSetTime				(__TmGetVtbl()->SetTime)
#	define TmGetUptime				(__TmGetVtbl()->GetUptime)
#	define TmGetMilliSeconds		(__TmGetVtbl()->GetMilliSeconds)

#	define TmGetTimeZoneInfo		(__TmGetVtbl()->GetTimeZoneInfo)
#	define TmSetTimeZoneInfo		(__TmGetVtbl()->SetTimeZoneInfo)
#	define TmGetLocalTime			(__TmGetVtbl()->GetLocalTime)
#	define TmSetLocalTime			(__TmGetVtbl()->SetLocalTime)
#	define TmConvertLocal2UTC		(__TmGetVtbl()->ConvertLocal2UTC)
#	define TmConvertUTC2Local		(__TmGetVtbl()->ConvertUTC2Local)

#	define TmGetTimeZoneOffset		(__TmGetVtbl()->GetTimeZoneOffset)
#	define TmSetTimeZoneOffset		(__TmGetVtbl()->SetTimeZoneOffset)
#	define TmGetDayLightSavingOffset	(__TmGetVtbl()->GetDayLightSavingOffset)
#	define TmSetDayLightSavingOffset	(__TmGetVtbl()->SetDayLightSavingOffset)

#	define TmSecondsToDateTime		(__TmGetVtbl()->SecondsToDateTime)
#	define TmDateTimeToSeconds		(__TmGetVtbl()->DateTimeToSeconds)
#	define TmDateToDays				(__TmGetVtbl()->DateToDays)
#	define TmDaysToDate				(__TmGetVtbl()->DaysToDate)
#	define TmGetDayOfWeek			(__TmGetVtbl()->GetDayOfWeek)
#	define TmAdjustDate				(__TmGetVtbl()->AdjustDate)
#	define TmAdjustTime				(__TmGetVtbl()->AdjustTime)
#	define TmProcessAlarm			(__TmGetVtbl()->ProcessAlarm)

#endif // _SHP_DLL


/*
 * ECOM Type
 */
typedef struct
{
	DECLARE_ECOM_VTBL(ETimeMgr)
	UINT32		refCount;

} TTimeMgr;

extern TTimeMgr	timeMgr;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif	// _TIME_ECOM_H_
