/*
 * Samsung Handset Platform
 * Copyright (c) 2000 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */

/*:Associate with "List Utility" */

#ifndef _UTY_LIST_H
#define _UTY_LIST_H

#ifdef __cplusplus
   extern "C" {
#endif

/*
 * List Type
 */
typedef enum
{
	LIST_TYPE_STATIC = 1,
	LIST_TYPE_DYNAMIC
} ListType;


#include "UtyNode.h"

/*
 * Dynamic List
 */
typedef struct _List	DList;

/*
 * Dynamic List Manipulation Functions
 */
BOOL	ListInit				(List* pList);
BOOL	ListIsEmpty				(List* pList);
BOOL	ListIsFull				(List* pList);
int		ListGetCount			(List* pList);
BOOL	ListAddObjectEx			(List* pList, void* pObject, const char* szFileName, int nLine);
#define ListAddObject(pList, pObject)		ListAddObjectEx(pList, pObject, __FILE__, __LINE__)
BOOL	ListAppendObjectEx		(List* pList, void* pObject, const char* szFileName, int nLine);
#define ListAppendObject(pList, pObject)		ListAppendObjectEx(pList, pObject, __FILE__, __LINE__)
BOOL	ListDeleteObject		(List* pList, void* pObject);
void*	ListDeleteFirstObject	(List* pList);
void*	ListDeleteLastObject	(List* pList);
void*	ListDeleteNthObject		(List* pList, int index);
BOOL	ListDeleteAllObject		(List* pList);
void*	ListGetFirstObject		(List* pList);
void*	ListGetLastObject		(List* pList);
void*	ListGetNthObject		(List* pList, int index);
int		ListGetObjectIndex		(List* pList, void* pObject);
BOOL	ListInsertObjectEx		(List* pList, void* pObject, int index, const char* szFileName, int nLine);
#define ListInsertObject(pList, pObject, index)		ListInsertObjectEx(pList, pObject, index, __FILE__, __LINE__)

/*
 * Static List Manipulation Functions
 */
BOOL	SListInit				(SList* pList, Node* pFreeNodes, int freeNodeCount);
BOOL	SListIsEmpty			(SList* pList);
BOOL	SListIsFull				(SList* pList);
int		SListGetCount			(SList* pList);
BOOL	SListAddObject			(SList* pList, void* pObject);
BOOL	SListAppendObject		(SList* pList, void* pObject);
BOOL	SListDeleteObject		(SList* pList, void* pObject);
void*	SListDeleteFirstObject	(SList* pList);
void*	SListDeleteLastObject	(SList* pList);
void*	SListDeleteNthObject	(SList* pList, int index);
BOOL	SListDeleteAllObject	(SList* pList);
void*	SListGetFirstObject		(SList* pList);
void*	SListGetLastObject		(SList* pList);
void*	SListGetNthObject		(SList* pList, int index);
int		SListGetObjectIndex		(SList* pList, void* pObject);
BOOL	SListInsertObject		(SList* pList, void* pObject, int index);

#ifdef __cplusplus
   }
#endif

#endif /* _UTY_LIST_H */
