/*
 * Samsung Handset Platform
 * Copyright (c) 2000 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */

/*:Associate with "Node Utility" */

#ifndef _UTY_NODE_H
#define _UTY_NODE_H

#ifdef __cplusplus
   extern "C" {
#endif
#define _LIST_DOUBLE

typedef struct _Node
{
	struct _Node*	pNext;
#ifdef _LIST_DOUBLE
	struct _Node*	pPrev;
#endif
	void*			pObject;
} Node;

/*
 * List
 */
typedef struct _List
{
	ListType	type;
	Node*		pNodes;
	int			nodeCount;
} List;

/*
 * Static List
 */
typedef struct _SList
{
	ListType	type;			/* List Type : Static or Dynamic */
	Node*		pNodes;
	int			nodeCount;
	Node*		pFreeNodes;
	int			freeNodeCount;
} SList;

/*
 * Node Manipulation Functions
 * Internal Function Prototypes
 */
Node*	ListNewNodeEx				(List* pList, const char* szFileName, int nLine);
#define ListNewNode(pList)		ListNewNodeEx(pList, __FILE__, __LINE__)

BOOL	ListAddNode				(List* pList, Node* pNode);
BOOL	ListAppendNode			(List* pList, Node* pNode);
BOOL	ListInsertNode			(List* pList, Node* pNode, int index);
Node*	ListGetNode				(List* pList, void* pObject);
Node*	ListGetFirstNode		(List* pList);
Node*	ListGetLastNode			(List* pList);
Node*	ListGetNthNode			(List* pList, int index);
BOOL	ListDeleteNode			(List* pList, Node* pNode);

/*
 * Node Manipulation Functions for Static List
 */
Node*	SListNewNode			(SList* pList);
Node*	SListNewNode			(SList* pList);
BOOL	SListAddNode			(SList* pList, Node* pNode);
BOOL	SListAppendNode			(SList* pList, Node* pNode);
BOOL	SListInsertNode			(SList* pList, Node* pNode, int index);
Node*	SListGetNode			(SList* pList, void* pObject);
Node*	SListGetFirstNode		(SList* pList);
Node*	SListGetLastNode		(SList* pList);
Node*	SListGetNthNode			(SList* pList, int index);
BOOL	SListDeleteNode			(SList* pList, Node* pNode);
BOOL	SListAddFreeNode		(SList* pList, Node* pNode);
BOOL	SListAppendFreeNode		(SList* pList, Node* pNode);
Node*	SListGetFreeNode		(SList* pList, void* pObject);
Node*	SListGetFirstFreeNode	(SList* pList);
Node*	SListGetLastFreeNode	(SList* pList);
BOOL	SListDeleteFreeNode		(SList* pList, Node* pNode);

#ifdef __cplusplus
   }
#endif

#endif /* _UTY_NODE_H */
