/*
 * SHP (Samsung Handset Platform)
 * Copyright (c) 2001, 2002 ITC, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
 /**
 * @file		ProtoWLAN.h
 * @brief		This file defines Protocol Manager for WLAN.
 * @version	1.0
 */
#ifndef _PROTO_WLAN_H_
#define _PROTO_WLAN_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "Shp.h"


#define PROTO_WLAN_CONTEXT_MAX			1
#define PROTO_WLAN_ACCOUNT_HANDLE		0
#define PROTO_WLAN_ACCOUNT_NAME			"IMS_DMS"
#define PROTO_WLAN_CONTEXT_ID			(0xFFFFFFF-1)
#define PROTO_MAC_ADDR_LEN_MAX			6		/**< Definition for WLAN 48-bit MAC address */

#define PROTO_WLAN_DHCP_EVENT_UP 		0 	/**< Event form QC WLAN - IP address allocation complete*/
#define PROTO_WLAN_DHCP_EVENT_DOWN 		1	/**< Event form QC WLAN - IP address allocation complete or IP address lost by some error*/

#define PROTO_WLAN_DHCP_ERROR_NONE 0			/**< Error cause QC WLAN - No error*/
#define PROTO_WLAN_DHCP_ERROR_UNSPECIFIED 1		/**< Error cause QC WLAN - unspecified error*/


typedef UINT8		MacAddr;

typedef enum
{	
	PROTO_WLAN_MODE_NONE,
	PROTO_WLAN_MODE_ADHOC,
	PROTO_WLAN_MODE_INFRA
}ProtoWlanMode;

/* DHCP information on WLAN */
typedef struct
{
	BOOL		bDhcpEnabled;					/**< Use DHCP or static configuration */
	IPV4Addr	localAddr;						/**< Local address*/
	IPV4Addr	dnsAddr1;						/**< Primary DNS address*/
	IPV4Addr	dnsAddr2;						/**< Secondary DNS address*/
	IPV4Addr	subnetMask;						/**< Subnet mask*/
	IPV4Addr	defaultGateway;					/**< Default gateway*/
	MacAddr		mac[PROTO_MAC_ADDR_LEN_MAX];	/**< 48-bit MAC address */
	BOOL		bAutoIPEnable;
} ProtoIpcfgInfo;

typedef struct
{
	IPV4Addr	ip;
	char 		szMacAddr[PROTO_MAC_ADDR_LEN_MAX];
	ULONG 		nMacAddrlen;
} ProtoArpCbkInfo;

typedef enum
{
	PROTO_WLAN_EVENT_CONNECTED			=1, 
	PROTO_WLAN_EVENT_CONNECTION_FAIL	=2,
	PROTO_WLAN_EVENT_DISCONNECTED		=3,
	PROTO_WLAN_EVENT_ARP_SUCCESS,
	PROTO_WLAN_EVENT_ARP_FAIL,

/* Not used. */	
	PROTO_WLAN_EVENT_LINK_DOWN,
	PROTO_WLAN_EVENT_LINK_UP,
	PROTO_WLAN_EVENT_DHCP_UP,
	PROTO_WLAN_EVENT_DHCP_DOWN,
	PROTO_WLAN_EVENT_DHCP_RENEW	
} ProtoWlanEvent;


BOOL	ProtoStartWLAN(HProtocol hProtocol, ProtoIpcfgInfo* pWlanInfo);
BOOL	ProtoStopWLAN(HProtocol hProtocol);
BOOL	ProtoStartUMTS(HProtocol hProtocol);
BOOL	ProtoStopUMTS(HProtocol hProtocol);
BOOL	ProtoSetImsHandle(HProtocol hProtocol);
BOOL	ProtoGetWlanInfo(ProtoIpcfgInfo* pWlanInfo);
BOOL	ProtoIsWlanConnected(void);
BOOL	ProtoIsWlanDomain(HProtocol hProtocol);
BOOL	ProtoIsImsRegistered(void);
BOOL	ProtoIsImsAvailable(HProtocol hProtocol);
BOOL	ProtoIsDmsHandover(void);
BOOL	ProtoIsUmtsLinkIdle(void);
BOOL	ProtoIsWlanLinkIdle(void);

#ifdef __cplusplus
}
#endif  /* __cplusplus */

#endif // _PROTO_WLAN_H_

