/*
 * Mocha
 * Copyright (c) 2001, 2002 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/**
 * @file		Socket.h
 * @brief		This file defines the Socket API.
 * @version	1.0
 * @addtogroup	SEC_COMM_SOCKET
 * @{
 */

#ifndef _SOCKET_H_
#define _SOCKET_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#include "Shp.h"
#include "ProtoMgr.h"

#ifdef _SHP_PROTO_SSL

typedef enum
{
	Socket_SSLv3 = 0x0300,
	Socket_TLSv1 = 0x0301
}SocketSecureVer;

typedef enum
{
	Socket_SslCertContinue = 0x0000,
	Socket_SslCertStop = 0x0001
}SocketSslCertFailure;

typedef enum
{	
	SOCKET_CS_ID_NONE = 0x0000,						// CS_ID_NULL_NULL_NULL					0x0000
	SOCKET_CS_ID_ANY = 0xFFFF,							// SSL_ANY_CIPHER_SUITES				0xFFFF
	SOCKET_CS_ID_RSA_DES_40_CBC_SHA = 0x0008,			// CS_ID_RSA_DES_40_CBC_SHA				0x0008// exportable
	SOCKET_CS_ID_RSA_DES_64_CBC_SHA = 0x0009,			// CS_ID_RSA_DES_64_CBC_SHA				0x0009
	SOCKET_CS_ID_RSA_DES_192_CBC3_SHA = 0x000a,		// CS_ID_RSA_DES_192_CBC3_SHA			0x000a
	SOCKET_CS_ID_DH_RSA_DES_40_CBC_SHA = 0x000e,		// CS_ID_DH_RSA_DES_40_CBC_SHA			0x000e// exportable
	SOCKET_CS_ID_DH_RSA_DES_64_CBC_SHA = 0x000f,		// CS_ID_DH_RSA_DES_64_CBC_SHA			0x000f
	SOCKET_CS_ID_DH_RSA_DES_192_CBC3_SHA = 0x0010,		// CS_ID_DH_RSA_DES_192_CBC3_SHA		0x0010
	SOCKET_CS_ID_EDH_RSA_DES_40_CBC_SHA = 0x0014,		// CS_ID_EDH_RSA_DES_40_CBC_SHA			0x0014// exportable
	SOCKET_CS_ID_EDH_RSA_DES_64_CBC_SHA = 0x0015,		// CS_ID_EDH_RSA_DES_64_CBC_SHA			0x0015
	SOCKET_CS_ID_EDH_RSA_DES_192_CBC3_SHA = 0x0016,	// CS_ID_EDH_RSA_DES_192_CBC3_SHA		0x0016
	SOCKET_CS_ID_ADH_DES_40_CBC_SHA = 0x0019,			// CS_ID_ADH_DES_40_CBC_SHA				0x0019
	SOCKET_CS_ID_ADH_DES_64_CBC_SHA = 0x001a,			// CS_ID_ADH_DES_64_CBC_SHA				0x001a
	SOCKET_CS_ID_ADH_DES_192_CBC_SHA = 0x001b,			// CS_ID_ADH_DES_192_CBC_SHA			0x001b
	SOCKET_CS_ID_RSA_SEED_CBC_MD5 = 0xff01,  // INITECH
	SOCKET_CS_ID_RSA_SEED_CBC_SHA = 0xff02,  // INITECH
	SOCKET_CS_ID_ECDH_ECDSA_3DES_EDE_CBC_SHA = 0x004a, // CS_ID_ECDH_ECDSA_3DES_EDE_CBC_SHA	0x004a
	SOCKET_CS_ID_ECDH_RSA_3DES_EDE_CBC_SHA = 0x0052,   // CS_ID_ECDH_RSA_3DES_EDE_CBC_SHA		0x0052
	SOCKET_CS_ID_ECDH_anon_RC4_128_SHA = 0x0058,       // CS_ID_ECDH_anon_RC4_128_SHA			0x0058
	SOCKET_CS_ID_ECDH_anon_3DES_EDE_CBC_SHA = 0x005a   // CS_ID_ECDH_anon_3DES_EDE_CBC_SHA		0x005a
} SocketSslCipherSuiteID;

#endif

#ifndef INADDR_ANY
#	define INADDR_ANY		(IPV4Addr)0x00000000
#endif /* INADDR_ANY */

#ifndef INADDR_NONE
#	define INADDR_NONE		(IPV4Addr)0xffffffff
#endif /* INADDR_NONE */

#define SOCKET_RESULT_ERROR			(-1)

#define SOCKET_NONEXIST				(SOCKET_COUNT_MAX + 1)
#define HOST_NAME_LEN_MAX			128
#define SOCKET_FD_SETSIZE			64

/**
* @def SOCKET_IPADDRESS_MAX	
*/
#define SOCKET_IPADDRESS_MAX		16

typedef void (*SocketEventCallback)(HSocket hSocket, int type, int event, int errorCode);
typedef	HObj	HDnsTr;

/* For High Speed of download, Added by HS.Kim. 07/03/2007. San Diego*/
typedef void (*SocketPollingCallback)(HSocket hSocket, char* pData, int size, int errorCode);

/**
* @enum  SocketState	
*/
typedef enum
{
	SOCKET_STATE_NONE,
	SOCKET_STATE_USED,
	SOCKET_STATE_CLOSING
} SocketState;

/**
 * This struct is SocketInfo.
 */
typedef struct
{
	SocketState			sockState;  /**< Socket State */
	int					sockDesc;				/**< Socket descriptor, which is socket table index */
	int					refSockDesc;		/**< refSockDesc */
	HEventHandler		hEventHandler;	/**< Event handler to receive socket events. */
	SocketEventCallback	eventInd;		/**< User callback to receive socket events. */
	int					message;				/**< message */
	int					protocol;				/**< protocol */
	HProtocol			hProtocol;	  /**< hProtocol */
	DID					did;					  /**< did*/

	HTimer				hPollingTimer;	/**< For High Speed of download, Added by HS.Kim. 07/03/2007. San Diego*/
	BOOL				bUsingPolling;	  /**< For High Speed of download, Added by HS.Kim. 07/03/2007. San Diego*/
	char*				pAppBuf;					/**< Application buffer which would be set by setting socket option.*/
	int					appEventSet;	/**< Event set application is using */
	int					bufSize;			/**< Application buffer size which would be set by setting socket option.*/
	SocketPollingCallback	pollingCB;/**< pollingCB */
} SocketInfo;

#define SOCKET_POLLING_TIMER_INTERVAL		5	/*5ms is minmum limit of timer.  For High Speed of download, Added by HS.Kim. 07/03/2007. San Diego*/
#define SOCKET_POLLING_TIMER_NAME_LEN_MAX	32

typedef struct
{
	BOOL				bEnable;
	char*				pBuf;
	int					bufSize;
	int					eventSet;
	SocketPollingCallback	pollingCB;
} SocketPollingCBInfo;


typedef enum
{
	SOCKET_IOCTLCMD_NONE	= -1,
	SOCKET_FIONBIO			= 0x00,
	SOCKET_FIONREAD,
	SOCKET_FIONWRITE,
	SOCKET_SIOCATMARK,
	SOCKET_IOCTLCMD_MAX
} SocketIoctlCmd;

typedef struct
{
	UINT	socketCount;
	HSocket	hSocket[SOCKET_FD_SETSIZE];
} SocketFdSet;

typedef struct
{
	int		sec;		/* Seconds */
	int		usec;		/* Micro seconds */
} SocketTimeval;

typedef struct
{
	USHORT	onoff;		/* option on/off */
	USHORT	linger;		/* linger time */
} SocketLinger;

typedef enum
{
	SOCKET_OPTLEVEL_NONE	= -1,
	SOCKET_IPPROTO_TCP		= 0x00,
	SOCKET_IPPROTO_IP,
	SOCKET_SOL_SOCKET,
	SOCKET_SOL_PROTO,
	SOCKET_OPTLEVEL_MAX
} SocketOptLevel;

typedef enum
{
	SOCKET_OPTNAME_NONE		= -1,
	SOCKET_TCP_NODELAY		= 0x00,	/* SOCKET_IPPROTO_TCP - SocketSetSockOpt()/SocketGetSockOpt() */
	SOCKET_IP_TTL,					/* SOCKET_IPPROTO_IP - SocketSetSockOpt()/SocketGetSockOpt() */
	SOCKET_IP_TOS,					/* SOCKET_IPPROTO_IP - SocketSetSockOpt()/SocketGetSockOpt() */
	SOCKET_SO_LINGER,				/* SOCKET_SOL_SOCKET - SocketSetSockOpt()/SocketGetSockOpt() */
	SOCKET_SO_DONTLINGER,			/* SOCKET_SOL_SOCKET - SocketSetSockOpt()/SocketGetSockOpt() */
	SOCKET_RCVTIMEO,				/* SOCKET_SOL_SOCKET - SocketSetSockOpt() */
	SOCKET_SNDTIMEO,				/* SOCKET_SOL_SOCKET - SocketSetSockOpt() */
	SOCKET_SO_ACCEPTCONN,			/* SOCKET_SOL_SOCKET - SocketGetSockOpt() */
	SOCKET_SO_ERROR,				/* SOCKET_SOL_SOCKET - SocketGetSockOpt() */
	SOCKET_SO_RCVBUF,				/* SOCKET_SOL_SOCKET - SocketGetSockOpt() */
	SOCKET_SO_SNDBUF,				/* SOCKET_SOL_SOCKET - SocketGetSockOpt() */
	SOCKET_SO_MAX_MSG_SIZE,			/* SOCKET_SOL_SOCKET - SocketGetSockOpt() */
	SOCKET_SO_TYPE,					/* SOCKET_SOL_SOCKET - SocketGetSockOpt() */
	SOCKET_SO_PROTOBEARER,			/* SOCKET_SOL_PROTO - SocketSetSockOpt()/SocketGetSockOpt() */
	SOCKET_SO_KEEPALIVE,			/* SOCKET_SOL_SOCKET - SocketSetSockOpt()/SocketGetSockOpt()*/
	SOCKET_SO_REUSEADDR,			/* SOCKET_SOL_SOCKET - SocketSetSockOpt()/SocketGetSockOpt() */

	SOCKET_SO_WS_TRICK,				/* SOCKET_SOL_SOCKET - SocketSetSockOpt() For High Speed of download, Added new socket option about WS trick changed by HS.Kim. 17/07/2007. Sydney*/
	SOCKET_SO_TIMER_POLLING,		/* SOCKET_SOL_SOCKET - SocketSetSockOpt() For High Speed of download, Added by HS.Kim. 07/03/2007. San Diego*/
	SOCKET_SO_IP_ADD_MEMBERSHIP,	/* SOCKET_IPPROTO_IP - SocketSetSockOpt() For multicast joining by sy3280.lee */
	SOCKET_SO_IP_DROP_MEMBERSHIP,	/* SOCKET_IPPROTO_IP - SocketSetSockOpt() For multicast leaving by sy3280.lee */

	SOCKET_SO_SSLVERSION,				/* SOCKET_SOL_SOCKET - SocketSetSockOpt()/SocketGetSockOpt() */
	SOCKET_SO_SSLSESSIONLIFETIME,		/* SOCKET_SOL_SOCKET - SocketSetSockOpt() */
	SOCKET_SO_SSLCONNECTIONWAITTIME,	/* SOCKET_SOL_SOCKET - SocketSetSockOpt() */
	SOCKET_SO_SSLCONNECTIONSTATE,		/* SOCKET_SOL_SOCKET - SocketGetSockOpt() */
	SOCKET_SO_SSLCIPHERSUITENAME,		/* SOCKET_SOL_SOCKET - SocketGetSockOpt() */
	SOCKET_SO_SSLCIPHERSUITEID,			/* SOCKET_SOL_SOCKET - SocketSetSockOpt()/SocketGetSockOpt() */
	SOCKET_SO_SSLCERTVERIFY,			/* SOCKET_SOL_SOCKET - SocketSetSockOpt() */ /* Changsoon, 20080312 */
	SOCKET_OPTNAME_MAX
} SocketOptName;


#define SOCKET_FD_CLR(fd, set) do {										\
	UCHAR __i;															\
	for (__i = 0; __i < ((SocketFdSet *)(set))->socketCount ; __i++)	\
	{																	\
		if (((SocketFdSet *)(set))->hSocket[__i] == fd)					\
		{																\
            while (__i < ((SocketFdSet *)(set))->socketCount-1)			\
			{															\
                ((SocketFdSet *)(set))->hSocket[__i] =					\
                    ((SocketFdSet *)(set))->hSocket[__i+1];				\
                __i++;													\
            }															\
            ((SocketFdSet *)(set))->socketCount--;						\
            break;														\
        }																\
    }																	\
} while(0)


#define SOCKET_FD_SET(fd, set) do {										\
	if (((SocketFdSet *)(set))->socketCount < SDP_FD_SETSIZE)			\
	{																	\
		((SocketFdSet *)(set))->hSocket[((SocketFdSet *)(set))->socketCount++]=(fd);	\
		SysDebug((MID_SOCKET, "SOCKET_FD_SET: Set info(Socket:%d Count:%d).\n", fd, ((SocketFdSet *)(set))->socketCount));\
	}																	\
	else																\
	{																	\
		SysDebug((MID_EXCEPTION, "SOCKET_FD_SET: Reached to MAX(Socket:%d Count:%d).\n", fd, ((SocketFdSet *)(set))->socketCount));\
	}																	\
} while(0)

#define SOCKET_FD_ZERO(set)			AcMemset((SocketFdSet *)(set),0, sizeof(SocketFdSet))
#define SOCKET_FD_ISSET(fd, set)	_SocketFdIsset((UINT)(fd), (SocketFdSet *)(set))


/* Macro to process the DNS request */
#define SOCKET_EVENT_DNS					0x1000
/* structure for Socket DNS APIs, coded by hskim.*/
#define SOCKET_HOSTINFO_LEN_MAX				3
#define SOCKET_HOSTINFO_ALIAS_SIZE_MAX		64

#define SOCKET_DNS_TIMER_NAME_LEN_MAX		32

/**
* @def  SOCKET_DNS_TIMEOUT	
*/
#define SOCKET_DNS_TIMEOUT					6000

/**
* @enum  SocketDnsError	
* DNS error enumeration 
*/
typedef enum
{
	SOCKET_DNS_SUCCESS					= 0x00, /**< SUCCESS */
	SOCKET_DNS_ERROR_INPROGRESS,				/**< INPROGRESS */
	SOCKET_DNS_ERROR_INVALIDSERVER,     /**< INVALIDSERVER */
	SOCKET_DNS_ERROR_INVALIDDOMAIN,		  /**< INVALIDDOMAIN */
	SOCKET_DNS_ERROR_NETWORKDOWN, 			/**< NETWORKDOWN */
	SOCKET_DNS_ERROR_OUTOFMEMORY, 			/**< OUTOFMEMORY */
	SOCKET_DNS_ERROR_INVALIDPARAM, 			/**< INVALIDPARAM */
	SOCKET_DNS_ERROR_TIMEOUT, 					/**< TIMEOUT */
	SOCKET_DNS_ERROR_NORESPONSE, 				/**< NORESPONSE */
	SOCKET_DNS_ERROR_NOTFOUND 					/**< NOTFOUND */
} SocketDnsError;

typedef struct _SocketHostInfo
{
	int			addrCount;
	IPV4Addr	addr[SOCKET_HOSTINFO_LEN_MAX];
	int			aliasCount;
	char		szAlias[SOCKET_HOSTINFO_LEN_MAX][SOCKET_HOSTINFO_ALIAS_SIZE_MAX+1];
}SocketHostInfo;

/* ProtoType of DNS callback function*/
typedef void (*SocketDnsCallback)(HDnsTr hDnsTr, SocketDnsError error, const SocketHostInfo* pHostInfo);

#define DNS_IP_COUNT_MAX					2
#define DNS_MAX_REQ							5
#define DNS_MAX_RETRY						2

#define DNS_RESULT_WIN_SOCK				1
#define DNS_RESULT_FROM_CACHE				2
#define DNS_RESULT_IP_DIRECT				3

typedef enum
{
	DNS_STATE_INVALID					= 0x00,
	DNS_STATE_INIT,
	DNS_STATE_PROCESS,
	DNS_STATE_READ,
	DNS_STATE_WRITE,
	DNS_STATE_WRITEBLOCKED,
	DNS_STATE_CLOSE,
	DNS_STATE_FREE
} SocketDnsReqState;

typedef enum
{
	SOCKET_DNS_SERVER_PRIMARY			= 0x00,
	SOCKET_DNS_SERVER_SECONDARY
} SocketDnsServerState;

typedef struct
{
	HTimer					hDnsTimer;
	ULONG					timeout;
	USHORT					retryCount;
	HSocket					hDnsSock;
	int						dnsReqID;
	SocketDnsReqState		status;
	SocketDnsServerState		serverState;
	IPV4Addr					dnsIpAddr;
	IPV4Addr					resultIP[DNS_IP_COUNT_MAX];
	SocketHostInfo			resultHostInfo;
	BOOL					bIPN;
	BOOL					bIPN2;
	char						szHostName[HOST_NAME_LEN_MAX];
	SocketDnsCallback			dnsResInd;
	DID						did;
} SocketDnsReqInfo;

typedef struct 
{
	HSocket					socket;
	int						resIndex;
	int						event;
	int						errorCode;
	int						action;
} SocketDnsRespInfo;

/* DNS event run with PROTO_DNS_ASYNC. */
typedef enum
{
	SOCKET_DNS_EVENT_REQUEST	= 0x11,	/* changed value because of value conflict with FD_READ event from main frame. */
	SOCKET_DNS_EVENT_RESPONSE	= 0x12,
	SOCKET_DNS_EVENT_MAX
} SocketDnsEvent;

/* DNS query entity */
typedef struct
{
	BOOL					usedFlag;
	int						connID;
	int						dnsReqID;
	int						did;
	SocketHostInfo			resultHostInfo;
	SocketDnsCallback			dnsResInd;
	ProtoEvent				protoEvent;
	int						queryID;		/* Native DNS query ID. Jinyup Kim 20060804. */
	char						szHostName[HOST_NAME_LEN_MAX];	/* Asynchronous DNS query. Jinyup Kim 2006/08/21 */
	/* ---------------------------------------------------------------------*/
	int						PDPProfileNum;	/* Reserved for Multi PDP DNS Query */
	HProtocol					hProtocol;		/* Reserved for Multi PDP DNS Query */
	/* ---------------------------------------------------------------------*/
} SocketDNSEntity;

/* multicast address */
typedef struct 
{
	Socket	multicastAddr; /* IP multicast address of group */
	Socket	interfaceAddr; /* local IP address of interface */
}SocketMReq;

#include "OemSocket.h"

/*
 * ECOM Interface
 */
#include "SocketECOM.h"

#ifdef __cplusplus
}
#endif  /* __cplusplus */

#endif // _SOCKET_H_
