/*
 * Mocha
 * Copyright (c) 2001, 2002 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/**
 * This file defines the Socket API.
 *
 */

#ifndef _SOCKET_ECOM_H_
#define _SOCKET_ECOM_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*
 * ECOM Interface
 */
ECOM_INTERFACE(ESocket)
{
	DECLARE_EBASE()

	/* Set up */
	HSocket	(*Create)(int af, int type, int protocol);
#if defined(_SHP_MULTI_PDP_SUPPORT)
	/*New interface CreateEx is added to ECOM Interface. This is required for mapping corresponding SocketCreateEx function. This is used for Multiple PDP scenario.*/
	HSocket	(*CreateEx)(int af, int type, int protocol,int hProtocol);
#endif
	BOOL	(*Bind)(HSocket hSocket, Socket bindAddr);  
	BOOL	(*Connect)(HSocket hSocket, Socket peerAddr);
	BOOL	(*AsyncSelect)(HSocket hSocket, SocketEventCallback eventInd, int message, int event);
	BOOL	(*AsyncSelectByEventHandler)(HSocket hSocket, char* szEventHandler, int message, int event);
	BOOL	(*Close)(HSocket hSocket);

	/* Data */
	int		(*SendTo)(HSocket hSocket, char* pBuf, int bufLen, UINT32 option, Socket destAddr);
	int		(*RecvFrom)(HSocket hSocket, char* pBuf, int bufLen, UINT32 option, Socket* pSourceAddr);
	int		(*Send)(HSocket hSocket, char* pBuf, int bufLen, UINT32 option);
	int		(*Recv)(HSocket hSocket, char* pBuf, int bufLen, UINT32 option);

	/* Socket option and I/O control*/
	BOOL	(*Ioctl)(HSocket hSocket, SocketIoctlCmd cmd, UINT32* pArg);
	int		(*Select)(int width, SocketFdSet* pReadSet, SocketFdSet* pWriteSet, SocketFdSet* pExceptSet, const SocketTimeval* pTimeout);
	BOOL	(*SetSockOpt)(HSocket hSocket, int level, int optName, const char* pOptVal, int optLen);
	BOOL	(*GetSockOpt)(HSocket hSocket, int level, int optName, char* pOptVal, int* pOptLen);
	BOOL	(*GetSockName)(HSocket hSocket, Socket* pSockName, int* pNameLen);
	BOOL	(*GetPeerName)(HSocket hSocket, Socket* pSockName, int* pNameLen);

	/* DNS */
	BOOL	(*SetDNSServer)(char* szServerAddr);
	HDnsTr	(*GetHostByName)(HProtocol hProtocol, const char* szHostName, SocketDnsCallback dnsResInd);
	HDnsTr	(*GetHostByAddr)(HProtocol hProtocol, const Socket* pAddr, SocketDnsCallback dnsResInd); 

	/* Convert an (Ipv4) Internet network address into a string, vice versa. */
	IPV4Addr	(*InetAddr)(char* szIpAddr);
	void 	(*InetNtoa)(IPV4Addr sourceAddr, char* szIpAddr);
	
	/* Byte order */
	UINT16	(*Htons)(UINT16 hostInt);
	UINT32	(*Htonl)(UINT32 hostInt);
	UINT16	(*Ntohs)(UINT16 netInt);
	UINT32	(*Ntohl)(UINT32 netInt);

	/* Server */
	BOOL	(*Listen)(HSocket hSocket, int backLog);
	HSocket	(*Accept)(HSocket hSocket, Socket* pPeerAddr);

	/* Security */
	BOOL	(*SwitchToTunnel)(HSocket hSocket, Socket peerAddr);
};

typedef struct
{
	DECLARE_ECOM_VTBL(ESocket)
	UINT32				refCount;

} TSocket;

extern TSocket socketEx;

#ifdef __cplusplus
}
#endif  /* __cplusplus */

#endif /* _SOCKET_ECOM_H_ */

