/** 
* Samsung Handset Platform
* Copyright (c) 2006 Samsung Electronics, Inc.
* All rights reserved. 
*
* This software is the confidential and proprietary information
* of Samsung Electronics, Inc. ("Confidential Information").  You
* shall not disclose such Confidential Information and shall use
* it only in accordance with the terms of the license agreement
* you entered into with Samsung Electronics.
*/
/*
/* FILE :
 *   This file defines the porting layer API's for the the components like HTTP
*
 * @name		SFBalRm.h [SHP HTTP interface]
 * @author		Ancil K George (ancilgeorge@samsung.com)
				Basavaraj P S (basavaraj.ps@samsung.com)
				Anshul Saxena (a.saxena@samsung.com)
				Rohit P Shetty (rohit.shetty@samsung.com)
*/


#ifndef _SFBAL_RM_H
#define _SFBAL_RM_H
#include "SFBal.h"
#include "SFBalProto.h"

#define SFB_UNZIP_INFLATE_FACTOR 11
#define SFB_UNZIP_REALLOC_FACTOR 4
#define SFB_UNZIP_RAW_DEFLATE -15
#define SFB_UNZIP_GZIP 31

#ifdef SFB_RM_UNZIP
typedef enum
{
	SFB_UNZIP_STATE_NONE	= 0x00,
	SFB_UNZIP_STATE_PROCESSING,
	SFB_UNZIP_STATE_DONE,
	SFB_UNZIP_STATE_ERROR
} SFBUnzipState;
#endif

/* Methods */
typedef enum
{
	SFB_HTTP_METHOD_GET		= 0x40,
	SFB_HTTP_METHOD_OPTIONS	= 0x41,
	SFB_HTTP_METHOD_HEAD	= 0x42, 
	SFB_HTTP_METHOD_DELETE	= 0x43,
	SFB_HTTP_METHOD_TRACE	= 0x44,
	SFB_HTTP_METHOD_POST		= 0x60,
	SFB_HTTP_METHOD_PUT		= 0x61,
	SFB_HTTP_METHOD_CONNECT	= 0x70,

	SFB_HTTP_METHOD_COUNT	= 8
} SFBMethodType;


typedef enum
{
	SFB_HTTP_CERT_RESUME=0x00,
	SFB_HTTP_CERT_PAUSE,

	SFB_HTTP_DOWNLOAD_RESUME=0x10,
	SFB_HTTP_DOWNLOAD_PAUSE
} SFBHttpMessage;


	/*DATA TYPES*/

typedef enum
{
    SFB_RM_HTTP_STATUS_100 = 100,			/* Continue */
    SFB_RM_HTTP_STATUS_101,					/* Switching Protocols */

    SFB_RM_HTTP_STATUS_200 = 200,			/* OK, Succes */
    SFB_RM_HTTP_STATUS_201,					/* Created */
    SFB_RM_HTTP_STATUS_202,					/* Accepted */
    SFB_RM_HTTP_STATUS_203,					/* Non-Authoritative Information */
    SFB_RM_HTTP_STATUS_204,					/* No Content */
    SFB_RM_HTTP_STATUS_205,					/* Reset Content */
    SFB_RM_HTTP_STATUS_206,					/* Partial Content */

    SFB_RM_HTTP_STATUS_300 = 300,			/* Multiple Choices */
    SFB_RM_HTTP_STATUS_301,					/* Moved Permanently */
    SFB_RM_HTTP_STATUS_302,					/* Found */
    SFB_RM_HTTP_STATUS_303,					/* See Other */
    SFB_RM_HTTP_STATUS_304,					/* Not Modified */
    SFB_RM_HTTP_STATUS_305,					/* Temporary Redirect */
	SFB_RM_HTTP_STATUS_306,					/* Not Modified */
    SFB_RM_HTTP_STATUS_307,					/* Temporary Redirect */

    SFB_RM_HTTP_STATUS_400 = 400,			/* Bad Request */
    SFB_RM_HTTP_STATUS_401,					/* Unauthorized */
    SFB_RM_HTTP_STATUS_402,					/* Payment Required */
    SFB_RM_HTTP_STATUS_403,					/* Forbidden */
    SFB_RM_HTTP_STATUS_404,					/* Not Found */
    SFB_RM_HTTP_STATUS_405,					/* Method Not Allowed */
    SFB_RM_HTTP_STATUS_406,					/* Not Acceptable */
    SFB_RM_HTTP_STATUS_407,					/* Proxy Authentication Required */
    SFB_RM_HTTP_STATUS_408,					/* Request Time-out */
    SFB_RM_HTTP_STATUS_409,					/* Conflict */
    SFB_RM_HTTP_STATUS_410,					/* Gone */
    SFB_RM_HTTP_STATUS_411,					/* Length Required */
    SFB_RM_HTTP_STATUS_412,					/* Precondition Failded */
    SFB_RM_HTTP_STATUS_413,					/* Request Entity Too Large */
    SFB_RM_HTTP_STATUS_414,					/* Request-URI Too Large */
    SFB_RM_HTTP_STATUS_415,					/* Unsupported Media Type */
	SFB_RM_HTTP_STATUS_416,					/*Requested Range Not Satisfiable*/
	SFB_RM_HTTP_STATUS_417,					/*Expectation Failed*/

    SFB_RM_HTTP_STATUS_500 = 500,			/* Internal Server Error */
    SFB_RM_HTTP_STATUS_501,					/* Not Implemented */
    SFB_RM_HTTP_STATUS_502,					/* Bad Gateway */
    SFB_RM_HTTP_STATUS_503,					/* Service Unavailable */
    SFB_RM_HTTP_STATUS_504,					/* Gateway Time-out */
    SFB_RM_HTTP_STATUS_505					/* HTTP Version not supported */
} SFBRmHTTPStatus;

#define SFB_DEFAULT_HTTP_ACCEPT_HEADERS \
		"Accept: text/html, " \
		  "application/xml, " \
		  "image/vnd.wap.wbmp, " \
		  "image/png, " \
		  "image/jpeg, " \
		  "image/gif, " \
		  "image/bmp, " \
		  "application/vnd.wap.xhtml+xml, " \
		  "application/xhtml+xml, " \
		  "application/vnd.wap.multipart.mixed, " \
		  "multipart/mixed, " \
		  "text/vnd.wap.wml, " \
		  "application/vnd.wap.wmlc, " \
		  "application/vnd.oma.dd+xml, " \
		  "text/vnd.sun.j2me.app-descriptor, " \
		  "application/java-archive, " \
		  "*/*\r\n"


#define SFB_DEFAULT_HTTP_WAP_HEADERS_DEFAULT \
		"Accept-Language: en\r\n" \
		"X-Wap-Proxy-Cookie: none\r\n" \
		"Accept-Charset: windows-1252;q=1.0,utf-8;q=1.0,utf-16;q=1.0," \
		  "iso-8859-1;q=0.6,iso-8859-2;q=0.6,iso-8859-3;q=0.6,iso-8859-4;q=0.6," \
		  "iso-8859-5;q=0.6,iso-8859-7;q=0.6,iso-8859-9;q=0.6,iso-8859-10;q=0.6," \
		  "*;q=0.1\r\n"
#ifdef SFB_RM_UNZIP
#define SFB_DEFAULT_HTTP_WAP_HEADERS \
		"Accept-Language: en\r\n" \
		"X-Wap-Proxy-Cookie: none\r\n" \
		"Accept-Charset: utf-8;q=1.0,iso-8859-1;q=0.6\r\n" \
		"Accept-Encoding: gzip,deflate\r\n"
#else
#define SFB_DEFAULT_HTTP_WAP_HEADERS \
		"Accept-Language: en\r\n" \
		"X-Wap-Proxy-Cookie: none\r\n" \
		"Accept-Charset: utf-8;q=1.0,iso-8859-1;q=0.6\r\n"
#endif
#define SFB_DEFAULT_HTTP_ACCEPT_HEADER_IMAGE \
		"image/vnd.wap.wbmp, " \
		"image/png, " \
		"image/jpeg, " \
		"image/gif, " \
		"image/bmp, " \
		"*/*" 

#define SFB_DEFAULT_HTTP_ACCEPT_HEADER_EXT_DOWNLOAD	\
		"image/vnd.wap.wbmp, " \
		"image/png, " \
		"image/jpeg, " \
		"image/gif, " \
		"image/bmp, " \
		"application/vnd.wap.multipart.mixed, " \
		"multipart/mixed, " \
		"application/java-archive, " \
		"*/*"


/*Callbacks for the HTTP responses*/
typedef void (*SFBalRmHttpInd)(int hHttptr, int status, char* szResponseHeader, char* pResponseBody, ULONG responseBodyLen);
typedef void (*SFBalRmHttpAbortInd)(int hHttptr, int reason, BOOL bData );
typedef void (*SFBalRmHttpChunkInd)(int hHttptr, char* pChunk, ULONG chunkLen);
typedef void (*SFBalRmHttpAddChunkInd)(int hHttptr, int recommendedChunkSize );

void SFBRmSSLReq(int hHttpTr, char* pOptionalData);
void SFBalRmSSLConfirm(int hHttpTr, BOOL iConfirm);

/*API for HTTP chunked request*/
int
SFBalRmHttpChunkedReq(char* szUrl, SFBMethodType method, char* szRequestHeader,
	char* pRequestBody,ULONG requestBodyLen, char* szProxyAddr,
	SFBalRmHttpInd httpInd, SFBalRmHttpAbortInd httpAbortInd, SFBalRmHttpChunkInd httpChunkInd, SFBalRmHttpAddChunkInd httpAddChunkInd, SFBalHProtocol hProtocol );

//Anshul


// API for adding chunk

BOOL
SFBalHttpAddChunk(int httpTr, char* pChunk, int chunkLen);
BOOL	
SFBalRmHttpAbortReq(int trId);

//gnana
//-------------------------------------------------------------------------------//
/* Http Pause and Resume for Download  */

BOOL SFBalHttpPauseReq(int trId,  SFBHttpMessage iMessage );

BOOL SFBalHttpResumeReq(int trId, SFBHttpMessage iMessage);
//-------------------------------------------------------------------------------//

int 
_SFBalRmModifyResponseStatus(int status);

char * SFBBalRmGetDefaultAcceptHeaders();
char * SFBBalRmGetDefaultHttpWapHeaders();
char * SFBBalRmGetDefaultAcceptHeaderImage();
char * SFBBalRmGetDefaultAcceptHeaderExtDownload();

#ifdef SFB_RM_UNZIP
SFBStatus SFBalUnCompress (const char *source, int sourceLen, int encType, char **dest, int *destLen, SFBUnzipState *unzipStatus, void **pUnZipStream);
SFBStatus _SFBalRmModifyUnzipErrorStatus(int status);
#endif
#endif
