/*
* ==============================================================================
*  Name        : BrowserControl.hpp
*  Part of     : Browser Control.
*  Description : Defines a browser control interface.
*  Version     :
*
*    Copyright (c) 2007,
*    All rights reserved.
* ==============================================================================
*/

#ifndef __BROWSERCONTROL_HPP__
#define __BROWSERCONTROL_HPP__

#include "BrowserControl_DataTypes.h"
#include "platform.h"
#include "CMMDefs.h"

class BrowserControl CMM_MODIFIED
{
public:
	//------------------------------------------------------------------------
	// WebKit initialize and finalize APIs
	//------------------------------------------------------------------------
	static ESTATUS BrowserControl_WebKit_Initialize(HWIN hMainWin);
	static ESTATUS BrowserControl_WebKit_Finalize();

	//------------------------------------------------------------------------
	// Creation and Destruction API
	//------------------------------------------------------------------------

	static ESTATUS BrowserControl_Init(HANDLE*& aBrowserHandle);
	static ESTATUS BrowserControl_Finalize(HANDLE aBrowserHandle);
	ESTATUS BrowserControl_Create(BrowserWidget** aWidget);
	ESTATUS BrowserControl_Delete();

	//------------------------------------------------------------------------
	// Settings API
	//------------------------------------------------------------------------

	ESTATUS BrowserControl_SetHwnd(const HWIN hWin);
	ESTATUS BrowserControl_SetHttpProxy(const SProxyInfo* aProxyInfo);
	ESTATUS BrowserControl_SetIAP(const IAPHANDLE aSelectedIapHandle);
	ESTATUS BrowserControl_SetCacheSize(const guint aCacheSize);
	ESTATUS BrowserControl_SetProperty(const EBrowserProperty aBrowserProperty,
										const gboolean aSettingValue);
	ESTATUS BrowserControl_GetHttpProxy(SProxyInfo* aProxyInfo);
	ESTATUS BrowserControl_GetIAP(IAPHANDLE* aSelectedIapHandle);
	ESTATUS BrowserControl_GetCacheSize(guint* aCacheSize);
	ESTATUS BrowserControl_GetProperty(const EBrowserProperty aBrowserProperty,
										gboolean* aSettingValue);

	ESTATUS BrowserControl_SetMinimumFontSize(const int aSettingsValue);
	ESTATUS BrowserControl_SetMinimumLogicalFontSize(const int aSettingsValue);
	ESTATUS BrowserControl_SetDefaultFontSize(const int aSettingsValue);
	ESTATUS BrowserControl_SetDefaultFixedFontSize(const int aSettingsValue);

	ESTATUS BrowserControl_SetDefaultTextEncodingName(const gchar *aSettingsValue);
	ESTATUS BrowserControl_SetFtpDirectoryTemplatePath(const gchar *aSettingsValue);
	ESTATUS BrowserControl_SetUserStyleSheetLocation(const gchar *aSettingsValue);
	ESTATUS BrowserControl_SetStandardFontFamily(const gchar *aSettingsValue);
	ESTATUS BrowserControl_SetFixedFontFamily(const gchar *aSettingsValue);
	ESTATUS BrowserControl_SetSerifFontFamily(const gchar *aSettingsValue);
	ESTATUS BrowserControl_SetSansSerifFontFamily(const gchar *aSettingsValue);
	ESTATUS BrowserControl_SetCursiveFontFamily(const gchar *aSettingsValue);
	ESTATUS BrowserControl_SetFantasyFontFamily(const gchar *aSettingsValue);
	ESTATUS BrowserControl_SetEditableLinkBehavior(const WebCore::EditableLinkBehavior aSettingsValue);
	ESTATUS BrowserControl_GetMinimumFontSize(int *  aSettingsValue);
	ESTATUS BrowserControl_GetMinimumLogicalFontSize(int *  aSettingsValue);
	ESTATUS BrowserControl_GetDefaultFontSize(int *  aSettingsValue);
	ESTATUS BrowserControl_GetDefaultFixedFontSize(int *  aSettingsValue);

	ESTATUS BrowserControl_GetDefaultTextEncodingName(gchar *aSettingsValue,gulong *aSize);
	ESTATUS BrowserControl_GetFtpDirectoryTemplatePath(gchar *aSettingsValue,gulong *aSize);
	ESTATUS BrowserControl_GetUserStyleSheetLocation(gchar *aSettingsValue,gulong *aSize);
	ESTATUS BrowserControl_GetStandardFontFamily(gchar *aSettingsValue,gulong *aSize);
	ESTATUS BrowserControl_GetFixedFontFamily(gchar *aSettingsValue,gulong *aSize);
	ESTATUS BrowserControl_GetSerifFontFamily(gchar *aSettingsValue,gulong *aSize);
	ESTATUS BrowserControl_GetSansSerifFontFamily(gchar *aSettingsValue,gulong *aSize);
	ESTATUS BrowserControl_GetCursiveFontFamily(gchar *aSettingsValue,gulong *aSize);
	ESTATUS BrowserControl_GetFantasyFontFamily(gchar *aSettingsValue,gulong *aSize);
	ESTATUS BrowserControl_GetEditableLinkBehavior(WebCore::EditableLinkBehavior *aSettingsValue);
	//------------------------------------------------------------------------
	// Page API
	//------------------------------------------------------------------------

#ifdef SFB_VISUAL_SUPPORT
	ESTATUS BrowserControl::BrowserControl_VisualHistory(WmBitmap** pWmBitmap,void* pUserData);
	//ESTATUS BrowserControl_VisualHistory(WmBitmap** pWmBitmap,void* pUserData);
	#endif

	ESTATUS BrowserControl_SwitchTab();
	ESTATUS BrowserControl_LoadUrl(const gchar* aUrl);
	ESTATUS BrowserControl_LoadFile(const gchar* aPath);
	ESTATUS BrowserControl_LoadData(const gchar* aData);
	ESTATUS BrowserControl_PostUrl(const gchar* aUrl,
									const gchar* aContentType,
									const gchar* aPostData,
									const gchar* aBoundary);
	ESTATUS BrowserControl_GetPageUrl(gchar* aUrl, gulong* aSize);
	ESTATUS BrowserControl_GetPageTitle(gchar* aTitle, gulong* aSize);
	ESTATUS BrowserControl_GetPageMimeType(gchar* aType, gulong* aSize);
	ESTATUS BrowserControl_GetPageSize(gulong* aSize);
	ESTATUS BrowserControl_GetPageContent(gchar* aContent, gulong* aSize);
	ESTATUS BrowserControl_GetPageLastModifiedTime(gchar* aTime, gulong* aSize);	ESTATUS BrowserControl_GetSelectedElementType(EPageItemType* aType);

       /**
 	*   bs0425.lee  modified for touch events 
 	*/

	ESTATUS BrowserControl_GetSelectedElementTypeByPoint(EPageItemType* aType,int x, int y );

	/**
 	*   bs0425.lee  modified for touch events 
 	*/

	ESTATUS BrowserControl_GetSelectedElementInfo(EPageItemType aType, ElementInfoBase* aInfo);
	ESTATUS BrowserControl_GetElementTypeUnderPoint(const guint aX,
													const guint aY,
													EPageItemType* aType);
	ESTATUS BrowserControl_GetItemCount(const EPageItemType aType, guint* aCount);
	ESTATUS BrowserControl_SavePage(const gchar* aPath);
	ESTATUS BrowserControl_SaveElement(const gchar* aPath);

	/**
 	*   bs0425.lee  modified for touch events 
 	*/
	ESTATUS BrowserControl_SaveElementByPoint(const gchar* aPath, int x , int y);
	/**
 	*   bs0425.lee  modified for touch events 
 	*/

	ESTATUS BrowserControl_GetImages(bool bVisibleOnly, ImageSaveInfo*** imageList);
	ESTATUS BrowserControl_GetImageCount(bool bVisibleOnly, guint* imageCount);
	ESTATUS BrowserControl_Reload();
	ESTATUS BrowserControl_StopLoading();
	ESTATUS BrowserControl_Find(const SFindInfo* aFindInfo);
	ESTATUS BrowserControl_FindNext(const SFindInfo* aFindInfo);
	ESTATUS BrowserControl_CanGoBack(gboolean* aRet);
	ESTATUS BrowserControl_CanGoForward(gboolean* aRet);
	ESTATUS BrowserControl_GoBack();
	ESTATUS BrowserControl_GoForward();
	ESTATUS BrowserControl_ZoomIn(const guint aScalePercent);
	ESTATUS BrowserControl_ZoomOut(const guint aScalePercent);
	ESTATUS BrowserControl_GetZoom(guint* aScalePercent);
	ESTATUS BrowserControl_CreatePageThumbnail(guint width, guint height, const gchar *thumbPath);
	ESTATUS BrowserControl_SetTextSize(guint size);
#ifdef SFB_RSS_LINK //vamsi Jan 28 Rss Link Tag Support for SFB.
	ESTATUS BrowserControl::BrowserControl_GetRssLinks(SRssLink** pRssLink, int* rssCount, gchar** url);
#endif

	//------------------------------------------------------------------------
	// Error Handling Interface
	//------------------------------------------------------------------------

	ESTATUS BrowserControl_SetErrorHandler(const EErrorType aType, const ErrorHandler aHandler);
	static void BrowserControl_GetLastError(ERRORCODE *aErrorCode);

	//------------------------------------------------------------------------
	// Observers API
	//------------------------------------------------------------------------

	ESTATUS BrowserControl_SetLoadHandler(const LoadHandler aHandler);

	//------------------------------------------------------------------------
	// General API
	//------------------------------------------------------------------------

	ESTATUS BrowserControl_GetVersion(gchar* aVersionInfo, gulong* aBufferSize);
	ESTATUS BrowserControl_GetCertificateInfo(SCertInfo* aCertInfo);
	ESTATUS BrowserControl_GetBrowserParameter(const EBrowserParameters
												aBrowserParameter,gchar* aParameterValue,
												gulong* aBufferSize);
	ESTATUS BrowserControl_SetBrowserParameter(const EBrowserParameters aBrowserParameter,
												const gchar* aParameterValue);

	ESTATUS BrowserControl_Paint( void* pUserData);

#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
	ESTATUS BrowserControl_SetPreview();
	ESTATUS BrowserControl_IsPreview(bool* bPreview);
#endif

	ESTATUS BrowserControl_HandleKeyEvent(  int keyCode, bool isKeyUp );

/**
 *   bs0425.lee  modified for touch events 
 */
	ESTATUS BrowserControl_HandleMouseEvent( int rectX, int rectY, int MouseEventType );
	ESTATUS BrowserControl_PaneScrollBy( int in_offsetX, int in_offsetY);
	gboolean  BrowserControl_IsFocusedByPoint( int pointX, int pointY);
	gboolean  BrowserControl_IsLink( int rectX, int rectY);
	gboolean BrowserControl_IsImageElement(int rectX, int rectY);
       ESTATUS BrowserControl_GetSelectedElementInfoByPoint(EPageItemType aType, ElementInfoBase* aInfo,int pointX, int pointY );


/**
 *   bs0425.lee  modified for touch events 
 */
#if defined SHP_SUPPORT_WML
	void BrowserControl_FetchRequest () ;
	int BrowserControl_WmlItemCount () ;
	int BrowserControl_WmlDoElement( int ) ;
#endif //SHP_SUPPORT_WML
#ifdef __SHP__
	void BrowserControl_SetProgressHandler(ProgressHandler aProgressHandler);
	void BrowserControl_SetTitleChangedHandler(TitleChangedHandler aTitleChangedHandler);
	void BrowserControl_SetLinkHoveringHandler(LinkHoveringHandler aLinkHoveringHandler);
	void BrowserControl_SetStatusbarTextChangedHandler(StatusbarTextChangedHandler aStatusbarTextChangedHandler);
	void BrowserControl_SetIconLoadedHandler(IconLoadedHandler aIconLoadedHandler);
	void BrowserControl_SetSelectionChangedHandler(SelectionChangedHandler aSelectionChangedHandler);
	void BrowserControl_SetUrlSchemeHandler(UrlSchemeHandler aHandler);
	void BrowserControl_SetUIStateHandler(UIStateHandler aHandler);

	//vamsi relayout
	void BrowserControl_ReLayout() ;

#endif
	//------------------------------------------------------------------------
	// Data members.
	//------------------------------------------------------------------------

	WebKitPage* mPage;

	HGC mGC;

	//------------------------------------------------------------------------
	// Internal functions.
	//------------------------------------------------------------------------
#ifdef SANITY_CHECK
	static gboolean IsValidHandle(HANDLE aBrowserHandle);
#endif // SANITY_CHECK
	static void SetLastError(ERRORCODE aErrorCode);

private:
	static ERRORCODE lastError;
#ifdef SANITY_CHECK
	static GSList* handleList;
#endif // SANITY_CHECK
};

#endif //__BROWSERCONTROL_HPP__


