/*
* ==============================================================================
*  Name        : BrowserControl_Interface.c
*  Part of     : Browser Control.
*  Description : Defines a browser control interface.
*  Version     :
*
*    Copyright (c) 2007,
*    All rights reserved.
* ==============================================================================
*/

#include "BrowserControl_Interface.h"
#include "BrowserControl.hpp"
#ifdef __SHP__
#include "BrowserSettings.h"
#endif //#ifdef __SHP__
/**
 * @defgroup Init Creation and Destruction
 */
/**
 * @defgroup Page Page related APIs
 */
/**
 * @defgroup Observer Observer related APIs
 */
/**
 * @defgroup Settings Setting related APIs
 */
/**
 * @defgroup Error Error handling related APIs
 */
/**
 * @defgroup ContextMenu Context Menu related APIs
 */
/**
 * @defgroup General General
 */



//------------------------------------------------------------------------
// WebKit initialize and finalize APIs
//------------------------------------------------------------------------
ESTATUS BrowserControl_WebKit_Initialize(HWIN hMainWin)
{
	return BrowserControl::BrowserControl_WebKit_Initialize(hMainWin);
}

ESTATUS BrowserControl_WebKit_Finalize()
{
	return BrowserControl::BrowserControl_WebKit_Finalize();
}

//------------------------------------------------------------------------
// Creation and Destruction API
//------------------------------------------------------------------------

ESTATUS BrowserControl_Init(HANDLE* aBrowserHandle)
{
	WEBKIT_INIT();
	return BrowserControl::BrowserControl_Init(aBrowserHandle);
}

ESTATUS BrowserControl_Finalize(HANDLE aBrowserHandle)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	((BrowserControl*)aBrowserHandle)->BrowserControl_Delete();
	return BrowserControl::BrowserControl_Finalize(aBrowserHandle);
}



ESTATUS BrowserControl_Create(HANDLE aBrowserHandle, BrowserWidget** aWidget)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aWidget)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_Create(aWidget);
}

ESTATUS BrowserControl_Delete(HANDLE aBrowserHandle)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_Delete();
}

//------------------------------------------------------------------------
// Settings API
//------------------------------------------------------------------------

ESTATUS BrowserControl_SetHwnd(HANDLE aBrowserHandle,const HWIN hWin)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetHwnd(hWin);
}

ESTATUS BrowserControl_SetHttpProxy(HANDLE aBrowserHandle,const SProxyInfo* aProxyInfo)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aProxyInfo)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetHttpProxy(aProxyInfo);
}

ESTATUS BrowserControl_SetIAP(HANDLE aBrowserHandle,const IAPHANDLE aSelectedIapHandle)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetIAP(aSelectedIapHandle);
}

ESTATUS BrowserControl_SetCacheSize(HANDLE aBrowserHandle
									,const guint aCacheSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetCacheSize(aCacheSize);
}


ESTATUS BrowserControl_SetProperty(HANDLE aBrowserHandle
		,const EBrowserProperty aBrowserProperty,const gboolean aSettingValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetProperty(aBrowserProperty, aSettingValue);
}

ESTATUS BrowserControl_GetHttpProxy(HANDLE aBrowserHandle,SProxyInfo* aProxyInfo)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	if(aProxyInfo)
	{
		return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetHttpProxy(aProxyInfo);	
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}
	
}

ESTATUS BrowserControl_GetIAP(HANDLE aBrowserHandle,IAPHANDLE* aSelectedIapHandle)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	if(aSelectedIapHandle)
	{
		return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetIAP(aSelectedIapHandle);
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

}

ESTATUS BrowserControl_GetCacheSize(HANDLE aBrowserHandle
								,guint* aCacheSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	if(aCacheSize)
	{
		return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetCacheSize(aCacheSize);
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}
}

ESTATUS BrowserControl_GetProperty(HANDLE aBrowserHandle
		,const EBrowserProperty aBrowserProperty,gboolean* aSettingValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	if(aSettingValue)
	{
		return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetProperty(aBrowserProperty, aSettingValue);
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}
}

#ifdef __SHP__
WEBKIT_SHP_API ESTATUS BrowserControl_SetDefaultFontSize(HANDLE aBrowserHandle
								,const int aSettingsValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetDefaultFontSize(aSettingsValue);
}
WEBKIT_SHP_API ESTATUS BrowserControl_SetDefaultFixedFontSize(HANDLE aBrowserHandle
								,const int aSettingsValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetDefaultFixedFontSize(aSettingsValue);
}
WEBKIT_SHP_API ESTATUS BrowserControl_SetMinimumLogicalFontSize(HANDLE aBrowserHandle
								,const int aSettingsValue)

{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetMinimumLogicalFontSize(aSettingsValue);
}
WEBKIT_SHP_API ESTATUS BrowserControl_SetMinimumFontSize(HANDLE aBrowserHandle
								,const int aSettingsValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetMinimumFontSize(aSettingsValue);
}
WEBKIT_SHP_API ESTATUS BrowserControl_SetDefaultTextEncodingName(HANDLE aBrowserHandle
								,const char * aSettingsValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetDefaultTextEncodingName(aSettingsValue);
}
WEBKIT_SHP_API ESTATUS BrowserControl_SetFtpDirectoryTemplatePath(HANDLE aBrowserHandle,
																  char *aSettingsValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetFtpDirectoryTemplatePath(aSettingsValue);

}

WEBKIT_SHP_API ESTATUS BrowserControl_SetUserStyleSheetLocation(HANDLE aBrowserHandle
								,const char *aSettingsValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetUserStyleSheetLocation(aSettingsValue);
}
WEBKIT_SHP_API ESTATUS BrowserControl_SetStandardFontFamily(HANDLE aBrowserHandle
								,const char * aSettingsValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetStandardFontFamily(aSettingsValue);
}
WEBKIT_SHP_API ESTATUS BrowserControl_SetFixedFontFamily(HANDLE aBrowserHandle
								,const char * aSettingsValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetFixedFontFamily(aSettingsValue);
}
WEBKIT_SHP_API ESTATUS BrowserControl_SetSerifFontFamily(HANDLE aBrowserHandle
								,const char * aSettingsValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetSerifFontFamily(aSettingsValue);
}
WEBKIT_SHP_API ESTATUS BrowserControl_SetSansSerifFontFamily(HANDLE aBrowserHandle
								,const char * aSettingsValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetSansSerifFontFamily(aSettingsValue);
}
WEBKIT_SHP_API ESTATUS BrowserControl_SetCursiveFontFamily(HANDLE aBrowserHandle
								,const char * aSettingsValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetCursiveFontFamily(aSettingsValue);
}
WEBKIT_SHP_API ESTATUS BrowserControl_SetFantasyFontFamily(HANDLE aBrowserHandle
								,const char * aSettingsValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetFantasyFontFamily(aSettingsValue);
}
WEBKIT_SHP_API ESTATUS BrowserControl_SetEditableLinkBehavior(HANDLE aBrowserHandle
								,const EEditableLinkBehavior aSettingsValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetEditableLinkBehavior((const WebCore::EditableLinkBehavior)aSettingsValue);
}
WEBKIT_SHP_API ESTATUS BrowserControl_GetDefaultFontSize(HANDLE aBrowserHandle
								,int *aSettingsValue)

{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	ESTATUS result = EFail;
	if (aSettingsValue)
	{
		result = ((BrowserControl*)aBrowserHandle)->BrowserControl_GetDefaultFontSize(aSettingsValue);
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
	}
	return result;
}
WEBKIT_SHP_API ESTATUS BrowserControl_GetDefaultFixedFontSize(HANDLE aBrowserHandle
								,int *aSettingsValue)

{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	ESTATUS result = EFail;
	if (aSettingsValue)
	{
		result = ((BrowserControl*)aBrowserHandle)->BrowserControl_GetDefaultFixedFontSize(aSettingsValue);
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
	}	
	return result;
}

WEBKIT_SHP_API ESTATUS BrowserControl_GetMinimumLogicalFontSize(HANDLE aBrowserHandle
								,int *aSettingsValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	ESTATUS result = EFail;
	if (aSettingsValue)
	{
		result = ((BrowserControl*)aBrowserHandle)->BrowserControl_GetMinimumLogicalFontSize(aSettingsValue);
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
	}
	return result;
}
WEBKIT_SHP_API ESTATUS BrowserControl_GetMinimumFontSize(HANDLE aBrowserHandle
								,int *aSettingsValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	ESTATUS result = EFail;
	if (aSettingsValue)
	{
	result =  ((BrowserControl*)aBrowserHandle)->BrowserControl_GetMinimumLogicalFontSize(aSettingsValue);
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
	}
	return result ;
}

ESTATUS BrowserControl_GetDefaultTextEncodingName(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetDefaultTextEncodingName(aSettingsValue ,aSize);
}
ESTATUS BrowserControl_GetFtpDirectoryTemplatePath(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetFtpDirectoryTemplatePath(aSettingsValue ,aSize);
}

ESTATUS BrowserControl_GetUserStyleSheetLocation(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetUserStyleSheetLocation(aSettingsValue ,aSize);
}
ESTATUS BrowserControl_GetStandardFontFamily(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetStandardFontFamily(aSettingsValue ,aSize);
}
ESTATUS BrowserControl_GetFixedFontFamily(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetFixedFontFamily(aSettingsValue ,aSize);
}

ESTATUS BrowserControl_GetSerifFontFamily(HANDLE aBrowserHandle
								,gchar *aSettingsValue
								,gulong *aSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetSerifFontFamily(aSettingsValue ,aSize);

}

ESTATUS BrowserControl_GetSansSerifFontFamily(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetSansSerifFontFamily(aSettingsValue ,aSize);
}
ESTATUS BrowserControl_GetCursiveFontFamily(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetCursiveFontFamily(aSettingsValue ,aSize);
}
ESTATUS BrowserControl_GetFantasyFontFamily(HANDLE aBrowserHandle,gchar *aSettingsValue,gulong *aSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetFantasyFontFamily(aSettingsValue ,aSize);
}

WEBKIT_SHP_API ESTATUS BrowserControl_GetEditableLinkBehavior(HANDLE aBrowserHandle
								,EEditableLinkBehavior *aSettingsValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	WebCore::EditableLinkBehavior lBehavior;
	ESTATUS result = EFail;
	if( aSettingsValue)
	{
		result = ((BrowserControl*)aBrowserHandle)->BrowserControl_GetEditableLinkBehavior(&lBehavior);
		*aSettingsValue =(EEditableLinkBehavior) lBehavior;
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
	}
	return result;
}
#endif //#ifdef __SHP__
//------------------------------------------------------------------------
// Page API
//------------------------------------------------------------------------

ESTATUS BrowserControl_SwitchTab(HANDLE aBrowserHandle)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SwitchTab();
}

ESTATUS BrowserControl_LoadUrl(HANDLE aBrowserHandle, const gchar* aUrl)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aUrl)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_LoadUrl(aUrl);
}

ESTATUS BrowserControl_LoadFile(HANDLE aBrowserHandle, const gchar* aPath)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aPath)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_LoadFile(aPath);
}

ESTATUS BrowserControl_LoadData(HANDLE aBrowserHandle, const gchar* aData)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // _DEBUG

	if(!aData)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_LoadData(aData);
}

ESTATUS BrowserControl_PostUrl(HANDLE aBrowserHandle,
								const gchar* aUrl,
								const gchar* aContentType,
								const gchar* aPostData,
								const gchar* aBoundary)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aUrl || !aContentType || !aPostData || !aBoundary)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_PostUrl(aUrl, aContentType, aPostData, aBoundary);
}

ESTATUS BrowserControl_GetPageUrl(HANDLE aBrowserHandle, gchar* aUrl, gulong* aSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aUrl || !aSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetPageUrl(aUrl, aSize);
}

ESTATUS BrowserControl_GetPageTitle(HANDLE aBrowserHandle, gchar* aTitle, gulong* aSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aTitle || !aSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetPageTitle(aTitle, aSize);
}

ESTATUS BrowserControl_GetPageMimeType(HANDLE aBrowserHandle, gchar* aType, gulong* aSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aType || !aSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetPageMimeType(aType, aSize);
}

ESTATUS BrowserControl_GetPageSize(HANDLE aBrowserHandle, gulong* aSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetPageSize(aSize);
}

ESTATUS BrowserControl_GetPageContent(HANDLE aBrowserHandle, gchar* aContent, gulong* aSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aContent || !aSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetPageContent(aContent,  aSize);
}

ESTATUS BrowserControl_GetPageLastModifiedTime(HANDLE aBrowserHandle, gchar* aTime, gulong* aSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	if(!aTime || !aSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetPageLastModifiedTime(aTime, aSize);
}

ESTATUS BrowserControl_GetSelectedElementType(HANDLE aBrowserHandle, EPageItemType* aType)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	if(aType)
	{
		return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetSelectedElementType(aType);
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}
}

/**
 *   bs0425.lee  modified for touch events 
 */
 
ESTATUS BrowserControl_GetSelectedElementTypeByPoint(HANDLE aBrowserHandle, EPageItemType* aType, int pointX, int pointY )
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	if(aType)
	{
		return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetSelectedElementTypeByPoint(aType, pointX, pointY );
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}
}

/**
 *   bs0425.lee  modified for touch events 
 */


ESTATUS BrowserControl_GetSelectedElementInfo(HANDLE aBrowserHandle, EPageItemType aType, ElementInfoBase* aInfo)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	if(aInfo)
	{
		return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetSelectedElementInfo(aType, aInfo);
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

}

ESTATUS BrowserControl_GetElementTypeUnderPoint(HANDLE aBrowserHandle,
												const guint aX,
												const guint aY,
												EPageItemType* aType)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aType)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetElementTypeUnderPoint(aX, aY, aType);
}

ESTATUS BrowserControl_GetItemCount(HANDLE aBrowserHandle, const EPageItemType aType, guint* aCount)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aCount)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetItemCount( aType, aCount);
}

ESTATUS BrowserControl_SavePage(HANDLE aBrowserHandle, const gchar* aPath)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aPath)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SavePage(aPath);
}

ESTATUS BrowserControl_GetImages(HANDLE aBrowserHandle, BOOL bVisibleOnly, ImageSaveInfo*** imageList)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetImages(bVisibleOnly, imageList);
}

ESTATUS BrowserControl_GetImageCount(HANDLE aBrowserHandle, BOOL bVisibleOnly, guint* imageCount)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetImageCount(bVisibleOnly, imageCount);
}

ESTATUS BrowserControl_SaveElement(HANDLE aBrowserHandle, const gchar* aPath)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aPath)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SaveElement(aPath);
}

/**
 *   bs0425.lee  modified for touch events 
 */


ESTATUS BrowserControl_SaveElementByPoint(HANDLE aBrowserHandle, const gchar* aPath, int pointX, int pointY)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aPath)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SaveElementByPoint(aPath, pointX, pointY );
}

/**
 *   bs0425.lee  modified for touch events 
 */



ESTATUS BrowserControl_Reload(HANDLE aBrowserHandle)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_Reload();
}

ESTATUS BrowserControl_StopLoading(HANDLE aBrowserHandle)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_StopLoading();
}

ESTATUS BrowserControl_Find(HANDLE aBrowserHandle, const SFindInfo* aFindInfo)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aFindInfo)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_Find(aFindInfo);
}

ESTATUS BrowserControl_FindNext(HANDLE aBrowserHandle, const SFindInfo* aFindInfo)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aFindInfo)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_FindNext(aFindInfo);
}

ESTATUS BrowserControl_CanGoBack(HANDLE aBrowserHandle, gboolean* aRet)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	if(!aRet)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_CanGoBack(aRet);
}

ESTATUS BrowserControl_CanGoForward(HANDLE aBrowserHandle, gboolean* aRet)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	if(!aRet)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_CanGoForward(aRet);
}

ESTATUS BrowserControl_GoBack(HANDLE aBrowserHandle)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GoBack();
}

ESTATUS BrowserControl_GoForward(HANDLE aBrowserHandle)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GoForward();
}

ESTATUS BrowserControl_ZoomIn(HANDLE aBrowserHandle, const guint aScalePercent)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_ZoomIn(aScalePercent);
}

ESTATUS BrowserControl_ZoomOut(HANDLE aBrowserHandle, const guint aScalePercent)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_ZoomOut(aScalePercent);
}

ESTATUS BrowserControl_GetZoom(HANDLE aBrowserHandle, guint* aScalePercent)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aScalePercent)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetZoom(aScalePercent);
}

ESTATUS BrowserControl_CreatePageThumbnail(HANDLE aBrowserHandle, guint width, guint height, const gchar *thumbPath)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!thumbPath)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_CreatePageThumbnail(width, height, thumbPath);
}

#ifdef SFB_RSS_LINK //vamsi Jan 28 Rss Link Tag Support for SFB.
ESTATUS BrowserControl_GetRssLinks(HANDLE aBrowserHandle, SRssLink** pRssLink, int* rssCount, gchar** url)
{
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetRssLinks(pRssLink, rssCount, url);
}
#endif

ESTATUS BrowserControl_SetTextSize(HANDLE aBrowserHandle, guint size)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetTextSize(size);
}

//------------------------------------------------------------------------
// Error Handling Interface
//------------------------------------------------------------------------

ESTATUS BrowserControl_SetErrorHandler(HANDLE aBrowserHandle, const EErrorType aType, const ErrorHandler aHandler)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetErrorHandler(aType, aHandler);
}

void BrowserControl_GetLastError(ERRORCODE *aErrorCode)
{
	BrowserControl::BrowserControl_GetLastError(aErrorCode);
}


//------------------------------------------------------------------------
// Observers API
//------------------------------------------------------------------------

ESTATUS BrowserControl_SetLoadHandler(HANDLE aBrowserHandle, const LoadHandler aHandler)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetLoadHandler(aHandler);
}

//------------------------------------------------------------------------
// General API
//------------------------------------------------------------------------

ESTATUS BrowserControl_GetVersion(HANDLE aBrowserHandle, gchar* aVersionInfo, gulong* aBufferSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aVersionInfo || !aBufferSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetVersion(aVersionInfo, aBufferSize);
}

ESTATUS BrowserControl_GetCertificateInfo(HANDLE aBrowserHandle, SCertInfo* aCertInfo)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aCertInfo)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetCertificateInfo(aCertInfo);
}

ESTATUS BrowserControl_GetBrowserParameter(HANDLE aBrowserHandle,
											const EBrowserParameters aBrowserParameter,
											gchar* aParameterValue,
											gulong* aBufferSize)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aParameterValue || !aBufferSize)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetBrowserParameter(aBrowserParameter, aParameterValue,aBufferSize);
}

ESTATUS BrowserControl_SetBrowserParameter(HANDLE aBrowserHandle,
											const EBrowserParameters aBrowserParameter,
											const gchar* aParameterValue)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!aParameterValue)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetBrowserParameter(aBrowserParameter, aParameterValue);
}

#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
ESTATUS BrowserControl_SetPreview(HANDLE aBrowserHandle)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK


	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetPreview();
}

ESTATUS BrowserControl_IsPreview(HANDLE aBrowserHandle, BOOL* bPreview)
{
	bool bIsPreview=false; //Ancil for removing linking error.
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(TRUE == *bPreview)
		bIsPreview = true;

	ESTATUS eStatus = ((BrowserControl*)aBrowserHandle)->BrowserControl_IsPreview(&bIsPreview);
	
	*bPreview = bIsPreview;

	return eStatus;
	
}
#endif

ESTATUS BrowserControl_Paint(HANDLE aBrowserHandle, void* pUserData)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	if(!pUserData)
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_Paint(pUserData);
}

ESTATUS BrowserControl_HandleKeyEvent(HANDLE aBrowserHandle, int keyCode,  BOOL isKeyUp)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_HandleKeyEvent(keyCode,  isKeyUp);
}


/**
 *   bs0425.lee  Handle for touch events 
 */
ESTATUS BrowserControl_HandleMouseEvent(HANDLE aBrowserHandle, int rectX, int rectY, int MouseEventType)
{

#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

 	return ((BrowserControl*)aBrowserHandle)->BrowserControl_HandleMouseEvent( rectX, rectY, MouseEventType);

}


ESTATUS BrowserControl_PaneScrollBy(HANDLE aBrowserHandle, int in_offsetX, int in_offsetY)
{

#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_PaneScrollBy( in_offsetX, in_offsetY);

}


gboolean BrowserControl_IsFocusedByPoint(HANDLE aBrowserHandle, int pointX, int pointY)
{

#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)-> BrowserControl_IsFocusedByPoint(pointX, pointY);

}

gboolean BrowserControl_IsLink(HANDLE aBrowserHandle, int rectX, int rectY)
{

#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return ESuccess;   // It is because, ESuccess means 0 
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)-> BrowserControl_IsLink(rectX, rectY);

}

gboolean BrowserControl_IsImageElement(HANDLE aBrowserHandle, int rectX, int rectY)
{

#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return ESuccess;   // It is because, ESuccess means 0 
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)-> BrowserControl_IsImageElement(rectX, rectY);

}

ESTATUS BrowserControl_GetSelectedElementInfoByPoint(HANDLE aBrowserHandle, EPageItemType aType, ElementInfoBase* aInfo,int pointX , int pointY)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK
	if(aInfo)
	{
		return ((BrowserControl*)aBrowserHandle)->BrowserControl_GetSelectedElementInfoByPoint(aType, aInfo, pointX , pointY);
	}
	else
	{
		BrowserControl::SetLastError(ENullInputParameter);
		return EFail;
	}
}


/**
 *   bs0425.lee  Handle for touch events 
 */




#if defined ( SHP_SUPPORT_WML )
int BrowserControl_WmlItemCount(HANDLE aBrowserHandle)
{
	return (static_cast<BrowserControl*>(aBrowserHandle))->BrowserControl_WmlItemCount() ;
}
int BrowserControl_WmlDoElement( HANDLE aBrowserHandle, int ID ) 
{
	return (static_cast<BrowserControl*>(aBrowserHandle))->BrowserControl_WmlDoElement( ID ) ;
}
#endif//SHP_SUPPORT_WML

#ifdef __SHP__
void BrowserControl_SetProgressHandler(HANDLE aBrowserHandle, ProgressHandler aProgressHandler)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetProgressHandler(aProgressHandler);
}

void BrowserControl_SetTitleChangedHandler(HANDLE aBrowserHandle, TitleChangedHandler aTitleChangedHandler)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetTitleChangedHandler(aTitleChangedHandler);
}

void BrowserControl_SetLinkHoveringHandler(HANDLE aBrowserHandle, LinkHoveringHandler aLinkHoveringHandler)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetLinkHoveringHandler(aLinkHoveringHandler);
}

void BrowserControl_SetStatusbarTextChangedHandler(HANDLE aBrowserHandle, StatusbarTextChangedHandler aStatusbarTextChangedHandler)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetStatusbarTextChangedHandler(aStatusbarTextChangedHandler);
}

void BrowserControl_SetIconLoadedHandler(HANDLE aBrowserHandle, IconLoadedHandler aIconLoadedHandler)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetIconLoadedHandler(aIconLoadedHandler);
}

void BrowserControl_SetSelectionChangedHandler(HANDLE aBrowserHandle, SelectionChangedHandler aSelectionChangedHandler)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetSelectionChangedHandler(aSelectionChangedHandler);
}

void BrowserControl_SetUrlSchemeHandler(HANDLE aBrowserHandle, UrlSchemeHandler aUrlSchemeHandler)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetUrlSchemeHandler(aUrlSchemeHandler);
}

void BrowserControl_SetUIStateHandler(HANDLE aBrowserHandle, UIStateHandler aUIStateHandler)
{
#ifdef SANITY_CHECK
	if(!IsValidHandle(aBrowserHandle))
	{
		BrowserControl::SetLastError(EInvalidHandle);
		return EFail;
	}
#endif // SANITY_CHECK

	return ((BrowserControl*)aBrowserHandle)->BrowserControl_SetUIStateHandler(aUIStateHandler);
}
#endif

//------------------------------------------------------------------------
// Internal functions
//------------------------------------------------------------------------

void BrowserControl_SetLastError(ERRORCODE aErrorCode)
{
	BrowserControl::SetLastError(aErrorCode);
}

#if defined SHP_SUPPORT_WML
void BrowserControl_FetchRequest (HANDLE aBrowserHandle)
{
	((BrowserControl*)aBrowserHandle)->BrowserControl_FetchRequest () ;
}
#endif //SHP_SUPPORT_WML

#ifdef SANITY_CHECK
gboolean IsValidHandle(HANDLE aBrowserHandle)
{
	return BrowserControl::IsValidHandle(aBrowserHandle);
}
#endif // SANITY_CHECK

#ifdef SFB_VISUAL_SUPPORT
ESTATUS BrowserControl_VisualHistory(HANDLE aBrowserHandle, WmBitmap** pWmBitmap,void* pUserData)
{
	return ((BrowserControl*)aBrowserHandle)->BrowserControl_VisualHistory(pWmBitmap,pUserData);
}
#endif 

//vamsi relayout
void BrowserControl_ReLayout(HANDLE aBrowserHandle)
{
	((BrowserControl*)aBrowserHandle)->BrowserControl_ReLayout();
}