
/*
 * Mocha
 * Copyright (c) 2002 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/**
 * This file implements Unicode SHP functions used by Webkit
 *
 * @author	Zafar Hussain (zafar.h@samsung.com)
 * @author	Pushparajan (rajan.vp@samsung.com) 
 */
//vprajan
#ifndef KJS_UNICODE_SHP_H
#define KJS_UNICODE_SHP_H

//#include "RsrcMgr.h"
#include "WmLngPack.h"
#include "WmGrpType.h" // For handling font direction
#include "AnsiCLib.h"
#include <wtf/Assertions.h>
//#include "AcWMap.c"
//ZAFAR: TO REMOVE ERROR DUE TO GRAMMAR.Y IN LEXER.CPP
#undef CONST
#undef VOID
#undef DELETE
#undef THIS
#undef IN

// vprajan - Replace the following typedefs with stdint.h of SHP
typedef unsigned   uint32_t;
typedef unsigned char   uint8_t;

//Hack done to make UChar compatible with JSChar (kjs)
# ifndef SHP_UCHAR_32
# define SHP_UCHAR_32
typedef uint32_t UChar32;
# endif
typedef wchar_t UChar;

#define U_MASK(x) ((uint32_t)1<<(x))
// some defines from ICU needed one or two places

typedef signed __int64 int64_t;
typedef unsigned __int64 uint64_t;
# ifndef uint16_t
typedef unsigned short uint16_t;
# endif
typedef enum UCharCategory
{
    /** See note !!.  Comments of the form "Cn" are read by genpname. */

    /** Non-category for unassigned and non-character code points. @stable ICU 2.0 */
    U_UNASSIGNED              = 0,
    /** Cn "Other, Not Assigned (no characters in [UnicodeData.txt] have this property)" (same as U_UNASSIGNED!) @stable ICU 2.0 */
    U_GENERAL_OTHER_TYPES     = 0,
    /** Lu @stable ICU 2.0 */
    U_UPPERCASE_LETTER        = 1,
    /** Ll @stable ICU 2.0 */
    U_LOWERCASE_LETTER        = 2,
    /** Lt @stable ICU 2.0 */
    U_TITLECASE_LETTER        = 3,
    /** Lm @stable ICU 2.0 */
    U_MODIFIER_LETTER         = 4,
    /** Lo @stable ICU 2.0 */
    U_OTHER_LETTER            = 5,
    /** Mn @stable ICU 2.0 */
    U_NON_SPACING_MARK        = 6,
    /** Me @stable ICU 2.0 */
    U_ENCLOSING_MARK          = 7,
    /** Mc @stable ICU 2.0 */
    U_COMBINING_SPACING_MARK  = 8,
    /** Nd @stable ICU 2.0 */
    U_DECIMAL_DIGIT_NUMBER    = 9,
    /** Nl @stable ICU 2.0 */
    U_LETTER_NUMBER           = 10,
    /** No @stable ICU 2.0 */
    U_OTHER_NUMBER            = 11,
    /** Zs @stable ICU 2.0 */
    U_SPACE_SEPARATOR         = 12,
    /** Zl @stable ICU 2.0 */
    U_LINE_SEPARATOR          = 13,
    /** Zp @stable ICU 2.0 */
    U_PARAGRAPH_SEPARATOR     = 14,
    /** Cc @stable ICU 2.0 */
    U_CONTROL_CHAR            = 15,
    /** Cf @stable ICU 2.0 */
    U_FORMAT_CHAR             = 16,
    /** Co @stable ICU 2.0 */
    U_PRIVATE_USE_CHAR        = 17,
    /** Cs @stable ICU 2.0 */
    U_SURROGATE               = 18,
    /** Pd @stable ICU 2.0 */
    U_DASH_PUNCTUATION        = 19,
    /** Ps @stable ICU 2.0 */
    U_START_PUNCTUATION       = 20,
    /** Pe @stable ICU 2.0 */
    U_END_PUNCTUATION         = 21,
    /** Pc @stable ICU 2.0 */
    U_CONNECTOR_PUNCTUATION   = 22,
    /** Po @stable ICU 2.0 */
    U_OTHER_PUNCTUATION       = 23,
    /** Sm @stable ICU 2.0 */
    U_MATH_SYMBOL             = 24,
    /** Sc @stable ICU 2.0 */
    U_CURRENCY_SYMBOL         = 25,
    /** Sk @stable ICU 2.0 */
    U_MODIFIER_SYMBOL         = 26,
    /** So @stable ICU 2.0 */
    U_OTHER_SYMBOL            = 27,
    /** Pi @stable ICU 2.0 */
    U_INITIAL_PUNCTUATION     = 28,
    /** Pf @stable ICU 2.0 */
    U_FINAL_PUNCTUATION       = 29,
    /** One higher than the last enum UCharCategory constant. @stable ICU 2.0 */
    U_CHAR_CATEGORY_COUNT
} UCharCategory;

// Taken directly from UnicodeQt4.h

#define U16_IS_LEAD(c) (((c)&0xfffffc00)==0xd800)
#define U16_IS_TRAIL(c) (((c)&0xfffffc00)==0xdc00)
#define U16_SURROGATE_OFFSET ((0xd800<<10UL)+0xdc00-0x10000)
#define U16_GET_SUPPLEMENTARY(lead, trail) \
    (((UChar32)(lead)<<10UL)+(UChar32)(trail)-U16_SURROGATE_OFFSET)

#define U16_LEAD(supplementary) (UChar)(((supplementary)>>10)+0xd7c0)
#define U16_TRAIL(supplementary) (UChar)(((supplementary)&0x3ff)|0xdc00)

#define U_IS_SURROGATE(c) (((c)&0xfffff800)==0xd800)
#define U16_IS_SURROGATE(c) U_IS_SURROGATE(c)
#define U16_IS_SURROGATE_LEAD(c) (((c)&0x400)==0)

#define U16_NEXT(s, i, length, c) { \
    (c)=(s)[(i)++]; \
    if(U16_IS_LEAD(c)) { \
        uint16_t __c2; \
        if((i)<(length) && U16_IS_TRAIL(__c2=(s)[(i)])) { \
            ++(i); \
            (c)=U16_GET_SUPPLEMENTARY((c), __c2); \
        } \
    } \
}

#define U_MASK(x) ((uint32_t)1<<(x))

namespace WTF {
  namespace Unicode {


    // Properties of Unicode strings
    
    enum Direction {
      LeftToRight = 0,
      RightToLeft ,
      EuropeanNumber ,
      EuropeanNumberSeparator ,
      EuropeanNumberTerminator,
      ArabicNumber ,
      CommonNumberSeparator,
      BlockSeparator,
      SegmentSeparator,
      WhiteSpaceNeutral,
      OtherNeutral,
      LeftToRightEmbedding,
      LeftToRightOverride,
      RightToLeftArabic,
      RightToLeftEmbedding,
      RightToLeftOverride,
      PopDirectionalFormat,
      NonSpacingMark,
      BoundaryNeutral
    };

	// Decomposition not available under SHP
    enum DecompositionType {
      DecompositionNone ,
      DecompositionCanonical ,
      DecompositionCompat ,
      DecompositionCircle ,
      DecompositionFinal ,
      DecompositionFont ,
      DecompositionFraction ,
      DecompositionInitial ,
      DecompositionIsolated ,
      DecompositionMedial ,
      DecompositionNarrow ,
      DecompositionNoBreak ,
      DecompositionSmall ,
      DecompositionSquare ,
      DecompositionSub ,
      DecompositionSuper ,
      DecompositionVertical ,
      DecompositionWide 
    };

	// Character Category not available under SHP
    enum CharCategory {
      NoCategory =  0,
      Other_NotAssigned = U_MASK(U_GENERAL_OTHER_TYPES),
      Letter_Uppercase = U_MASK(U_UPPERCASE_LETTER),
      Letter_Lowercase = U_MASK(U_LOWERCASE_LETTER),
      Letter_Titlecase = U_MASK(U_TITLECASE_LETTER),
      Letter_Modifier = U_MASK(U_MODIFIER_LETTER),
      Letter_Other = U_MASK(U_OTHER_LETTER),

      Mark_NonSpacing = U_MASK(U_NON_SPACING_MARK),
      Mark_Enclosing = U_MASK(U_ENCLOSING_MARK),
      Mark_SpacingCombining = U_MASK(U_COMBINING_SPACING_MARK),

      Number_DecimalDigit = U_MASK(U_DECIMAL_DIGIT_NUMBER),
      Number_Letter = U_MASK(U_LETTER_NUMBER),
      Number_Other = U_MASK(U_OTHER_NUMBER),

      Separator_Space = U_MASK(U_SPACE_SEPARATOR),
      Separator_Line = U_MASK(U_LINE_SEPARATOR),
      Separator_Paragraph = U_MASK(U_PARAGRAPH_SEPARATOR),

      Other_Control = U_MASK(U_CONTROL_CHAR),
      Other_Format = U_MASK(U_FORMAT_CHAR),
      Other_PrivateUse = U_MASK(U_PRIVATE_USE_CHAR),
      Other_Surrogate = U_MASK(U_SURROGATE),

      Punctuation_Dash = U_MASK(U_DASH_PUNCTUATION),
      Punctuation_Open = U_MASK(U_START_PUNCTUATION),
      Punctuation_Close = U_MASK(U_END_PUNCTUATION),
      Punctuation_Connector = U_MASK(U_CONNECTOR_PUNCTUATION),
      Punctuation_Other = U_MASK(U_OTHER_PUNCTUATION),

      Symbol_Math = U_MASK(U_MATH_SYMBOL),
      Symbol_Currency = U_MASK(U_CURRENCY_SYMBOL),
      Symbol_Modifier = U_MASK(U_MODIFIER_SYMBOL),
      Symbol_Other = U_MASK(U_OTHER_SYMBOL),

      Punctuation_InitialQuote = U_MASK(U_INITIAL_PUNCTUATION),
      Punctuation_FinalQuote = U_MASK(U_FINAL_PUNCTUATION) 
    };

	inline UChar32 foldCase(UChar32 c)
    {
       return (UChar32)AcTolower((int)c);
    }

	inline UChar32 toLower(UChar32 ch)
    {
        return (UChar32)AcTolower((int)ch);
    }

	 inline int toLower(UChar* result, int resultLength, const UChar* src, int srcLength,  bool* error)
    {
      *error = false;
      if (resultLength < srcLength) {
        *error = true;
        return srcLength;
      }
      for (int i = 0; i < srcLength; ++i)
        result[i] = (UChar) foldCase((UChar32)src[i]);
      return srcLength;
    }

    inline UChar32 toUpper(UChar32 ch)
    {
      return (UChar32)AcToupper((int)ch);
    }

    inline int toUpper(UChar* result, int resultLength, const UChar* src, int srcLength,  bool* error)
    {
      *error = false;
      if (resultLength < srcLength) {
        *error = true;
        return srcLength;
      }
      for (int i = 0; i < srcLength; ++i)
        result[i] = (UChar) AcTowupper(src[i]);
      return srcLength;
    }

    inline int toTitleCase(UChar32 c)
    {
      return (UChar32)AcToupper((int)c);
    }

    

    inline int foldCase(UChar* result, int resultLength, UChar* src, int srcLength,  bool* error)
    {
      return toLower(result, resultLength, src, srcLength, error);
    }

/*This API is based on SHP unicode functionality.
 This API provides the length of the output for encoding the given input.
 Plz use this function to calculate the outBufSize for ConvertToSingleByteCharacterString
*/
	inline int calcLengthForEncode(UChar* mszText , int charCount)
	{
	int		nBuffSize = 0;
	SysAssert( mszText );

	if ( mszText[0] == MB_NULL )
	return 0;

	while( charCount > 0 && *mszText != MB_NULL )
	{

	if( 0x0001 <= *mszText && *mszText <= 0x007F ) 
	{
	nBuffSize++;
	mszText++;
	charCount--;
	}
	else if( (*mszText == 0x0000) || (0x0080 <= *mszText && *mszText <= 0x07FF) )
	{
	nBuffSize += 2;
	mszText++;
	charCount--;
	}
	else if( 0x0800 <= *mszText && *mszText <= 0xFFFF )
	{
	nBuffSize += 3;
	mszText++;
	charCount--;
	}
	else 
	{
	SysSetLastError( WMERR_INVALID_UNICODE_RANGE );
	return -1;
	}
	}
	return nBuffSize+1;	/* NULL ڱ  */

	}
	inline int ConvertToSingleByteCharacterString(char* pszOutText, int outBufSize, UChar* mszInText, int charCount)
	{		
		return	WmConvertPCode2UTF((unsigned char *)pszOutText,outBufSize,(MCHAR*)mszInText,charCount);
	}

/*This API is based on SHP unicode functionality.
 This API provides the length of the output for decoding the given input.
 Plz use this function to calculate the outBufSize for ConvertToDoubleByteCharacterString
*/
	inline int calcLengthForDecode( UCHAR* szInText, int byteCount )
	{
		int nCount = 0;
		while( byteCount > 0 && (*szInText != '\0') )
		{
			if (*szInText < 0x80) 
			{
				szInText++;
				byteCount--;
			}
			else if  (((0xC0 <= *szInText) && (*szInText < 0xE0)) && (*(szInText+1) >= 0x80))
			{
				szInText += 2;
				byteCount -= 2;
			}
			else if  ((*szInText >= 0xE0) && (*(szInText+1) >= 0x80) && (*(szInText+2) >= 0x80))
			{
				szInText += 3;
				byteCount -= 3;
			}
			else 
			{
				szInText++;
				byteCount--;
				SysDebug(( MID_LANGPACK, "__WmLngStrlenByByte: utf8 incorrect range!\n"));
			}
			nCount++;
		}
		return (nCount+1)*sizeof(MCHAR);
	}
	

	inline int ConvertToDoubleByteCharacterString(UChar* pmszOutText, int outBufSize, char*  szInText,int inBufSize)
	{		
		return	WmConvertUTF2PCode ((MCHAR*)pmszOutText, outBufSize, (UCHAR*)szInText, inBufSize);
	}
#if 0
	inline bool isFormatChar(UChar32 c)//TO DO::::Getting No idea of what this is meant to do. So, need to review on this further.
    {
	  //REVIEW: Review required !!
      return (c & 0xffff0000) == 0 && AcIswcntrl((wint_t)c);
    }
#endif 

    inline bool isPrintableChar(UChar32 c)
    {
      return c >= 32 && c <= 126 || c >= 160 && c <= 254;

	}

    inline bool isSeparatorSpace(UChar32 c)
    {
	  /*Zafar: Plz refer http://www.fileformat.info/info/unicode/category/Zs/list.htm for more details.
	  NOTE: purposefully implemented ONLY for ASCII characters & not for other char set. 
	  Use the above link to implement for the rest.*/
		if(c==0x0020 || c==0x00A0)
		  return TRUE;
	  else 
		  return FALSE;
    }
	inline bool isSpace(UChar32 c)
	{
		if (AcIsspace((int) c))
			return TRUE;
		else
			return FALSE;
	}

	inline bool isWideCharSpace(UChar c)/*ZAFAR: Added this for unicode char. Used in TextBreakIteratorSHP.cpp*/
	{
		if (AcIswdigit(c))
			return TRUE;
		else
			return FALSE;
		
	}

    inline bool isPunct(UChar32 c)
    {
		if(((c) >= 33 && (c) <= 47) || ((c) >= 58 && (c) <= 64)  || ((c) >= 91 && (c) <= 96) || ((c) >= 123 && (c) <= 126))
			return true;
		else
			return false;
    }


    inline bool isDigit(UChar32 c)
    {
      if (AcIsdigit((int)c))
		  return TRUE;
	  else 
		  return FALSE;
    }

    inline bool isLower(UChar32 c)
    {
      if( AcIslower((int) c))
		  return TRUE;
	  else 
		  return FALSE;
    }

    inline bool isUpper(UChar32 c)
    {
      if( AcIsupper((int) c))
		  return TRUE;
	  else 
		  return FALSE;
    }

    inline int digitValue(UChar32 c)
    {
      return AcIsdigit((int)c);
    }

    inline UChar32 mirroredChar(UChar32 c)
    {
      /*ZAFAR::	Mirrored char as per unicode 4.0
	  Bidi Mirrored is a normative property of characters such as parentheses, whose images are
mirrored horizontally in text that is laid out from right to left. For example, U+0028 is interpreted as opening parenthesis; in a left-to-right context it will appear as
(, while in a right-to-left context it will appear as the mirrored glyph ). The list of mirrored
characters appears in the Unicode Character Database. Note that mirroring is not
limited to paired characters, but that any character with the mirrored property will need
two mirrored glyphsfor example, U+222B . This requirement is necessary to
render the character properly in a bidirectional context. It is the default behavior in Unicode
text.*/
		//Zafar: Note: As we are supporting only latin (& thus only left to right charset), I am returning dummy value.
		return c; 

    }


    inline uint8_t combiningClass(UChar32 c)
    {
		ASSERT(0);//NEEDS TO BE IMMPLEMENTED.
		uint8_t temp=(uint8_t)c;
		return temp;
     
    }


    inline DecompositionType decompositionType(UChar32 c)
    {
      if (c > 0xffff)
        return DecompositionNone;
      //return (DecompositionType)QChar(c).decompositionTag();
    }

    inline int umemcasecmp(const UChar* a, const UChar* b, int len)
    {
	  for (int i = 0; i < len; ++i) {
        UChar c1 = (UChar)foldCase(a[i]);
        UChar c2 = (UChar)foldCase(b[i]);
        if (c1 != c2)
          return c1 < c2;
      }
      return 0;
    }

    inline Direction direction(UChar32 c)
    {
      if (c > 0xffff)
        return LeftToRight;
	  //ZAFAR:: LATIN CHARACTERS ALWAYS HAVE A DIRECTION OF LEFT TO RIGHT. 
	  //SINCE CUURENTLY SHP UNICODE SUPPORT IS LIMITED TO LATINC CHAR SET, WE ARE JUST RETURNING LEFTTORIGHT ALWAYS      
	  //return (Direction)QChar(c).direction();
	  return LeftToRight;
    }


    inline CharCategory category(UChar32 c)
     {
       if (c < 0)
         return NoCategory;
       if (c < 0x000000ff) {
           static const CharCategory cats[] = {
           Other_Control, Other_Control, Other_Control, Other_Control,
           Other_Control, Other_Control, Other_Control, Other_Control,
           Other_Control, Other_Control, Other_Control, Other_Control,
           Other_Control, Other_Control, Other_Control, Other_Control,
           Other_Control, Other_Control, Other_Control, Other_Control,
           Other_Control, Other_Control, Other_Control, Other_Control,
           Other_Control, Other_Control, Other_Control, Other_Control,
           Other_Control, Other_Control, Other_Control, Other_Control,
           Separator_Space, Punctuation_Other, Punctuation_Other,
           Punctuation_Other, Symbol_Currency, Punctuation_Other,
           Punctuation_Other, Punctuation_Other, Punctuation_Open,
           Punctuation_Close, Punctuation_Other, Symbol_Math,
           Punctuation_Other, Punctuation_Dash, Punctuation_Other,
           Punctuation_Other, Number_DecimalDigit, Number_DecimalDigit,
           Number_DecimalDigit, Number_DecimalDigit, Number_DecimalDigit,
           Number_DecimalDigit, Number_DecimalDigit, Number_DecimalDigit,
           Number_DecimalDigit, Number_DecimalDigit, Punctuation_Other,
           Punctuation_Other, Symbol_Math, Symbol_Math, Symbol_Math,
           Punctuation_Other, Punctuation_Other,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Punctuation_Open,
           Punctuation_Other, Punctuation_Close, Symbol_Modifier,
           Punctuation_Connector, Symbol_Modifier, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Punctuation_Open, Symbol_Math, Punctuation_Close,
           Symbol_Math, Other_Control, Other_Control, Other_Control,
           Other_Control, Other_Control,Other_Control, Other_Control,
           Other_Control, Other_Control, Other_Control, Other_Control,
           Other_Control, Other_Control, Other_Control, Other_Control,
           Other_Control, Other_Control, Other_Control, Other_Control,
           Other_Control, Other_Control, Other_Control, Other_Control,
           Other_Control, Other_Control, Other_Control, Other_Control,
           Other_Control, Other_Control, Other_Control, Other_Control,
           Other_Control, Other_Control, Separator_Space, Punctuation_Other,
           Symbol_Currency, Symbol_Currency, Symbol_Currency, Symbol_Currency,
           Symbol_Other, Symbol_Other, Symbol_Modifier, Symbol_Other,
           Letter_Lowercase, Punctuation_InitialQuote, Symbol_Math,
           Other_Format, Symbol_Other, Symbol_Modifier, Symbol_Other,
           Symbol_Math, Number_Other, Number_Other, Symbol_Modifier,
           Letter_Lowercase, Symbol_Other, Punctuation_Other,
           Symbol_Modifier, Number_Other, Letter_Lowercase,
           Punctuation_FinalQuote, Number_Other, Number_Other,
           Number_Other, Punctuation_Other, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Uppercase, Symbol_Math, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Uppercase, Letter_Uppercase, Letter_Uppercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Symbol_Math, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase,
           Letter_Lowercase, Letter_Lowercase, Letter_Lowercase
         };
         //BrPlSysAssert(sizeof(cats)/sizeof(CharCategory) == 0x0100);
         return cats[c];
       }
 
       // FIXME: implement for the rest ...
       return NoCategory;
     }

  }
}
#endif
