#ifndef WEBCOREBRIDGE_H
#define WEBCOREBRIDGE_H

//  INCLUDES
//vprajan: RVCT fix
#include "wtf/Vector.h"

#include "BridgeTypes.h"
#include "CMMDefs.h"

// CONSTANTS

// MACROS

// DATA TYPES

// FUNCTION PROTOTYPES

// FORWARD DECLARATIONS

namespace WebCore {
    class Frame;
    class Image;
	class WebCorePageDataItem;
    class ResourceLoader;
    class ResourceRequest;
}

class IContentListener;
class IWebClientBridge;

typedef WebCore::Frame XMLFrame;

// CLASS DECLARATION

/**
* WebCoreBridge, the main bridge interface to the WebCore library
*
* WebCoreBridge objects are used by WebCore to abstract away operations that need
* to be implemented by library clients, for example WebKit. The objects are also
* used in the opposite direction, for simple access to WebCore functions without dealing
* directly with the HTML C++ classes.
*
* A WebCoreBridge creates and holds a reference to a HTMLPart.
*
* The WebCoreBridge interface contains methods for use by the non-WebCore side of the bridge.
*
*  @lib WebCore
*/
class WebCoreBridge
    CMM_MODIFIED
{
    public:  // Constructors and destructor

        /**
        * Constructor.
        */
        WebCoreBridge( XMLFrame* _frame,
                        IWebClientBridge* _clientBridge );

        /**
        * Destructor.
        */
        virtual ~WebCoreBridge();

    public: // New functions

        IWebClientBridge* Client() { return m_clientBridge; }

		Vector<WebCore::WebCorePageDataItem *> GetPageData(int _typeMask,XMLFrame* child, const char *path);
	
        Vector<WebCore::Image *> GetImageData();
                       
        void SetFrame(XMLFrame* _frame) { m_frame = _frame; }
        XMLFrame* GetFrame() { return m_frame; }

    protected:  // Data

    	XMLFrame* m_frame;
        
    	// pointer to the client side bridge interface
	    IWebClientBridge* m_clientBridge;
    };



// CLASS DECLARATION
//
/**
* The IWebCoreBridge interface contains methods for use by the WebCore side of the bridge.
*
*  @lib WebCore
*/
class IWebClientBridge

{
    public: // Constructors and destructor
        /**
        * Constructor.
        */
        IWebClientBridge() {}
        /**
        * Destructor.
        */
        virtual ~IWebClientBridge() {}

    public: // IWebClientBridge methods

        /**
        * LoadPage
        * Public Class Method
        * Purpose - Load the main page
        * @param ResourceLoader* _resourceLoader
        * @param ResourceRequest* _request
        * @return void
        */
        virtual void LoadPage( WebCore::ResourceLoader* _resourceLoader,
                                WebCore::ResourceRequest* _request ) = 0;

        /**
        * LoadResource
        * Public Class Method
        * Purpose - Load a sub-resource
        * @param ResourceLoader* _resourceLoader
        * @param ResourceRequest* _request
        * @return void
        */
        virtual void LoadResource( WebCore::ResourceLoader* _resourceLoader,
                                    const WebCore::ResourceRequest* _request ) = 0;

        /**
        * CancelTransaction
        * Public Class Method
        * Purpose - Cancel a particular transaction for the given resource loader
        * @param _resourceLoader : The resource loader whose transaction is to be canceled
        * @return void
        */
        virtual void CancelTransaction( WebCore::ResourceLoader* _resourceLoader ) = 0;

        /**
        * CancelAll
        * Public Class Method
        * Purpose - Cancel all outstanding requests
        * @return void
        */
        virtual void CancelAll( ) = 0;

        /**
        * GetWebCoreBridge
        * Public Class Method
        * Purpose - Returns the WebCoreBridge
        * @return WebCoreBridge
        */
        virtual WebCoreBridge*  GetWebCoreBridge() const = 0;

        /**
        * SetWebCoreBridge
        * Public Class Method
        * Purpose - Stores the WebCoreBridge with the client bridge
        * @param _webCoreBridge : The bridge instance to be stored
        * @return void
        */
        virtual void SetWebCoreBridge( WebCoreBridge* _webCoreBridge ) = 0;

        /**
        * RedirectTransaction
        * Public Class Method
        * Purpose - Redirects the contents to the given content listener
        * @param _resourceLoader : The resource loader whose transaction is to be redirected
		* @param _transportHandle : This is the out parameter, returns transport handle
        * @return eErrorCode
        */
        virtual eErrorCode RedirectTransaction( WebCore::ResourceLoader* _resourceLoader, 
																int* _transportHandle) = 0;

		/*
		 * Update meta Cache
		 * @param
         * @return
		 */
		virtual void UpdateMetaCache( const char* szUrl, 
										const char* szMetaField , 
										const char* szMetaFieldContent) = 0;

		/*
		 * Update meta cookie
		 * @param
         * @return
		 */
		virtual void SetCookie( const char* szUrl, 
								const char* szContent) = 0;

		/*
		 * Update meta cookie
		 * @param
         * @return
		 */
		virtual char* GetCookie(const char* szUrl) = 0;

		/*
		 * Update meta cookie
		 * @param
         * @return
		 */
		virtual BOOL GetCookieEnabled() = 0;

}; // IWebClientBridge

#endif      // WEBCOREBRIDGE_H

// End of File

