#if defined(SHP_SUPPORT_WML)

#include "WMLBrowserContext.h"
#include "WMLDocument.h"
#include "BackForwardList.h"
#include "HistoryItem.h"
#include "Page.h"
#include "Frame.h"
#include "FrameLoader.h"
#include "wtf/asciictype.h"

using namespace WTF;

namespace WebCore {

WMLBrowserContext::WMLBrowserContext(Frame *frame, Page *page)
	:m_frame(frame)
	,m_page(page)
	,m_accessUrl(""),
	m_allowDeskAccess(true)
{
	ASSERT (frame) ;
	ASSERT (page) ;
}

WMLBrowserContext::~WMLBrowserContext()
{
	AttributeMap::iterator end = m_attrMaps.end() ;
	WMLVariable *var = 0 ;
	for (AttributeMap::iterator it = m_attrMaps.begin(); it != end ; ++it)	{
		var = it->second ;
		ASSERT (var) ;
		delete var ;
	}
	m_attrMaps.clear ();
}

void WMLBrowserContext::InitBrowserContext()
{
}

void WMLBrowserContext::ResetBrowserContext()
{
	ResetHistoryStack() ;
	ResetVariableInstances () ;
	//m_attrMaps.clear();
}

void WMLBrowserContext::ResetHistoryStack()
{
	PassRefPtr<HistoryItem> newCurrItem = 0 ;
	BackForwardList *list = m_page->backForwardList() ;
	HistoryItem *currentItem = list->currentItem() ;
	newCurrItem = currentItem->copy () ; 
	ASSERT (list) ;
	list->close() ;
	list->addItem (newCurrItem) ;
}

void WMLBrowserContext::GoBackInHistory(int aNoOfSteps)
{
	m_frame->loader()->goBackOrForward(-aNoOfSteps) ;
	/*BackForwardList *list = m_page->backForwardList() ;
	ASSERT (list) ;
	HistoryItem* item = list->itemAtIndex(-aNoOfSteps) ;
	list->goToItem(item) ;*/
}

void WMLBrowserContext::addHistoryItem (const String &url)
{
	PassRefPtr<HistoryItem> item = m_frame->loader()->createHistoryItemTree (m_frame, false) ;
	if(item)	{
		item->setURLString (url) ;
		BackForwardList *list = m_page->backForwardList() ;
		list->addItem (item) ;
	}
}

void WMLBrowserContext::GoForwardInHistory(int aNoOfSteps )
{
	m_frame->loader()->goBackOrForward(aNoOfSteps) ;
	/*BackForwardList *list = m_page->backForwardList() ;
	ASSERT (list) ;
	HistoryItem* item = list->itemAtIndex(-aNoOfSteps) ;
	list->goToItem(item) ;*/
}

void WMLBrowserContext::ResetVariableInstances()
{
	AttributeMap::iterator begin = m_attrMaps.begin() ;
	AttributeMap::iterator end = m_attrMaps.end() ;
	WMLVariable *var = 0 ;
	String value("") ;
	for (AttributeMap::iterator it = begin; it != end ; ++it)	{
		var = it->second ;
		ASSERT (var) ;
		var->setVariableValue(value) ;
	}
}

void WMLBrowserContext::AddVariable(const String& aVariableName, const String& aVariableValue, \
																EVariableConversionType type)
{
	if(isVariableNameValid(aVariableName))
	{
	    if (m_attrMaps.contains(aVariableName))
		return ;
	    WMLVariable *var = new WMLVariable (aVariableValue, type) ;

	    m_attrMaps.add(aVariableName, var);
        }
}

void WMLBrowserContext::SetVariableValue(const String& aVariableName, const String& aNewVariableValue)
{
	if (!m_attrMaps.contains(aVariableName))
		return ;
	WMLVariable *var = m_attrMaps.get(aVariableName);
	ASSERT (var) ;
	/* this is for debug purpose.. if the variable value is errornous, the we 
	do not store that variable so we need not have to assert if variable name
	is not present, else if its present and WmlVariable is not set then its an 
	issue in the code
	*/
	if (var)
		var->setVariableValue(aNewVariableValue);
}

bool WMLBrowserContext::isVariableNameValid(const String & aVariableName)
{
	if(!((isASCIIUpper(aVariableName[0])) ||
	     (isASCIILower(aVariableName[0])) ||
	     (aVariableName[0] == '_')))
	{
		return false;
	}

	int index = 1;
	while(index < aVariableName.length())
	{
		if((isASCIIUpper(aVariableName[index])) ||
		   (isASCIILower(aVariableName[index])) ||
		   (isASCIIDigit(aVariableName[index])) ||
		   (aVariableName[index] == '_'))
		{
			index++;
			continue;
		}
		else
		{
			return false;
		}
	}
	return true;
}
bool WMLBrowserContext::contains (String & aVariableName)
{
	return m_attrMaps.contains(aVariableName) ;
}

String WMLBrowserContext::GetVariableValue(String& aVariableName)
{
	WMLVariable *var = m_attrMaps.get(aVariableName);
	if (var)
		return var->getVariable ();
	return String();
}
void WMLBrowserContext::SetAccessDomain(String& aDomain)
{
	if (!aDomain.isEmpty())
		m_accessUrl.setHost(aDomain.deprecatedString());
}

void WMLBrowserContext::SetAccessPath(String& aPath)
{
		if (!aPath.isEmpty())
			m_accessUrl.setPath(aPath.deprecatedString()) ;
}

// This function will be mainly used for generating random names 
// for anonymous variables.These names will have the format
// @#%_var_<time_stamp> to maintain uniqueness.
String* WMLBrowserContext::RandomVarName()
{
	srand(static_cast<unsigned>(time(0)));
	String* randVarName = new String("");
	randVarName->append(String().format("@#%%_var_%ld",rand()));
	return randVarName;
}
AttributeMap& WMLBrowserContext::GetVariableAttributeMap()
{
	return m_attrMaps;
}

} //WebCore
#endif //SHP_SUPPORT_WML