/******************************************************************************************************
 * PROJECT :  Samsung Safari Browser
 * Copyright (c) 2007 Software Center, Samsung Electronics, Inc.
 * All right reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. (Confidential Information). You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics.
 */

/* FILE :
 * This file describes the Plugin specific structure/variable definitions 
 * which will be used by Webkit
 *
 * @name		HTMLBgSoundElement.h 
 * @author		SameerPatil (sameer.p@siso.com)  [M]
 * @version		$1.0$  [O]
 * @see		    related information [O]
 ******************************************************************************************************/


#ifndef HTMLBgSoundElement_h
#define HTMLBgSoundElement_h

#ifdef SUPPORT_NP_PLUGIN

#include "HTMLPlugInElement.h"

//FORWARD DECLARATIONS


#if USE(JAVASCRIPTCORE_BINDINGS)
#include <bindings/runtime.h>
#endif

namespace WebCore {
class PluginView;
class SVGDocument;


class HTMLBgSoundElement : public HTMLPlugInElement {

public:
    
	HTMLBgSoundElement(Document*);
    ~HTMLBgSoundElement();

    virtual HTMLTagStatus endTagRequirement() const { return TagStatusForbidden; }
    virtual int tagPriority() const { return 0; }

    virtual bool mapToEntry(const QualifiedName& attrName, MappedAttributeEntry& result) const;
    virtual void parseMappedAttribute(MappedAttribute*);

    virtual void attach();
    virtual void detach();
    virtual void insertedIntoDocument();
    virtual void removedFromDocument();
    virtual void attributeChanged(Attribute*, bool preserveDecls = false);
    virtual bool isURLAttribute(Attribute*) const;
    void createPlugin();	

#if USE(JAVASCRIPTCORE_BINDINGS)
    virtual KJS::Bindings::Instance* getInstance() const;
#endif

    String src() const;
    void setSrc(const String&);

	String type() const;
    void setType(const String&);

	float volume();
	void setVolume(const String& value);
	
	int loop();
	void setLoop(const String& value);
    
#if ENABLE(SVG)
    SVGDocument* getSVGDocument(ExceptionCode&) const;
#endif

    
public:	
	PluginView*			m_pluginView;//owned
	DeprecatedString	url;
    String				m_serviceType;
	INT32				m_loopCount;
	INT32				m_volume;
	Document*			m_doc;				//not owned
};

}

#endif//SUPPORT_NP_PLUGIN

#endif//HTMLBgSoundElement_h
