/*
 * This file is part of the DOM implementation for KDE.
 *
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2000 Simon Hausmann <hausmann@kde.org>
 * Copyright (C) 2004, 2006 Apple Computer, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#ifndef HTMLIFrameElement_h
#define HTMLIFrameElement_h

#include "HTMLFrameElementBase.h"

namespace WebCore {

class HTMLIFrameElement : public HTMLFrameElementBase
{
public:
    HTMLIFrameElement(Document* doc);

    virtual HTMLTagStatus endTagRequirement() const { return TagStatusRequired; }
    virtual int tagPriority() const { return 1; }

    virtual bool mapToEntry(const QualifiedName&, MappedAttributeEntry&) const;
    virtual void parseMappedAttribute(MappedAttribute*);

    virtual void insertedIntoDocument();
    virtual void removedFromDocument();
    
    virtual void attach();

    virtual bool rendererIsNeeded(RenderStyle*);
    virtual RenderObject* createRenderer(RenderArena*, RenderStyle*);
    
    virtual bool isURLAttribute(Attribute*) const;
#if PLATFORM(SHP) && defined(SFB_SSR_RENDERING)
// CELUNITE_SSR_Change_Begin
    virtual void handleContentChange();
// CELUNITE_SSR_Change_End
#endif
    String align() const;
    void setAlign(const String&);

    String height() const;
    void setHeight(const String&);

    String width() const;
    void setWidth(const String&);
#if PLATFORM(SHP) && defined(SFB_SSR_RENDERING)
	AtomicString url(){return m_URL;}
private :
    void enforceLay0utChange(int width, int height);
#endif
private:
    String oldNameAttr;
#if PLATFORM(SHP) && defined(SFB_SSR_RENDERING)
// CELUNITE_SSR_Change_Begin
    int m_iOrgFrameWidth;
    int m_iOrgFrameHeight;
// CELUNITE_SSR_Change_End
#endif
};

} // namespace WebCore

#endif // HTMLIFrameElement_h
