#if defined(SHP_SUPPORT_WML)
#include "config.h"
#include "AtomicString.h"
#include "EventNames.h"
#include "QualifiedName.h"
#include "WMLDocument.h"
#include "XMLTokenizer.h"
#include "HTMLNames.h"
#include "PlatformString.h"
#include "WMLBrowserContext.h"
#include "Frame.h"
#include "FrameLoader.h"
#include "FrameLoaderClient.h"
#include "WMLDoElement.h"
#include "KURL.h"

#if defined ( SFB_SUPPORT_WMLSCRIPT )
#include "WMLScriptInterface.h"
#endif//SFB_SUPPORT_WMLSCRIPT
#include "AnsiCLib.h" //For SHP memory apis

namespace WebCore {

using namespace HTMLNames;

//Kishore: Definitions of member functions of WMLDocument
WMLDocument::WMLDocument(DOMImplementation* implementation, Frame* frame)
    : Document(implementation, frame)
	,m_count(0)
	,m_doElementMap(new NodeMap)
#if defined (SFB_SUPPORT_WMLSCRIPT)
	,wmlscript(new WMLScript(this))
#endif
	,m_bIsOOM(false)
	,m_bSendReferer(true)
{
}

//Kishore: Added this to allow wml element
bool WMLDocument::childAllowed( Node *newChild )
{
    return newChild->hasTagName(wmlTag);
}


Tokenizer *WMLDocument::createTokenizer()
{
	//ASSERT(view());
	return new WMLTokenizer (this, view());
}

void WMLDocument::detach()
{
	Document::detach() ;

	NodeMap::const_iterator begin = m_doElementMap->begin() ;
	NodeMap::const_iterator end = m_doElementMap->end() ;
	int nodeId ;
	for (NodeMap::const_iterator it = begin; it != end ; ++it)	{
		nodeId = it->first ;
		frame()->loader()->client()->removeDoElementinUserMenu (nodeId) ;
	}
	m_count = 0 ;
	//m_doElementMap.clear() ;
	delete m_doElementMap ;
	m_doElementMap = new NodeMap ;
}

void WMLDocument::setDoElement(Node *node)
{
	WMLDoElement *element = static_cast<WMLDoElement *>(node) ;
	ASSERT(element) ;

	if (m_doElementMap->size() > 0)
	{
		WMLDocument *wmldoc = static_cast<WMLDocument *>(document()) ;
		WMLDoElement *wmlDoElem = static_cast<WMLDoElement *>(node) ;
		String name = wmlDoElem->name() ;
		String typeValue = wmlDoElem->type () ;
		WMLDoElement *pNode = 0;
		NodeMap::const_iterator it = m_doElementMap->begin();
		NodeMap::const_iterator end = m_doElementMap->end() ;
		Node *endNode = 0 ;
		Node *fstNode = it->second ;
		int count = m_doElementMap->size() ;

		endNode = end->second ;
		while (count)	{
			pNode = static_cast<WMLDoElement *>(it->second) ;
			ASSERT (pNode) ;
			if ( ((!name.isEmpty()) && name == pNode->name()) ||
				(name.isEmpty() && !typeValue.isEmpty() && typeValue == pNode->type() ))	{
				Node *parent = pNode->parent() ;
				if (parent && parent->hasTagName(templateTag))	{
					wmldoc->removeDoElement (it->first) ;
					break ;
				}
			}
			count -- ;
			if (count)
				++it ;
		}
	}

	Node *pChild = element->firstChild();
	while(pChild)
	{
		if(pChild->hasTagName(noopTag))
		{
			return;
		}
		pChild = pChild->nextSibling();
	}

	m_doElementMap->add(++m_count, node) ;
	String label = element->getDisplayString () ;
	frame()->loader()->client()->delegateDoElementToUserMenu (m_count, label) ;
}

void WMLDocument::removeDoElement (int nodeId)
{
	frame()->loader()->client()->removeDoElementinUserMenu (nodeId) ;
	m_doElementMap->remove (nodeId) ;
	m_count--;
	NodeMap tempDoMap;
	int index = 1;
	NodeMap::const_iterator it = m_doElementMap->begin();
	NodeMap::const_iterator end = m_doElementMap->end() ;
	while(it != end)
	{
		tempDoMap.add(index,it->second);
		++it;
		index++;
	}
	m_doElementMap->clear();
	it = tempDoMap.begin();
	end = tempDoMap.end();
	for(it = tempDoMap.begin(); it != end ; ++it)
	{
		m_doElementMap->add(it->first, it->second);
	}
}

void WMLDocument::handleRequestedGoEvent(int nodeId)
{
	WMLDoElement *doElement = static_cast<WMLDoElement *>(m_doElementMap->get(nodeId)) ;
	ASSERT(doElement) ;
	bool bIsEventHandled = false;
	doElement->handleEvent (DoMenuEvent, true, bIsEventHandled) ;
}

int WMLDocument::WML_ItemCount()
{
	return m_doElementMap->size();
}

} //WebCore
#endif
