/*
 * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
 *           (C) 1999 Antti Koivisto (koivisto@kde.org)
 *           (C) 2001 Dirk Mueller (mueller@kde.org)
 * Copyright (C) 2004, 2005, 2006, 2007 Apple Inc. All rights reserved.
 *           (C) 2006 Alexey Proskuryakov (ap@nypop.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */
 
#if defined(SHP_SUPPORT_WML)

#include "config.h"
#include "WMLSelectElement.h"

#include "CSSPropertyNames.h"
#include "CSSStyleSelector.h"
#include "CharacterNames.h"
#include "Document.h"
#include "Event.h"
#include "EventHandler.h"
#include "EventNames.h"
#include "FormDataList.h"
#include "Frame.h"
#include "HTMLFormElement.h"
#include "HTMLNames.h"
#include "HTMLOptionElement.h"
#include "HTMLOptionsCollection.h"
#include "KeyboardEvent.h"
#include "MouseEvent.h"
#include "RenderListBox.h"
#include "RenderMenuList.h"
#include <math.h>
#include <wtf/Vector.h>

#include "WMLDocument.h"
#include "WMLOptionElement.h"

namespace WebCore {

using namespace EventNames;
using namespace HTMLNames;

WMLSelectElement::WMLSelectElement(Document* doc, HTMLFormElement* f)
    : HTMLSelectElement(doc, f)
	, m_inRestore(false)
{
}

String WMLSelectElement::iName() const
{
	/*WMLDocument *wmlDoc = static_cast<WMLDocument *>(document()) ;
	String labelVal(wmlDoc->getVariableReplacedString(getAttribute(inameAttr).impl())) ;
	return labelVal ;*/
	return getAttribute(inameAttr) ;
}

String WMLSelectElement::iValue() const
{
	/*WMLDocument *wmlDoc = static_cast<WMLDocument *>(document()) ;
	String labelVal(wmlDoc->getVariableReplacedString(getAttribute(ivalueAttr).impl())) ;
	return labelVal ;*/
	return getAttribute(ivalueAttr) ;
}

String WMLSelectElement::valueAttribute() const
{
	String value ;
	String *temp = document()->frame()->getVariableReplacedString(getAttribute(valueAttr).domString()) ;
	if (temp)
		value = temp->copy() ;
	return value ;
}

void WMLSelectElement::parseMappedAttribute(MappedAttribute *attr)
{
    String varName; 
	String varValue = "";
	if ((attr->name() == inameAttr)||
		(attr->name() == nameAttr))
	{		
        varName = attr->value();
		WMLDocument *doc = static_cast<WMLDocument *>(document()) ;
		doc->BrowserContext()->AddVariable(varName,varValue,WMLBrowserContext::ENoEscape);
	}
	else if(attr->name() == ivalueAttr)
	{
	}
	else if(attr->name() == valueAttr)
	{
	}
	HTMLSelectElement::parseMappedAttribute(attr);
}

void WMLSelectElement::recalcListItems(bool updateSelectedStates) const
{
    m_recalcListItems = true;
    m_listItems.clear();
	bool bAppendListItems = true;
	bool isSelected = false;

	String tempIName = iName() ;
	//Default option selection is based on priorities. iname > ivalue > name > value
	//if(bCheckForOtherAttributes && !(getAttribute(inameAttr).isEmpty()))
	if(updateSelectedStates)
	{
		if (!tempIName.isEmpty())
		{
			//String varName = iName() ;//getAttribute(inameAttr);
			String value = ((WMLDocument *)document())->BrowserContext()->GetVariableValue(tempIName);
			((WMLDocument *)document())->BrowserContext()->SetVariableValue(tempIName, "");
			if(value.impl()->length())
			{
				isSelected = setDefaultOptionsUsingIndexString(value);
			}
		}
		String tempiValueAttr = iValue() ;
		//if(bCheckForOtherAttributes && !(getAttribute(ivalueAttr).isEmpty()))
		if(!isSelected && !tempiValueAttr.isEmpty())
		{
			//String varName = "";
			//String value =  String (((WMLDocument *)document())->getVariableReplacedString (getAttribute(ivalueAttr).impl(), WMLBrowserContext::ENoEscape)) ;
			//String value = iName() ;
			if(tempiValueAttr.impl()->length()) //if variable has some value
			{
				isSelected = setDefaultOptionsUsingIndexString(tempiValueAttr);
			}
		}
		String tempNameAttr = name() ;
		//if(bCheckForOtherAttributes && !(getAttribute(nameAttr).isEmpty()))
		if(!isSelected && !tempNameAttr.isEmpty())
		{
			//String varName = getAttribute(nameAttr);
			String value = ((WMLDocument *)document())->BrowserContext()->GetVariableValue(tempNameAttr);
			((WMLDocument *)document())->BrowserContext()->SetVariableValue(tempNameAttr, "");//for reload case
			if(value.impl()->length())
			{
				isSelected = setDefaultOptionsUsingValuesString(value);
			}
		}
		String tempValueAttr = valueAttribute() ;
		//if(bCheckForOtherAttributes && !(getAttribute(valueAttr).isEmpty()))
		if(!isSelected && !tempValueAttr.isEmpty())
		{
			//String varName = "";
			//String value =  String (((WMLDocument *)document())->getVariableReplacedString (getAttribute(valueAttr).impl(), WMLBrowserContext::ENoEscape)) ;
			String value = valueAttribute() ;
			if(value.impl()->length())
			{
				isSelected = setDefaultOptionsUsingValuesString(value);
			}
		}
		if(!isSelected)
		{
			//Set first option as default as there are no attributes / values
			setFirstOptionAsDefault();
			//list items appended in the above call, set flag to false.
			bAppendListItems = false;
		}
	}
	if(bAppendListItems)
	{
		Node *current = firstChild();
		while (current) 
		{
			if (current->hasTagName(optgroupTag) && current->firstChild()) {
				// ### what if optgroup contains just comments? don't want one of no options in it...
				m_listItems.append(static_cast<HTMLElement*>(current));
				current = current->firstChild();
			}
			if (current->hasTagName(optionTag))
			{
				m_listItems.append(static_cast<HTMLElement*>(current));
			}
			if (current->hasTagName(hrTag))
				m_listItems.append(static_cast<HTMLElement*>(current));

			Node* parent = current->parentNode();
			current = current->nextSibling();
			if (!current) 
			{
				if (parent != this)
					current = parent->nextSibling();
			}
		}
	}
	m_recalcListItems = false;			
}


bool WMLSelectElement::setDefaultOptionsUsingIndexString(String value) const
{
	//find ; in the value string. ; seperates multiple selection options
	bool retVal = false ;
	if(multiple())
	{
		int pos = 0;
		while((pos = value.find(';')) != -1)
		{
			String optionIndex = value.substring(0,pos);
			WMLOptionElement *pOption = getOptionWithIndex(optionIndex.toInt());
			if(pOption)
			{
				pOption->setSelectedState(true);
				retVal = true ;
			}
			value = value.substring(pos+1);
		}
		//2;3;4  In this index string, now we need get option for index 4
		WMLOptionElement *pOption = getOptionWithIndex(value.toInt());
		if(pOption)
		{
			pOption->setSelectedState(true);
			retVal = true ;
		}
	}
	else
	{
		//Single selection
		//search for value in the option list and focus it
		WMLOptionElement *pOption = getOptionWithIndex(value.impl()->toInt());
		if(pOption)
		{
			pOption->setSelectedState(true);
			retVal = true ;
		}
	}
	return retVal ;
}

bool WMLSelectElement::setDefaultOptionsUsingValuesString(String value) const
{
	//find ; in the value string. ; seperates multiple selection options
	bool retVal = false;

	if(multiple())
	{
		int pos;
		while((pos = value.find(';')) != -1)
		{
			String optionValue = value.substring(0,pos);
			WMLOptionElement *pOption = getOptionWithValue(optionValue);
			if(pOption)
			{
				pOption->setSelectedState(true);
				retVal = true;
			}
			value = value.substring(pos+1);
		}
		//2;3;4  In this index string, now we need get option for index 4
		WMLOptionElement *pOption = getOptionWithValue(value);
		if(pOption)
		{
			pOption->setSelectedState(true);
			retVal = true;
		}
	}
	else
	{
		//Single selection
		//search for value in the option list and focus it
		WMLOptionElement *pOption = getOptionWithValue(value);
		if(pOption)
		{
			pOption->setSelectedState(true);
			retVal = true;
		}
	}
	return retVal ;
}

void WMLSelectElement::restoreState(const String& state)
{
	m_inRestore = true ;
	HTMLSelectElement::restoreState(state) ;
	m_inRestore = false ;
}

void WMLSelectElement::setFirstOptionAsDefault() const
{
	WMLOptionElement* foundSelected = 0;
	Node *current = firstChild();
    while (current)
	{
        if (current->hasTagName(optgroupTag) && current->firstChild()) 
		{
            // ### what if optgroup contains just comments? don't want one of no options in it...
            m_listItems.append(static_cast<HTMLElement*>(current));
            current = current->firstChild();
        }
        if (current->hasTagName(optionTag)) 
		{
            m_listItems.append(static_cast<HTMLElement*>(current));
            if (!foundSelected && (usesMenuList() || (!m_multiple && static_cast<WMLOptionElement*>(current)->selected()))) 
			{
                foundSelected = static_cast<WMLOptionElement*>(current);
                foundSelected->setSelectedState(true);
            }
			else if (foundSelected && !m_multiple && static_cast<WMLOptionElement*>(current)->selected()) 
			{
                foundSelected->setSelectedState(false);
                foundSelected = static_cast<WMLOptionElement*>(current);
            }
        }
        if (current->hasTagName(hrTag))
            m_listItems.append(static_cast<HTMLElement*>(current));

        Node* parent = current->parentNode();
        current = current->nextSibling();
        if (!current) 
		{
            if (parent != this)
                current = parent->nextSibling();
        }
    }
}

int WMLSelectElement::getOptionIndex(String value) const
{
	int count = 0;
	for(Node *pChild = firstChild();pChild; pChild = pChild->nextSibling())
	{
		if(pChild->hasTagName(optionTag))
		{
			count++;
			WMLOptionElement *pOption = static_cast<WMLOptionElement *>(pChild);
			String optionValue = pOption->valueAttribute() ;//((WMLDocument *)document())->getVariableReplacedString (pOption->getAttribute(valueAttr).impl(), WMLBrowserContext::ENoEscape);
			if(optionValue == value)
			{
				return count;
			}
		}
	}
	return -1;
}

WMLOptionElement * WMLSelectElement::getOptionWithValue(String value) const
{
	for(Node *pChild = firstChild(); pChild; pChild = pChild->nextSibling())
	{
		if(pChild->hasTagName(optionTag))
		{
			WMLOptionElement *pOption = static_cast<WMLOptionElement *>(pChild);
			String optionValue = pOption->valueAttribute().copy();
			if(optionValue == value)
			{
				return pOption;
			}
		}
	}
	return NULL;
}

WMLOptionElement * WMLSelectElement::getOptionWithIndex(int index) const
{
	int count = 0;
	for(Node *pChild = firstChild(); pChild; pChild = pChild->nextSibling())
	{
		if(pChild->hasTagName(optionTag))
		{
			count++;
			WMLOptionElement *pOption = static_cast<WMLOptionElement *>(pChild);
			if(count == index)
			{
				return pOption;
			}
		}
	}
	return NULL;
}

} // namespace

#endif //SHP_SUPPORT_WML