#if defined(SHP_SUPPORT_WML)
#include "config.h"
#include "Element.h"
#include "WMLTemplateElement.h"
#include "AtomicString.h"
#include "CSSHelper.h"
#include "CSSPropertyNames.h"
#include "CSSValueKeywords.h"
#include "EventNames.h"
#include "HTMLDocument.h"
#include "HTMLNames.h"
#include "QualifiedName.h"
#include "WMLDocument.h"
#include "WMLBrowserContext.h"
#include "HTMLCollection.h"
#include "WMLDocument.h"
#include "Frame.h"
#include "FrameLoader.h"

namespace WebCore {

using namespace HTMLNames;

//WML <template> tag att

String WMLTemplateElement :: onenterbackward () const
{
    return getAttribute(onenterbackwardAttr);
}

void WMLTemplateElement::setOnenterbackward (const String &value)
{
    setAttribute(onenterbackwardAttr, value);
}


String WMLTemplateElement::onenterforward  () const
{
    return getAttribute(onenterforwardAttr);
}

void WMLTemplateElement::setOnenterforward  (const String &value)
{
    setAttribute(onenterforwardAttr, value);
}


String WMLTemplateElement :: ontimer () const
{
    return getAttribute(ontimerAttr);
}

void WMLTemplateElement::setOntimer  (const String &value)
{
    setAttribute(ontimerAttr, value);
}

WMLTemplateElement::WMLTemplateElement(Document *doc)
    :HTMLElement( templateTag,doc) 
{
}
WMLTemplateElement::~WMLTemplateElement()
{
}

void WMLTemplateElement::attach()
{
	if (attached())
		return ;

	HTMLCollection *collection = new HTMLCollection(this, HTMLCollection::DoTag) ;
	WMLDocument *wmldoc = static_cast<WMLDocument *>(document()) ;
	for(Node *node = collection->firstItem(); node; \
							node = collection->nextItem())
		wmldoc->setDoElement (node) ;
		/* here i get the wmlDoElement, present in the card, i need to add this to the document.
		Document API calls the frame API and it inturn calls the frameLoaderClient API.
		FrameLoaderClient passes the control to the UI using browserControl APIs
		I am not calling the same at the deck level bcos it adds all DoTag guys from 
		all the card guys
		*/
	delete collection ;

	HTMLElement::attach() ;
}

void WMLTemplateElement::detach()
{
	HTMLElement::detach() ;
}

void WMLTemplateElement::handleEvent(EventType eventType, bool bForwardNavigation, bool & bIsEventHandled)
{
	if(NavigationEvent == eventType)
	{
		if(document()->m_bTemplateNavigationEventHandled)
		{
			return;
		}
		document()->m_bTemplateNavigationEventHandled = true;
	
   		if((!(onenterforward().isNull())) && bForwardNavigation)
		{
			bIsEventHandled = true;
		}
		if((!(onenterbackward().isNull())) && !bForwardNavigation)
		{
			bIsEventHandled = true;
		}
	}
	else if((TimerEvent == eventType) &&
		    (!ontimer().isNull()))
	{
		bIsEventHandled = true;
	}
			

	//Check for ontimer, onenterforward and onenterbackward attribs here
	String url;
	if((!ontimer().isNull())&& eventType == TimerEvent)
	{
		url = ontimer();
	}
	else if(bForwardNavigation)
	{
		url = onenterforward();
	}
	else
	{
		url = onenterbackward();
	}

	if(!url.isNull())
	{
		int pos	= url.reverseFind('#');
		if(pos != -1)
		{
			String cardId = url.substring(pos+1);
			
			//create a History item with url of next card, and then set new card id.
			//RefPtr<HistoryItem> item = document()->frame()->page()->mainFrame()->loader()->createHistoryItemTree(document()->frame(), true);
			//item->setURLString(url);
			//document()->frame()->page()->backForwardList()->addItem(item);
			String completeUrl = "#";
			//completeUrl.append(document()->getActiveCardId());
			completeUrl.append(cardId);
			completeUrl = document()->completeURL(completeUrl);
			document()->setURL(completeUrl.deprecatedString());
			if(bForwardNavigation)
			{
				((WMLDocument *)(document()))->BrowserContext()->addHistoryItem(completeUrl);
			}
			// now that the entry to the history has been made replace the card id
			document()->setActiveCardId(cardId);
		}
		else
		{
			String empty ;
			document()->frame()->loader()->urlSelected(document()->completeURL(url),empty, NULL, false, true);
		}
	}

	for(Node *pChild = firstChild(); pChild; pChild = pChild->nextSibling())
	{
		pChild->handleEvent(eventType, bForwardNavigation, bIsEventHandled);
	}
}

} //namespace
#endif