/*
 *  Copyright (c)  SISO, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics.
 */
/**
 * @file		AdBlocker.cpp
 * @brief		implement Smartlink
 * @author	Gnanasekar.s (Gnanasekar.s@samsung.com) 
 * @version	1.0
 * @date		2008.08.18	v1.0		Created			Gnanasekar.S				
 **/
#ifdef SFB_ADBLOCKER_FEATURE
#include "CString.h"
#include "AdBlocker.h"
#include <wtf/Forward.h>

AdBlocker* AdBlocker::m_AdBlocker = NULL;
#ifdef SFB_GLOBAL_STATIC_HANDLING
void initAdBlocker()
{
    AdBlocker::Init();
}

void deInitAdBlocker()
{
    AdBlocker::Finalize();
}
#endif

AdBlocker::~AdBlocker()
{
}

AdBlocker::AdBlocker()
{
}

void AdBlocker::Init()
{
	if( !m_AdBlocker)
	{
		m_AdBlocker = new AdBlocker();
		m_AdBlocker->CreateStateMachineObject();
	}
}

void AdBlocker::Finalize()
{
	if(m_AdBlocker)
	{
		m_AdBlocker->m_abRegExp = NULL;
		delete m_AdBlocker;
	}
}

void AdBlocker::CreateStateMachineObject()
{
	KJS::UString* pattern = new KJS::UString("-adspace|&adspace=|=viewAdJs|&affiliate=$subdocument|&ClientType=*&AdID=|&google_adpage=|\
											  &partner$subdocument?ad_id=?adtype=?affiliate?getad=&$~other,~object-subrequest?showbanner=|\
											  _ad.aspx|_adbrite|_adfunction|_ads/$image|_ads.php?|_adspace|_banner_ad|_bannerid*random|\
											  _companionad.|_files/*htm|$subdocument|adaffiliate|AdIFrame$match-case|admentor|ADTECH;cookie=|\
											  ads.sv.publicus.$script|adsfac.net|affiliatebrand.|audienceprofiler.|aurora-*marketing.co|\
											  banner-ad|bannerad|BannerMangement|banners&id=|blog.tmcnet.*/overlib.js|brandcentral.|\
											  Click*Advertiser|clicktag=*/ad|content.search|adserving.cpxinteractive|cubics.com/|dbbsrv.com|\
											  dgmaustralia.|download-door.search.com/search|dynamicad.|earthlink*/promos|eas.blocket.|\
											  engine.awaps.net/|exitexplosion.*/exit.js|expedia_ad.|faireagle.com|favinfo.com/ad.|\
											  gamesbanner.net/|geocities.com/js_source/|google.com*adfetch|google.com*/promo_|hera.hardocp.|\
											  imageshack*tagworld|interclick.|js.worthathousandwords.|js2.yimg.*_popup_|kanoodle|link_ads|\
											  mediacorpsingapore|medrx.sensis.com.au/content/|nebuad.com|network.ninemsn.*/share/|nbjmp.com/|\
											  openbanner$match-case|page.grabclick.|phpadsnew$~other,~object-subrequest|popinads.|\
											  popunder$script|popup_ad.|precisionclick.|pro-market.|promopop|ptnrcontent|publicidad|quigo.com|\
											  rad.live.com/ADSAdClient|rankz.armorbot.|richmedia.*yimg.|sponslink|sponsor.gif|sponsorads|\
											  sponsoredlink$~stylesheet|sponsored_links|sponsors*banner|storefronts$~stylesheet|\
											  sys-con.com/common/$script|targetpoint.com|textlink-ads.$~image|themis.yahoo.|vs20060817.com/|\
											  worsethanfailure.*/Tizes/$object|www.cb.cl/*banner|www.ad.tomshardware.*=banner|xcelsiusadserver.|\
											  yceml.net|.1100i.com/|.188server.|.2mdn.net/|.360ads.|.43plc.com|.about.com/0/$subdocument|\
											  .accuserveadsystem.|.ad.tomshardware.com/$script|.ad-flow.|.ad20.net/|.ad4cash.|.adaction.|\
											  .adbard.net/ab|.adbrite.com/mb/|.adbureau.|.adbutler.|.adcentriconline.|.adchap.|.adecn.com/|\
											  .adengage.|.adfusion.|.adgardener.|.adgine.|.adgroups.|.adhese.|.adicate.|.adition.com/|\
											  .adinterax.|.adireland.|.adjuggler.|.admarketplace.|.adnet.biz|.adlink.net|.adnet.ru|.adocean.|\
											  .adoperator.|.adpark.|.adpinion.|.adsdk.com/|.adserver.*?|.adservinginternational.|.adsforindians.|\
											  .adshopping.|.adshuffle.|.adsmarket.|.adsonar.|.adspeed.|.adtext.|.adtmedia.|.adtology3.|.adtrgt.|\
											  .adultadworld.|.adultfriendfinder.com/banners/|.adverserve.|.advertarium.|.adverticum.net/|\
											  .advertising.|.advertlets.|.advertserve.|.adviva.|.adxpower.|.afcyhf.|.affiliate.|.affiliatefuel.|\
											  .affiliatefuture.|.affiliatesensor.|.affilimatch.|.aim4media.|.akamai.*sponsor|.alphagodaddy.|\
											  .anrdoezrs.|.arcadebanners.|.as5000.|.assoc-amazon.|.atdmt.com/|.atwola.|.auspipe.|.awin1.com|\
											  .awltovhc.|.axill.com/|.azads.com/|.azjmp.com/|.azoogleads.|.bannerbank.ru/|.bannerconnect.|\
											  .bannersmania.|.bbc.co.uk/*/vs.js|.begun.ru/|.belointeractive.|.bestofferdirect.|.bidvertiser.|\
											  .bimedia.net/video/$object|.blogads.|.bluestreak.|.bravenetmedianetwork.|.bravenet.*/rover/|\
											  .bridgetrack.|.btrll.com/*|.burstnet.|.c8.net.ua/|.casalemedia.|.cc-dt.com/|.centralmediaserver.$~image|\
											  .cgecwm.org/|.checkm8.|.checkmystats.|.chitika.|.cjt1.net|.clash-media.|.claxonmedia.|.clickad.|\
											  .clickbooth|.clickexperts.|.clickintext.$~script|.clickthrucash.|.clixgalore.|.cogsdigital.|\
											  .com/banner$subdocument|.commissionmonster.|.connextra.|.contextweb.|.cpaclicks|.cpays.com/|\
											  .cpmstar.|.cpuim.com/|.crashextads.|.crispads.|.crwdcntrl.net/cc.js|.decisionmark.|.decisionnews.|\
											  .deepmetrix.|.directorym.|.dl-rms.com/|.domainsponsor.|.doubleclick.|.dpbolvw.|\
											  .ebayrtm.com/rtm?RtmCmd&a=inline&$script|.ebaystatic./adserver|.edge.ru4.|.empoweredcomms.|\
											  .etology.|.euroclick.|.exponential.|.eyereturn.|.eyewonder.|.fairfax.$~stylesheet|.falkag.|.fastclick.|\
											  .feedburner.com/~a/|.fimserve.|.firstadsolution.|.fixionmedia.|.flux.com/*?|.fmpub.net/|.forrestersurveys.|\
											  .fluxads.|.flyordie.com/games/free/b/*--?p=|.ftjcfx.|.funklicks.|\
					                          .g.akamai.*/ads.|.game-advertising-online.|.gamecetera.|.gamersbanner.|.geopromos.|.gestionpub.|\
											  .gklmedia.|.go.com/*ads.js|.googleadservices.|.grabmyads.|.henwo.com/$script|\
										      .hit-now.|.hosticanaffiliate.|.httpool.|.hypemakers.|.hypervre.$script|.ibatom.*/syndication/|.ic-live.|\
											  .icdirect.|.idg.com.au/images/*_promo$image|.imagesatlantic.|.imedia.co.il/|.infinite-ads.|.imglt.com/|\
											  .impresionesweb.|.imrworldwide.|.indiads.|.industrybrains.|.inetinteractive.|.infocious.|.intellitxt.|.interpolls.|.ivwbox.|\
											  .jdoqocy.|.jydesign.|.ketoo.com/|.klipmart.|.kontera.|.kqzyfj.|.leadacceptor.|.lduhtrp.|\
											  .lightningcast.$~other,~object-subrequest|.linkads.*?|.linkexchange.|.linkworth.|.litres.ru/static/banner/|\
											  .ltassrv.|.maxserving.|.mb01.com/|.mbn.com.ua/|.medialand.ru/|.mediaonenetwork.|.mediaplex.|.mediatarget.|.mediavantage.|\
											  .megaupload.com*/aff2.php|.mercuras.|.metaffiliation.|.microsoftaffiliates.*.aspx?|.miva.com/|\
											  .mochiads.com/srv/$other,object-subrequest|.mootermedia.|.msn.com/?adunitid|.myway.com/gca_iframe.|.neoseeker.com/*_pc.html|\
										      .netavenir.|.newanglemedia.com/clients/|.newsadstream.|.ngads.com/|.northmay.|.ng/*&affiliate=|.nwsource.*/adv.gif|.nyadmcncserve-|\
											  .obibanners.|.onenetworkdirect.|.openx.org/a*.php|.overture.|.oxado.com/|.pc-ads.com/|.peelawayads.com/affads/|.perfb.com|.pgpartner.|\
											  .pheedo.|.php?bannerid|.php?adclass|.platinumadvertisement.|.playertraffic.|.pointroll.|.predictad.|.pricegrabber.$subdocument|\
											  .pricespy.co.nz/adds/|.primaryads.|.pro-advertising.|.probannerswap.|.profitpeelers.$script|.projectwonderful.|\
											  .proximic.com/js/widget.js|.pulse360.|.qksrv.net/|.qksz.net/|.questionmarket.|.revresda.|.revsci.|.rmxads.|\
											  .rottentomatoes.*size=*x*&dechannel|.rovion.*?AffID=|.rwpads.|.scanscout.|.shareasale.|.shareresults.|.smartadserver.|\
											  .smarttargetting.|.snap.com/$script|.snopes.com/*/*ad$subdocument|.socialmedia.com/|.sonnerie.|\
											  .space.com/promo/|.sparkstudios.|.specificclick.|.specificmedia.|.speedsuccess.net/|.sponsorpalace.|\
											  .spotplex.*widget|.srtk.net/|.sta-ads.|.survey-poll.|.swf?clickTag=|.tacoda.|.targetnet.|.thebigchair.com.au/egnonline/|\
											  .tiser.com|.tkqlhce.|.torrentspy.*staticframe|.total-media.net/|.tqlkg.com|.tradedoubler.|.trafficmasterz.|.trafic.|.tribalfusion.|\
											  .trigami.|.typepad.com/sponsors/|.tyroo.com|.uimserv.net/|.unicast.|.universalhub.com/bban/|.utarget.|.valuead.|.valueclick.|.vibrantmedia.|\
											  .videosift.com/bnr.php?|.visitorglobe.*record|.vpico.com/|.webads.co.nz|.webmasterplan.|.widgetbucks.|.worlddatinghere.|.xchangebanners.|\
											  .y.megaclick.|.yimg.com/a/1-$~stylesheet|.yimg.com/adv/|.yimg.com/*/fairfax/$image|\
											  .zanox.com/|.zangocash.*/detectenvironment|.zeads.com/|.zedo.com/|.zoomdirect.com.au/|\
											  /63.225.61.|/64.73.24.44|/207.67.9.41/*|/213.239.222.7/ad/*|/217.15.94.117|/.adserv/*|/a.clearlightdigital.|/a.kerg.net/*|/a.lakequincy.|/ab.vcmedia.|\
											  /abmw.aspx|/ad/*promo|/ad/code$script|/ad.asp?|/ad.aspx?|/ad2.aspx?|/ad.php?|/ad/frame|/ad/mercury$object|/ad/serve|/ad_*.gif|\
											  /ad_$subdocument|/ad_functions|/ad_insert.|/ad_refresher.|/ad_wrapper|/ad2games.|/adbanner|/adbrite$subdocument|/adbrite.|/adclick|/adcode.js|/adconfig/*|\
											  /adcontent.$~other,~object-subrequest|/adcycle/*|/adf.cgi?|/adframe.|/adframe_|/adfshow?|/adgraphics/*|/adheader|/adhoc/js/swfobject.js|/AdIFrame.|/adimages/*|\
											  /adfunction|/adimage.|/adinsert.|/adjs.php?|/adjsmp.php?|/adlabel|/adlinks.js|/adman/www/*|/admanagement/*|/ad_manager.js|/admanager$script|/admatch-syndication.|\
											  /adpage.|/adpeeps/*|/adpeeps.php|/Adplayer/*|/adproducts/*|/adRelated.|/adsreporting/*|\
											  /adrevolver/*|/adrotator|/ads/*|/ads_reporting/*|/ads_v2.js|/ads.htm|/ads.php?|/ads*.php$subdocument|/Ads-Leader|\
											  /Ads-Rec|/Ads-Sky|/ads2.php?|/ads2/*|/ADSAdClient31.dll?GetAd?$other,object-subrequest|/adscript|/adsense_$script|/adsense.|/Adserver?|\
											  /adsfolder/*|/adshow?|/AdsIframe/*|/adsimage/*|/AdsManager/*|/adspro/*|/adsonar.|/adSwap.js|/adsyndication.|/adtags/*|/ADTECH;|/adtext.|/adtext_|/adtraff.|\
											  /adtype.php?|/advert_|/advert/ms|/adverti|/advertpro/*|/adverts_|/adverts/*|/adview.|/AdWorks/*|/adwrapper/*|/adx.js|/affiliate_$image|/affiliate*/ad/*|/AffiliateBanners/*|\
											  /affiliates.babylon.|/AffiliateWiz/*|/afr.php?|/ah.pricegrabber.com/cb_table.php|/aj.600z.|/ajrotator/*|/ajs.php?|/anchor.captainad.|/annonser/*|/au.video.yahoo.com/ads?|/autoPromo|\
											  /banimpress.|/banman.asp?$subdocument|/banman/*$subdocument,script|/banman.isoftmarketing.|/banmanpro/*&ad|/banner/Ad|/banner_db.php?|/banner_ads.|/Banner_Management/*|/banner.php?http|\
											  /banner_file.php?|/bannermedia/*|/banners/*$script,object|/banners/banners.jsp?|/banners.adultfriendfinder|/banner*ClickTag=|/banners/*.gif$~background|/bannery/*?banner=|\
											  /bbccom.js?|/bbc.com/script/1/config.js|/blogad_|/blogads|/bmp/banman.asp?|/bnrsrv.*?|/bs.yandex.ru|/cas.clickability.com/*|/clickserv|/cm8adam|/cm8space_call$script|/cms/Profile_Display/*|\
											  /cnnSLads.js|/cnwk.*widgets.js|/commercials/splash|/content.yieldmanager.|/ContextAd.|/csDynamic$match-case|/ctxtlink/*|/d.m3.net/*|/data.resultlinks.|/de*.myspace.$subdocument|/descPopup.js|\
											  /destacados/*$subdocument|/direct_ads.|/directads.|/dontblockthis/*|/DisplayAds$match-case|/DNSads.html?|/dsg/bnn/*|/DynamicAd?|/dynBanner/flash/*|\
										      /eBayISAPI.dll?EKServer&|/ecustomeropinions.com/popup/*|/ekmas.com|/export_feeds.php?&banner|/eyoob.com/elayer/*|/fairadsnetwork.|/farm.sproutbuilder.|/flashAds.|/flashbanner/*$match-case|\
											  /flipmedia|/forms.aweber.|/freetrafficbar.|/fuseads/*|/gamecast/ads|/gamersad.|/gampad/google_service.js|/getad.php?|/get_ad.php?|/getbanner.cfm?$subdocument|/google_ads/*|/google-adsense$subdocument|\
											  /googleAd.js|/googleframe.|/hits.europuls.|/hits4pay.|/hotjobs_module.js|/houseads/*|/html.ng/*$subdocument|/httpads/*|/iframead.|/iframed_*sessionid=|/images/ad/*|/index_files/*.htm|\
											  /IndianRailways/*$match-case|/intext.js|/invideoad.|/itunesaffiliate|/job_ticker.|/js/ysc_csc_|/js.ng/site=|/kermit.macnn.|/kestrel.ospreymedialp.|/l.yimg.com/a/a/1-/flash/promotions/*/0*|\
											  /launch/testdrive.gif|/layer-ads.|/LinkExchange/*|/linkreplacer.js|/linkshare/*|/logos/adLogo|/lw/ysc_csc_$script|/marketing*partner$image|/mac-ad?|/magic-ads/*|/media.funpic.*/layer.|/mediamgr|/medrx.sensis.com.au/*|\
									          /miva_ads.|/MNetOrfad.js|/mod_ad/*|/nascar/*/defector.js|/nascar/*/promos/*$script,object|/network.sportsyndicator.|\
											  /oas_logic.|/oasc03.$other,object-subrequest|/oascentral.|/oasisi.php?|/oasisi-*.php?|/openads/*?|/openads2/*|/outsidebanners/*|/overture/*$subdocument|/overture_|/pagead/*&videoad_start_delay=1|/pagead/imgad?|/pageear/*|\
											  /pagepeel$script|/partner*rotate|/partner.sbaffiliates.|/partner.video.syndication.msn.com/*$~other,~object-subrequest|/partners/*$script|/partnership/*affiliate|/peel.js|/peelad/*$script|/phpads/*|/phpadserver/*|/pilot_ad.|\
											  /play/ad/*$other,object-subrequest|/podimages/*$match-case|/popupjs.|/promos/*banner.gif|/printads/*|/public/ad?|/publisher.shopzilla.$subdocument|/r.mail.ru$object|/rad.*?GetSAd=|/resources.infolinks.|\
											  /rok.com.com/*|/rotateAds.|/rotating_banner|/rotating.php$subdocument|/rotation/*.php?|\
										      /rtq.careerbuilder.|/s3.buysellads.|/s7121.vsservers.|/shared/promos/*|/show.asp?_sky$subdocument|\
											  /show_ad_|/show_afs_ads.js|/show_deals.js|/show_i.php?|/showad.$subdocument|/showads.|/showbanner.php?|/skyad.php|/small_ad.|/socialads.js|/softsale/*$subdocument|\
											  /Splash/Page_Header/*|/spinbox.freedom.|/sponsImages/*|/sponsorad.|/sponsored$subdocument|/support.biemedia.|/surveyCookie.js|/svgn.com/*|/textad?|/textAd.js|/tii_ads.js|/tizes/a.aspx?|\
											  /trusearch.net/affblock/*|/ttz_ad.js|/userbanners/*|/vendshow/*|/video.ap.org/*/ad_js.|/videoad.$match-case|/videoads.$match-case|/videoads/*$match-case|\
											  /w1.buysub.$~image|/webadImg/*|/webads_|/whiteglove.jsp?|/widget.blogrush.com/show.js|/ws.amazon.*/widgets/q?|/ygames_e/embed/src/embedPlayer.js|/ysmads.html|//wrapper.3dgamers.|\
											  http://171.66.124.80/|http://196.41.0.207/|http://72.32.71.244|http://a.ads.|http://ad*.emidiate.|http://abm.hothardware.|http://ad-uk.|http://ad.|http://ad0.|http://ad1.|http://ad2.|http://adbureau.|\
                                              http://adclient|http://adcreative.|http://adfarm.|http://adimg.|http://adimages.|http://adinterax.|http://adnet.|http://adq.|http://adremote.|http://adonline.|http://ads.|http://ads1.|http://ads2.|http://ads3.|http://ads4.|\
                                              http://ads5.|http://adsatt.|http://adsearch.|http://adserv|http://adsnew.|http://adsremote.|http://adstream.*.cgi|http://adsvr.|http://adsys.|http://adt.|http://adtags.|http://adtology|http://adv.|http://advert.|http://adverts.|\
                                              http://advision.*/getad|http://adworks.|http://adx.|http://affiliates.*.aspx?|http://afimages.|http://almobty.com/|http://altfarm.|http://avpa.|http://banner.|http://banners*&Random=|http://banners.*http|http://biz28.|\
                                              http://body.imho.ru/|http://bwp.$subdocument|http://cbanners.|http://cdn.nvero.|http://circads.|http://common.*/slider.js|http://dclk.*.ng/|http://feeds.*/~a/|http://fdads.sv.|http://ffxcam.|http://getad.|http://images.*/banners/|\
                                              http://jazad.|http://lads.*-gutter-|http://mads.|http://marketing.*http|http://ng3.ads.|http://oas-central.|http://pagead2.$~other,~object-subrequest|http://partner.$subdocument|http://promo.|http://promo2.|http://rcm*.amazon.|\
                                              http://reklama.|http://richmedia2.|http://rss.*/~a/|http://servedbyadbutler.|http://smartad.|http://stocker.bonnint.|http://synad|http://ttsrc.aroq.|http://video.flashtalking.|http://wrapper.*/a?|http://*/partners.$subdocument|\
											  /adv598.php|antivirus2009-freescan.|antivirus-scanonline.com/|/avsystemcare.|/bleso.com/*|cdpuvbhfzz.com|.defender-scanner.|freestat.ws|.malwarecrush.|/onlinexpscanner.com/*|spysheriff.com/|.spyshredderscanner.|\
									          windows-virus-scanner.com|/virus-onlinescanner.|.virus-scanonline.|/virus-scanonline.");
	KJS::UString* flags = new KJS::UString();
	m_AdBlocker->m_abRegExp = KJS::RegExp::create( *pattern, *flags);
}

bool AdBlocker::isAd_Banner(WebCore::String& s)
{
	OwnArrayPtr<int>* tmpOvector = NULL;
	
	KJS::UString* InputString = new KJS::UString(s);

	int result = m_AdBlocker->m_abRegExp.get()->match( *InputString, 0, tmpOvector);
	
	delete InputString;
	
	if(	result == -1)
		return false;

	SysDebug(( MID_EXCEPTION, "AdBlocker Blocked: %s ", s.latin1().mutableData() ));

	return true;
}



#endif
