/*
 * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer. 
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution. 
 * 3.  Neither the name of Apple Computer, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "NetscapePlugInStreamLoader.h"

#include "FrameLoader.h"
#include "DocumentLoader.h"
/* <TBD> revert the m_client check unwanted !!!*/
namespace WebCore {

NetscapePlugInStreamLoader::NetscapePlugInStreamLoader(Frame* frame, NetscapePlugInStreamLoaderClient* client)
    : ResourceLoader(frame, true, true)
    , m_client(client)
{
	SysDebug((MID_BROWSER, "NetscapePlugInStreamLoader:: constructor called for this pointr 0x%08x\n", this));
}

NetscapePlugInStreamLoader::~NetscapePlugInStreamLoader()
{
	SysDebug((MID_BROWSER, "NetscapePlugInStreamLoader:: destructor called for this pointr 0x%08x\n", this));
}

PassRefPtr<NetscapePlugInStreamLoader> NetscapePlugInStreamLoader::create(Frame* frame, NetscapePlugInStreamLoaderClient* client)
{
	SysDebug((MID_BROWSER, "NetscapePlugInStreamLoader::create called \n"));
    return new NetscapePlugInStreamLoader(frame, client);
}

bool NetscapePlugInStreamLoader::isDone() const
{
	SysDebug((MID_BROWSER, "NetscapePlugInStreamLoader::isDone called \n"));
    return !m_client;
}

void NetscapePlugInStreamLoader::releaseResources()
{
	SysDebug((MID_BROWSER, "NetscapePlugInStreamLoader::releaseResources called \n"));
	m_client = 0;
    ResourceLoader::releaseResources();
}

void NetscapePlugInStreamLoader::didReceiveResponse(const ResourceResponse& response)
{
	SysDebug((MID_BROWSER, "NetscapePlugInStreamLoader::didReceiveResponse called for this pointr 0x%08x\n", this));

	RefPtr<NetscapePlugInStreamLoader> protect(this);
    if (!m_client)
        return;
    m_client->didReceiveResponse(this, response);

    // Don't continue if the stream is cancelled

    if (!m_client)
        return;

    ResourceLoader::didReceiveResponse(response);

    // Don't continue if the stream is cancelled
    if (!m_client)
        return;

    if (response.isHTTP() && (response.httpStatusCode() < 100 || response.httpStatusCode() >= 400))
        didCancel(frameLoader()->fileDoesNotExistError(response));
}

void NetscapePlugInStreamLoader::didReceiveData(const char* data, int length, long long lengthReceived, bool allAtOnce)
{
	SysDebug((MID_BROWSER, "NetscapePlugInStreamLoader::didReceiveData called \n"));

    RefPtr<NetscapePlugInStreamLoader> protect(this);
	if (!m_client)
        return;

    m_client->didReceiveData(this, data, length);
    
    ResourceLoader::didReceiveData(data, length, lengthReceived, allAtOnce);
}

void NetscapePlugInStreamLoader::didFinishLoading()
{
	SysDebug((MID_BROWSER, "NetscapePlugInStreamLoader::didFinishLoading called \n"));

    // If load has been cancelled after finishing (which could happen with a 
    // JavaScript that changes the window location), do nothing.
    if(cancelled())
        return;

    RefPtr<NetscapePlugInStreamLoader> protect(this);

    m_documentLoader->removePlugInStreamLoader(this);
    if (!m_client)
	    return;
	m_client->didFinishLoading(this);
    ResourceLoader::didFinishLoading();
}

void NetscapePlugInStreamLoader::didFail(const ResourceError& error)
{
	SysDebug((MID_BROWSER, "NetscapePlugInStreamLoader::didFail called \n"));
    RefPtr<NetscapePlugInStreamLoader> protect(this);

    m_documentLoader->removePlugInStreamLoader(this);
    if (!m_client)
        return;
    m_client->didFail(this, error);
    ResourceLoader::didFail(error);
}

void NetscapePlugInStreamLoader::didCancel(const ResourceError& error)
{
	SysDebug((MID_BROWSER, "NetscapePlugInStreamLoader::didCancel called \n"));
    RefPtr<NetscapePlugInStreamLoader> protect(this);

    m_documentLoader->removePlugInStreamLoader(this);
    if (!m_client)
        return;
    m_client->didFail(this, error);
    ResourceLoader::didCancel(error);
}

}
