/*
* ==============================================================================
*  Name         : ScrollNavigation.h
*  Part of      : WEBCORE / Page
*  Description  : Interface of Navigation
*  Date         : April 29, 2008
*  Authors      : Vineet Chaudhary (vineet.c@siso.com),
*                 Prashant Nevase (n.prashant@siso.com)
* ==============================================================================
*/


#ifndef __SCROLL_NAVIGATION_H__
#define __SCROLL_NAVIGATION_H__

#if PLATFORM(SHP)
#include "CMMDefs.h"
#endif // PLATFORM(SHP)

#include "Navigation.h"

namespace WebCore {
class Page;

class CScrollNavigation : public CNavigation
{
public:
	//STATIC METHODS
	//for static global handling
	static void initSNStatics();

	static void init();
	static void finalize();

	static CScrollNavigation* createInstance();
	static void destroyInstance(CScrollNavigation* scrollNavigation);

	//OVERRIDDEN METHODS
	virtual void setFrameView(FrameView* frameView);

	//for new page load in the same window
	virtual void resetPage();
	virtual void pageLoadingFinished();

	virtual void updateVerticalStats() {}
	virtual void updateHorizontalStats() {}

	virtual void handleNavigationEvent(int keyCode, bool isKeyUp);

	virtual void getPointerLocation(FrameView** frameView, int* x, int* y) {}
	virtual void setPointerLocation(int x, int y) {}

	virtual void scrollBy(int xOffset, int yOffset) {}

#ifdef SUPPORT_TOUCH_SCREEN
	virtual void touchPressed(IntPoint wndPoint);
	virtual void touchMoved(IntPoint wndPoint);
	virtual void touchReleased(IntPoint wndPoint);
#endif

	virtual void doPostPaintAction() {}
	virtual void doDefaultAction() {}

	virtual Node* getHoveredNode();

private:
	CScrollNavigation();
	~CScrollNavigation();

	FrameView* frameViewAtPoint(IntPoint wndPoint);

};

} //namespace WebCore

#endif