/*
 * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
 * Copyright (C) 2006 Michael Emmel mike.emmel@gmail.com 
 * Copyright (C) 2007 Holger Hans Peter Freyther
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include "config.h"
#include "Frame.h"

//Jan-16 Vamsi/Pramod
#ifdef _SFB_SEARCH_STRING_IN_PAGE
#include "FramePrivate.h"
#include "FrameView.h"
#include "GraphicsContext.h"
#include "RenderTheme.h"
#endif

#include "NotImplemented.h"


namespace WebCore {

DragImageRef Frame::dragImageForSelection() 
{
    notImplemented();
    return 0;
}

void Frame::dashboardRegionsChanged()
{
    notImplemented();
}

//Jan-16 Vamsi/Pramod
#ifdef _SFB_SEARCH_STRING_IN_PAGE

/*
This function is tailored for presence of Edit Field on top of GC
*/

#define SEARCH_EDIT_FIELD_ADJUSTMENT 40

void Frame::revealSelectionTextSearch()
{
    IntRect rect;
	FrameView* frameView = Frame::view();
	RenderLayer::ScrollAlignment gAlignSHP = { RenderLayer::noScroll, RenderLayer::alignCenter, RenderLayer::alignToClosestEdge };

    switch (selectionController()->state()) {
        case Selection::RANGE:
            rect = enclosingIntRect(selectionRect(false));
            break;
    }

    Position start = selectionController()->start();

    ASSERT(start.node());
	
	IntRect viewRect = enclosingIntRect(frameView->visibleContentRectConsideringExternalScrollers());

	//Scroll condition
	if( (rect.y() > (viewRect.y() + viewRect.height() - SEARCH_EDIT_FIELD_ADJUSTMENT) )
		||  (rect.y() < viewRect.y()) )
	{
		gAlignSHP.m_rectVisible = RenderLayer::alignCenter;
		gAlignSHP.m_rectHidden	= RenderLayer::alignCenter;
		gAlignSHP.m_rectPartial = RenderLayer::alignCenter;
	}

    if (start.node() && start.node()->renderer()) {
        if (RenderLayer *layer = start.node()->renderer()->enclosingLayer())
            layer->scrollRectToVisible(rect, gAlignSHP, gAlignSHP);
    }
}
#endif

}
