/*
 * This file is part of the internal font implementation.  It should not be included by anyone other than
 * FontMac.cpp, FontWin.cpp and Font.cpp.
 *
 * Copyright (C) 2006, 2007 Apple Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

#include "config.h"
#include "FontPlatformData.h"

#include "PlatformString.h"
#include "StringHash.h"
#include "FontDescription.h"

#define _SFBAL_SUPPORT_VECTOR_FONT

#include "ShpJasmine.h"

namespace WebCore {

FontPlatformData::FontPlatformData(const FontDescription& fontDescription, const AtomicString& family)
{
	FontPlatformData::init();
    m_syntheticBold = m_fontAttr.bBold = fontDescription.bold();
    m_syntheticOblique = m_fontAttr.bItalic = fontDescription.italic();
    m_size = m_fontAttr.iSize = (int)fontDescription.computedSize();
    SFBalGwesSetFontAttr(m_fontAttr);
}

FontPlatformData::~FontPlatformData()
{

}

unsigned 
FontPlatformData::hash() const
{ 
    uintptr_t hashCodes[2] = { (uintptr_t)0, m_syntheticBold << 1 | (int)m_syntheticOblique };
    return StringImpl::computeHash(reinterpret_cast<UChar*>(hashCodes), sizeof(hashCodes) / sizeof(UChar));
}

bool 
FontPlatformData::operator==(const FontPlatformData& other) const
{ 
    return m_syntheticBold == other.m_syntheticBold && m_syntheticOblique == other.m_syntheticOblique && m_size == other.m_size;
}


HFNT
FontPlatformData::getFontHandle()const
{
	SFBalGwesSetFontAttr(m_fontAttr);
    return SFBalGwesGetHFontSreenInfo();
}

bool FontPlatformData::isFixedPitch()
{
	// as we have no way of finding the spacing pattern we assume the spacing not to be uniform
	return false;
}

bool FontPlatformData::init()
{
	return true;
}

void FontPlatformData::setFont(HGC* gc) const
{
	SFBalGwesSetFontAttr(m_fontAttr);
    HFNT hFont = SFBalGwesGetHFontSreenInfo();
	WmSetFont(*gc, hFont);
}


#ifdef SFB_GLOBAL_STATIC_HANDLING

void initFontPlatformData()
{
    SFBalGwesInitializeGraphic();
}

void deInitFontPlatformData()
{
    SFBalGwesFinalizeGraphic();
}

#endif

}