#ifndef HTTPLOADER_H
#define HTTPLOADER_H

//  INCLUDES
#include "BridgeTypes.h"
#include "UrlLoader.h"
#include "WebkitLoader.h"

// CONSTANTS

// MACROS

// DATA TYPES

// FUNCTION PROTOTYPES

// FORWARD DECLARATIONS
class ContentDispatcher;

// CLASS DECLARATION

/**
*  This is a http loader class. It handles http tranasactions like http://google.html
*  @lib resLoader.lib
*/
class HttpLoader : public UrlLoader
    {
    public:  // Constructors and destructor        

        /**
        * Constructor
        * @param _contentDispatcher This dispatcher takes the incoming chunks
        * @return Loader object.
        */
        HttpLoader( ContentDispatcher* _contentDispatcher, WebKitLoader *pWebkitLoader );
        
        /**
        * Destructor.
        */
        virtual ~HttpLoader();
        
    public: // from UrlLoader
       
        /**
        * Request a url.
        * @param _requestInfo Url request info created by the resloader client.
        * @return OS wide error code.
        */
        eErrorCode Request( UrlRequestInfo* _requestInfo );

        /**
        * Function is called when headers are received.
        * @param _transaction Observed transaction.
        * @param _response response object
        * @return OS wide error code.
        */
        virtual eErrorCode Headers(  BaseTransaction* _transaction, UrlResponseInfo* _response );

        /**
        * Close the url loader/session.
        * @since 3.1
        * @param
        * @return 
        */
        void Close();       
        
        /**
        * This function is called when an error happens to the transaction.
        * @since 3.1
        * @param _transaction Observed transaction.
        * @param _error HTTP error
        * @return
        */
        bool HandleError( BaseTransaction* _transaction, eHttpStatus _error );
        
        /**
        * Cancel all outstanding transactions.
        * @param _error This error code is propagated up to
        * the resource loader client through Complete()
        * @return ErrNotFound if there is not outstanding transaction.
        */
        eErrorCode CancelAll( eErrorCode _error = ERR_OK );
       

        WebKitLoader* GetWebkitLoader() { return m_pWebkitLoader; }
       
    protected: // from UrlLoader
        
        /**
        * Create a transaction.
        * @param _urlRequestInfo Url request info created by the resource loader client.
        * @return Transaction object.
        */
        BaseTransaction* CreateTransaction( UrlRequestInfo* _urlRequestInfo );
        
        WebKitLoader* m_pWebkitLoader;
        
    };

#endif      // CHTTPLOADER_H
            
// End of File
