/*
* ==============================================================================
*  Name        : RequestDispatcher.cpp
*  Part of     : WebKit / ResourceLoader
*  Interface   :
*  Description : Implementation of RequestDispatcher
*  Version     : %Version%
* ==============================================================================
*/

// INCLUDE FILES


#include "types.h"
#include "WebControl.h"
#include "RequestDispatcher.h"
#include "UrlLoader.h"
#include "HttpLoader.h"
#include "ContentDispatcher.h"
#include "WebKitBridge.h"
#include "CString.h"
#include "Allocs.h"

// EXTERNAL DATA STRUCTURES

// EXTERNAL FUNCTION PROTOTYPES

// CONSTANTS

// MACROS

// LOCAL CONSTANTS AND MACROS

// MODULE DATA STRUCTURES

unsigned int RequestDispatcher::m_transactionCount = 0;
#define PAGE_SAVE_PATH  "/User/Br/SavedPage"
#ifdef SFB_GLOBAL_STATIC_HANDLING
void InitRequestDispatcher()
{
	RequestDispatcher::initCount();
}
#endif


// LOCAL FUNCTION PROTOTYPES

// FORWARD DECLARATIONS

// ============================= LOCAL FUNCTIONS ===============================

// ============================ MEMBER FUNCTIONS ===============================

// -----------------------------------------------------------------------------
// RequestDispatcher::RequestDispatcher
// C++ default constructor can NOT contain any code, that
// might leave.
// -----------------------------------------------------------------------------
//
RequestDispatcher::RequestDispatcher(
    WebKitLoader* _webKitLoader,
    ContentDispatcher* _contentDispatcher )
    : m_contentDispatcher( _contentDispatcher ),
      m_httpLoader( NULL ),
      m_fileLoader( NULL ),
      m_contentLoader( NULL ),
      m_webkitloader( _webKitLoader )
    {
    }

// Destructor
RequestDispatcher::~RequestDispatcher()
    {
    if( m_httpLoader )
        {
        m_httpLoader->Deref();
        }
    if( m_fileLoader )
        {
        m_fileLoader->Deref();
        }
    }

// -----------------------------------------------------------------------------
// RequestDispatcher::Dispatch
//
//
// -----------------------------------------------------------------------------
//
eErrorCode RequestDispatcher::Dispatch(UrlRequestInfo* _requestInfo, IContentLoader** _urlLoader ,unsigned char _isMainResource,StateChangeInfo state_info)
    {
    *_urlLoader = NULL;
    // not found by default
    eErrorCode status = ERR_NOT_SUPPORTED;

    if( _requestInfo )
        {
        // set transaction id
        _requestInfo->SetTransId( m_transactionCount++ );
        }
    String scheme(_requestInfo->CurrentUrl());
    String schemeToLowerCase(scheme.lower());

    UrlLoader* loader = IsSupportedScheme(schemeToLowerCase.latin1().data());
	WebControl::WebControl* iControl = WebControl::kit(m_webkitloader->GetWebKitBridge()->GetWebCoreBridge()->GetFrame()->page());	
	if( loader )
	{
        // Lock the loader
        loader->Ref();

		//shkim
		// set loadgroup for this request
		XMLFrame* pFrame = m_webkitloader->GetWebKitBridge()->GetWebCoreBridge()->GetFrame();	
		WebControl::WebFrame *pWebFrame = WebControl::kit( pFrame );
		_requestInfo->SetWebControl( pWebFrame->GetWebControl() );
		
		status = loader->Request( _requestInfo );
		*_urlLoader = loader;		

        // Unlock the loader
        loader->Deref();
    }
	// Added when loader returns NULL from IsSupportedScheme() and scheme is invalid URL type
	else
	{
		int iStatus = WebControl::HRESULT_TRUE;
		if( iControl->GetContentListener() )
		{
			iStatus = iControl->GetContentListener()->exScheme((char*)schemeToLowerCase.latin1().data(),NULL);
		}

		if( iStatus == WebControl::HRESULT_NOTSUPPORT )
		{
			status = ERR_INVALID_URL;
		}
		else
		{
			status = ERR_OK;
		}
	}

    //
    return status;
    }

// -----------------------------------------------------------------------------
// RequestDispatcher::CancelTransaction
//
//
// -----------------------------------------------------------------------------
//
void RequestDispatcher::CancelTransaction( int _transactionId, eErrorCode _error )
    {
    // <TODO> : Cancel transaction with other loaders too
    if( m_httpLoader )
        {
        m_httpLoader->Cancel( _transactionId, _error );
        }
    if( m_fileLoader )
        {
        m_fileLoader->Cancel( _transactionId, _error );
        }
    }


/**
* Redirect Transaction
*
*/
eErrorCode RequestDispatcher::RedirectTransaction( int transID, int *transportHandle  )
{
	BaseTransaction* pTransaction = NULL;

    if( m_httpLoader )
	{
        pTransaction = m_httpLoader->RemoveTransaction( transID );
		m_contentDispatcher->RemoveContentListener( transID );
		if( pTransaction )
		{
			*transportHandle = pTransaction->GetTransportHandle();
			pTransaction->Deref();
			return ERR_OK ;
		}
	}
    if( m_fileLoader )
	{
		pTransaction = m_fileLoader->RemoveTransaction( transID );
		m_contentDispatcher->RemoveContentListener( transID );
		if( pTransaction )
		{
			*transportHandle = pTransaction->GetTransportHandle();
			pTransaction->Deref();
			return ERR_OK;
		}
	}

	//fail
	return ERR_CANCEL;
}


// -----------------------------------------------------------------------------
// RequestDispatcher::CancelAllLoaders
//
//
// -----------------------------------------------------------------------------
//
void RequestDispatcher::CancelAllLoaders(eErrorCode _error )
    {
    // <TODO> : Cancel other loaders too
    if( m_httpLoader )
        {
        m_httpLoader->CancelAll( _error );
        }
    if( m_fileLoader )
        {
        m_fileLoader->CancelAll( _error );
        }
    }

// -----------------------------------------------------------------------------
// RequestDispatcher::IsSupportedScheme
//
//
// -----------------------------------------------------------------------------
//
UrlLoader* RequestDispatcher::IsSupportedScheme(const char* _scheme)
    {
    // don't change the order.
    // 1. performance reasons
    // 2. default request handler must be the last one

    // buffer request handler is managed separately
	
    UrlLoader* loader = NULL;
        String scheme("");
        scheme = _scheme;

	WebControl::WebControl* iControl = WebControl::kit(m_webkitloader->GetWebKitBridge()->GetWebCoreBridge()->GetFrame()->page());
        if( scheme.startsWith("http") || scheme.startsWith("https") || scheme.startsWith("data") )
    {
		
        if( !m_httpLoader )
        {
            m_httpLoader = new HttpLoader( m_contentDispatcher, m_webkitloader );
        }
        loader = m_httpLoader;
    }
    // create file loader for file:// request
        else if( scheme.startsWith("file") )
    {
        if(AcStrstr(_scheme,PAGE_SAVE_PATH) == NULL )
		{
			scheme.replace("file","http");
			AcStrcpy((char*)_scheme,(const char*)scheme.latin1().data());				
			if( !m_httpLoader )
			{
				m_httpLoader = new HttpLoader( m_contentDispatcher, m_webkitloader );
			}
			loader = m_httpLoader;
		}
		else
		{
			if( !m_fileLoader )
            {
				m_fileLoader = new FileLoader( m_contentDispatcher);
            }
	        loader = m_fileLoader;
        }
	}

    return loader;
    }

//  End of File

