/*
* ==============================================================================
*  Name        : UrlRequestInfo.cpp
*  Part of     : WebKit / ResourceLoader
*  Interface   :
*  Description : Implementation of UrlRequestInfo
*  Version     : %Version%
* ==============================================================================
*/

// INCLUDE FILES
#include "types.h"
#include "UrlRequestInfo.h"


#ifdef __CMM__
#include <AnsiCLib.h>
#include <allocs.h>
#else
#include "Shp.h"
#endif

// EXTERNAL DATA STRUCTURES

// EXTERNAL FUNCTION PROTOTYPES

// CONSTANTS

// MACROS

// LOCAL CONSTANTS AND MACROS

// MODULE DATA STRUCTURES

// LOCAL FUNCTION PROTOTYPES

// FORWARD DECLARATIONS

// ============================= LOCAL FUNCTIONS ===============================


// ============================ MEMBER FUNCTIONS ===============================

// -----------------------------------------------------------------------------
// UrlRequestInfo::UrlRequestInfo
// C++ default constructor can NOT contain any code, that
// might leave.
// -----------------------------------------------------------------------------
//
UrlRequestInfo::UrlRequestInfo()
    : m_transId( -1 ),
      m_loadMethod( URL_GET ),
      m_cacheMode( UseProtocolCachePolicy ), // normal cache mode
      m_loadType( URL_REQUEST_TYPE_ANY ),
      m_defaultHeadersMask( DEFAULT_HEADERS_ALL ),
      // no priority by default
      m_priority( -1 ),
      m_savedPageRequest ( FALSE ),
      m_url( NULL ),
      m_currentUrl( NULL ),
      m_refererUrl( NULL ),
      m_redirectedUrl( NULL ),
      m_postContentType( NULL ),
      m_postBoundary( NULL ),
      m_acceptHeader( NULL ),
	m_pWebControl(NULL)
	{
    }

// Destructor
UrlRequestInfo::~UrlRequestInfo()
    {
    free(m_url);
    free(m_currentUrl);
    free(m_refererUrl);
    free(m_redirectedUrl);
    free(m_postContentType);
    free(m_postBoundary);
    free(m_acceptHeader);
    }

// -----------------------------------------------------------------------------
// UrlRequestInfo::SetUrl
//
//
// -----------------------------------------------------------------------------
//
void UrlRequestInfo::SetUrl(
    const char* _url )
    {
    free(m_url);
    m_url = NULL;
    if( _url )
        {
        m_url = (char *)malloc(AcStrlen(_url) + 1);
        AcStrcpy(m_url, _url);

        }
    }

// -----------------------------------------------------------------------------
// UrlRequestInfo::SetCurrentUrl
//
//
// -----------------------------------------------------------------------------
//
void UrlRequestInfo::SetCurrentUrl(
    const char* _currentUrl )
    {
    free(m_currentUrl);
    m_currentUrl = NULL;
    //
    if( _currentUrl )
        {
        m_currentUrl = (char *)malloc(AcStrlen(_currentUrl) + 1);
        AcStrcpy(m_currentUrl, _currentUrl);

        }
    }

// -----------------------------------------------------------------------------
// UrlRequestInfo::SetRedirectUrl
//
//
// -----------------------------------------------------------------------------
//
void UrlRequestInfo::SetRedirectedUrl(
    const char* _redirectUrl )
    {
    free(m_redirectedUrl);
    m_redirectedUrl = NULL;
    //
    //
    if( _redirectUrl )
        {
        m_redirectedUrl = (char *)malloc(AcStrlen(_redirectUrl) + 1);
        AcStrcpy(m_redirectedUrl, _redirectUrl);
        }
    }

// -----------------------------------------------------------------------------
// UrlRequestInfo::SetPostContentType
//
//
// -----------------------------------------------------------------------------
//
void UrlRequestInfo::SetPostContentType(
    const char* _postContentType)
    {
    free(m_postContentType);
    m_postContentType = NULL;
    //
    if( _postContentType )
        {
        m_postContentType = (char *)malloc(AcStrlen(_postContentType) + 1);
        AcStrcpy(m_postContentType, _postContentType);

        }
    }


void UrlRequestInfo::SetPostDataStr( String _postDataStr )
    {
        m_postDataStr = _postDataStr;
    }
// -----------------------------------------------------------------------------
// UrlRequestInfo::SetPostBoundary
//
//
// -----------------------------------------------------------------------------
//
void UrlRequestInfo::SetPostBoundary(
    const char* _postBoundary )
    {
    free(m_postBoundary);
    m_postBoundary = NULL;
    //
    if( _postBoundary )
        {
        m_postBoundary = (char *)malloc(AcStrlen(_postBoundary) + 1);
        AcStrcpy(m_postBoundary, _postBoundary);
        }
    }

// -----------------------------------------------------------------------------
// UrlRequestInfo::SetRefererUrl
//
//
// -----------------------------------------------------------------------------
//
void UrlRequestInfo::SetRefererUrl(
    const char* _refererUrl )
    {
    free(m_refererUrl);
    m_refererUrl = NULL;
    //
    if( _refererUrl )
        {
        m_refererUrl = (char *)malloc(AcStrlen(_refererUrl) + 1);
        AcStrcpy(m_refererUrl, _refererUrl);

        }
    }

// -----------------------------------------------------------------------------
// UrlRequestInfo::SetAcceptHeader
//
// -----------------------------------------------------------------------------
//
void UrlRequestInfo::SetAcceptHeader(
    const char* _acceptHeader )
    {
    free(m_acceptHeader);
    m_acceptHeader = NULL;
    //
    if( _acceptHeader )
        {
        m_acceptHeader = (char *)malloc(AcStrlen(_acceptHeader) + 1);
        AcStrcpy(m_acceptHeader, _acceptHeader);

        }
    }

//  End of File

