#ifndef URLRESPONSEINFO_H
#define URLRESPONSEINFO_H

//  INCLUDES
#include "BridgeTypes.h"
#include "WebCoreUrlResponseInfo.h"
#include "UrlResponseHeaderInfo.h"
#include "UrlRequestInfo.h"

#ifdef __CMM__
#include <AnsiCLib.h>
#include <allocs.h>
#include "CMMDefs.h"
#else
#include "Shp.h"
#endif


#if PLATFORM(GDK) || (PLATFORM(SHP) && defined (USE_CURL))
#include <curl/curl.h>
#endif
// CONSTANTS

// MACROS

// DATA TYPES

// FUNCTION PROTOTYPES

// FORWARD DECLARATIONS

// CLASS DECLARATION

/**
*  This class holds the response information
*  @lib resLoader.lib
*  @since 3.1
*/
class UrlResponseInfo : public IWebCoreUrlResponseInfo CMM_MODIFIED_ADD
    {

    public:  // Constructors and destructor        
        
        /**
        * Constructor
        */
        UrlResponseInfo();
        
        /**
        * Destructor.
        */
        virtual ~UrlResponseInfo();

    public: // New functions

        /**
        * Set response body
        * @param _body Response body
        * @return void
        */
        inline void SetBody( const char* _body, int _len ) 
            {
				// shkim
				// don't copy data, it will decrease performace
				//if(m_body)
				//{
				//	free(m_body);
				//	m_body = NULL;
				//}
				//m_body = (char*) malloc(_len  + 1);
				//AcMemcpy( m_body, _body, _len );
				//m_body[ _len ] = '\0';
				//m_bodyLength = _len;
				m_body = (char*)_body;
				m_bodyLength = _len;
            }

        /**
        * Set header information
        * @param _headerInfo Header information
        * @return void
        */
        inline void SetHeaderInfo( UrlResponseHeaderInfo* _headerInfo ) { m_headerInfo = _headerInfo; }

		/**
        * Set request information
        * @param _requestInfo Request information
        * @return void
        */
        inline void SetRequestInfo( UrlRequestInfo* _requestInfo ) { m_requestInfo = _requestInfo; }
        
        /**
        * Sets the transport handle
        * @param _handle - transport handle
        * @return 
        */
        inline void SetTransportHandle( TransportHandle _handle ) { m_transportHandle = _handle; }

        /**
        * Gets the transport handle
        * @param 
        * @return transport handle
        */
        inline TransportHandle GetTransportHandle() const { return m_transportHandle; }

    public: // from IWebCoreUrlResponseInfo

        /**
        * Get response body
        * @since 3.1
        * @param 
        * @return Response body 
        */
        inline const void* Body() const { return m_body; }

        inline const int BodyLength() const { return m_bodyLength; }

        /**
        * Get header information
        * @return Header information 
        */
        inline UrlResponseHeaderInfo* HeaderInfo() const 
            { return m_headerInfo; }

		/**
        * Get request information
        * @return request information 
        */
        inline UrlRequestInfo* RequestInfo() const 
            { return m_requestInfo; }

    private: // Data
    
        // The body of the request
        char*           m_body;
        // Length of the body
        int             m_bodyLength;              
        // header info
        UrlResponseHeaderInfo*  m_headerInfo;    // owned
		//Request info
		UrlRequestInfo*	        m_requestInfo;   // not owned
        //
        TransportHandle     m_transportHandle;
    };

#endif      // URLRESPONSEINFO_H           

// End of File
