#ifndef FILETRANSACTION_H
#define FILETRANSACTION_H

//  INCLUDES
#include <wtf/Platform.h>
#include "BaseTransaction.h"

// CONSTANTS

// MACROS

// FUNCTION PROTOTYPES

// FORWARD DECLARATIONS
class HttpLoader;

// CLASS DECLARATION

/**
*  This class handles http transactions.
*  @lib resLoader.lib
*/
class FileTransaction : public BaseTransaction
{
    public:  // Constructors

        /**
        * Constructor
        * @param _urlRequestInfo Url reques info
        * @param _transactionObserver Transaction observer-Url loader.
        * @param _acceptHeaderUtil accept header util class
        */
        FileTransaction( UrlRequestInfo* _urlRequestInfo,
        ITransactionObserver* _transactionObserver);

    protected: //  destructor

        /**
        * Destructor
        * We are making the destructor protected since BaseTransaction is ref counted
        */
        virtual ~FileTransaction();

    public: // new functions

        inline void SetIsRedirected( bool _isRedirected ) {m_isRedirected = _isRedirected;}
        inline bool IsRedirected() {return m_isRedirected;}

        inline TransportHandle GetTransportHandle() { return m_handle; }

        inline void SetTransportHandle(TransportHandle _handle) { m_handle = _handle; }

    public: // from BaseTransaction

        /**
        * Cancel this transaction.
        * @param _error This error code is propagated up to
        * the resource loader client through Complete()
        * @return void
        */
        void CancelTransaction( eErrorCode _error = ERR_OK );

        /**
        * This function is called when the header info is available.
        * @param _contentLength Expected max lenght of the total content
        * @param _url Response url
        * @param _httpStatus Http status code
        * @return void
        */
        void HandleGotResponseHeaders();

        /**
        * This functions is called when a new chunk is available.
        * @param _body
        * @oaram _len Length of the body data
        * @return void
        */
        void HandleResponseBodyData(void* _body, int _len);

        /**
        * Create header information object
        * @param
        * @return Header information object
        */
        void CreateHeaderResponse();

    protected: // from BaseTransaction

        /**
        * Sumbmit transaction
        * @param
        * @return OS wide error code
        */
        eErrorCode DoSubmit();


    private:

        /**
        * Add default request headers to the transaction
        * @param
        * @return void
        */
        void AddRequestHeaders();

    private:    // Data

        bool                        m_transCompleted;
        bool                        m_isRedirected;

        // Transport handle for the transaction
        TransportHandle            m_handle;
};

#endif // FILETRANSACTION_H

