#ifndef TransactionManager_h
#define TransactionManager_h

#include "Timer.h"
#include "BaseTransaction.h"
#include <curl/curl.h>

class TransactionList {
public:
    TransactionList(BaseTransaction* job, TransactionList* next)
        : m_job(job)
        , m_next(next)
        , m_removed(false)
    {}
    TransactionList* next() const { return m_next; }
    BaseTransaction* job() const { return m_job; }
    void setRemoved(bool removed) { m_removed = removed; }
    bool removed() const { return m_removed; }

private:
    BaseTransaction* m_job;
    TransactionList* m_next;
    bool m_removed;
};

class TransactionManager {
public:
    static TransactionManager* sharedInstance();
    void add(BaseTransaction*);
    void cancel(BaseTransaction*);
    void setCookieJarFileName(const char* cookieJarFileName);

    void setupPOST(BaseTransaction*);
    void setupPUT(BaseTransaction*);

private:
    TransactionManager();
    ~TransactionManager();
    void downloadTimerCallback(Timer<TransactionManager>*);
    void removeFromCurl(BaseTransaction*);
    bool removeScheduledJob(BaseTransaction*);
    void startJob(BaseTransaction*);
    bool startScheduledJobs();

    void TempHeaders(curl_slist** headers);
    
    Timer<TransactionManager> m_downloadTimer;
    CURLM* m_curlMultiHandle;
    CURLSH* m_curlShareHandle;
    char* m_cookieJarFileName;
    char m_curlErrorBuffer[CURL_ERROR_SIZE];
    TransactionList* m_TransactionListHead;
};


#endif // TransactionManager_h

// End of File

