/*
 * Copyright (C) 2007 Holger Hans Peter Freyther
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer. 
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution. 
 * 3.  Neither the name of Apple, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "webkitshpframe.h"
#include "webkitshppage.h"
#include "webkitshpprivate.h"

#include "FrameLoader.h"
#include "FrameLoaderClientshp.h"
#include "FrameView.h"
#include "IFrameView.h"
#include "HTMLNames.h"
#include "HTMLFrameOwnerElement.h"

using namespace WebCore;

extern "C" {


enum {
    CLEARED,
    LOAD_DONE,
    TITLE_CHANGED,
    HOVERING_OVER_LINK,
    LAST_SIGNAL
};

void webkit_shp_frame_finalize(WebKitShpFrame* frame)
{
    WebKitShpFramePrivate* privateData = frame->privateFrame;

    delete privateData;
    delete frame;
    frame = NULL;
}

WebKitShpFrame*
webkit_shp_frame_new(WebKitShpPage* page)
{
    WebKitShpFrame* frame = new WebKitShpFrame();
    frame->privateFrame = new WebKitShpFramePrivate();

    WebKitShpFramePrivate* frameData = frame->privateFrame;
    WebKitShpPagePrivate* pageData = page->privatePage;

    frameData->client = new WebKit::FrameLoaderClient(frame);
	frameData->frame = new Frame(pageData->page, 0, frameData->client ); //,&pageData->hWin,&pageData->state_info);

    FrameView* frameView = new FrameView(frameData->frame);

	frameData->frame->setView(frameView);
    frameData->frame->init();
    frameData->page = page;

    return frame;
}
WebKitShpFrame* webkit_shp_frame_init_with_page(WebKitShpPage* page, HTMLFrameOwnerElement* element)
{
    WebKitShpFrame* frame = new WebKitShpFrame();
	frame->privateFrame = new WebKitShpFramePrivate();

    WebKitShpFramePrivate* frameData = frame->privateFrame;
    WebKitShpPagePrivate* pageData = page->privatePage;

    frameData->client = new WebKit::FrameLoaderClient(frame);
	frameData->frame = new Frame(pageData->page, element, frameData->client ); //,&(page->privatePage->hWin),&(page->privatePage->state_info));
 
	FrameView* frameView;

	if(element->hasTagName(HTMLNames::iframeTag))
	{
		frameView = new IFrameView(frameData->frame, pageData->hWin);
	}
	else
	{
		frameView = new FrameView(frameData->frame);
	}
    
	frameData->frame->setView(frameView);
    frameData->frame->init();
    frameData->page = page;

    return frame;
} 

WebKitShpPage*
webkit_shp_frame_get_page(WebKitShpFrame* frame)
{
    return frame->privateFrame->page;
}

}
