/*
 * Copyright (C) 2007 Holger Hans Peter Freyther
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer. 
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution. 
 * 3.  Neither the name of Apple, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef WEBKIT_SHP_PRIVATE_H
#define WEBKIT_SHP_PRIVATE_H

/*
 * Internal class. This class knows the shared secret of WebKitShpFrame,
 * WebKitShpNetworkRequest and WebKitShpPage.
 * They are using WebCore which musn't be exposed to the outer world.
 */

#include "webkitshppage.h"
#include "webkitshpframe.h"


#include "Settings.h"
#include "Page.h"
#include "Frame.h"
#include "FrameLoaderClient.h"
#include "shptypes.h"

namespace WebKit {

    WebKitShpFrame*  getFrameFromPage(WebKitShpPage*);
    WebKitShpPage* getPageFromFrame(WebKitShpFrame*);

    WebCore::Frame* core(WebKitShpFrame*);
    WebKitShpFrame* kit(WebCore::Frame*);
    WebCore::Page* core(WebKitShpPage*);
    WebKitShpPage* kit(WebCore::Page*);
}

namespace WebCore {
class CVirtualCursor;
}

extern "C" {

	typedef enum {
	STATECHANGE_LOADING,
	STATECHANGE_PROGRESS,
    STATECHANGE_SECURE,
    STATECHANGE_SECURE_INCLUSION,
	STATECHANGE_ERROR
	} WEBKIT_CHANGE_STATE;

	typedef struct _StateChangeInfo StateChangeInfo;

	/*struct _StateChangeInfo{
		WEBKIT_CHANGE_STATE		m_stateChange;
		BOOL					m_prevHttpState;
		BOOL					m_prevHttpInclusionState;
	};
*/
	

    typedef struct _WebKitShpPagePrivate WebKitShpPagePrivate;
    struct _WebKitShpPagePrivate {

		 _WebKitShpPagePrivate() : 
			page(NULL), 
			settings(NULL),
			mainFrame(NULL),
			applicationNameForUserAgent(""),
			userAgent(NULL),
			hGc(INVALID_HOBJ),
#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
			hGc_restore(INVALID_HOBJ),
			m_border_rect(),
			gPreview(0),
#endif
			hWin(INVALID_HOBJ),
			placeholderImage(NULL)
		{

		}

        WebCore::Page* page;
        WebCore::Settings* settings;

        WebKitShpFrame* mainFrame;
        WebCore::String applicationNameForUserAgent;
        WebCore::String* userAgent;
		HGC hGc;
#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
		HGC hGc_restore;
		WebCore::IntRect*		m_border_rect;
		BOOL gPreview;
#endif
		HWIN hWin;
		StateChangeInfo state_info;			
		WebCore::CVirtualCursor* pageVirtualCursor;

#ifdef SFB_PROGRESSIVE_RENDERING
        WmBitmap* placeholderImage;
#endif // SFB_PROGRESSIVE_RENDERING
    };
    
    typedef struct _WebKitShpFramePrivate WebKitShpFramePrivate;
    struct _WebKitShpFramePrivate {
        WebCore::Frame* frame;
        WebCore::FrameLoaderClient* client;
        WebKitShpPage* page;
    };

    WebKitShpFrame* webkit_shp_frame_init_with_page(WebKitShpPage*, WebCore::HTMLFrameOwnerElement*);
}

#endif
