/*
     File:       CFNetwork/CFHTTPCookiesPriv.h
     Copyright:  (c) 2007 by Apple Inc., all rights reserved
*/
#ifndef __CFHTTPCOOKIESPRIV__
#define __CFHTTPCOOKIESPRIV__

#ifndef __CFNETWORKDEFS__
#include <CFNetwork/CFNetworkDefs.h>
#endif

#ifndef __COREFOUNDATION__
#include <CoreFoundation/CoreFoundation.h>
#endif




#include <AvailabilityMacros.h>

#if PRAGMA_ONCE
#pragma once
#endif

#ifdef __cplusplus
extern "C" {
#endif

#pragma pack(push, 2)

#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint on
#endif

typedef struct OpaqueCFHTTPCookieStorage*  CFHTTPCookieStorageRef;
typedef struct OpaqueCFHTTPCookie*      CFHTTPCookieRef;
enum {
  kCFHTTPCookieStorageAcceptPolicyAlways = 0,
  kCFHTTPCookieStorageAcceptPolicyNever = 1,
  kCFHTTPCookieStorageAcceptPolicyOnlyFromMainDocumentDomain = 2
};

enum {
  CFHTTPCookieStorageAcceptPolicyAlways = kCFHTTPCookieStorageAcceptPolicyAlways,
  CFHTTPCookieStorageAcceptPolicyNever = kCFHTTPCookieStorageAcceptPolicyNever,
  CFHTTPCookieStorageAcceptPolicyOnlyFromMainDocumentDomain = kCFHTTPCookieStorageAcceptPolicyOnlyFromMainDocumentDomain
};

typedef int                             CFHTTPCookieStorageAcceptPolicy;
enum CFHTTPCookieFlags {
  CFHTTPCookieSecureFlag        = 1 << 0,
  CFHTTPCookieSessionOnlyFlag   = 1 << 1
};
typedef enum CFHTTPCookieFlags CFHTTPCookieFlags;

/*
 *  CFHTTPCookieGetTypeID()
 *  
 */
APPLE_WIN_API CFTypeID 
CFHTTPCookieGetTypeID(void)                                   AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieCreateWithProperties()
 *  
 */
APPLE_WIN_API CFHTTPCookieRef 
CFHTTPCookieCreateWithProperties(
  CFAllocatorRef    inAllocator,
  CFDictionaryRef   inProperties)                             AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieCreateWithResponseHeaderFields()
 *  
 */
APPLE_WIN_API CFArrayRef 
CFHTTPCookieCreateWithResponseHeaderFields(
  CFAllocatorRef    inAllocator,
  CFDictionaryRef   headerFields,
  CFURLRef          inURL)                                    AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieCopyRequestHeaderFields()
 *  
 */
APPLE_WIN_API CFDictionaryRef 
CFHTTPCookieCopyRequestHeaderFields(
  CFAllocatorRef   inAllocator,
  CFArrayRef       inCookies)                                 AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieCopyProperties()
 *  
 */
APPLE_WIN_API CFDictionaryRef 
CFHTTPCookieCopyProperties(CFHTTPCookieRef inCookie)          AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieGetVersion()
 *  
 */
APPLE_WIN_API CFIndex 
CFHTTPCookieGetVersion(CFHTTPCookieRef inCookie)              AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieGetName()
 *  
 */
APPLE_WIN_API CFStringRef 
CFHTTPCookieGetName(CFHTTPCookieRef inCookie)                 AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieGetValue()
 *  
 */
APPLE_WIN_API CFStringRef 
CFHTTPCookieGetValue(CFHTTPCookieRef inCookie)                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieGetExpiratonDate()
 *  
 */
APPLE_WIN_API CFDateRef 
CFHTTPCookieGetExpiratonDate(CFHTTPCookieRef inCookie)        AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieGetCreateDate()
 *  
 */
APPLE_WIN_API CFDateRef 
CFHTTPCookieGetCreateDate(CFHTTPCookieRef inCookie)           AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieGetDomain()
 *  
 */
APPLE_WIN_API CFStringRef 
CFHTTPCookieGetDomain(CFHTTPCookieRef inCookie)               AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieGetPath()
 *  
 */
APPLE_WIN_API CFStringRef 
CFHTTPCookieGetPath(CFHTTPCookieRef inCookie)                 AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieGetFlags()
 *  
 */
APPLE_WIN_API unsigned int 
CFHTTPCookieGetFlags(CFHTTPCookieRef inCookie)                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieGetComment()
 *  
 */
APPLE_WIN_API CFStringRef 
CFHTTPCookieGetComment(CFHTTPCookieRef inCookie)              AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieGetCommentURL()
 *  
 */
APPLE_WIN_API CFURLRef 
CFHTTPCookieGetCommentURL(CFHTTPCookieRef inCookie)           AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieGetPortArray()
 *  
 */
APPLE_WIN_API CFArrayRef 
CFHTTPCookieGetPortArray(CFHTTPCookieRef inCookie)            AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


typedef CALLBACK_API_C( void , CFHTTPCookieStorageChangedProcPtr )(CFHTTPCookieStorageRef inStorage, void *context);
/*
 *  CFHTTPCookieStorageAddObserver()
 *  
 */
APPLE_WIN_API void 
CFHTTPCookieStorageAddObserver(
  CFHTTPCookieStorageRef              inStorage,
  CFRunLoopRef                        inRunloop,
  CFStringRef                         inRunloopMode,
  CFHTTPCookieStorageChangedProcPtr   callback,
  void *                              context)                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageRemoveObserver()
 *  
 */
APPLE_WIN_API void 
CFHTTPCookieStorageRemoveObserver(
  CFHTTPCookieStorageRef              inStorage,
  CFRunLoopRef                        inRunloop,
  CFStringRef                         inRunloopMode,
  CFHTTPCookieStorageChangedProcPtr   callback,
  void *                              context)                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageGetTypeID()
 *  
 */
APPLE_WIN_API CFTypeID 
CFHTTPCookieStorageGetTypeID(void)                            AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


struct CFHTTPCookieStorageFilterContext {
  CFIndex             version;
  void *              info;
  CFAllocatorRetainCallBack  retain;
  CFAllocatorReleaseCallBack  release;
  CFAllocatorCopyDescriptionCallBack  copyDescription;
};
typedef struct CFHTTPCookieStorageFilterContext CFHTTPCookieStorageFilterContext;
typedef CALLBACK_API_C( Boolean , CFHTTPCookieStorageFilterProcPtr )(CFHTTPCookieStorageRef inStorage, CFHTTPCookieRef inTestCookie, void *inInfo);
/*
 *  _CFHTTPCookieStorageGetDefault()
 *  
 */
APPLE_WIN_API CFHTTPCookieStorageRef 
_CFHTTPCookieStorageGetDefault(CFAllocatorRef inAllocator)    AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  _CFHTTPCookieStorageCreateInMemory()
 *  
 */
APPLE_WIN_API CFHTTPCookieStorageRef 
_CFHTTPCookieStorageCreateInMemory(
  CFAllocatorRef                      inAllocator,
  CFHTTPCookieStorageRef              baseStorage,
  CFHTTPCookieStorageFilterProcPtr    filterCallback,
  CFHTTPCookieStorageFilterContext *  context)                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  _CFHTTPCookieStorageCreateFromFile()
 *  
 */
APPLE_WIN_API CFHTTPCookieStorageRef 
_CFHTTPCookieStorageCreateFromFile(
  CFAllocatorRef                      inAllocator,
  CFURLRef                            inStorageLocation,
  CFHTTPCookieStorageRef              baseStorage,
  CFHTTPCookieStorageFilterProcPtr    filterCallback,
  CFHTTPCookieStorageFilterContext *  context)                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageCreateInMemory()
 *  
 */
APPLE_WIN_API CFHTTPCookieStorageRef 
CFHTTPCookieStorageCreateInMemory(
  CFAllocatorRef           inAllocator,
  CFHTTPCookieStorageRef   baseStorage)                       AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageCreateFromFile()
 *  
 */
APPLE_WIN_API CFHTTPCookieStorageRef 
CFHTTPCookieStorageCreateFromFile(
  CFAllocatorRef           inAllocator,
  CFURLRef                 inStorageLocation,
  CFHTTPCookieStorageRef   baseStorage)                       AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageCreateArchive()
 *  
 */
APPLE_WIN_API CFArrayRef 
CFHTTPCookieStorageCreateArchive(
  CFAllocatorRef           inAllocator,
  CFHTTPCookieStorageRef   inStorage)                         AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageCreateFromArchive()
 *  
 */
APPLE_WIN_API CFHTTPCookieStorageRef 
CFHTTPCookieStorageCreateFromArchive(
  CFAllocatorRef   inAllocator,
  CFArrayRef       inArchive)                                 AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageScheduleWithRunLoop()
 *  
 */
APPLE_WIN_API void 
CFHTTPCookieStorageScheduleWithRunLoop(
  CFHTTPCookieStorageRef   inStorage,
  CFRunLoopRef             inRunloop,
  CFStringRef              inRunloopMode)                     AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageUnscheduleFromRunLoop()
 *  
 */
APPLE_WIN_API void 
CFHTTPCookieStorageUnscheduleFromRunLoop(
  CFHTTPCookieStorageRef   inStorage,
  CFRunLoopRef             inRunloop,
  CFStringRef              inRunloopMode)                     AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageGetBaseStorage()
 *  
 */
APPLE_WIN_API CFHTTPCookieStorageRef 
CFHTTPCookieStorageGetBaseStorage(CFHTTPCookieStorageRef inStorage) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageGetCookieAcceptPolicy()
 *  
 */
APPLE_WIN_API CFHTTPCookieStorageAcceptPolicy 
CFHTTPCookieStorageGetCookieAcceptPolicy(CFHTTPCookieStorageRef inCookieStorage) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageSetCookieAcceptPolicy()
 *  
 */
APPLE_WIN_API void 
CFHTTPCookieStorageSetCookieAcceptPolicy(
  CFHTTPCookieStorageRef            inCookieStorage,
  CFHTTPCookieStorageAcceptPolicy   inCookieAcceptPolicy)     AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageCopyCookiesForURL()
 *  
 */
APPLE_WIN_API CFArrayRef 
CFHTTPCookieStorageCopyCookiesForURL(
  CFHTTPCookieStorageRef   inCookieStorage,
  CFURLRef                 inURL,
  Boolean                  sendSecureCookies)                 AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageSetCookie()
 *  
 */
APPLE_WIN_API void 
CFHTTPCookieStorageSetCookie(
  CFHTTPCookieStorageRef   inCookieStorage,
  CFHTTPCookieRef          inCookie)                          AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageSetCookies()
 *  
 */
APPLE_WIN_API void 
CFHTTPCookieStorageSetCookies(
  CFHTTPCookieStorageRef   inCookieStorage,
  CFArrayRef               inCookies,
  CFURLRef                 inURL,
  CFURLRef                 inMainDocumentURL)                 AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageCopyCookies()
 *  
 */
APPLE_WIN_API CFArrayRef 
CFHTTPCookieStorageCopyCookies(CFHTTPCookieStorageRef inCookieStorage) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageDeleteCookie()
 *  
 */
APPLE_WIN_API void 
CFHTTPCookieStorageDeleteCookie(
  CFHTTPCookieStorageRef   inCookieStorage,
  CFHTTPCookieRef          inCookie)                          AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFHTTPCookieStorageDeleteAllCookies()
 *  
 */
APPLE_WIN_API void 
CFHTTPCookieStorageDeleteAllCookies(CFHTTPCookieStorageRef inCookieStorage) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  _CFHTTPCookieStorageSetCookiesWithResponseHeaderFields()
 *  
 */
APPLE_WIN_API void 
_CFHTTPCookieStorageSetCookiesWithResponseHeaderFields(
  CFHTTPCookieStorageRef            inCookieStorage,
  CFURLRef                          inURL,
  CFDictionaryRef                   headerFields,
  CFURLRef                          inPolicyURL,
  CFHTTPCookieStorageAcceptPolicy   inCookieAcceptPolicy)     AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  _CFHTTPCookieStorageCopyRequestHeaderFieldsForURL()
 *  
 */
APPLE_WIN_API CFDictionaryRef 
_CFHTTPCookieStorageCopyRequestHeaderFieldsForURL(
  CFAllocatorRef           inAllocator,
  CFHTTPCookieStorageRef   inCookieStorage,
  CFURLRef                 inRequestURL)                      AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint reset
#endif

#pragma pack(pop)

#ifdef __cplusplus
}
#endif

#endif /* __CFHTTPCOOKIESPRIV__ */

