/*
     File:       CFNetwork/CFURLAuthChallengePriv.h
     Copyright:  (c) 2007 by Apple Inc., all rights reserved
*/
#ifndef __CFURLAUTHCHALLENGEPRIV__
#define __CFURLAUTHCHALLENGEPRIV__

#ifndef __CFNETWORKDEFS__
#include <CFNetwork/CFNetworkDefs.h>
#endif

#ifndef __COREFOUNDATION__
#include <CoreFoundation/CoreFoundation.h>
#endif

#ifndef __CFURLCREDENTIALPRIV__
#include <CFNetwork/CFURLCredentialPriv.h>
#endif

#ifndef __CFURLPROTECTIONSPACEPRIV__
#include <CFNetwork/CFURLProtectionSpacePriv.h>
#endif

#ifndef __CFURLRESPONSEPRIV__
#include <CFNetwork/CFURLResponsePriv.h>
#endif



#include <AvailabilityMacros.h>

#if PRAGMA_ONCE
#pragma once
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint on
#endif


/*
 *  CFURLAuthChallengeRef
 *  
 *  Discussion:
 *    This is the type of a reference CFURLAuthChallenge.
 */
typedef struct _CFURLAuthChallenge*     CFURLAuthChallengeRef;
typedef CFURLAuthChallengeRef           CFURLAuthenticationChallengeRef;
/*
 *  CFURLAuthChallengeGetTypeID()
 *  
 *  Discussion:
 *    Returns the type identifier of all CFURLAuthChallenge instances.
 *  
 */
APPLE_WIN_API CFTypeID 
CFURLAuthChallengeGetTypeID(void)                             AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLAuthChallengeCreate()
 *  
 *  Discussion:
 *    Creates a new CFURLAuthChallenge object.
 *  
 *  Parameters:
 *    
 *    allocator:
 *      The CFAllocator which should be used to allocate memory for the
 *      CFURLAuthChallenge. If this reference is not a valid
 *      CFAllocator, the behavior is undefined.
 *    
 *    protectionSpace:
 *      The protection space for the CFURLAuthChallenge object.
 *    
 *    proposedCredential:
 *      The proposed creadential for the CFURLAuthChallenge object, or
 *      NULL if there is no proposed creadential.
 *    
 *    previousFailureCount:
 *      The count of the receiver's failed authentication attempts.
 *    
 *    failureResponse:
 *      The CFURLResponse representing the last authentication failure.
 *    
 *    error:
 *      The last authentication failure.
 *  
 *  Result:
 *    A valid CFURLAuthChallengeRef, or NULL if unsuccessful.
 *  
 */
APPLE_WIN_API CFURLAuthChallengeRef 
CFURLAuthChallengeCreate(
  CFAllocatorRef            allocator,
  CFURLProtectionSpaceRef   protectionSpace,
  CFURLCredentialRef        proposedCredential,
  CFIndex                   previousFailureCount,
  CFURLResponseRef          failureResponse,
  CFStreamError             error)                            AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLAuthChallengeGetProtectionSpace()
 *  
 *  Discussion:
 *    Returns the receiver's protection space.
 *  
 *  Parameters:
 *    
 *    authChallenge:
 *      The CFURLAuthChallenge.
 *  
 *  Result:
 *    A valid CFURLProtectionSpaceRef, or NULL if unsuccessful.
 *  
 */
APPLE_WIN_API CFURLProtectionSpaceRef 
CFURLAuthChallengeGetProtectionSpace(CFURLAuthChallengeRef authChallenge) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLAuthChallengeGetProposedCredential()
 *  
 *  Discussion:
 *    Returns the proposed credential for this challenge.
 *  
 *  Parameters:
 *    
 *    authChallenge:
 *      The CFURLAuthChallenge.
 *  
 *  Result:
 *    A valid CFURLCredentialRef, or NULL if unsuccessful.
 *  
 */
APPLE_WIN_API CFURLCredentialRef 
CFURLAuthChallengeGetProposedCredential(CFURLAuthChallengeRef authChallenge) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLAuthChallengeGetPreviousFailureCount()
 *  
 *  Discussion:
 *    Returns the receiver's count of failed authentication attempts.
 *  
 *  Parameters:
 *    
 *    authChallenge:
 *      The CFURLAuthChallenge.
 *  
 *  Result:
 *    The count of failed authentication attempts, or -1 if
 *    unsuccessful.
 *  
 */
APPLE_WIN_API CFIndex 
CFURLAuthChallengeGetPreviousFailureCount(CFURLAuthChallengeRef authChallenge) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLAuthChallengeGetError()
 *  
 *  Discussion:
 *    Returns the CFStreamError representing the last authentication
 *    failure.
 *  
 *  Parameters:
 *    
 *    authChallenge:
 *      The CFURLAuthChallenge.
 *  
 *  Result:
 *    The CFStreamError representing the last authentication failure.
 *  
 */
APPLE_WIN_API CFStreamError 
CFURLAuthChallengeGetError(CFURLAuthChallengeRef authChallenge) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLAuthChallengeGetFailureResponse()
 *  
 *  Discussion:
 *    Returns the response representing the last authentication failure.
 *  
 *  Parameters:
 *    
 *    authChallenge:
 *      The CFURLAuthChallenge.
 *  
 *  Result:
 *    The response representing the last authentication failure, or
 *    NULL if unsuccessful.
 *  
 */
APPLE_WIN_API CFTypeRef 
CFURLAuthChallengeGetFailureResponse(CFURLAuthChallengeRef authChallenge) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint reset
#endif

#ifdef __cplusplus
}
#endif

#endif /* __CFURLAUTHCHALLENGEPRIV__ */

