/*
     File:       CFNetwork/CFURLCachePriv.h
     Copyright:  (c) 2007 by Apple Inc., all rights reserved
*/
#ifndef __CFURLCACHEPRIV__
#define __CFURLCACHEPRIV__

#ifndef __CFURLREQUESTPRIV__
#include <CFNetwork/CFURLRequestPriv.h>
#endif

#ifndef __CFURLRESPONSEPRIV__
#include <CFNetwork/CFURLResponsePriv.h>
#endif



#include <AvailabilityMacros.h>

#if PRAGMA_ONCE
#pragma once
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint on
#endif


/*
 *  CFURLCachePriv
 *  
 *  Discussion:
 *    CFURLCache is a CFType that represents a cache of URLs.
 */
typedef const struct _CFCachedURLResponse*  CFCachedURLResponseRef;
typedef const struct _CFURLCache*       CFURLCacheRef;
/*
 *  CFURLCacheGetTypeID()
 *  
 */
APPLE_WIN_API CFTypeID 
CFURLCacheGetTypeID(void)                                     AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFCachedURLResponseGetTypeID()
 *  
 */
APPLE_WIN_API CFTypeID 
CFCachedURLResponseGetTypeID(void)                            AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFCachedURLResponseCreateWithData()
 *  
 */
APPLE_WIN_API CFCachedURLResponseRef 
CFCachedURLResponseCreateWithData(
  CFAllocatorRef     alloc,
  CFURLResponseRef   response,
  CFDataRef          data)                                    AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFCachedURLResponseCreateWithUserInfo()
 *  
 */
APPLE_WIN_API CFCachedURLResponseRef 
CFCachedURLResponseCreateWithUserInfo(
  CFAllocatorRef            alloc,
  CFURLResponseRef          response,
  CFDataRef                 data,
  CFPropertyListRef         userInfo,
  CFURLCacheStoragePolicy   storagePolicy)                    AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFCachedURLResponseGetWrappedResponse()
 *  
 */
APPLE_WIN_API CFURLResponseRef 
CFCachedURLResponseGetWrappedResponse(CFCachedURLResponseRef cachedResponse) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFCachedURLResponseGetReceiverData()
 *  
 */
APPLE_WIN_API CFDataRef 
CFCachedURLResponseGetReceiverData(CFCachedURLResponseRef cachedResponse) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFCachedURLResponseGetUserInfo()
 *  
 */
APPLE_WIN_API CFPropertyListRef 
CFCachedURLResponseGetUserInfo(CFCachedURLResponseRef cachedResponse) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFCachedURLResponseGetStoragePolicy()
 *  
 */
APPLE_WIN_API CFURLCacheStoragePolicy 
CFCachedURLResponseGetStoragePolicy(CFCachedURLResponseRef cachedResponse) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFCachedURLResponseSetStoragePolicy()
 *  
 */
APPLE_WIN_API void 
CFCachedURLResponseSetStoragePolicy(
  CFCachedURLResponseRef    cachedResponse,
  CFURLCacheStoragePolicy   storagePolicy)                    AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCacheSharedURLCache()
 *  
 */
APPLE_WIN_API CFURLCacheRef 
CFURLCacheSharedURLCache(void)                                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCacheSetSharedURLCache()
 *  
 */
APPLE_WIN_API void 
CFURLCacheSetSharedURLCache(CFURLCacheRef cache)              AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCacheCreate()
 *  
 */
APPLE_WIN_API CFURLCacheRef 
CFURLCacheCreate(
  CFAllocatorRef   alloc,
  CFIndex          memoryCapacity,
  CFIndex          diskCapacity,
  CFStringRef      path)                                      AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCacheCopyResponseForRequest()
 *  
 */
APPLE_WIN_API CFCachedURLResponseRef 
CFURLCacheCopyResponseForRequest(
  CFURLCacheRef     cacheRef,
  CFURLRequestRef   request)                                  AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCacheAddCachedResponseForRequest()
 *  
 */
APPLE_WIN_API void 
CFURLCacheAddCachedResponseForRequest(
  CFURLCacheRef            cacheRef,
  CFCachedURLResponseRef   cachedResponse,
  CFURLRequestRef          request,
  CFTypeRef                nsCachedURLResponse)               AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCacheRemoveCachedResponseForRequest()
 *  
 */
APPLE_WIN_API void 
CFURLCacheRemoveCachedResponseForRequest(
  CFURLCacheRef     cacheRef,
  CFURLRequestRef   request)                                  AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCacheRemoveAllCachedResponses()
 *  
 */
APPLE_WIN_API void 
CFURLCacheRemoveAllCachedResponses(CFURLCacheRef cacheRef)    AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCacheMemoryCapacity()
 *  
 */
APPLE_WIN_API CFIndex 
CFURLCacheMemoryCapacity(CFURLCacheRef cacheRef)              AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCacheDiskCapacity()
 *  
 */
APPLE_WIN_API CFIndex 
CFURLCacheDiskCapacity(CFURLCacheRef cacheRef)                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCacheSetMemoryCapacity()
 *  
 */
APPLE_WIN_API void 
CFURLCacheSetMemoryCapacity(
  CFURLCacheRef   cacheRef,
  CFIndex         memoryCapacity)                             AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCacheSetDiskCapacity()
 *  
 */
APPLE_WIN_API void 
CFURLCacheSetDiskCapacity(
  CFURLCacheRef   cacheRef,
  CFIndex         diskCapacity)                               AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCacheCurrentMemoryUsage()
 *  
 */
APPLE_WIN_API CFIndex 
CFURLCacheCurrentMemoryUsage(CFURLCacheRef cacheRef)          AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCacheCurrentDiskUsage()
 *  
 */
APPLE_WIN_API CFIndex 
CFURLCacheCurrentDiskUsage(CFURLCacheRef cacheRef)            AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint reset
#endif

#ifdef __cplusplus
}
#endif

#endif /* __CFURLCACHEPRIV__ */

