/* 
 * Copyright (c) 2007 SISO, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 /* FILE :
 *   This file defines the RM Queue structure/Data Types
 *
 * @name		SFBRmCoreTypes.h
 * @author		Ancil K George (ancilgeorge@samsung.com)
				Basavaraj P S (basavaraj.ps@samsung.com)
				Anshul Saxena (a.saxena@samsung.com)
				Rohit P Shetty (rohit.shetty@samsung.com)
  */

#ifndef _SFB_CORE_TYPES_H_
#define _SFB_CORE_TYPES_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <SFBRmMgr.h>

/*MACROS*/ 

/**
SFBRmResponseInfo
This structure defines the params present in the response Info.
*/

typedef struct _SFBRmResponseInfo
{
	int statusCode;
	char* pResponseheader;
	int responseHeaderLen;
	char* pResponseBody;
	int resposeBodyLen;
	BOOL bLastChunk;
}SFBRmResponseInfo;

//Begin Fix for Redirection Issue
typedef enum _SFBRedirectState
{
	REDIRECT_NONE = 0,
	REDIRECT_ABOSULUTE_URI,
	REDIRECT_RELATIVE_URI,
}SFBRedirectState;
//End Fix for Redirection Issue
//EncodeID specifies the encoding of data received-gzip or deflate
typedef enum _SFBContentEncodeID
{
	SFB_IS_NONE,
	SFB_IS_GZIP,
	SFB_IS_DEFLATE
}SFBContentEncodeID;
/* 
Enum SFBRmRequestStatus
Status of the request in the queue */
typedef enum 
{
	SFB_REQUEST_NONE,
	SFB_REQUEST_PENDING,
	SFB_REQUEST_FILE_PENDING,
	SFB_REQUEST_CHECKING_CACHE,
	SFB_REQUEST_NOT_IN_CACHE,
	SFB_REQUEST_PROCESSING,
	SFB_REQUEST_HDR_RCVD,
	SFB_REQUEST_HDR_BODY_RCVD,
	SFB_REQUEST_BODY_RCVD,
	SFB_REQUEST_LAST_CHUNK_RCVD,
	SFB_REQUEST_HEADER_PROCESSING,
	SFB_REQUEST_HDR_PROCESSED,
	SFB_REQUEST_BODY_PROCESSING,
	SFB_REQUEST_BODY_PROCESSED,
	SFB_REQUEST_FINISHED,
	SFB_REQUEST_NEEDAUTH, //SFB00000029
	SFB_REQUEST_AUTHRCVD, //SFB00000029
	SFB_REQUEST_CANCEL,
	SFB_REQUEST_ABORT,
	SFB_REQUEST_HEADER_PROCESSING_PAUSE,
	SFB_REQUEST_BODY_PROCESSING_PAUSE
}SFBRmRequestStatus;

/**
SFBRmEntity
This structure defines one instance of the request.
It handles all the request and response info for the particular request.
These request entities shall be queued as linked list
*/
typedef struct _SFBRmEntity_st
{	
	/* Module responsible for the request*/
	//SFBAppID appId; 
	SFBRmHClient	hClient;
	

	/* Request ID returned by RM to the requesting application*/
	SFBRmReqId reqId; 

	/*Transaction ID returned by underlying HTTP or WSP layer*/
	int trId;  
	
	/* Request Info of the request filled by requesting module */
	SFBRmRequestInfo* requestInfo;

	/*Response Info of the request*/
	SFBRmResponseInfo* responseInfo;	
	
	/*Status of the request in the queue*/
	SFBRmRequestStatus requestStatus;

	/*Cache Entry structure updated by cache module*/
	void * pCacheEntry; // SFB00000031

	/*Auth Data updated by Auth module*/
	void * pAuthData; //SFB00000029

	/*boolean ReUseAuth updated by Auth module*/
	BOOL  bReUseAuth; //SFB00000029
	
		//EncodeID specifies the encoding of data received-gzip or deflate
	SFBContentEncodeID EncID;
	/* redirection state */
	SFBRedirectState redirectState; // Fix for Redirection Issue

	/* callback function to send Header response to the corresponding application */
	SFBRmHeaderCallback headerCallback; 
	
	/* callback function to send content response to the corresponding application */
	SFBRmContentCallback contentCallback;
	
	/* callback function to send error response to the corresponding application */
	SFBRmErrorCallback errorCallback;

	/* callback function to send redirect url to corresponding application */
	SFBRmRedirectCallback redirectCallback;
	
	/* callback function to send ------- to the corresponding application */
	SFBRmCacheCallback cacheCallback;
	
	/* callback function to send cookie confirmation to the corresponding application */
	SFBRmCookieCallback cookieCallback;

	/* callback function to send authentifiation confirmation to the corresponding application */
	SFBRmAuthenticationCallback authenticationCallback;

	/* callback function to get the confirmation for SSL certificate */
	SFBRmSSLConfirmationCallback sslConfirmationCallback;

	/* callback function to send posted datalength to the corresponding application */
	SFBRmPostCallback postCallback; 

	SFBRmSendNextPostChunkCallback sendNextPostChunkCallback;//Anshul July 3

	/* Application Data which is argument to callback function */
	void * appData;

	SFBalHSenderContext hSenderContext;

#ifdef SFB_RM_UNZIP
	/* Stream Data used for gzip and deflate uncompress */
	void *pUnZipStream; 
	/* State of gzip and deflate uncompress */
	SFBUnzipState unzipStatus;
#endif

	struct _SFBRmEntity_st* pNext;

}SFBRmEntity;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif //_SFB_CORE_TYPES_H_