/* 
 * Copyright (c) 2007 SISO, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 /* FILE :
 *   This file defines the porting layer API's for the the components like HTTP
 *
 * @name		SFBRmHTTP.h [SHP HTTP interface]
 * @author		Ancil K George (ancilgeorge@samsung.com)
				Basavaraj P S (basavaraj.ps@samsung.com)
				Anshul Saxena (a.saxena@samsung.com)
				Rohit P Shetty (rohit.shetty@samsung.com)
  */
#ifndef _SFB_RMHTTP_H_
#define _SFB_RMHTTP_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

	/*INCLUDES*/
#include <SFBRmCoreTypes.h>
#include <SFBalProto.h>
	/*MACROS*/ 

#define HTTP_RESP_HDR_LOCATION "Location:" //SFB00000378
#define HTTP_RESP_HDR_TRANSFER_ENCODING "Transfer-Encoding:" 
#define HTTP_SCHEME "http://"
#define HTTPS_SCHEME "https://"
#define HOST_HEADER "Host: "
#define TRANSFER_ENCODING_HEADER "Transfer-Encoding: chunked"
#define HTTP_HDR_CONTENT_LENGTH "Content-Length: "
#define SFB_HTTP_POST_MAX      (25*1024)//Anshul
#define HTTP_CONTENT_TYPE		"Content-Type: "
#define SFB_TEXT_HTML			"text/html"
#define HTTP_HDR_CONTENT_ENCODING  "Content-Encoding:"
#define ENCODE_GZIP "gzip"
#define ENCODE_DEFLATE "deflate"
#define SFB_SAVED_PAGE_PATH "/User/Br/SavedPage"
#define HTTP_IF_MODIFIED_SINCE "If-Modified-Since:"
#define HTTP_IF_NONE_MATCH "If-None-Match:"

// move to BAL
//	/*DATA TYPES*/
//
//typedef enum
//{
//    SFB_RM_HTTP_STATUS_100 = 100,			/* Continue */
//    SFB_RM_HTTP_STATUS_101,					/* Switching Protocols */
//
//    SFB_RM_HTTP_STATUS_200 = 200,			/* OK, Succes */
//    SFB_RM_HTTP_STATUS_201,					/* Created */
//    SFB_RM_HTTP_STATUS_202,					/* Accepted */
//    SFB_RM_HTTP_STATUS_203,					/* Non-Authoritative Information */
//    SFB_RM_HTTP_STATUS_204,					/* No Content */
//    SFB_RM_HTTP_STATUS_205,					/* Reset Content */
//    SFB_RM_HTTP_STATUS_206,					/* Partial Content */
//
//    SFB_RM_HTTP_STATUS_300 = 300,			/* Multiple Choices */
//    SFB_RM_HTTP_STATUS_301,					/* Moved Permanently */
//    SFB_RM_HTTP_STATUS_302,					/* Found */
//    SFB_RM_HTTP_STATUS_303,					/* See Other */
//    SFB_RM_HTTP_STATUS_304,					/* Not Modified */
//    SFB_RM_HTTP_STATUS_305,					/* Temporary Redirect */
//
//    SFB_RM_HTTP_STATUS_400 = 400,			/* Bad Request */
//    SFB_RM_HTTP_STATUS_401,					/* Unauthorized */
//    SFB_RM_HTTP_STATUS_402,					/* Payment Required */
//    SFB_RM_HTTP_STATUS_403,					/* Forbidden */
//    SFB_RM_HTTP_STATUS_404,					/* Not Found */
//    SFB_RM_HTTP_STATUS_405,					/* Method Not Allowed */
//    SFB_RM_HTTP_STATUS_406,					/* Not Acceptable */
//    SFB_RM_HTTP_STATUS_407,					/* Proxy Authentication Required */
//    SFB_RM_HTTP_STATUS_408,					/* Request Time-out */
//    SFB_RM_HTTP_STATUS_409,					/* Conflict */
//    SFB_RM_HTTP_STATUS_410,					/* Gone */
//    SFB_RM_HTTP_STATUS_411,					/* Length Required */
//    SFB_RM_HTTP_STATUS_412,					/* Precondition Failded */
//    SFB_RM_HTTP_STATUS_413,					/* Request Entity Too Large */
//    SFB_RM_HTTP_STATUS_414,					/* Request-URI Too Large */
//    SFB_RM_HTTP_STATUS_415,					/* Unsupported Media Type */
//
//    SFB_RM_HTTP_STATUS_500 = 500,			/* Internal Server Error */
//    SFB_RM_HTTP_STATUS_501,					/* Not Implemented */
//    SFB_RM_HTTP_STATUS_502,					/* Bad Gateway */
//    SFB_RM_HTTP_STATUS_503,					/* Service Unavailable */
//    SFB_RM_HTTP_STATUS_504,					/* Gateway Time-out */
//    SFB_RM_HTTP_STATUS_505					/* HTTP Version not supported */
//} SFBRmHTTPStatus;


/* Function Declarations */


/* Header Encoding operation Functions */




/**
Fucntion: SFBRmSendHTTPRequest

* This function is called to get the HTTP request header Buffer, inside this function 
* BAL layer External API's are called to get the All types of HTTP Headers
* this function used only for HTTP connection type
* @param	SFBRmEntity [in] Request in a RmEntity that is to be processed
* @param	HBrProtoAccount [in] hProtoAccount Proto Account Handle 
* @param	HBrProtocol [in] hProtocol protocol Handle

* @return	Returns the SFB_RM_OK if success or SFB_INVALID_OBJ if fail
*/

SFBResult SFBRmSendHTTPRequest (SFBRmEntity * pRmEntity, char * Proxy, SFBalHProtocol hProtocol);


/*
  	Fucntion: SFBRmHttpAbortRequest

 * This function is called by RM module to Abort the Request by giving input transaction Id
 *
 * @param   trId			[in] TransactionId of HTTP request
 * @return	Returns the SFB_RM_OK if success or any other failure if fail
 */


SFBResult SFBRmHttpAbortRequest (int TrId);


/**
Fucntion: SFBRmHttpRequestIndication

* This function is called by HTTP module to indicate the result of HTTP Request(trId).
* As a response of SFBRmSendHTTPRequest(), HTTP module calls this function to
* give the responseHeader & 1st chunk of responseBody.
*
* @param   trId				[in] TransactionId of HTTP request
* @param   status				[in] Http server status
* @param   pResponseHeader	[in] Http Response Header
* @param   pResponseBody		[in] Http Response Body
* @param   responseBodyLen	[in] Http Response Body Length
*/

void  SFBRmHttpRequestIndication (int trId, int status, char* pResponseHeader,
char* pResponseBody, int responseBodyLen);


/*

  Fucntion: SFBRmHttpChunkedRequestIndication

 * This function is called by HTTP module to indicate the result of HTTP Request.
 * As a response of SFBRmSendHTTPRequest(), HTTP module calls this
 * to give a chunk data except header and 1st chunk which is given to SFBRmHttpRequestIndication()
 *
 * @param   trId		[in] TransactionId of HTTP request, this is ID of different HTTP requests.
 * @param   pChunk	[in] Http Response chunk Body
 * @param   chunkLen	[in] Http Response chunk Body Length 
 */

void  SFBRmHttpChunkedRequestIndication (int trId, char* pChunk,int chunkLen);


/*

	Fucntion: SFBRmHttpAbortRequestIndication

 * This function is called by HTTP module 
 * to indicate that HTTP request is aborted due to "abortreason"
 *
 * @param   trId			[in] TransactionId of HTTP request
 * @param   abortreason	[in] Abort reason of teh request
 * @param   bData			[in] If there is any data received just before "Aborted", 
						      it will delivered through SFBRmHttpRequestIndication() or 
							SFBRmHttpChunkedRequestIndication()
 * 
 */
 
void  SFBRmHttpAbortRequestIndication (int trId, int abortReason, BOOL bData);

/*

  	Fucntion: SFBRmHttpAbortRequestIndication

 * This function is called by HTTP module to indicate the addition of next chunk body.
 *
 * @param   trId			[in] TransactionId of HTTP request
 * @param   chunkSize		[in] Size of next chunk body
 */


void SFBRmHttpAddChunkIndication (int trId, int chunkSize);



//gnana
//-------------------------------------------------------------------------------//
/* Http Pause and Resume for Download  */

BOOL SFBRmHttpPauseReq(int trId,  SFBHttpMessage iMessage );

BOOL SFBRmHttpResumeReq(int trId, SFBHttpMessage iMessage);
//-------------------------------------------------------------------------------//


/*
  	Fucntion: SFBRmGetHeaderfieldValue

 * This function is called to get the header value by passing Header Name.
 *
 * @param   szHeaderString			[in] NULL terminated Response Header string
 * @param   szHeaderField		[in] Header Field name
 * @param   szHeaderField		[out] return Header Value
 */
char* SFBRmGetHeaderfieldValue(char* szHeaderString, char* szHeaderField);


/*
  	Fucntion: SFBRmIsChunkedMode

 * This function is called to see wheather header value has "chunked"
 *
 * @param   pHeader			[in] NULL terminated Response Header string
 * returns  TRUE or FALSE
 */

BOOL SFBRmIsChunkedMode(char* pHeader);
void SFBRmMakeHostUrlScheme(char *pszUrl , char *outputUrl);
void SFBRmMakeRedirectUrlScheme(char *pszUrl , char *outputUrl, char** pRedirectUrl);
BOOL SFBRmCheckDomainInUrl(char *pszUrl);
char* SFBRmValidateUrl(char *pszUrl);
BOOL SFBRmEscapeBadCharsFromUrl(const char* in_url, char** out_ppurl);
BOOL SFBRmParseURL (const char * url, char** out_ppurl);
char* SFBRmDecodeURLEscapeSequences(char* str);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif //_SFB_RMHTTP_H_