/* 
 * Copyright (c) 2007 SISO, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 /* FILE :
 *   This file defines the porting layer API's for the the components like Webkit
 *
 * @name		SFBRmMgr.h [RM External interface]
 * @author		Ancil K George (ancilgeorge@samsung.com)
				Basavaraj P S (basavaraj.ps@samsung.com)
				Anshul Saxena (a.saxena@samsung.com)
				Rohit P Shetty (rohit.shetty@samsung.com)
  */
#ifndef _SFB_REQMNGR_H_
#define _SFB_REQMNGR_H_

#include "SFBalMain.h"
#include "SFBalRm.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define SFB_REDIRECT_URL_HEADER "X-SFB-RedirectedUrl: "

/*SFBRmRqId
Id returned by Request Manager to the calling module*/
typedef int SFBRmReqId;

/* 
Enum SFBAppID
Defines Module which is sending the request*/
/*
typedef enum 
{
	SFB_APP_WEBKIT_BRIDGE,
	SFB_APP_CONTENT_DOWNLOADER
}SFBAppID;*/

typedef HObj SFBRmHClient;

typedef SFBalHProtoAccount SFBRmHPDPAccount;


/* 
Enum SFBCacheMode
Defines the Mode of the cache
This enum may be defined in Cache header file, need to include that header
*/
typedef enum
{
	SFB_CACHE_NORMAL,
	SFB_CACHE_NOCACHE,
	SFB_CACHE_HISTORY,
	SFB_CACHE_ONLYCACHE
}SFBCacheMode;

/**
callback data types used by RM while sending the response to corresponding module
*/

/* 
Enum SFBRmRespInd
Response Indication Type , indicates whether response is header or data or abort or stop
*/
typedef enum _SFBRmRespInd
{
	SFB_HEADER_IND, /* Response Header Indication*/
	SFB_ABORT_IND, /* when network fails or any other error */ 
	SFB_STOP_IND, /* when User press Stop  */
	SFB_REDIRECT_IND /* during redirection status */
}SFBRmResponseInd;

/**
SFBRmRequestInfo
This structure defines the params needed to send a request via Request Manager.
This structure is input to a RM Module called WebKit or CD, need to include corresponding header file
*/

typedef struct _SFBRmRequestInfo
{
	char* szURL;
	char* szMainDocURL;
	char* pPostData;
	int postDataLen;// total size of data to be posted
	int postedDataLength;//size of data posted till now
	char* pRequestHeader;	
	char* pAuthHeader;
	SFBMethodType method;
	SFBCacheMode cacheMode;
	BOOL bAllowCookies;
//	int totalPostDataLen;//Anshul July 3
	int postDataReceivedLength;//size of data to be posted in one shot
	int postedDataLenPerChunk;//Oct 15
//	int totalPostedDataLen; //Anshul Aug 11

}SFBRmRequestInfo;

// Because of circular dependency, it's impossible to use specific structure of cache/chookie as this handle
// let's use void* instead
typedef void* SFBHCookie;
typedef void* SFBHCache;
#define SFB_INVALID_CACHE_HANDLE	NULL
#define SFB_INVALID_COOKIE_HANDLE	NULL

/**
Fucntion Ptr: SFBRmHeaderCallback

* This function ptr specifies the params of the callback that shall be called by RM 
* This callback shall be called for each line of the header separated by  \r\n 

* @param	reqId Id of the request 
* @param	appData [in] application Data
* @param	statusCode [in] response status code
* @param	responseType [in] Specifies whether the response is for Header, Abort, Stop or redirection
* @param	respHdr [in] response Header buffer separated by \r\n
* @param	respHdr [in] response Header Length
*/

typedef void (*SFBRmHeaderCallback) (
									SFBRmHClient hClient,
									SFBRmReqId reqID, 
									 void * appData, 
									 int statusCode, 
									 SFBRmResponseInd responseType, 
									 char* respHdr, 
									 int respHdrLen);


/**
Fucntion Ptr: SFBRmContentCallback

* This function ptr specifies the params of the callback that shall be called by RM 

* @param	reqId Id of the request 
* @param	appData [in] application Data
* @param	respHdr [in] response Body
* @param	respHdr [in] response Body Length
* @param	bLastChunk [in] boolean flag to indicate Last Chunk
*/


typedef void (*SFBRmContentCallback) (
									SFBRmHClient hClient,
									SFBRmReqId reqID, 
									  void * appData, 
									  char* respBody, 
									  int respBodyLen, 
									  BOOL bLastChunk);


/**
Fucntion Ptr: SFBRmErrorCallback

* This function ptr specifies the params of the callback that shall be called by RM 
* This callback shall be called for each line of the header separated by  \r\n 

* @param	reqId Id of the request 
* @param	appData [in] application Data
* @param	errorCode [in] response error code
*/

typedef void (*SFBRmErrorCallback) (
									SFBRmHClient hClient,
									SFBRmReqId reqID, 
									 void * appData, 
									 SFBStatus errorCode);


/**
 * The client call this callback response to continut to process the redirect.
 * @param pRedirectedReq The actual request to be excuted. The client can chage attributes of the original redirected request.
 * 
 */
 typedef void (*SFBRmRedirectCallbackResponse) ( SFBRmHClient hClient,
											SFBRmReqId reqID,
												 SFBRmRequestInfo* pRedirectedReq, 
												 BOOL userConfirmation
												);//Anshul



/**
 * Called before processing the redirected request
 * Client can determine where to continue to process the request or not and can chage attributes of the redirected request.
 * In case client does not specify this callback, SFBRm will continue to process the request automatically.
 * If the client implements this method, the request will suspend until the client call fpRespCallback.
 *
 * @param pRedirectedReq[in] The redirected request. 
 * @param szResHeaders[in] Thre redirect response headers of the original request. 
 *
 * @return TRUE if client want the request to continue to process or FALSE if client want to stop the request.
 *
 */
typedef BOOL (*SFBRmRedirectCallback) ( SFBRmHClient hClient,
											SFBRmReqId reqID,
											void* appData,
											SFBRmRequestInfo* pRedirectedReq,
											char* pResHeaders,
											int nLenResHeaders,
											SFBRmRedirectCallbackResponse fpCallbackResponse );

typedef enum
{
	SFBRM_AUTH_HTTPBASIC,
	SFBRM_AUTH_HTTPDIGEST,
	SFBRM_AUTH_HTMLFORM
}SFBRmAuthMethod;


typedef struct
{
	SFBRmAuthMethod nAuthMethod;
	char* pszHost;
	int nPort;
	int nProxyType;
	char* pszRealm; /**< authentication realm, or nil if no realm has been set. */
}SFBRmProtectionSpace;



/**
 * The client call this callback to continue process the authentication challenge.
 *
 * 1. If client want to provide credentials : set bCancel = FALSE, szUser, szPassword
 * 2. If client want to continue without providing credential : set bCancel = FALSE, szUser = NULL, szPasswork = NULL
 * 3. If client want to cancel the authentication challenge, set bCancel = TRUE
 */
typedef void (*SFBRmAuthenticationCallbackResponse)( SFBRmHClient hClient,
											SFBRmReqId reqID,
											BOOL bCancel,
											char* szUser,
											char* szPassword
											);

/**
 * Called when SFBRm must authenticate a challenge in order to process the request
 * This method gives the client the opportunity to determine the course of action taken 
 * for the challenge: 1.provide credentials, 2.continue without providing credentials, or 3.cancel the authentication challenge.
 * If the client implements this method, the request will suspend until the client call fpRespCallback.
 *
 * @param nError[in] The error representing the last authentication failure
 * @param prevFailCount The count of failed authentication attempts
 * @param szProposedUser[in] The proposed user for this challenge. 
 						    SFBRm propose this information from the request URL or the credential storage
 * @param szProposedPass[in] The proposed password for this challenge. 
 						     SFBRm propose this information from the request URL or the credential storage
 * @param pProtectionSpace[in] The protection space
 *
 */
typedef void (*SFBRmAuthenticationCallback) ( SFBRmHClient hClient,
											SFBRmReqId reqID,
											void* appData,
											SFBStatus nError,
											int prevFailCount,
											char* szProposedUser,
											char* szProposedPass,
											SFBRmProtectionSpace *pProtectionSpace,
											SFBRmAuthenticationCallbackResponse fpRespCallback
											);
											





/**
 * The client call this callback response in order to continue to process the cache.
 *
 * @param bCancel TRUE if the client want to save cache, otherwise FALSE
 * @param pResHeaders[in] 	The actual headers to be stored
 * @param pResData[in]		The actual headers to be stored
 * @param hCacheStorage[in] 	If client wish to use a different cache storage to store this response instead of the default storage specified by SFBRmSetCacheStorage,
 *						client must set the handle of the different cache storage. If client want to use the default storage, set SFB_INVALID_CACHE_HANDLE
 						For example, If pResHeaders include the operator specific header, client can specify the handle of the operator cache.
 */
typedef void (*SFBRmCacheCallbackResponse)( SFBRmHClient hClient,
											SFBRmReqId reqID,
											BOOL bCancel,
											SFBHCache hCacheStorage,
											char* pResHeaders,
											int nLenResHeaders,
											char* pResData,
											int nLenResData );


/**
 * Called before storing the response to cache storage in order to give the client a chance to change it.
  * In case the client does not specify this callback, SFBRm will save the cache using the default cache storages.
 * If the client implements this method, the request will suspend until the client call fpCallbackResponse.
 *
 * @param pResHeaders[in] 	The all headers of the response
 * @param pResData[int]		The all data of the response
 */
typedef void (*SFBRmCacheCallback)( SFBRmHClient hClient,
											SFBRmReqId reqID,
											void* appData,
											char* pResHeaders,
											int nLenResHeaders,
											char* pResData,
											int nLenResData,
											SFBRmCacheCallbackResponse fpCallbackResponse );






/**
 * The client call this callback response in order to continue to process the cookie.
 
 *
 * @param szCurrentUrl The actual cookie information to be sotred
 * @param szMainDocUrl The actual cookie information to be sotred
 * @param szHeaders The actual cookie information to be sotred
 * @param bCancel TRUE if the client want to save cookie, otherwise FALSE
 * @param hNewCookieStorage[in] 	If client wish to use a different cookie storage instead of the default storage specified by SFBRmSetCookieStorage,
 *						the client must set the handle of the different cookie storage. If client want to use the default storage, set SFB_INVALID_COOKIE_HANDLE
 * 						For example, If szHeaders include the operator specific header, client can specify the handle of the operator cookie storage.
 */

typedef void (*SFBRmCookieCallbackResponse)( SFBRmHClient hClient,
											SFBRmReqId reqID,
											unsigned int iConfirm,
											void* pCookie
											);


/**
 * Called before storing the response cookies to cookie storage in order to give the client a chance to change it.
 * In case the client does not specify this callback, SFBRm will save the cookies using the default cookie storages.
 * If the client implements this method, the request will suspend until the client call fpCallbackResponse.
 * 
 * @param szCurrentUrl[in] 	
 * @param szMainDocUrl[in]		
 * @param szHeaders[in] The http headers
 */
typedef void (*SFBRmCookieCallback)( SFBRmHClient hClient,
											SFBRmReqId reqID,
											void* appData,
											char* szHostName,
											BOOL bCookieExist,
											char* szCookieString,
											void* pCookie,
											SFBRmCookieCallbackResponse fpCallbackResponse
											);

/*
 * Added for SSL Callback function
 */
typedef void (*SFBalRmSSLCallbackResponse)(int hHttpTr, BOOL iConfirm);
typedef void (*SFBRmSSLConfirmationCallback)( SFBRmHClient hClient, SFBRmReqId reqID, void* appData, char* pCertData, SFBalRmSSLCallbackResponse fpRespCallback);

/**
 * Called whenever the data chunk is posted
 *  For example, the client can use this callbak to update progress bar of UI.
 *
 * @param nLenPostedData The length of data which is posted at this time
 * @param nLenTotalPostedData The length of data which has been posted until now
 * @param nLenTotalPostData The length of total data which will be posted
 */
 typedef void (*SFBRmPostCallback)( SFBRmHClient hClient,
											SFBRmReqId reqID,
											void* appData,
										//	int nLenPostedData,
											int nLenTotalPostedData,
											int nLenTotalPostData
											);
											

 typedef void (*SFBRmSendNextPostChunkCallback)(void* appData, void* _pReqInfo, char** postBuffer, int* postDataLen);

typedef enum
{
	SFB_PDPCTX_STARTED,
	SFB_PDPCTX_STARTING,
	SFB_PDPCTX_STOPPING,
	SFB_PDPCTX_STOPPED,
	SFB_PDPCTX_CONFIRM_DISCONECT_OTHER_PDP,

	// For 3G Indicator - Sumdra
        SFB_PDPCTX_TRANSACTION,
	SFB_PDPCTX_NO_TRANSACTION
}SFBRmPDPContextEvent;



typedef struct
{
	
	SFBRmPDPContextEvent nEvent;
	int nError;
}SFBRmPDPContextEventParams;


/**
 * The client call this callback response to continue to process the new PDP connection.
 * 
 */
 typedef void (*SFBRmPDPContextCallbackResponse) ( SFBRmHClient hClient, BOOL bConfirm);


/**
 * Called when some PDP related events happens 
 */
typedef void (*SFBRmPDPContextCallback)( SFBRmHClient hClient, 
										SFBRmPDPContextEventParams *pParams , 
										SFBRmPDPContextCallbackResponse fpCallbackResponse);
							 

/**
Fucntion: SFBRmInit
* This funtion initialize SFBRm module. Initialize global variables .
* @return	Returns SFB_RM_OK if success or SFB_INVALID_OBJ if fail
*/
SFBResult SFBRmInit();


/**
Fucntion: SFBRmFinal
* This funtion finalize SFBRm module.
* @return	Returns SFB_RM_OK if success or SFB_INVALID_OBJ if fail
*/
SFBResult SFBRmFinal();


/**
 * This funtion register a client to SFBRm module.
 * If client want to use SFBRm, Client must register to SFBRm first.
 * Even though szClientName is same, each client have different PDP context if the each client call SFBRmRegisterClient seperately.
 * That means the "A" client won't be disconnected when the "B" client call SFBRmRequestStopAll even though the two client use same PDP account.
 * PDP context is used to handle SFBRmRequestUrl and Each SFBRmRequest will share a PDP context registered by SFBRmRegisterClient
 *
 * @param fpPDPCtxCallback The client will be notified of PDP related events
 */
SFBRmHClient SFBRmRegisterClient (const char * szClientName, SFBRmPDPContextCallback fpPDPCtxCallback );

/**
 * This funtion unregister a client from SFBRm module.
 */
SFBResult SFBRmUnregisterClient ( SFBRmHClient hClient  );


/**
 * This funtion set PDP account to be used to handle SFBRmRequestUrl
 */
SFBResult SFBRmSetPDPAccount( SFBRmHClient hClient , SFBRmHPDPAccount hAccount  );


/**
 * Set the cookie storage which SFBRm use to store cookie.
 * Each client can have different cookie storage.
 * @see SFBInitCache
 */
SFBResult SFBRmSetCookieStorage( SFBRmHClient hClient, SFBHCookie hCookie );

/**
 * Get the cookie storage which SFBRm use to store cookie.
 * Each client can have different cookie storage.
 * @see 
 */
SFBHCookie SFBRmGetCookieStorage( SFBRmHClient hClient );

/**
 * Set the cache storage which SFBRm use to store cookie.
 * Each client can have different cache storage.
 * @see SFBInitCache
 */
SFBResult SFBRmSetCacheStorage( SFBRmHClient hClient, SFBHCache hCache );

/**
 * Get the cache storage which SFBRm use to store cookie.
 * Each client can have different cache storage.
 * @see SFBInitCache
 */
SFBHCache SFBRmGetCacheStorage( SFBRmHClient hClient );

/**
* The client call this funtion to send a new URL request.
* For each URL request, the client can specify a callbacks.
** 
* @param	hClient [in] The handle of the client who send this request
* @param	pRequestInfo [in] The request information  
* @param	appData [in] The application data needs to be passed  back in the response callbacks
* @param	headerCallback [in] header callback function to give response Header when Response comes
* @param	contentCallback [in] content callback function to give response body when Response comes
* @param	errorCallback [in] error callback function to give error response other than 400 500 status
* 
* @param	redirectCallback [in] The callback which handle the redireciton, see  SFBRmRedirectCallback for details.
							If the client does not implement this callback, SFBRm will redirect automatically.
* @param	cacheCallback [in] The callback which handle the cache, see SFBRmCacheCallback for the details.
							If the client does not implement this callback, SFBRm will store the cache using the default cache storage.
							The default cache storeage will be set by SFBRmSetCacheStorage
* @param	cookieCallback [in] The callback which handle the cache, see SFBRmCacheCallback for the details.
							If the client does not implement this callback, SFBRm will store the cookie using the default cookie storage.
							The default cookie storeage will be set by SFBRmSetCookieStorage
* @param	SFBRmAuthenticationCallback [in] The callback which handle the authentication, see SFBRmAuthenticationCallback for the details.
							If the client does not implement this callback, SFBRm will use credentials in the default credential storage.
* @param	SFBRmPostCallback [in] The callback which handle the POST, see SFBRmPostCallback for the details.
*
* @return	Returns the reqId if success or SFB_INVALID_OBJ if fail
*/

SFBRmReqId SFBRmRequestUrl (SFBRmHClient hClient,
					 SFBRmRequestInfo* pRequestInfo, 
					 void * appData, 
					 SFBRmHeaderCallback headerCallback, 
					 SFBRmContentCallback contentCallback,
					 SFBRmErrorCallback errorCallback,
					 SFBRmRedirectCallback redirectCallback,
					 SFBRmCacheCallback cacheCallback,
					 SFBRmCookieCallback cookeCallback,
					 SFBRmAuthenticationCallback authenticationCallback,
					 SFBRmSSLConfirmationCallback sslConfirmationCallback,
					 SFBRmPostCallback postCallback,
					 SFBRmSendNextPostChunkCallback	sendNextPostChunkCallback//Anshul July 3
					 );

/**
 * used to redirect the existing stream
 */
SFBResult SFBRmSetAppData( SFBRmHClient hClient, 
								  SFBRmReqId reqId,
								  void* appData );

/**
 * used to redirect the existing stream
 */
SFBResult SFBRmSetContentCallback( SFBRmHClient hClient, 
								  SFBRmReqId reqId,
								  SFBRmContentCallback contentCallback );

/**
 * used to redirect the existing stream
 */
SFBResult SFBRmSetErrorCallback( 
								SFBRmHClient hClient, 
								SFBRmReqId reqId,
								SFBRmErrorCallback errorCallback );


SFBResult SFBRmResetAllCallbacks( 
								SFBRmHClient hClient, 
								SFBRmReqId reqId);

/**
	This function will take the input Application ID and Request ID 
	to stop the request and SFBRmCallback with STOP_IND indication will
	get called
**/

/**
Fucntion: SFBRmRequestStop

* This function stops the request based on the hClient.
* The callback of the request shall be called to inform the requesting module about the stop via SFB_STOP_IND .

* @param	reqId [in] Id of the request to stop

* @return	SFBResult [out] Returns TRUE if success else returns SFB_INVALID_OBJ
*/

SFBResult SFBRmRequestStop ( SFBRmHClient hClient, SFBRmReqId reqId);


/**
Fucntion: SFBRmRequestStopAll

* This function stops all the requests of the Application based on the hClient
* The callback of the request shall be called to inform the requesting module about the stop via SFB_STOP_IND .

* @param	AppId [in] Id of the Application

* @return	SFBResult [out] Returns TRUE if success else returns SFB_INVALID_OBJ
*/

SFBResult SFBRmRequestStopAll (SFBRmHClient hClient);

//gnana
//-------------------------------------------------------------------------------//
/* Http Pause and Resume for Download  */

BOOL SFBRmPauseReq(SFBRmHClient hClient, SFBRmReqId reqId, SFBHttpMessage iMessage );

BOOL SFBRmResumeReq(SFBRmHClient hClient, SFBRmReqId reqId, SFBHttpMessage iMessage);
//-------------------------------------------------------------------------------//

/* Reset the Linger timer from clients.*/
void SFBRmResetLingerTimer(SFBRmHClient hClient); //AKG for SFB00001702

void SFBRmCancelLingerTimer(SFBRmHClient hClient);

void SFBRmSetLcdDimmingMode(SFBRmHClient hClient, BOOL bIsLcdMode);

/* This APIs are for handling 3G indicator*/
void SFBRmSetRequestCount(SFBRmHClient hClient, BOOL	bIsSend);
void SFBRmResetRequestCount(SFBRmHClient hClient);

int	SFBRmGetRequestCount(void);

// This function is for handling after receiving user confirmation when other net application is using PDP
// Add by sumdra. 2008.06.18
void SFBRmDisconnectOtherNetCallback(SFBRmHClient hClient, BOOL bConfirm);

/**
Fucntion: SFBRmUtilsInit

* This function provides interface to the requesting modules to initialize the RM Util Modules
* This Function initializes Auth, Cache and Cookie

* @return	Returns SFB_RM_OK if success or SFB_INVALID_OBJ if fail
*/

//SFBResult SFBRmUtilsInit (char*	pCookieStorage, SFBCookieConfig* pCookieConfig, char* pCacheStorage, SFBCacheConfig* pCacheConfig);

/**
Fucntion: SFBRmUtilsExit

* This function provides interface to the requesting modules to exit the RM Util Modules
* This Function exits Auth, Cache and Cookie

* @return	Returns SFB_RM_OK if success or SFB_INVALID_OBJ if fail
*/

//SFBResult SFBRmUtilsExit (void);

/***************

Cache/Cookie related 
****************/


int SFBRmGetHProtoAccountHandle( SFBRmHClient hClient );

int SFBRmGetReqCount(void); //Oct 17

/**
 * Internal used Types / funtions
 */









#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif //_SFB_REQMNGR_H_