/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		IAuthInformation..h
 * @brief		Define interface of URILoader IAuthInformation
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.05	v1.0		Created
 */
#ifndef __IAuthInformation_Defined
#define __IAuthInformation_Defined

namespace URILoader
{

class IAuthInformation  : public IUnknown
{
public:
        static const unsigned long  AUTH_HOST = 1;
        static const unsigned long  AUTH_PROXY = 2;
        static const unsigned long NEED_DOMAIN = 4;
		static const unsigned long ONLY_PASSWORD = 8;
	/**
	* Auth prompts should ignore flags they don't understand; especially, they
	* should not throw an exception because of an unsupported flag.
	*/
	virtual Result GetFlags( unsigned long *pValue ) = 0;

	/**
	* The server-supplied realm of the authentication as defined in RFC 2617.
	* Can be the empty string if the protocol does not support realms.
	*/
	virtual Result GetRealm( String *pValue ) = 0;
	
	

	/**
	* The authentication scheme used for this request, if applicable. If the
	* protocol for this authentication does not support schemes, this will be
	* the empty string.
	*/
	virtual Result GetAuthScheme( String *pValue ) = 0;
	

	/**
	* The initial value should be used to prefill the dialog or be shown
	* in some other way to the user.
	* On return, this parameter contains the username entered by
	* the user.
	*/
	virtual Result GetUsername( String *pValue ) = 0;
	virtual Result SetUsername( String *aValue ) = 0;

	/**
	* The initial value should be used to prefill the dialog or be shown
	* in some other way to the user.
	* On return, this parameter contains the password entered by
	* the user.
	*/
	virtual Result GetPassword( String *pValue ) = 0;
	virtual Result SetPassword( String *aValue ) = 0;

	/**
	* The initial value should be used to prefill the dialog or be shown
	* in some other way to the user.
	* On return, this parameter contains the domain entered by
	* the user.
	*/
	virtual Result GetDomain( String *pValue ) = 0;
	virtual Result SetDomain( String *aValue ) = 0;
};	




} // namespace 

#endif //__IAuthInformation_Defined
