/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		IChannelEventSender..h
 * @brief		Define interface of URILoader IChannelEventSender
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.05	v1.0		Created
 */
#ifndef __IChannelEventSender_Defined
#define __IChannelEventSender_Defined

#include "IAuthInformation.h"
#include "ICookie.h"

namespace URILoader
{

class	IChannel;

class IChannelEventSender : public IUnknown
{
public:

	/**
	* @param bCancel Decide whether to cancle the redirection or not
	* @param aNewChannel The channel to be redirected. If bCancel is TRUE, this parameter will be ignored
	*/
	virtual Result OnContinueRedirect( BOOL bCancel, IChannel *aNewChannel  ) = 0;

	/**
	* @param bCancel Decide whether to cancle the authentication or not
	* @param aAuthInfo The imformation which user entered. If bCancel is TRUE, this parameter will be ignored
	*/
	virtual Result OnContinueAuthentication( BOOL bCancel, IAuthInformation* aAuthInfo ) = 0;


	/**
	*
	*/
	virtual Result OnContinueCertificateError( BOOL bCancel );

	

	static const unsigned int DENY_COOKIE = 0;
	static const unsigned int ACCEPT_COOKIE = 1;
	static const unsigned int ACCEPT_COOKIE_FOR_CURRENT_SESSION = 2;

	virtual Result OnContinueProcessCookie( unsigned int decision, ICookie* aCookie );
	


};

} // namespace 

#endif //__IChannelEventSender_Defined
