/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		ICookie.h
 * @brief		Define interface of URILoader ICooke
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.05	v1.0		Created
 */
#ifndef __ICooke_Defined
#define __ICooke_Defined

namespace URILoader
{

class ICookie : public IUnknown
{
public:

	/**
	 * the name of the cookie
	 */
	//readonly attribute ACString name;
	virtual Result GetName( String *pValue ) = 0;

	/**
	 * the cookie value
	 */
	//readonly attribute ACString value;
	virtual Result GetValue( String *pValue ) = 0;

	/**
	 * true if the cookie is a domain cookie, false otherwise
	 */
	//readonly attribute boolean isDomain;
	virtual Result GetIsDomain( BOOL *pValue ) = 0;


	/**
	 * the host (possibly fully qualified) of the cookie
	 */
	//readonly attribute AUTF8String host;
	virtual Result GetHost( String *pValue ) = 0;

	/**
	 * the path pertaining to the cookie
	 */
	//readonly attribute AUTF8String path;
	virtual Result GetPath( String *pValue ) = 0;

	/**
	 * true if the cookie was transmitted over ssl, false otherwise
	 */
	//readonly attribute boolean isSecure;
	virtual Result GetIsSecure( BOOL *pValue ) = 0;


	/**
	* ICookie2
	*/


	/**
	 * the host (possibly fully qualified) of the cookie,
	 * without a leading dot to represent if it is a
	 * domain cookie.
	 */
	//readonly attribute AUTF8String rawHost;
	virtual Result GetRawHost( String *pValue ) = 0; 


	/**
	 * true if the cookie is a session cookie.
	 * note that expiry time will also be honored
	 * for session cookies (see below); thus, whichever is
	 * the more restrictive of the two will take effect.
	 */
	//readonly attribute boolean isSession;
	virtual Result GetIsSession( BOOL *pValue ) = 0;

	/**
	 * the actual expiry time of the cookie, in seconds
	 * since midnight (00:00:00), January 1, 1970 UTC.
	 *
	 * this is distinct from nsICookie::expires, which
	 * has different and obsolete semantics.
	 */
	//readonly attribute PRInt64 expiry;
	virtual Result GetExpiry( long *pValue ) = 0;

	/**
	 * true if the cookie is an http only cookie
	 */
	//readonly attribute boolean isHttpOnly;
	virtual Result GetIsHttpOnly( BOOL *pValue ) = 0;
};




} // namespace 

#endif //__ICooke_Defined
