/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		IRequest.h
 * @brief		This file define interface for URILoader IChannel.
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.05	v1.0		Created
 */

#ifndef __IRequest_Defined
#define __IRequest_Defined

#include "URILoader.h"


namespace URILoader {

typedef unsigned long LoadFlags;


class IRequest : public IUnknown
{
public:

	/** Constants 
	  */
	/** Listed below are the various load flags which may be or'd together.
	 */

	/**
	 * No special load flags:
	 */
	static const unsigned long LOAD_NORMAL = 0;

	/** 
	 * Don't deliver status notifications to the IProgressEventSink, or keep 
	 * this load from completing the ILoadGroup it may belong to.
	 */
	static const unsigned long LOAD_BACKGROUND = 1 << 0; 

	/**************************************************************************
	 * The following flags control the flow of data into the cache.
	 */

	static const unsigned long INHIBIT_CACHING = 1 << 7;

	static const unsigned long INHIBIT_PERSISTENT_CACHING = 1 << 8;

	static const unsigned long LOAD_BYPASS_CACHE = 1 << 9;

	static const unsigned long LOAD_FROM_CACHE   = 1 << 10;

	static const unsigned long VALIDATE_ALWAYS           = 1 << 11;
	static const unsigned long VALIDATE_NEVER            = 1 << 12;
	static const unsigned long VALIDATE_ONCE_PER_SESSION = 1 << 13;	



	/***************************************************************************
	*
	* Properties
	*
	****************************************************************************/
	/**
	 * The name of the request. Often this is the URI of the request.
	 */
	virtual Result GetName( String *pValue ) = 0;

	/**
	 * The error status associated with the request.
	 */
	virtual Result GetStatus( Result *pValue  ) = 0;

	/**
	 * The load flags of this request.  Bits 0-15 are reserved.
	 *
	 */
	virtual Result GetLoadFlags( LoadFlags *pValue ) = 0;
	virtual Result SetLoadFlags( LoadFlags aValue ) = 0;



	/***************************************************************************
	*
	* Funtions
	*
	****************************************************************************/
	
	/**
	 * Indicates whether the request is pending.
	 */
	virtual Result IsPending( BOOL *pIsPending ) = 0;



	/**
	* Cancels the current request.
	*/
	virtual Result Cancel( Result aStatus ) = 0;


	/**
	 * Suspends the current request.
	 */
	virtual Result Suspend() = 0;


	virtual Result Pause() = 0;

	/**
	 * Resumes the current request.
	 */
	 virtual Result Resume() = 0;
	
};

} // namespace URILoader


#endif