/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		IStreamListener..h
 * @brief		Define interface of URILoader IStreamListener
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.05	v1.0		Created
 */

#ifndef _IStreamListener_Defined
#define _IStreamListener_Defined

#include "URILoader.h"
#include "IRequestObserver.h"
#include "IInputStream.h"

namespace URILoader
{

class IStreamListener : public IRequestObserver
{
public:
	
	/***************************************************************************
	*
	* Funtions
	*
	****************************************************************************/
	/**
	 * Called when the next chunk of data (corresponding to the request) may
	 * be read without blocking the calling thread.  The onDataAvailable impl
	 * must read exactly |aCount| bytes of data before returning.
	 *
	 * @param aRequest request corresponding to the source of the data
	 * @param aContext user defined context
	 * @param aInputStream input stream containing the data chunk
	 * @param aOffset
	 *        Number of bytes that were sent in previous onDataAvailable calls
	 *        for this request. In other words, the sum of all previous count
	 *        parameters.
	 *        If that number is greater than or equal to 2^32, this parameter
	 *        will be PR_UINT32_MAX (2^32 - 1).
	 * @param aCount number of bytes available in the stream
	 *
	 * NOTE: The aInputStream parameter must implement readSegments.
	 *
	 * An exception thrown from onDataAvailable has the side-effect of
	 * causing the request to be canceled.
	 */
	//virtual void OnDataAvailable( IRequest* aRequest,
	//                     void* aContext, 
	//                     IInputStream* aInputStream,
	//                     unsigned long aOffset,
	//                     unsigned long aCount) = 0;
	virtual void OnDataAvailable(  IRequest* aRequest,
	                     void* aContext, 
	                     char* inputStream,
	                     unsigned long aOffset,
	                     unsigned long aCount) = 0;


}; // IStreamListener



} // namespace 

#endif //_IRequestObserver_Defined
