/**
 * Samsung Handset Platform
 * Copyright (c) 2000 Software Center, Samsung Electronics, Inc.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics.
 */

/**
 *  This file is about ECOM service.
 * 
 * @name        UrlLoaderECOM.h
 * @author      Zafar Hussain ( zafar.h@samsung.com )
 */

#if defined (_JASMINE_ECOM)
#ifndef _URLLOADER_ECOM_H_
#define _URLLOADER_ECOM_H_

#if !defined(__cplusplus)
#error This header is not compatible with C files.
#endif

//#include "UrlLoaderECOMDecl.h"

#ifdef __cplusplus

extern "C" {
#endif /* __cplusplus */


#if !defined(_SHP_SIMUL) && defined(_SHP_CALL_DLL_URLLOADER)
extern __declspec(dllimport) EcomFactory* pDllBaseEcomFactory;
extern __declspec(dllimport)  ADDR dllBaseSb;
#endif 

	void SFBRmEventHandler( SFBalEventDataList * evt );

/*
 * ECOM Interface
 */
ECOM_INTERFACE(EUrlLoader)
{
	DECLARE_EBASE()
	/* DECLARATION*/
	
	// SFBalShp.c functionalities
	char*						(*Strdup)					(const char *pszStr);
	char*						(*Itoa)						(int n, char* str, int b);
	int							(*Strnicmp)					(const char* dst, const char* src, size_t n);
	int							(*Isspace)					(int ch);

	//UrlLoader
	URILoader::Result			  (*UrlCreateIURI)				( char *,URILoader::IURI **) ;
	URILoader::Result			  (*UrlCreateLoadGroup)			( URILoader::String , URILoader::ILoadGroup **);	
	void						  (*UrlDestroyLoadGroup)		( URILoader::ILoadGroup *);

	//Request Manager:
	SFBResult		  (*RmInit)							();
	SFBResult		  (*RmFinal)						();
	SFBResult		  (*RmRequestStop)					(SFBRmHClient, SFBRmReqId);
	SFBRmReqId		  (*RmRequestUrl)					(SFBRmHClient,SFBRmRequestInfo*,void *,SFBRmHeaderCallback,SFBRmContentCallback,SFBRmErrorCallback,SFBRmRedirectCallback,SFBRmCacheCallback,SFBRmCookieCallback,SFBRmAuthenticationCallback,SFBRmSSLConfirmationCallback,SFBRmPostCallback,SFBRmSendNextPostChunkCallback);
	void			  (*RmEventHandler)					(SFBalEventDataList *);
	SFBResult		  (*RmSetPDPAccount)				(SFBRmHClient , SFBRmHPDPAccount);
	SFBRmHClient	  (*RmRegisterClient)				(const char *, SFBRmPDPContextCallback);
	SFBResult		  (*RmUnregisterClient)				(SFBRmHClient);
	SFBResult		  (*RmSetContentCallback)			(SFBRmHClient, SFBRmReqId, SFBRmContentCallback);
	SFBResult		  (*ResetAllCallbacks)				(SFBRmHClient, SFBRmReqId);
	SFBResult		  (*RmSetAppData)					(SFBRmHClient , SFBRmReqId ,void*);
	SFBResult		  (*RmSetErrorCallback)				(SFBRmHClient , SFBRmReqId ,SFBRmErrorCallback );	
	BOOL			  (*RmPauseReq)						(SFBRmHClient, SFBRmReqId , SFBHttpMessage);
	BOOL			  (*RmResumeReq)					(SFBRmHClient, SFBRmReqId , SFBHttpMessage);
	void			  (*RmResetLingerTimer)				(SFBRmHClient);
	void			  (*RmCancelLingerTimer)			(SFBRmHClient);
	BOOL			  (*RmSetLingerTimer)				(SFBRmHClient, ULONG);
	BOOL			  (*RmEscapeBadCharsFromUrl)		(const char* , char** );

	void			  (*RmSetLcdDimmingMode)			(SFBRmHClient, BOOL);

	//cache:
	SFBResult		  (*RmSetCacheStorage)				(SFBRmHClient , SFBHCache );
	SFBHCache		  (*RmGetCacheStorage)				( SFBRmHClient);
	void			  (*InitCache)						(SFBHCache );
	void			  (*ExitCache)						(SFBHCache);	
	void			  (*StopCacheOperation)				(SFBHCache ,SFBRmReqId );
	void			  (*ClearAllCache)					(SFBHCache hCacheHandle);
	void			  (*UpdateCacheMetaData)			(SFBHCache,char*,char*, char*);

	//Cookie:

	SFBResult		  (*RmSetCookieStorage)				(SFBRmHClient , SFBHCookie);	
	SFBHCookie		  (*RmGetCookieStorage)				( SFBRmHClient);	
	SFBHCookie		  (*InitCookie)						(char* , SFBCookieConfig* );	
	void			  (*ExitCookie)						(SFBHCookie );	
	void			  (*RemoveAllCookie)				(SFBHCookie );	
	void			  (*SetCookieUIConfirmCallback)		(SFBRmHClient,SFBRmReqId,	unsigned int, void*);
	BOOL			  (*SetJavaScriptCookie)			(SFBCookieCallback ,SFBRmReqId,SFBRmHClient,void*, SFBHCookie,const char* , const char* );	
    char*			  (*GetJavaScriptCookie)			(SFBHCookie, const char*);
	BOOL			  (*GetCookieEnable)				(SFBHCookie);

	//Authentication
	
	void			   (*AuthInit)					();
	void			   (*AuthExit)					();
	void			   (*GetPasswdCallback)			(SFBRmHClient, SFBRmReqId, BOOL, char*, char*);

//All the below are BAL interfaces.They are used only by the engine present in WebControl
	//Memory manipulation
	void*			  (*BALMemAlloc)				(UINT32 );
	void*			  (*BALMemCalloc)          	 	(UINT32 , UINT32);
	void*             (*BALMemRealloc)				(void *, UINT32);
	void			  (*BALMemFree)					(void *);
	int				  (*MemGetStatEx)				();	
	void* 			  (*BALMemAllocWithPreCheck) 	(UINT32 memSize);
	BOOL 			  (*BalCheckPlatformOOM)		();
	char* 			  (*BalGetMimeFromExt)			(const char* szExt);
	BOOL			  (*BalIsExposeEventPresent)	();
	BOOL			  (*BalFSWriteAutoFormFill)		(char* szUrl,char* szForm, char*	NameValuePair );
	void 			  (*BalRemoveEventByID)			(SFBalAppId appId,SFBalEventId eventId, void* pWebControlInstance );
	SFBHCache 		  (*CreateCache) (char* pCurCacheDir, SFBCacheConfig* pTSFBCacheConfig);
	int 			  (*RmGetHProtoAccountHandle)		( SFBRmHClient hClient );
	BOOL 			  (*BalFSReadAutoFormFill)			(AddEntryToPageMap constructEntry);
	//Directory operations
	BOOL			  (*FmIsExistDir)			(const char* );
	BOOL			  (*RemoveDir)				(const char* );	
	BOOL			  (*CreateDir)				(const char* );
	BOOL			  (*SeekFile)				(BalFile , INT32 , INT32);
	//File operations
	BalFile			  (*CreateFile)				(const char* fileName , BalFileMode mode);
	BOOL			  (*IsFileExist)			(const char*);	
	BalFile			  (*OpenFile)				(const char *,UINT32);
	BOOL			  (*CloseFile)				(BalFile);
	BOOL			  (*ReadFile)				(BalFile  , void*  , UINT32  , UINT32 *);

	BOOL              (*WriteFile)				(BalFile ,  void * , UINT32 );
	UINT32			  (*GetFileSize)			(const char* );	
	int				  (*GetValidFilename)		(char* , char* , int);	
	BOOL			  (*RemoveFile)				(const char* );
	BOOL			  (*IsValidPathName)        (const char *,SFBalFileType);
	
	void			  (*SetEventCallbacks)		(SFBalEventCallbacks *);
	void			  (*ExitEvent)				();

	//Debugging 
	void			  (*Debug)					( UINT32 , const char* , ... );

	//List Utility
	int					(*ListGetCount)			(List* pList);
	void* 				(*ListGetNthObject)		(List* pList, int index);
	BOOL				(*ListAppendObjectEx)	(List* pList, void* pObject, const char* szFileName, int nLine);
	
	// Graphics
	BOOL			(*GwesInitializeGraphic)	();
	BOOL			(*GwesFinalizeGraphic)		( void );
	//Miscellaneous
	char *			  (*GetUserAgent)			();
	char *			  (*GetAppName)				();
	char *			  (*GetAppVersion)			();
	int				  (*ExtractNameFromURL)		(const char* , char* , int);
	char *			  (*GetHostName)			(SFBalHProtoClient );
	ULONG             (*SysGetTick)();
	BOOL              (*IsLcdHorizontal)(int ith);
	Diagnose          (*GetDiagnoseInfo)(int iComp);
	int				  (*GetDebugLevel) ();
	BOOL		      (*SetEditFieldInputTypeCategory) ( int, int, BOOL );
	
	BOOL		      (*SetEditFieldInputAlphabetMode) ( int, int );
	BOOL		      (*GetEditFieldInputAlphabetMode) ( int, int *);
	BOOL              (*GetEditAreaInputAlphabetMode)  ( int, int *); 
#if defined ( _SFBAL_SUPPORT_VECTOR_FONT )
	HFNT			(*GwesGetHFontSreenInfo)	( void );
	BOOL			(*GwesSetFontAttr)			( SFBalFontAttr  );
#endif
		//Event Operations
#ifdef MULTIPLE_INSTANCE_SUPPORT // Multiple instance support
	void			  (*SendEvent)				(SFBalAppId ,SFBalEventId ,void* ,int ,UINT32,void* ); 
	void			  (*BalInstanceExitEvent)	(void* pWebControlInstance, void *pRmClient);
#else
	void			  (*SendEvent)				(SFBalAppId ,SFBalEventId ,void* ,int ,UINT32); 
#endif 

#if defined ( SUPPORT_NP_PLUGIN )
		//Plugin
	NPGluePluginDLLDirectory* (*GetPluginDLLDirectory)		();
#endif


#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)

	//ListBox Related functions used by NativeListBoxShp.cpp
	int				  (*BalGwesCreateListBox)				(int  hWinParent, int style, int x,int y,int w,int  h,WmEventMapEntry* gBrPlWsListEventMap);
	BOOL			  (*BalGwesAddListItemText)				(int  hWin, const char* szItem);
	int				  (*BalGwesGetListExFocusedItem)		(HWIN hWin, int *index );
	BOOL			  (*BalGwesSetListExFocusItem)			(HWIN hWin, WsListExTreeDepth depth, int ListIndex);
	BOOL			  (*BalGwesSetListExAppInfo)			(HWIN hWin, void *pAppInfo, int depth,int ListIndex);
	void*			  (*BalGwesGetListExAppInfo)			(HWIN hWin, WsListExTreeDepth depth, int ListIndex);
	BOOL			  (*BalGwesSetListExItemBgBitmap)		(HWIN hWin, WmBitmap *pbmItemBg);
	BOOL			  (*BalGwesGetListExColumnForm)			(HWIN hWin, HFNT (*colFont)[MAX_LISTEX_COLUMN], COLOR (*colNomalColor)[MAX_LISTEX_COLUMN], COLOR (*colSelectColor)[MAX_LISTEX_COLUMN], int ListIndex);
	BOOL			  (*BalGwesSetListExColumnForm)			(HWIN hWin, HFNT (*colFont)[MAX_LISTEX_COLUMN], COLOR (*colNomalColor)[MAX_LISTEX_COLUMN], COLOR (*colSelectColor)[MAX_LISTEX_COLUMN], WsListExTreeDepth depth, int ListIndex);
	int				  (*BalGwesGetListExAllItemCount)		(HWIN hWin);
	BOOL			  (*BalGwesSetListExFocusItemBgColor)	(HWIN hWin, COLOR color);
	COLOR			  (*BalGwesGetListExFocusItemBgColor)	(HWIN hWin);
	WsListExTreeDepth (*BalGwesGetListExTouchPressedItemIndex)	(HWIN hWin, int XPos, int YPos, int *index);

#endif //#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)
	BOOL              (*BalWsSetVeEnableCR)  ( int, int ); 
};

/*
 * Interface Accessor
 */
#if (defined(USING_URLLOADER)||defined(_SHP_SIMUL) && defined(_SHP_CALL_DLL_URLLOADER)) || (defined(_SHP_DLL) && ! defined(_SHP_DLL_URLLOADER) && ! defined(_SHP_CALL_DLL_URLLOADER))

//Plugin
#define			SFB_GetPluginDLLDirectory	(__UrlLoaderGetVtbl()->GetPluginDLLDirectory)
 
 // SFBalShp.c
#define			SFBStrdup					(__UrlLoaderGetVtbl()->Strdup)
#define			SFBItoa						(__UrlLoaderGetVtbl()->Itoa)
#define			SFBStrnicmp					(__UrlLoaderGetVtbl()->Strnicmp)
#define			SFBIsspace					(__UrlLoaderGetVtbl()->Isspace)


//UrlLoader
#define			CreateIURI					(__UrlLoaderGetVtbl()->UrlCreateIURI)
#define			CreateLoadGroup				(__UrlLoaderGetVtbl()->UrlCreateLoadGroup)
#define			DestroyLoadGroup			(__UrlLoaderGetVtbl()->UrlDestroyLoadGroup)

//Utility
#define			SFBGetPasswdCallback		(__UrlLoaderGetVtbl()->GetPasswdCallback)
#define			SFBAuthInit					(__UrlLoaderGetVtbl()->AuthInit)
#define			SFBAuthFinal				(__UrlLoaderGetVtbl()->AuthFinal)
#define			SFBRmInit					(__UrlLoaderGetVtbl()->RmInit)
#define			SFBRmFinal					(__UrlLoaderGetVtbl()->RmFinal)
#define         SFBSetCookieUIConfirmCallback (__UrlLoaderGetVtbl()->SetCookieUIConfirmCallback)
#define			SFBRmGetCacheStorage		(__UrlLoaderGetVtbl()->RmGetCacheStorage)
#define			SFBRmGetCookieStorage		(__UrlLoaderGetVtbl()->RmGetCookieStorage)
#define			SFBRmRequestStop			(__UrlLoaderGetVtbl()->RmRequestStop)
#define         SFBRmGetHProtoAccountHandle  (__UrlLoaderGetVtbl()->RmGetHProtoAccountHandle)
#define			SFBInitCookie				(__UrlLoaderGetVtbl()->InitCookie)
#define			SFBInitCache				(__UrlLoaderGetVtbl()->InitCache)
#define			SFBExitCookie				(__UrlLoaderGetVtbl()->ExitCookie)
#define			SFBExitCache				(__UrlLoaderGetVtbl()->ExitCache)
#define         SFBCacheStopOperation		(__UrlLoaderGetVtbl()->StopCacheOperation)					
#define			SFBRmSetCacheStorage		(__UrlLoaderGetVtbl()->RmSetCacheStorage)
#define			SFBRmSetCookieStorage		(__UrlLoaderGetVtbl()->RmSetCookieStorage)
#define			SFBRmRegisterClient			(__UrlLoaderGetVtbl()->RmRegisterClient)
#define			SFBRmUnregisterClient		(__UrlLoaderGetVtbl()->RmUnregisterClient)
#define			SFBRmSetPDPAccount			(__UrlLoaderGetVtbl()->RmSetPDPAccount)
#define			SFBRmCancelLingerTimer  (__UrlLoaderGetVtbl()->RmCancelLingerTimer)
#define			SFBalSetLingerTimer			(__UrlLoaderGetVtbl()->RmSetLingerTimer)
#define			SFBClearAllCache			(__UrlLoaderGetVtbl()->ClearAllCache)
#define			SFBRemoveAllCookie			(__UrlLoaderGetVtbl()->RemoveAllCookie)
#define			SFBRmEventHandler			(__UrlLoaderGetVtbl()->RmEventHandler)
#define			SFBRmResetLingerTimer		(__UrlLoaderGetVtbl()->RmResetLingerTimer)
#define			SFBRmSetErrorCallback		(__UrlLoaderGetVtbl()->RmSetErrorCallback)
#define			SFBRmSetContentCallback		(__UrlLoaderGetVtbl()->RmSetContentCallback)
#define			SFBRmResetAllCallbacks		(__UrlLoaderGetVtbl()->ResetAllCallbacks)
#define			SFBRmSetAppData				(__UrlLoaderGetVtbl()->RmSetAppData)
#define			SFBRmPauseReq				(__UrlLoaderGetVtbl()->RmPauseReq)
#define			SFBRmResumeReq				(__UrlLoaderGetVtbl()->RmResumeReq)
#define			SFBRmRequestUrl				(__UrlLoaderGetVtbl()->RmRequestUrl)
#define			SFBRmEscapeBadCharsFromUrl	(__UrlLoaderGetVtbl()->RmEscapeBadCharsFromUrl)
#define			SFBRmSetLcdDimmingMode		(__UrlLoaderGetVtbl()->RmSetLcdDimmingMode)

#define			SFBUpdateCacheMetaData		(__UrlLoaderGetVtbl()->UpdateCacheMetaData)
#define			SFBSetJavaScriptCookie		(__UrlLoaderGetVtbl()->SetJavaScriptCookie)
#define			SFBGetJavaScriptCookie		(__UrlLoaderGetVtbl()->GetJavaScriptCookie)
#define			SFBGetCookieEnable			(__UrlLoaderGetVtbl()->GetCookieEnable)
#define         SFBCreateCache				(__UrlLoaderGetVtbl()->CreateCache)


#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)

//ListBox
#define     SFBalGwesCreateListBox			(__UrlLoaderGetVtbl()->BalGwesCreateListBox)
#define     SFBalGwesAddListItemText		(__UrlLoaderGetVtbl()->BalGwesAddListItemText)
#define		SFBalGwesGetListExFocusedItem	(__UrlLoaderGetVtbl()->BalGwesGetListExFocusedItem)
#define		SFBalGwesSetListExFocusItem		(__UrlLoaderGetVtbl()->BalGwesSetListExFocusItem)
#define		SFBalGwesSetListExAppInfo		(__UrlLoaderGetVtbl()->BalGwesSetListExAppInfo)
#define		SFBalGwesGetListExAppInfo		(__UrlLoaderGetVtbl()->BalGwesGetListExAppInfo)
#define		SFBalGwesSetListExItemBgBitmap	(__UrlLoaderGetVtbl()->BalGwesSetListExItemBgBitmap)
#define		SFBalGwesGetListExColumnForm	(__UrlLoaderGetVtbl()->BalGwesGetListExColumnForm)
#define		SFBalGwesSetListExColumnForm	(__UrlLoaderGetVtbl()->BalGwesSetListExColumnForm)
#define		SFBalGwesGetListExAllItemCount	(__UrlLoaderGetVtbl()->BalGwesGetListExAllItemCount)
#define		SFBalGwesSetListExFocusItemBgColor	(__UrlLoaderGetVtbl()->BalGwesSetListExFocusItemBgColor)
#define		SFBalGwesGetListExFocusItemBgColor	(__UrlLoaderGetVtbl()->BalGwesGetListExFocusItemBgColor)
#define		SFBalGwesGetListExTouchPressedItemIndex (__UrlLoaderGetVtbl()->BalGwesGetListExTouchPressedItemIndex)

#endif //#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)

//BAL
#define     SFBalIsExposeEventPresent		(__UrlLoaderGetVtbl()->BalIsExposeEventPresent)
#define		SFBalMemAllocEx			        (__UrlLoaderGetVtbl()->BALMemAlloc)
#define		SFBalMemCallocEx			        (__UrlLoaderGetVtbl()->BALMemCalloc)
#define		SFBalMemReallocEx					(__UrlLoaderGetVtbl()->BALMemRealloc)
#define		SFBalMemFree					(__UrlLoaderGetVtbl()->BALMemFree)
#define		SFBalMemGetStatEx			    (__UrlLoaderGetVtbl()->MemGetStatEx)
#define		SFBalMemAllocWithPreCheck	    (__UrlLoaderGetVtbl()->BALMemAllocWithPreCheck)
#define		SFBalCheckPlatformOOM		    (__UrlLoaderGetVtbl()->BalCheckPlatformOOM)
#define		SFBBalGetMimeFromExt		    (__UrlLoaderGetVtbl()->BalGetMimeFromExt)
#define		SFBalFSReadAutoFormFill			(__UrlLoaderGetVtbl()->BalFSReadAutoFormFill)
#define		SFBalFmIsExistDir				(__UrlLoaderGetVtbl()->FmIsExistDir)
#define		SFBalRemoveDir					(__UrlLoaderGetVtbl()->RemoveDir)
#define		SFBalCreateDir					(__UrlLoaderGetVtbl()->CreateDir)

#define		SFBalListGetCount				(__UrlLoaderGetVtbl()->ListGetCount)
#define		SFBalListGetNthObject			(__UrlLoaderGetVtbl()->ListGetNthObject)
#define		SFBalListAppendObjectEx			(__UrlLoaderGetVtbl()->ListAppendObjectEx)


#define		SFBalCreateFile					(__UrlLoaderGetVtbl()->CreateFile)
#define		SFBalIsFileExist				(__UrlLoaderGetVtbl()->IsFileExist)
#define		SFBalOpenFile					(__UrlLoaderGetVtbl()->OpenFile)
#define		SFBalCloseFile					(__UrlLoaderGetVtbl()->CloseFile)
#define		SFBalReadFile					(__UrlLoaderGetVtbl()->ReadFile)
#define		SFBalWriteFile					(__UrlLoaderGetVtbl()->WriteFile)
#define		SFBalSeekFile					(__UrlLoaderGetVtbl()->SeekFile)
#define		SFBalGetFileSize				(__UrlLoaderGetVtbl()->GetFileSize)
#define		SFBalGetValidFilename			(__UrlLoaderGetVtbl()->GetValidFilename)
#define		SFBalRemoveFile					(__UrlLoaderGetVtbl()->RemoveFile)
#define		SFBalIsValidPathName			(__UrlLoaderGetVtbl()->IsValidPathName)
#define		SFBalSendEvent					(__UrlLoaderGetVtbl()->SendEvent)
#define		SFBalSetEventCallbacks			(__UrlLoaderGetVtbl()->SetEventCallbacks)
#define		SFBalExitEvent					(__UrlLoaderGetVtbl()->ExitEvent)

#define		SFBalDebug						(__UrlLoaderGetVtbl()->Debug)

#define		SFBalGetUserAgent				(__UrlLoaderGetVtbl()->GetUserAgent)
#define		SFBalGetAppName					(__UrlLoaderGetVtbl()->GetAppName)
#define		SFBalGetAppVersion				(__UrlLoaderGetVtbl()->GetAppVersion)
#define		SFBalExtractNameFromURL			(__UrlLoaderGetVtbl()->ExtractNameFromURL)
#define		SFBalGetHostName				(__UrlLoaderGetVtbl()->GetHostName)
#define     SFBalFSWriteAutoFormFill		(__UrlLoaderGetVtbl()->BalFSWriteAutoFormFill)
#define 	SFBalRemoveEventByID			(__UrlLoaderGetVtbl()->BalRemoveEventByID)

#define SFBSysGetTick (__UrlLoaderGetVtbl()->SysGetTick)
#define SFBalIsLcdHorizontal (__UrlLoaderGetVtbl()->IsLcdHorizontal)

#define		SFBalGetDiagnoseInfo			(__UrlLoaderGetVtbl()->GetDiagnoseInfo)
#define		SFBalGetDebugLevel				(__UrlLoaderGetVtbl()->GetDebugLevel)
#define		SFBalSetEditFieldInputTypeCategory (__UrlLoaderGetVtbl()->SetEditFieldInputTypeCategory)


#define		SFBalSetEditFieldInputAlphabetMode (__UrlLoaderGetVtbl()->SetEditFieldInputAlphabetMode)
#define		SFBalGetEditFieldInputAlphabetMode (__UrlLoaderGetVtbl()->GetEditFieldInputAlphabetMode)
#define		SFBalGetEditAreaInputAlphabetMode (__UrlLoaderGetVtbl()->GetEditAreaInputAlphabetMode)


#define     SFBalGwesInitializeGraphic		(__UrlLoaderGetVtbl()->GwesInitializeGraphic)
#define		SFBalGwesFinalizeGraphic		(__UrlLoaderGetVtbl()->GwesFinalizeGraphic)
#if defined ( _SFBAL_SUPPORT_VECTOR_FONT )	
	#define		SFBalGwesGetHFontSreenInfo		(__UrlLoaderGetVtbl()->GwesGetHFontSreenInfo)
	#define		SFBalGwesSetFontAttr			(__UrlLoaderGetVtbl()->GwesSetFontAttr)
#endif 
#if defined ( MULTIPLE_INSTANCE_SUPPORT )	
	#define		SFBalInstanceExitEvent			(__UrlLoaderGetVtbl()->BalInstanceExitEvent)
#endif 

#define SFBalWsSetVeEnableCR					(__UrlLoaderGetVtbl()->BalWsSetVeEnableCR)

#endif 


/**
 * @struct 
 * 
 *  ECOM Type
 * @see        related information ( function name, module name, etc )
 */
typedef struct
{
	DECLARE_ECOM_VTBL(EUrlLoader)
	UINT32		refCount;
} TUrlLoader;

extern TUrlLoader gUrlLoader;

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif // _URLLOADER_ECOM_H_
#endif


