/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		IControl.h
 * @brief		Define interface IControl of WebControl
 * @author	Chaehwan Li (chaehwan.li@samsung.com)
 * @version	1.0
 * @date		2008.03.14	v1.0		Created
 */
#ifndef __IControl_Defined
#define __IControl_Defined
#if defined (_JASMINE_ECOM)
#include "Shp.h" // Zafar ECOM
#endif
#include "WebDefs.h"
#include "IContents.h"
#include "ISetting.h"
#include "ICertificate.h"
#include "IWML.h"
#include "ILoadGroup.h"



#ifndef IMAGESAVEINFO
#define IMAGESAVEINFO
	typedef struct
	{
		char* url;							/**<Url of the image. */
		char* name;						/**<Name of the image. */
		char* extn;
		const char* data;
		unsigned int size;
	} ImageSaveInfo;
#endif


namespace	WebControl
{

class CallBackHandler
{
private:
	typedef BOOL(*callJsAlert)		( const char * );
	typedef BOOL(*callJsConfirm)	( const char * );
	typedef BOOL(*callJsPrompt)		( const char * );
	typedef BOOL(*callWmlsAlert)	( const char * );
	typedef BOOL(*callWmlsConfirm)	( const char *, const char *, const char * );
	typedef BOOL(*callWmlsPrompt)	( const char *, const char * );
//SFB00001637 
	typedef BOOL(*callWmlAlert)	( const char * );



	callJsAlert		BrwCallJsAlert;
	callJsConfirm	BrwCallJsConfirm;
	callJsPrompt	BrwCallJsPrompt;
	callWmlsAlert	BrwCallWmlsAlert;
	callWmlsConfirm	BrwCallWmlsConfirm;
	callWmlsPrompt	BrwCallWmlsPrompt;
//SFB00001637 
	callWmlAlert	BrwCallWmlAlert;



public:
	CallBackHandler( callJsAlert JsAlert, callJsConfirm JsConfirm ,
						callJsPrompt JsPrompt, callWmlsAlert WmlsAlert,
						callWmlsConfirm WmlsConfirm, callWmlsPrompt WmlsPrompt	,callWmlAlert WmlAlert)

	{
		BrwCallJsAlert		= JsAlert;
		BrwCallJsConfirm	= JsConfirm;
		BrwCallJsPrompt		= JsPrompt;
		BrwCallWmlsAlert	= WmlsAlert;
		BrwCallWmlsConfirm	= WmlsConfirm;
		BrwCallWmlsPrompt	= WmlsPrompt;
//CQ 1637 and 3030: Partho
	if(WmlAlert)
		BrwCallWmlAlert	= WmlAlert ;
	else
		BrwCallWmlAlert	= WmlsAlert ;



	}
	~CallBackHandler() {} //Adding definition

	callJsAlert		getJsAlertFuncHandler()		{ return BrwCallJsAlert;	}
	callJsConfirm	getJsConfirmFuncHandler()	{ return BrwCallJsConfirm;	}
	callJsPrompt	getJsPromptFuncHandler()	{ return BrwCallJsPrompt;	}
	callWmlsAlert	getWmlsAlertFuncHandler()	{ return BrwCallWmlsAlert;	}
	callWmlsConfirm getWmlsConfirmFuncHandler()	{ return BrwCallWmlsConfirm; }
	callWmlsPrompt	getWmlsPromptFuncHandler()	{ return BrwCallWmlsPrompt; }
//SFB00001637 
	callWmlAlert	getWmlAlertFuncHandler()	{ return BrwCallWmlAlert;	}


};

class IControl : public IUnknown, public IContents
{

public:
	// Use getSetting/setSetting
	///*
	// * @desc sets a property of Browser Engine.
	// * @param in_PropertyID : Property ID to setting
	// * @param in_PropertyValue : pointer to Property Value to setting
	// * @return HRESULT TRUE / FALSE
	// */
	//virtual HRESULT setProperty( int in_PropertyID, void* in_PropertyValue ) = 0;

	///*
	// * @desc gets a property of Browser Engine.
	// * @param in_PropertyID : Property ID to setting
	// * @param out_PropertyValue : pointer to Property Value to setting
	// * @return HRESULT TRUE / FALSE
	// */
	//virtual HRESULT getProperty( int in_PropertyID, void* out_PropertyValue ) = 0;

	virtual HRESULT getImagesInfo(bool bVisibleOnly ,ImageSaveInfo*** imageList) = 0 ;

	virtual void getLastViewedImageUrl(char *) = 0;


	/*
	 * @desc sets account for proxy
	 * @param in_Account : account handle of Mocha
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT setAccessPoint( int in_Account ) = 0;

	virtual	URILoader::ILoadGroup*	getLoadGroup(void) = 0;

	virtual	void	setLoadGroup(URILoader::ILoadGroup* clinetID) = 0;

	virtual void setLcdDimmingMode(bool bIsDimming) = 0;

#if defined(JS_MULTIPLE_WINDOW_SUPPORT)/* Anjana - UI changes for JS New Window */
	virtual HRESULT setSecureMode( BOOL bMode ) = 0;

	virtual BOOL getSecureMode() = 0;
#endif

	/*
	 * @desc sets a request header property of Browser Engine.
	 * @param in_pszHttpHeader : pointer to the HttpHeader
	 * @param in_pszHeader : pointer to the HttpHeader Value
	 * @return HRESULT TRUE / FALSE
	 */
	// use ISetting
	//virtual HRESULT setHttpHeaderProperty( char* in_pszHttpHeader, char* in_pszHeaderBody ) = 0;

	/*
	 * @desc goes to forward or backward history
	 * @param 
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT goBackwardHistory( void ) = 0;


	/*
	 * @desc goes to forward or backward history
	 * @param 
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT goForwardHistory( void ) = 0;

	/**
	* @desc check if gobackward is possible
	* @return HRESULT TRUE / FALSE
	*/
	virtual HRESULT canGoBackwardHistory() = 0;

	/**
	* @desc check if goforward is possible
	* @return HRESULT TRUE / FALSE
	*/
	virtual HRESULT canGoForwardHistory() = 0;

	/*
	 * @desc clears history
	 * @param 
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT clearHistory( void ) = 0;

	/*
	 * @desc clears cache
	 * @param 
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT clearCache( void ) = 0;

	/*
	 * @desc Init cache
	 * @param 
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT InitCache( void ) = 0;

	/*
	 * @desc emptiess cookie
	 * @param 
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT emptyCookie( void ) = 0;

	/*
	 * @desc gets IWMLDo
	 * @param 
	 * @return pointer to the IWMLDo
	 */
	// removed
	//virtual  HRESULT getIWMLDo( IWMLDo** out_WMLDo ) = 0;

	/*
	 * @desc gets ICertificate
	 * @param 
	 * @return pointer to the ICertificate
	 */
	virtual  HRESULT getICertificate( ICertificate** out_Certificate ) = 0;

	/*
	 * @desc gets ISetting
	 * If clients does not set any specific ISetting, the default ISetting will be returned.
	 * The default ISetting is shared by all IControl instance.
	 * @param 
	 * @return pointer to the ISetting
	 */
	virtual  HRESULT getISetting( ISetting** out_Setting ) = 0;

	/*
	 * @desc sets ISetting
	 * @param 
	 * @return pointer to the ISetting
	 */
	virtual  HRESULT setISetting( ISetting* in_Setting ) = 0;

	/*
	 * @desc hides the Browsing window
	 * @param 
	 * @return pointer to the ISetting
	 */
	virtual HRESULT	 BrowsingWindowHide() = 0;

    virtual void AddURLToVisitedLinks(char* szURL) = 0;	

	
	/**
	 * @desc to get Smart Touch Enabled.
	 */
       virtual  bool isSmartTouchEnabled() = 0;

	

	/**
	 * @desc set the selected file path
	 * @param      in_szFilePath  [in] The file path
	 * @param      in_iPathLen  [in] The path length
	 */
	virtual HRESULT setSelectedFilePath( const char* in_szFilePath, int in_iPathLen) = 0;



#ifdef SFB_UPDATE_URL_BAR  //Pramod - Bresson Model - update Url ComboBox bar
	/* 
	 * @desc gets the current processing url
	 * @param  : None
	 * @return : char* - Url string
	 */
	virtual void getCurrentUrlFromHistoryList( char* szUrl ) = 0;
	virtual void getUrlFromHistoryList( int nDirect, char* szUrl ) = 0;/* Fix:CQ 2813:Anjana, check for forward/backward url available */
#endif
#ifdef SFB_DRAG_DROP
	/*
	 * @desc gets a current drag element rect.
	 * @return HRESULT TRUE/FALSE
	 */
        virtual HRESULT getCurrentDragElementRect(int*height, int*width,int* actualX,int* actualY, int x, int y,  int* ctElement, int* elementType )= 0;
	
        /*
	 * @desc gets a current drag element Info.
	 * @return HRESULT TRUE/FALSE
	 */
	virtual HRESULT getCurrentDragElementInfo(void* ctElement, char* srcUrl, char* hrefUrl, char** textString )= 0;
#endif



// [todo]
// need to check if these funtions are required
#if 0
	/***************************************************************************
	* For ACCESS
	****************************************************************************/
	/*
	 * @desc sets a application's code name for script.
	 * @param in_pszCodeName : pointer to application's code name ( ie : Mozilla )
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT setScriptAppCodeName( char* in_pszCodeName ) = 0;

	/*
	 * @desc sets a application's name for script.
	 * @param in_pszName : pointer to application's name ( ie : Samsung Browser, Netfront Browser )
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT setScriptAppName( char* in_pszName ) = 0;

	/*
	 * @desc sets a application's version for script.
	 * @param in_pszVersion : pointer to application's version ( ie : 1.0 )
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT setScriptAppVersion( char* in_pszVersion ) = 0;

	/*
	 * @desc sets a application's language for script.
	 * @param in_pszLanguage : pointer to application's language ( ie : en-US )
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT setScriptAppLanguage( char* in_pszLanguage ) = 0;

	/*
	 * @desc sets a application's platform for script.
	 * @param in_pszPlatform : pointer to application's platform ( ie : WIN32 )
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT setScriptAppPlatform( char* in_pszPlatform ) = 0;
#endif

	virtual void stopLingerTimerAfterAjaxRequest() = 0;

	public:
		CallBackHandler *pFuncHandler;
	
};

/*
 * @desc creates factory function to create IControl
 * @param in_pszDomainName : pointer to Domain ( it means mocha application ) name
 * @param in_ppIControl 
 * @return HRESULT TRUE / FALSE
*/
HRESULT CreateIControl( char* in_pszDomainName, 
					   HWIN hWin, 
					   int x, int y, int w, int h, 
					   IControl** in_ppIControl,
					   GCType gcType );


URILoader::ILoadGroup*	CreateClientLoadGroup(char* appName);

void	DestroyClientLoadGroup(URILoader::ILoadGroup* clinetID);

int GetWebControlCount();

} // namespace  WebControl

#if defined (_JASMINE_ECOM)
// I need to add some flag to make sure that it is included only for Jasmine Browser.
#include "WebControlECOM.h" // Zafar ECOM Since WebControl.h also includes this and this is the one included in BrCoreControl, I am preferring this.
#endif
#endif // __IControl_Defined