/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		IDecisionListner.h
 * @brief		Define interface IDecisionListner of WebControl
 * @author	Chaehwan Li (chaehwan.li@samsung.com)
 * @version	1.0
 * @date		2008.03.14	v1.0		Created
 */

#ifndef __IDecisionListner_Defined
#define __IDecisionListner_Defined

#include "WebDefs.h"
#include "IDecisionSender.h"

namespace	WebControl
{

class IDecisionListener
{
public:
	/*
	 * @desc listner function called when notification is requested by contents from browser engine
	 * @param in_trID transation ID // removed
	 * @param in_pszMimeType pointer to mimetype
	 * @param in_psHttpHeader pointer to http header
	 * @param in_HttpHeaderLen length of http header
	 * @param in_pIDecisionSender pointer to sender Interface
	 * @param in_isMainFrame true if the request is from a main frame
	 * @return bool TRUE / FALSE
	 */
	virtual HRESULT DecisionMimeListener( /*int in_trID,*/ char* in_pszMimeType, char* in_psHttpHeader, int in_HttpHeaderLen, IDecisionSender* in_pIDecisionSender, bool in_isMainFrame ) = 0;

	/*
	 * @desc listner function called when notification is requested by contents from browser engine
	 * @param in_pszUrl pointer to url
	 * @param in_pIDecisionSender pointer to sender Interface
	 * @return bool TRUE / FALSE
	 */
	virtual HRESULT DecisionUrlListener( char* in_pszUrl, IDecisionSender* in_pIDecisionSender ) = 0;

	/*
	 * @desc listner function called when notification is requested by contents from browser engine
	 * @param in_pNewWindowAction action key map
	 * @param in_pIDecisionSender pointer to sender Interface
	 * @return bool TRUE / FALSE
	 */
	virtual HRESULT DecisionNewWindowListener( NewWindowAction* in_pNewWindowAction, IDecisionSender* in_pIDecisionSender ) = 0;

};

} // namespace WebControl

#endif // __IDecisionListner_Defined