/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		WebDefs.h
 * @brief		Define interface IControl of WebControl
 * @author	Chaehwan Li (chaehwan.li@samsung.com)
 * @version	1.0
 * @date		2008.03.14	v1.0		Created
 */

#ifndef __WebDefs_Defined
#define __WebDefs_Defined

#ifdef STLPORT_ENABLED
#include "map"
#include "string"
#else
#include <map>
#include <string>
#endif

namespace WebControl
{
typedef int HWIN;
typedef signed int	HRESULT;
typedef signed int	DomNodeType;
typedef int ErrorCode;
// in target, STLPort is not available now
#ifdef STLPORT_ENABLED
typedef stlport::map< int, void* > NewWindowAction;
typedef stlport::map< int, void* > ElementProperty;
typedef stlport::string String;
#else
typedef std::map< int, void* > NewWindowAction;
typedef std::map< int, void* > ElementProperty;
typedef std::string String;
#endif

#undef BOOL;
typedef int BOOL;
typedef signed int ImgCodecType;
typedef signed int ConfirmType;
typedef signed int GCType;


/***************************************************************************
* For HRESULT of WebControl
****************************************************************************/

// Error
const signed int HRESULT_NOK = -6;
const signed int HRESULT_NOT_ALLOW = -5;
const signed int HRESULT_NOT_FOUND = -4;
const signed int HRESULT_INVALID_PARAM = -3;
const signed int HRESULT_NO_MEMORY = -2;
const signed int HRESULT_NOTSUPPORT = -1;
const signed int HRESULT_FALSE = 0;

// Success
const signed int HRESULT_TRUE = 1;
const signed int HRESULT_OK = 1;
const signed int HRESULT_ALLOW = 1;
const signed int HRESULT_FOUND = 1;

#define HRESULT_ISFAIL(x) (x <= 0)

/***************************************************************************
* For state of WebControl::Contents
****************************************************************************/
const signed int STATE_START = 1;
const signed int STATE_PROGRESS = 2;
const signed int STATE_FORMATTING = 3; // ACCESS
const signed int STATE_FINISH = 4;
const signed int STATECHANGE_SECURE = 5;
const signed int STATECHANGE_SECURE_INCLUSION = 6;

// For 3G Indicator - Sumdra
const signed int STATECHANGE_TRANSACTION = 7;
const signed int STATECHANGE_NO_TRANSACTION = 8;

/***************************************************************************
* Confirm Type of Listener::confirm
****************************************************************************/
const int CONFIRM_BASE = 300;
const int CONFIRM_CERTIFICATE = CONFIRM_BASE + 1;
const int CONFIRM_PDP_OTHERUSE = CONFIRM_BASE + 2;
const int CONFIRM_SMART_LINK = CONFIRM_BASE + 3;
const int HRESULT_URL_SCHEME_HANDLED = CONFIRM_BASE + 4;

/***************************************************************************
* ErrorCode of WebControl::Contents
****************************************************************************/
// etc. error
const int ERROR_ETC = 200;
const int ERROR_UNKNOWN = ERROR_ETC + 0;
const int ERROR_REQUEST_TIMEOUT = ERROR_ETC + 1;
const int ERROR_DNS_ERROR = ERROR_ETC + 2;
const int ERROR_TCP_ERROR = ERROR_ETC + 3; // socket related error
const int ERROR_SSL_ERROR = ERROR_ETC + 4;
const int ERROR_INVALID_URL = ERROR_ETC + 5;
const int ERROR_OUTOF_MEMORY = ERROR_ETC + 6;
const int ERROR_FILE_ERROR = ERROR_ETC + 7;
const int ERROR_COLLECTING_MEMORY = ERROR_ETC + 8;
const int ERROR_PROFILE_SETTING_ERROR = ERROR_ETC + 9;

// PDP Context error
const int ERROR_PROTO = 1;
const int ERROR_PROTO_NO_SERVICE = ERROR_PROTO + 1;		
const int ERROR_PROTO_NOT_SUBSCRIBED = ERROR_PROTO + 2;	
const int ERROR_PROTO_INVALID_APN = ERROR_PROTO + 3;
const int ERROR_PROTO_TIME_OUT = ERROR_PROTO + 4;
const int ERROR_PROTO_INVALID_PARAM = ERROR_PROTO + 5;
const int ERROR_PROTO_AUTH_FAIL = ERROR_PROTO + 6;
const int ERROR_PROTO_FDN_NOT_ALLOWED = ERROR_PROTO + 7; 
const int ERROR_PROTO_NOT_SUPPORT_3GCSD = ERROR_PROTO + 8; 
const int ERROR_PROTO_NETWORK_FAILURE = ERROR_PROTO + 9; 
const int ERROR_PROTO_UNKNOWN = ERROR_PROTO + 10; 
const int ERROR_PROTO_NETWORK_USED = ERROR_PROTO + 11; 
const int ERROR_PROTO_MAX_CONNECTED = ERROR_PROTO + 12;
const int ERROR_PROTO_NO_RESOURCE = ERROR_PROTO + 13;
const int ERROR_PROTO_MODEM_IN_USE_CANCEL = ERROR_PROTO + 14;

// HTTP Error- Gnana
const int ERROR_HTTPERR = 20;
const int ERROR_HTTPERR_INVALID_PARAM = ERROR_HTTPERR + 1;	
const int ERROR_HTTPERR_UNKNOWN = ERROR_HTTPERR + 2;			
const int ERROR_HTTPERR_INVALID_PROXY = ERROR_HTTPERR + 3;	
const int ERROR_HTTPERR_OUT_OF_MEMORY = ERROR_HTTPERR + 4;				
const int ERROR_HTTPERR_NO_RESPONSE = ERROR_HTTPERR + 5;				
const int ERROR_HTTPERR_INIT = ERROR_HTTPERR + 6;					
const int ERROR_HTTPERR_NETDOWN = ERROR_HTTPERR + 7;					
const int ERROR_HTTPERR_TIMEOUT = ERROR_HTTPERR + 8;				
const int ERROR_HTTPERR_HOST_UNREACH = ERROR_HTTPERR + 9;
const int ERROR_HTTPERR_CONN_RESET = ERROR_HTTPERR + 10;			
const int ERROR_HTTPERR_INTERNAL = ERROR_HTTPERR + 11;				
const int ERROR_HTTPERR_CHUNKEDTR = ERROR_HTTPERR + 12;				
const int ERROR_HTTPERR_USER = ERROR_HTTPERR + 13;				
const int ERROR_HTTPERR_TOO_BIG = ERROR_HTTPERR + 14;
const int ERROR_HTTPERR_NOT_SUPPORTED_SVC = ERROR_HTTPERR + 15;
const int ERROR_HTTPERR_NO_CONTENT = ERROR_HTTPERR + 16;		
const int ERROR_HTTPERR_WRITEFAIL = ERROR_HTTPERR + 17;				
const int ERROR_HTTPERR_AUTHFAIL = ERROR_HTTPERR + 18;
const int ERROR_HTTPERR_DNSFAIL = ERROR_HTTPERR + 19;				
const int ERROR_HTTPERR_UAGENT_NOT_ALLOWED = ERROR_HTTPERR + 20;

//-------------

const int ERROR_NOCONTENT = 50;
const int ERROR_NOCONTENT_DEFAULT = ERROR_NOCONTENT + 0;
const int ERROR_NOCONTENT_HTTP400 = ERROR_NOCONTENT + 1;
const int ERROR_NOCONTENT_HTTP401 = ERROR_NOCONTENT + 2;
const int ERROR_NOCONTENT_HTTP402 = ERROR_NOCONTENT + 3;
const int ERROR_NOCONTENT_HTTP403= ERROR_NOCONTENT + 4;
const int ERROR_NOCONTENT_HTTP404= ERROR_NOCONTENT + 5;
const int ERROR_NOCONTENT_HTTP405= ERROR_NOCONTENT + 6;
const int ERROR_NOCONTENT_HTTP406= ERROR_NOCONTENT + 7;
const int ERROR_NOCONTENT_HTTP407= ERROR_NOCONTENT + 8;
const int ERROR_NOCONTENT_HTTP408= ERROR_NOCONTENT + 9;
const int ERROR_NOCONTENT_HTTP409= ERROR_NOCONTENT + 10;
const int ERROR_NOCONTENT_HTTP410= ERROR_NOCONTENT + 11;
const int ERROR_NOCONTENT_HTTP411= ERROR_NOCONTENT + 12;
const int ERROR_NOCONTENT_HTTP412= ERROR_NOCONTENT + 13;
const int ERROR_NOCONTENT_HTTP413= ERROR_NOCONTENT + 14;
const int ERROR_NOCONTENT_HTTP414= ERROR_NOCONTENT + 15;
const int ERROR_NOCONTENT_HTTP415= ERROR_NOCONTENT + 16;
const int ERROR_NOCONTENT_HTTP500= ERROR_NOCONTENT + 17;
const int ERROR_NOCONTENT_HTTP501= ERROR_NOCONTENT + 18;
const int ERROR_NOCONTENT_HTTP502= ERROR_NOCONTENT + 19;
const int ERROR_NOCONTENT_HTTP503= ERROR_NOCONTENT + 20;
const int ERROR_NOCONTENT_HTTP504= ERROR_NOCONTENT + 21;
const int ERROR_NOCONTENT_HTTP505= ERROR_NOCONTENT + 22;

//Download Errors
const int ERROR_CONTENTDOWNLOAD_UNSUPPORTED_MIMETYPE = 42;

//Auth Error- gnana
const int ERROR_AUTH_INVALID_FORMAT = 41;

/***************************************************************************
* image codec
****************************************************************************/
const signed int IMG_CODEC_GIF = 1;
const signed int IMG_CODEC_JPEG = 2;
const signed int IMG_CODEC_PNG = 3;
const signed int IMG_CODEC_BMP = 4;
const signed int IMG_CODEC_WBMP = 5;
const signed int IMG_CODEC_TIFF = 6;
const signed int IMG_CODEC_UNKNOWN = 7; 

/***************************************************************************
* cookie policy
****************************************************************************/
const signed int COOKIE_POLICY_ACCEPT = 1;
const signed int COOKIE_POLICY_REJECT = 2;
const signed int COOKIE_POLICY_PROMPT = 3;

/***************************************************************************
* Page Rendering mode
****************************************************************************/
const signed int STANDARD_RENDERING_MODE = 1;
const signed int SMART_WIDTH_RENDERING_MODE = 2;

/***************************************************************************
* Page View mode
****************************************************************************/
const signed int STANDARD_VIEW_MODE = 1;
const signed int MAP_VIEW_MODE = 2;

/***************************************************************************
* For Window Action Key
****************************************************************************/
const signed int ACTION_ELEMENT_PROPERTY_KEY = 1; // IElementProperty
const signed int ACTION_URL_KEY = 2; // IURI

/***************************************************************************
* IME WINDOW TYPE
****************************************************************************/
const unsigned short	EDITFIELD_IME_WINDOW = 1;
const unsigned short	EDITAREA_IME_WINDOW = 2;

/***************************************************************************
* IME Usage
****************************************************************************/
const unsigned short	NONE_IME_USAGE = 1;
const unsigned short	URL_IME_USAGE = 2;
const unsigned short	ALPHA_IME_USAGE = 3;
const unsigned short	NUM_IME_USAGE = 4;

/***************************************************************************
* for DOM NODE TYPE
****************************************************************************/
const unsigned short	UNKNOWN_NODE       = 0;
const unsigned short	ELEMENT_NODE       = 1;
const unsigned short	ATTRIBUTE_NODE     = 2;
const unsigned short	TEXT_NODE          = 3;
const unsigned short	CDATA_SECTION_NODE = 4;
const unsigned short	ENTITY_REFERENCE_NODE = 5;
const unsigned short	ENTITY_NODE        = 6;
const unsigned short	PROCESSING_INSTRUCTION_NODE = 7;
const unsigned short	COMMENT_NODE       = 8;
const unsigned short	DOCUMENT_NODE      = 9;
const unsigned short	DOCUMENT_TYPE_NODE = 10;
const unsigned short	DOCUMENT_FRAGMENT_NODE = 11;
const unsigned short	NOTATION_NODE      = 12;

/***************************************************************************
* For ElementProperty Key
****************************************************************************/
const signed int ELEMENT_PROPERTY_DOM_NODE_KEY = 1;
const signed int ELEMENT_PROPERTY_IMAGE_KEY = 2;

/***************************************************************************
* for DRAG and DROP Element TYPE
****************************************************************************/
const unsigned int DRAG_DROP_NIL = 0;
const unsigned int DRAG_DROP_IMAGE = 1;
const unsigned int DRAG_DROP_LINK = 2;
const unsigned int DRAG_DROP_TEXT = 3;
const unsigned int DRAG_DROP_LINK_TEXT = 4;
const unsigned int DRAG_DROP_LINK_IMAGE = 5;

/***************************************************************************
* for Property of WebControl
****************************************************************************/
// INT  is int type
// STR  is WebControl::String type
const unsigned int	PROPERTY_INT_BASE = 100;
const unsigned int	PROPERTY_STR_BASE = 4000;


const unsigned int	PROPERTY_CACHE_INT_BASE = PROPERTY_INT_BASE;
const unsigned int	PROPERTY_COOKIE_INT_BASE = PROPERTY_INT_BASE + 100;
const unsigned int	PROPERTY_SCRIPT_INT_BASE = PROPERTY_INT_BASE + 200;
const unsigned int	PROPERTY_SWIFTNAVI_INT_BASE = PROPERTY_INT_BASE + 300;
const unsigned int	PROPERTY_RENDERING_MODE_INT_BASE = PROPERTY_INT_BASE + 400;
const unsigned int	PROPERTY_VIEW_MODE_INT_BASE = PROPERTY_INT_BASE + 500;
const unsigned int	PROPERTY_VIRTUALPOINT_MODE_INT_BASE = PROPERTY_INT_BASE + 600;
const unsigned int	PROPERTY_CONTENTS_INT_BASE = PROPERTY_INT_BASE + 700;
const unsigned int	PROPERTY_HTTPHEADER_INT_BASE = PROPERTY_INT_BASE + 800;
const unsigned int	PROPERTY_DOWNLOAD_INT_BASE = PROPERTY_INT_BASE + 900;
const unsigned int	PROPERTY_SECURITY_INT_BASE = PROPERTY_INT_BASE + 1000;
const unsigned int	PROPERTY_MEMORY_INT_BASE = PROPERTY_INT_BASE + 1100;
const unsigned int	PROPERTY_FONT_INT_BASE = PROPERTY_INT_BASE + 1200;
const unsigned int	PROPERTY_HTTP_INT_BASE = PROPERTY_INT_BASE + 1300;



const unsigned int	PROPERTY_RENDERING_MODE_STR_BASE = PROPERTY_STR_BASE;
const unsigned int	PROPERTY_SWIFTNAVI_STR_BASE = PROPERTY_STR_BASE + 100;
const unsigned int	PROPERTY_SCRIPT_STR_BASE = PROPERTY_STR_BASE + 200;
const unsigned int	PROPERTY_COOKIE_STR_BASE = PROPERTY_STR_BASE + 300;
const unsigned int	PROPERTY_CACHE_STR_BASE = PROPERTY_STR_BASE + 400;
const unsigned int	PROPERTY_VIEW_MODE_STR_BASE = PROPERTY_STR_BASE + 500;
const unsigned int	PROPERTY_VIRTUALPOINT_MODE_STR_BASE = PROPERTY_STR_BASE + 600;
const unsigned int	PROPERTY_CONTENTS_STR_BASE = PROPERTY_STR_BASE + 700;
const unsigned int	PROPERTY_HTTP_STR_BASE = PROPERTY_STR_BASE + 800;
const unsigned int	PROPERTY_HTTPHEADER_STR_BASE = PROPERTY_STR_BASE + 900;
const unsigned int	PROPERTY_DOWNLOAD_STR_BASE = PROPERTY_STR_BASE + 1000;
const unsigned int	PROPERTY_SECURITY_STR_BASE = PROPERTY_STR_BASE + 1100;
const unsigned int	PROPERTY_MEMORY_STR_BASE = PROPERTY_STR_BASE + 1200;
const unsigned int	PROPERTY_FONT_STR_BASE = PROPERTY_STR_BASE + 1300;






	/***************************************************************************
	* for Cache
	****************************************************************************/
const unsigned int	CACHE_L1CONTENTSIZE = PROPERTY_CACHE_INT_BASE + 1;
const unsigned int	CACHE_L1ENTRIES = PROPERTY_CACHE_INT_BASE + 2;
const unsigned int	CACHE_L1TOTALSIZE = PROPERTY_CACHE_INT_BASE + 3;
const unsigned int	CACHE_L2CONTENTSIZE = PROPERTY_CACHE_INT_BASE + 4;
const unsigned int	CACHE_L2ENTRIES = PROPERTY_CACHE_INT_BASE + 5;
const unsigned int	CACHE_L2TOTALSIZE = PROPERTY_CACHE_INT_BASE + 6;
const unsigned int	CACHE_L2DIR = PROPERTY_CACHE_INT_BASE + 7;
const unsigned int	PAGESAVE_ON_CACHECONTROL_NOSTORE = PROPERTY_CACHE_INT_BASE + 8;
const unsigned int	CACHE_EXPIRE_PERIOD = PROPERTY_CACHE_INT_BASE + 9;
const unsigned int	L2CACHESYNC_FILES = PROPERTY_CACHE_INT_BASE + 10;
const unsigned int	CACHE_MODE = PROPERTY_CACHE_INT_BASE + 11;

	/***************************************************************************
	* for Cookie
	****************************************************************************/
const unsigned int	COOKIE_ALLOW = PROPERTY_COOKIE_INT_BASE + 1;
const unsigned int	COOKIE_MAX_TOTAL_COOKIES = PROPERTY_COOKIE_INT_BASE + 2;
const unsigned int	COOKIE_MAX_COOKIES_PER_DOMAIN = PROPERTY_COOKIE_INT_BASE + 3;
const unsigned int	COOKIE_MAX_LEN_PER_COOKIE = PROPERTY_COOKIE_INT_BASE + 4;
const unsigned int	COOKIE_ENTRIES_FILENAME = PROPERTY_COOKIE_INT_BASE + 5;
const unsigned int	SESSION_COOKIE_ENTRIES_FILENAME = PROPERTY_COOKIE_INT_BASE + 6;
const unsigned int	EXT_DNLD_COOKIE_ENTRIES_FILENAME = PROPERTY_COOKIE_INT_BASE + 7;
const unsigned int	COOKIE_TYPE = PROPERTY_COOKIE_INT_BASE + 8;
const unsigned int	COOKIE_POLICY = PROPERTY_COOKIE_INT_BASE + 9;

	/***************************************************************************
	* for HTTP
	****************************************************************************/
const unsigned int	HTTP_MAX_STREAMS = PROPERTY_HTTP_INT_BASE + 1;
const unsigned int	TCP_MAX_STREAMS = PROPERTY_HTTP_INT_BASE + 2;
const unsigned int	MAX_PIPELINES = PROPERTY_HTTP_INT_BASE + 3;
const unsigned int	HTTP11_PIPELINE = PROPERTY_HTTP_INT_BASE + 4;
const unsigned int	SEND_PROXY_KEEP_ALIVE = PROPERTY_HTTP_INT_BASE + 5;
const unsigned int	HTTP_KEEP_ALIVE_TIMEOUT = PROPERTY_HTTP_INT_BASE + 6;

const unsigned int	HTTP_USERAGENT = PROPERTY_HTTP_STR_BASE + 7;

	/***************************************************************************
	* for Script
	****************************************************************************/
const unsigned int	JAVASCRIPT_ENABLE = PROPERTY_SCRIPT_INT_BASE + 1;
const unsigned int	DISABLE_JS_SCHEME = PROPERTY_SCRIPT_INT_BASE + 2;

	/***************************************************************************
	* for Contents
	****************************************************************************/
const unsigned int	WORDWRAP = PROPERTY_CONTENTS_INT_BASE + 1;
const unsigned int	SCROLLSIZE = PROPERTY_CONTENTS_INT_BASE + 2;
const unsigned int	PAGESCROLLSIZE = PROPERTY_CONTENTS_INT_BASE + 3;
const unsigned int	ENABLE_FILE_SCHEME = PROPERTY_CONTENTS_INT_BASE + 4;
const unsigned int	ENABLE_BG_SOUND = PROPERTY_CONTENTS_INT_BASE + 5;
const unsigned int	WEBDATA_FILENAME = PROPERTY_CONTENTS_INT_BASE + 6;
const unsigned int	AUTHINFO_FILENAME = PROPERTY_CONTENTS_INT_BASE + 7;
const unsigned int	FOCUSOUTLINE_COLOR = PROPERTY_CONTENTS_INT_BASE + 8;
const unsigned int	FOCUSOUTLINE_TYPE = PROPERTY_CONTENTS_INT_BASE + 9;
const unsigned int	FOCUSOUTLINE_WIDTH = PROPERTY_CONTENTS_INT_BASE + 10;
const unsigned int	VIRTUALCANVAS_WIDTH = PROPERTY_CONTENTS_INT_BASE + 11;
const unsigned int	VIRTUALCANVAS_HEIGHT = PROPERTY_CONTENTS_INT_BASE + 12;
const unsigned int	MAX_PAGE_AUTH = PROPERTY_CONTENTS_INT_BASE + 13;
const unsigned int	MARQUEE_SPEED_FAST = PROPERTY_CONTENTS_INT_BASE + 14;
const unsigned int	MARQUEE_SPEED_NORMAL = PROPERTY_CONTENTS_INT_BASE + 15;
const unsigned int	MARQUEE_SPEED_SLOW = PROPERTY_CONTENTS_INT_BASE + 16;
const unsigned int	KEEP_AUTHINFO = PROPERTY_CONTENTS_INT_BASE + 17;
const unsigned int	DEFAULTSTYLESHEET = PROPERTY_CONTENTS_INT_BASE + 18;
const unsigned int	USERSTYLESHEET = PROPERTY_CONTENTS_INT_BASE + 19;
const unsigned int	MAXSTAYTIME_CONTENTPARSER = PROPERTY_CONTENTS_INT_BASE + 20;
const unsigned int	MAXSTAYTIME_PAGEMAKER = PROPERTY_CONTENTS_INT_BASE + 21;
const unsigned int	MAXSTAYTIME_EDITOR_BG = PROPERTY_CONTENTS_INT_BASE + 22;
const unsigned int	MAXSTAYTIME_EDITOR_FG = PROPERTY_CONTENTS_INT_BASE + 23;
const unsigned int	AUTOLOAD_IMAGEBOUND = PROPERTY_CONTENTS_INT_BASE + 24;
const unsigned int	MAXVISITS = PROPERTY_CONTENTS_INT_BASE + 25;
const unsigned int	MAXREDIRECT = PROPERTY_CONTENTS_INT_BASE + 26;
const unsigned int	MULTIPLESELECT_DEFAULTSIZE = PROPERTY_CONTENTS_INT_BASE + 27;
const unsigned int	IE_COMPATIBLE_REDIRECT = PROPERTY_CONTENTS_INT_BASE + 28;
const unsigned int	MAX_REFRESH_COUNT = PROPERTY_CONTENTS_INT_BASE + 29;
const unsigned int	WML_MAX_LOOP_COUNT = PROPERTY_CONTENTS_INT_BASE + 30;
const unsigned int	AUTOFOCUS_USING_TABINDEX1 = PROPERTY_CONTENTS_INT_BASE + 31;
const unsigned int	RSS = PROPERTY_CONTENTS_INT_BASE + 32; // ACCESS
const unsigned int	MAXHISTORIES = PROPERTY_CONTENTS_INT_BASE + 33;
const unsigned int	CYCLIC_FOCUS = PROPERTY_CONTENTS_INT_BASE + 34;
const unsigned int	MAXSTAYTIME_IMAGEDECODER = PROPERTY_CONTENTS_INT_BASE + 35; // ACCESS
const unsigned int	FOCUSNAVIGATION = PROPERTY_CONTENTS_INT_BASE + 36;
const unsigned int	MAX_TOTAL_CONTENTSIZE = PROPERTY_CONTENTS_INT_BASE + 37;
const unsigned int	MAXSTAYTIME_ALL = PROPERTY_CONTENTS_INT_BASE + 38; // ACCESS
const unsigned int	HANDLECMD_REQUESTCONNECTION = PROPERTY_CONTENTS_INT_BASE + 39; // ACCESS
const unsigned int	HANDLECMD_ACCEPTCOOKIE = PROPERTY_CONTENTS_INT_BASE + 40; // ACCESS
const unsigned int	HANDLECMD_REDIRECT = PROPERTY_CONTENTS_INT_BASE + 41; // ACCESS
const unsigned int	HANDLECMD_REQUESTSERVERCERTCONFIRM = PROPERTY_CONTENTS_INT_BASE + 42; // ACCESS
const unsigned int	ENABLE_HELPER_BY_FILE_SCHEME = PROPERTY_CONTENTS_INT_BASE + 43;
const unsigned int	CONNECT_IN_PAGELOAD = PROPERTY_CONTENTS_INT_BASE + 44; // ACCESS
const unsigned int	CONNECT_IN_CHANGEPROPERTY = PROPERTY_CONTENTS_INT_BASE + 45; // ACCESS
const unsigned int	WML_TEXTBOX_DEFAULTSIZE = PROPERTY_CONTENTS_INT_BASE + 46;
const unsigned int	REPLACE_SPLASH_FILENAME = PROPERTY_CONTENTS_INT_BASE + 47;
const unsigned int	ANCHORCOLOR = PROPERTY_CONTENTS_INT_BASE + 48;
const unsigned int	TOOLTIP_LONGLABEL = PROPERTY_CONTENTS_INT_BASE + 49; // ACCESS
const unsigned int	ENABLE_IME_WHILE_PAGELOADING = PROPERTY_CONTENTS_INT_BASE + 50; // ACCESS
const unsigned int	DOCWRITE_IN_XHTML = PROPERTY_CONTENTS_INT_BASE + 51; // ACCESS
const unsigned int	DOCUMENT_SHRINK_STYLE = PROPERTY_CONTENTS_INT_BASE + 52; // ACCESS
const unsigned int	NOT_USING_COMPO_FOCUS = PROPERTY_CONTENTS_INT_BASE + 53; // ACCESS
const unsigned int	AJUST_GAP_OF_TEXTLINE = PROPERTY_CONTENTS_INT_BASE + 54;
const unsigned int	NEED_IGNORE_SCROLLBAR_INROOTPANE = PROPERTY_CONTENTS_INT_BASE + 55; // ACCESS
const unsigned int	MAXSTAYTIME_JSPARSER = PROPERTY_CONTENTS_INT_BASE + 56; // ACCESS
const unsigned int	OVERRIDE_PANEHANDLENEXTCONTENT = PROPERTY_CONTENTS_INT_BASE + 57; // ACCESS
const unsigned int	DISPLAYIMAGE = PROPERTY_CONTENTS_INT_BASE + 58;
const unsigned int	SHOULD_PRINT_BACKGROUND = PROPERTY_CONTENTS_INT_BASE + 59;




const unsigned int LOADS_IMAGE_AUTOMATICALLY = PROPERTY_CONTENTS_INT_BASE +59;
const unsigned int JAVASCRIPT_ENABLED = PROPERTY_CONTENTS_INT_BASE +60;
const unsigned int JAVASCRIPT_CAN_OPENWINDOW_AUTOMATICALLY = PROPERTY_CONTENTS_INT_BASE +61;
const unsigned int JAVA_ENABLED = PROPERTY_CONTENTS_INT_BASE +62;
const unsigned int PLUGIN_ENABLED = PROPERTY_CONTENTS_INT_BASE +63;
const unsigned int PAGE_OVERVIEW_ENABLED = PROPERTY_CONTENTS_INT_BASE +64;

const unsigned int DEFAULT_TEXT_ENCODING_NAME = PROPERTY_CONTENTS_STR_BASE +1;
const unsigned int USER_STYLESHEET_LOCATION = PROPERTY_CONTENTS_STR_BASE +2;


	/***************************************************************************
	* for SwiftNavi ( For ACCESS )
	****************************************************************************/
const unsigned int	SWIFTNAVI_FIRSTVELOCITY = PROPERTY_SWIFTNAVI_INT_BASE + 1;
const unsigned int	SWIFTNAVI_FRICTION = PROPERTY_SWIFTNAVI_INT_BASE + 2;
const unsigned int	SWIFTNAVI_PROCESSCYCLE = PROPERTY_SWIFTNAVI_INT_BASE + 3;

	/***************************************************************************
	* for Rendering Mode
	****************************************************************************/
const unsigned int	SFRFORVFG = PROPERTY_RENDERING_MODE_INT_BASE + 1; // for ACCESS
const unsigned int	SMARTFIT_MINIMAGEWIDTH = PROPERTY_RENDERING_MODE_INT_BASE + 2;
const unsigned int	SMARTFIT_MINIMAGEHEIGHT = PROPERTY_RENDERING_MODE_INT_BASE + 3;
const unsigned int	SMARTFIT_DISPLAY_BGIMAGE = PROPERTY_RENDERING_MODE_INT_BASE + 4;
const unsigned int	COLUMN_RENDERING = PROPERTY_RENDERING_MODE_INT_BASE + 5;

	/***************************************************************************
	* for View Mode
	****************************************************************************/
const unsigned int	PAGEPILOT_WINDOWZOOM = PROPERTY_VIEW_MODE_INT_BASE + 1;
const unsigned int	PAGEPILOT_MINWIDTHSCALE = PROPERTY_VIEW_MODE_INT_BASE + 2;
const unsigned int	PAGEPILOT_MINHEIGHTSCALE = PROPERTY_VIEW_MODE_INT_BASE + 3;
const unsigned int	PAGEPILOT_MARGIN_TOP = PROPERTY_VIEW_MODE_INT_BASE + 4;
const unsigned int	PAGEPILOT_MARGIN_BOTTOM = PROPERTY_VIEW_MODE_INT_BASE + 5;
const unsigned int	PAGEPILOT_MARGIN_LEFT = PROPERTY_VIEW_MODE_INT_BASE + 6;
const unsigned int	PAGEPILOT_MARGIN_RIGHT = PROPERTY_VIEW_MODE_INT_BASE + 7;
const unsigned int	PAGEPILOT_BACKGROUNDCOLOR = PROPERTY_VIEW_MODE_INT_BASE + 8;
const unsigned int	PAGEPILOT_SHADECOLOR = PROPERTY_VIEW_MODE_INT_BASE + 9;
const unsigned int	PAGEPILOT_FLOAT_X = PROPERTY_VIEW_MODE_INT_BASE + 10;
const unsigned int	PAGEPILOT_FLOAT_Y = PROPERTY_VIEW_MODE_INT_BASE + 11;
const unsigned int	PAGEPILOT_FRAMECOLOR = PROPERTY_VIEW_MODE_INT_BASE + 12;
const unsigned int	PAGEPILOT_FRAMESHADECOLOR = PROPERTY_VIEW_MODE_INT_BASE + 13;
const unsigned int	PAGEPILOT_FRAMEFLOAT_X = PROPERTY_VIEW_MODE_INT_BASE + 14;
const unsigned int	PAGEPILOT_FRAMEFLOAT_Y = PROPERTY_VIEW_MODE_INT_BASE + 15;
const unsigned int	PAGEPILOT_FRAMEMOVE_X = PROPERTY_VIEW_MODE_INT_BASE + 16;
const unsigned int	PAGEPILOT_FRAMEMOVE_Y = PROPERTY_VIEW_MODE_INT_BASE + 17;
const unsigned int	PAGEPILOT_FRAMETHICKNESS = PROPERTY_VIEW_MODE_INT_BASE + 18;
const unsigned int	PAGEPILOT_FONT_MIN_SIZE_THRESHOLD = PROPERTY_VIEW_MODE_INT_BASE + 19;
const unsigned int	PAGEPILOT_FRAMEWHEELMOVE = PROPERTY_VIEW_MODE_INT_BASE + 20;

	/***************************************************************************
	* for Virtual Point Mode
	****************************************************************************/
const unsigned int	VP_MOVEAMOUNT = PROPERTY_VIRTUALPOINT_MODE_INT_BASE + 1;
const unsigned int	VP_ENABLE_CONTEXTMENU = PROPERTY_VIRTUALPOINT_MODE_INT_BASE + 2;
const unsigned int	VP_DENOMINATORPAGESCROLL = PROPERTY_VIRTUALPOINT_MODE_INT_BASE + 3;
const unsigned int	POINTER_MODE = PROPERTY_VIRTUALPOINT_MODE_INT_BASE + 4;
const unsigned int	FINGER_POINTING = PROPERTY_VIRTUALPOINT_MODE_INT_BASE + 5;
const unsigned int	FINGER_POSITION_DELTA = PROPERTY_VIRTUALPOINT_MODE_INT_BASE + 6;
const unsigned int	FINGER_POINTING_HELPER = PROPERTY_VIRTUALPOINT_MODE_INT_BASE + 7;
const unsigned int	FINGER_POINTING_THUMBNAIL_WIDTH = PROPERTY_VIRTUALPOINT_MODE_INT_BASE + 8;
const unsigned int	FINGER_POINTING_THUMBNAIL_HEIGHT = PROPERTY_VIRTUALPOINT_MODE_INT_BASE + 9;
const unsigned int	VP_ENABLED = PROPERTY_VIRTUALPOINT_MODE_INT_BASE + 10;

	/***************************************************************************
	* for Download
	****************************************************************************/
const unsigned int	EXTDOWNLOAD_BUFFER_SIZE = PROPERTY_DOWNLOAD_INT_BASE + 1;
const unsigned int	EXTDOWNLOAD_SLEEP_TIME = PROPERTY_DOWNLOAD_INT_BASE + 2;
const unsigned int	EXTDOWNLOAD_KEEP_AUTHINFO = PROPERTY_DOWNLOAD_INT_BASE + 3;
const unsigned int	ALINKMULTIPLEDOWNLOAD = PROPERTY_DOWNLOAD_INT_BASE + 4;

	/***************************************************************************
	* for HTTP Header
	****************************************************************************/
const unsigned int	AUTO_DETECT_LANGSET = PROPERTY_HTTPHEADER_INT_BASE + 1;
const unsigned int	UA_PROF = PROPERTY_HTTPHEADER_INT_BASE + 2;
const unsigned int	DEFAULT_HTTP_HEADERS = PROPERTY_HTTPHEADER_INT_BASE + 3;
const unsigned int	CURRENT_LANGSET = PROPERTY_HTTPHEADER_INT_BASE + 4;
const unsigned int	CURRENT_CHARSET = PROPERTY_HTTPHEADER_INT_BASE + 5;
const unsigned int	NAVIGATOR_APPCODENAME = PROPERTY_HTTPHEADER_INT_BASE + 6;
const unsigned int	NAVIGATOR_APPNAME = PROPERTY_HTTPHEADER_INT_BASE + 7;
const unsigned int	NAVIGATOR_APPVERSION = PROPERTY_HTTPHEADER_INT_BASE + 8;
const unsigned int	NAVIGATOR_LANGUAGE = PROPERTY_HTTPHEADER_INT_BASE + 9;
const unsigned int	NAVIGATOR_PLATFORM = PROPERTY_HTTPHEADER_INT_BASE + 10;
const unsigned int	HTTP_BYTE_RANGE_REQUEST = PROPERTY_HTTPHEADER_INT_BASE + 11;

	/***************************************************************************
	* for Security
	****************************************************************************/
const unsigned int	SSL_VERSION = PROPERTY_SECURITY_INT_BASE + 1;
const unsigned int	MIXEDSECURE_POLICY = PROPERTY_SECURITY_INT_BASE + 2;

	/***************************************************************************
	* for Memory
	****************************************************************************/
const unsigned int	TANKMEMORY_AUTOEXTEND = PROPERTY_MEMORY_INT_BASE + 1; 
const unsigned int	TANKMEMORY_MINSUBTANKBYTES = PROPERTY_MEMORY_INT_BASE + 2;
const unsigned int	TANKMEMORY_MAXTOTALBYTES = PROPERTY_MEMORY_INT_BASE + 3;
const unsigned int	MEMORYCRISIS_RESERVE = PROPERTY_MEMORY_INT_BASE + 4;


	/***************************************************************************
	* for FONT
	****************************************************************************/
const unsigned int MINIMUM_FONT_SIZE = PROPERTY_FONT_INT_BASE +1;
const unsigned int MINIMUM_LOGICAL_FONT_SIZE = PROPERTY_FONT_INT_BASE +2;
const unsigned int DEFAULT_FONT_SIZE = PROPERTY_FONT_INT_BASE +3;
const unsigned int DEFAULT_FIXED_FONT_SIZE = PROPERTY_FONT_INT_BASE +4;

const unsigned int STANDARD_FONT_FAMILY = PROPERTY_FONT_STR_BASE +1;
const unsigned int FIXED_FONT_FAMILY = PROPERTY_FONT_STR_BASE +2;
const unsigned int SERIF_FONT_FAMILY = PROPERTY_FONT_STR_BASE +3;
const unsigned int SANSSERIF_FONT_FAMILY = PROPERTY_FONT_STR_BASE +4;
const unsigned int CURSIVE_FONT_FAMILY = PROPERTY_FONT_STR_BASE +5;
const unsigned int FANTASY_FONT_FAMILY = PROPERTY_FONT_STR_BASE +6;


	/***************************************************************************
	* for Color format of GC
	****************************************************************************/
const int BIT_16_GC = 0;
const int BIT_32_GC = 1;




        













} // namespace WebControl

#endif // __WebDefs_Defined
