/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics.
 */
/**
 * @file		WebBackingStore.h
 * @brief		
 * @author	Sangheon Kim (sangheon77.kim@samsung.com) @n
					
 * @version	1.0
 * @date		2008.07.23	v1.0		Created							shkim @n
 */

#ifndef WebBackingStore_Defined
#define WebBackingStore_Defined

#include "WebControlConfig.h"
#include "IntRect.h"
#include "IntPoint.h"
#include "FloatSize.h"
#include "GWES.h"
#include "Vector.h"
#include "HistoryItem.h"
#include "ScrollBar.h"
#include "PlatformScrollbar.h"

//#define _BACKINGSTORE_DEBUG
//#define _ADDTODIRTYREGION_DEBUG
#if defined(_SFB_PERF_LOG)
#define _PERFORMANCE_DEBUG
#endif
#define DYNAMIC_SCROLLBAR

#define WEB_TOTAL_PLATFORM_HEAP_SIZE						MemGetStat()->totalHeapSize
#define WEB_CURRENTLY_ALLOCATED_PLATFORM_HEAP_SIZE			MemGetStat()->currentlyAllocatedMemSize
#define WEB_MAX_MEM_USAGE									0.85 /*if memory usage crosses 85 % then throw OOM */


namespace WebControl
{

class WebControl;

class WebBackingStore : WebCore::ScrollbarClient
{
public:
	WebBackingStore( WebControl *pWebControl, int id );
	~WebBackingStore();

	void AddToDirtyRegion(const WebCore::IntRect& rect);
	void AddToPaintRegion( const WebCore::IntRect & rect );
	void ScrollBackingStore( int dx, int dy, const WebCore::IntRect& scrollViewRect, const WebCore::IntRect& clipRect);
	void UpdateBackingStore( bool bCompleteDirty );
	bool ValidateBackingStore();
	void PaintToWindow( int x, int y, int w, int h, bool bDraw, bool bShow );
	bool AdjustInitialScale(bool bCheckCondition = true );
	void PageLoadStarted();
	void PageLoadFinished();
	void RotateBackingStore( int w, int h );
	void ScrollBy( int dx, int dy );
	WebCore::IntPoint WindowToBackingStore( const WebCore::IntPoint& point) ;
	WebCore::IntPoint BackingStoreToWindow( const WebCore::IntPoint& point) ;
	void SetBackingStoreMultiplier( float multiplier );
	float GetBackingStoreMultiplier();
	void Zooming( float newMultiplier, WebCore::IntPoint centerPoint = WebCore::IntPoint(-1,-1) );
	void Zoom( float multiplier, WebCore::IntPoint center = WebCore::IntPoint(-1,-1) );
	float GetZoom();
	void ResetViewState();
	void SaveViewState(WebCore::HistoryItem* item);
	void RestoreViewState();
	WebCore::IntPoint GetViewPoint();
	int GetBackingStoreGC() { return m_backingStoreGC; }
	void DispatchDidFirstLayout();
	void DispatchDidLoadMainResource(WebCore::DocumentLoader*) ;
	void Paint();
	void PaintScrollbar( int hGC , bool bShow);
	WebCore::IntRect GetViewRect();
	// bs0425.lee modified for touch scroll. 
	WebCore::IntRect getBackingScrollbarData(const WebCore::IntPoint& mousePosition );
	WebCore::IntRect getBackingTrackRect() const;
	void SetPageOverview( const WebCore::IntPoint & centerpoint );
	float GetPossibleMinZoomMultiplier();
	float GetMinZoomFactor();
	void SetMinZoomFactor(float factor );
	void SetMaxZoomFactor(float factor );
	float GetMaxZoomFactor();
	void MaximizeBackingStore(bool bForce = false);
	void MinimizeBackingStore();
	void CopyScreen( int destGC, int destx, int desty, int srcx, int srcy, int srcw, int srch );
	bool ScrollTo( WebCore::IntPoint point );
	void SetInitialOverviewMode( bool bEnable );
	void HandleUpdateContents( WebCore::IntRect rect );
	HGC GetWindowGC() { return m_hWindowGC; }

public: // ScrollbarClient
	virtual void valueChanged(WebCore::Scrollbar*);

    // Used to obtain a window clip rect.
	virtual WebCore::IntRect windowClipRect() const;

    // FIXME: It would be nice to set this state on the scroll bar instead of
    // having to ask for it from the client at paint time.
    virtual bool isActive() const;


private:

	void GetUpdateRects( WmRegion* region, const WebCore::IntRect& dirtyRect, WTF::Vector<WebCore::IntRect> &rects );
	void CopyGC( HGC hDestGC, int destX, int destY, HGC hSrcGC, int srcX, int srcY, int srcW, int srcH );
	WebCore::FloatRect CalculateZoomArea( float newMultiplier, WebCore::IntPoint& centerPoint );
	bool ChangeBackingStoreSize( const WebCore::IntSize &newSize, bool bForce = false );
	bool ChangeFrameViewSize( const WebCore::IntSize &newSize, bool bForce = false );
public:
	WebCore::IntRect GetBackingStoreRect();
private:
	WebCore::IntRect GetFrameViewRect();
	WebCore::IntRect GetWindowRect();
	WebCore::IntSize GetBackingStoreSize();
	WebCore::IntSize GetMaxBackingStoreSize();
	void SetViewPos(  const WebCore::IntPoint & pos );
	void SetViewSize( const WebCore::IntSize & size );
	void SetBackingStorePos(  const WebCore::IntPoint & pos );
	void SetBackingStoreSize( const WebCore::IntSize & size );
	bool IsLoading() { return m_bPageLoading; }
	void __ScrollBackingStore( int dx, int dy );
	void __ScrollBy( int dx, int dy );
	WebCore::IntRect CalculateRectContaingCenterRect( const WebCore::IntRect & centerRect, const WebCore::IntRect & boundaryRect, const WebCore::IntSize & rectSize );
	void AdjustViewPos();
	void UpdateContents( WebCore::IntRect rect );
	void RequestPaintToWindow( int x, int y, int w, int h );
#if !defined(SFB_GWES_TRANSFORM)
	void AdjustBackingStorePos( const WebCore::IntRect& viewRect );
	void AdjustViewPos();
	WebCore::IntRect BackingStoreRect();
	WebCore::IntSize CalculateBackingStoreSize( const WebCore::IntSize & contentsSize );
#endif //SFB_GWES_TRANSFORM

	//friends
	friend void WebBackingStore_UpdateContentsTimerCB(HTimer hTimer, ULONG param);
#ifdef DYNAMIC_SCROLLBAR
	friend void WebBackingStore_ScrollBarTimerCB(HTimer hTimer, ULONG param); 
#endif

#ifdef _BACKINGSTORE_DEBUG
void SaveBackingStoreToImageFile( HGC hGC, int w, int h, char* namePrefix );
int m_WebControlID;
int m_filecount;
#endif


#ifdef _PERFORMANCE_DEBUG
	int m_addToDirtyCount,	m_adjustInitialScaleCount,	m_updateBackingStoreCount,	m_showToWindowCount , m_resizeBackingStoreCount;
#endif

private:

	WebControl	*m_pWebControl;
	WmRegion	*	m_pDirtyRegion;
	WmRegion* m_pPaintRegion;


	HGC	m_backingStoreGC;
	bool	m_bPageLoading;
	bool m_bHideScrollbar;

	WebCore::PlatformScrollbar m_hBar, m_vBar;
	//bs0425.lee modified for touch scroll. 
	WebCore::IntRect m_trackRect;
	
	float	m_zoomMultiplier, m_minZoomMultiplier, m_maxZoomMultiplier;
	WebCore::FloatSize m_scrollError;
	WebCore::FloatSize m_scrollErrorCorrection;
	bool m_bDidLoadMainResource;
	bool m_bDidFirstLayout;
	bool m_bDidUserZoomed;
	WebCore::IntRect m_rcBackingStore;
	WebCore::IntRect m_rcView;
	bool m_bEnableInitialOverviewMode;
	HTimer m_hUpdateContentsTimer;
	bool m_bNeedToUpdateContents;
	HGC m_hWindowGC;
#ifdef DYNAMIC_SCROLLBAR
	HTimer m_hTouchScrollBarTimer;
#endif
#if !defined(SFB_GWES_TRANSFORM)
	WebCore::IntRect		m_viewRect;
	WebCore::IntSize m_prevContentsSize;
	WebCore::IntPoint m_zoomingPoint;
	bool m_bCompleteDirty;
#endif //SFB_GWES_TRANSFORM

	bool m_bUseOldBackingStore;

};

} // namespace WebControl

#endif //WebBackingStore_Defined