/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		WebContentsSender.cpp
 * @brief		
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.04.14	v1.0		Created
 */

#include "WebContentsSender.h"




namespace WebControl {

WebContentsSender::WebContentsSender( WebControl *pWebControl )
{

}

void WebContentsSender::SetAuthenticationInfo(  SFBRmHClient hClient,
												SFBRmReqId reqID,
												int transID,
												SFBRmAuthenticationCallbackResponse fpRespCallback )
{
	m_hClient = hClient;
	m_reqID = reqID;
	m_transID = transID;
	m_fpAuthResp = fpRespCallback;
}

void WebContentsSender::SetCookieInfo( SFBRmHClient hClient,SFBRmReqId reqID, SFBRmCookieCallbackResponse fpCallbackResponse )
{
		m_hClient = hClient;
		m_reqID = reqID;
		m_fpCookieResp = fpCallbackResponse;
}

void WebContentsSender::SetRedirectionInfo(  SFBRmHClient hClient,
												SFBRmReqId reqID,
												SFBRmRequestInfo* pRedirectedReq,
												SFBRmRedirectCallbackResponse fpCallbackResponse )
{
	m_hClient = hClient;
	m_reqID = reqID;
	m_pRedirectReqInfo = pRedirectedReq;
	m_fpRedirectResp = fpCallbackResponse;
}

void WebContentsSender::SetSSLInfo(  SFBRmHClient hClient,
												SFBRmReqId reqID,
												int transID,
												SFBalRmSSLCallbackResponse fpCallbackResponse )
{
	m_hClient = hClient;
	m_reqID = reqID;
	m_transID = transID;
	m_fpSSLResp = fpCallbackResponse;
}

void WebContentsSender::SetPDPEventInfo(  SFBRmHClient hClient, SFBRmPDPContextCallbackResponse fpCallbackResponse )
{
	m_hClient = hClient;
	m_fpPDPEventResp = fpCallbackResponse;
}

/*
 * @desc sender function called when sends result about authentication.
 * @param in_pszID pointer to ID
 * @param in_pszPassword pointer to password
 * @return
 */
HRESULT WebContentsSender::auth( BOOL bCancel, char* in_pszID, char* in_pszPassword )
{
	//if( bCancel ) // cancel resourceloader of WebCore
	//{
	//	WebKitLoader *pLoader = m_pWebControl->GetMainWebFrame()->GetFrame()->getWebKitBridge()->Loader();
	//	pLoader->CancelAuthentication( m_transID );
	//}
	SysDebug(( MID_BROWSER, "WebContentsSender::auth Accept[%d], ID[%s], PWD[%s] \n",bCancel, in_pszID,  in_pszPassword));

	if( m_fpAuthResp )
			m_fpAuthResp( m_hClient, m_reqID,  bCancel, in_pszID, in_pszPassword  );

	delete this;
	return HRESULT_OK;
}

/*
 * @desc sender function called when sends result about redirect.
 * @param in_bOK : HRESULT_NOK / HRESULT_OK
 * @return
 */
HRESULT WebContentsSender::redirect( HRESULT in_bOK )
{
	if(!m_fpRedirectResp)
	{
		delete this;
		return HRESULT_FALSE;
	}

	SysDebug(( MID_BROWSER, "WebContentsSender::redirect Accept[%d] \n",in_bOK));

	if(!in_bOK)
	{
		m_fpRedirectResp(m_hClient, m_reqID, m_pRedirectReqInfo, FALSE);
		delete this;
		return HRESULT_FALSE;
	}
	
	m_fpRedirectResp(m_hClient, m_reqID, m_pRedirectReqInfo, TRUE);

	delete this;
	return HRESULT_OK;
}

/*
 * @desc sender function called when sends result about cookie.
 * @param in_bAllow : HRESULT_NOT_ALLOW / HRESULT_ALLOW
 * @return
 */
HRESULT WebContentsSender::cookie( HRESULT in_bAllow, void* pCookie )
{
	SysDebug(( MID_BROWSER, "WebContentsSender::cookie Accept[%d] \n",in_bAllow));

	if(m_fpCookieResp )
		m_fpCookieResp( m_hClient, m_reqID, in_bAllow, pCookie );

	delete this;
	return HRESULT_OK;
}

/*
 * @desc sender function called when sends result about ExScheme
 * @param
 * @return
 */
HRESULT WebContentsSender::exScheme( void )
{

	__NotImplemented(); 
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc sender function called when sends result about upload file
 * @param in_pszFilePath : selected file and path to upload
 * @return
 */
HRESULT WebContentsSender::upload( char* in_pszFilePath )
{

	__NotImplemented(); 
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc sender function called when sends result about Upload State
 * @param
 * @return
 */
HRESULT WebContentsSender::uploadState( void )
{

	__NotImplemented(); 
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc sender function called when sends result about forward
 * @param in_YesNo : HRESULT_NOK / HRESULT_OK
 * @return
 */
HRESULT WebContentsSender::forwardHistory( HRESULT in_OK )
{

	__NotImplemented(); 
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc sender function called when sends result about backward
 * @param in_YesNo : HRESULT_NOK / HRESULT_OK
 * @return
 */
HRESULT WebContentsSender::backwardHistory( HRESULT in_OK )
{

	__NotImplemented(); 
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc sender function called when sends result about notify
 * @param in_YesNo : HRESULT_NOK / HRESULT_OK
 * @return
 */
HRESULT WebContentsSender::notify( HRESULT in_YesNo )
{

	__NotImplemented(); 
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc sender function called when sends result about confirm
 * @param in_YesNo : HRESULT_NOK / HRESULT_OK
 * @return
 */
HRESULT WebContentsSender::confirm( ConfirmType in_ConfirmType, HRESULT in_YesNo )
{

	switch(in_ConfirmType)
	{
		case CONFIRM_CERTIFICATE:

			SysDebug(( MID_BROWSER, "WebContentsSender::confirm CONFIRM_CERTIFICATE \n"));

			if(m_fpSSLResp)
				m_fpSSLResp(m_transID, in_YesNo);

			break;
                 // For handling case that other app is using PDP - Sumdra
		case CONFIRM_PDP_OTHERUSE:

			SysDebug(( MID_BROWSER, "WebContentsSender::confirm CONFIRM_PDP_OTHERUSE \n"));

			if(m_fpPDPEventResp)
				m_fpPDPEventResp(m_hClient,in_YesNo);
			break;

		default:
			SysDebug(( MID_EXCEPTION, "WebContentsSender::Confirm Unknown Type(%d) \n", in_ConfirmType ));
			break;
	}

	delete this;
	return HRESULT_OK;
}

/*
 * @desc sender function called when sends result about prompt
 * @param in_YesNo : YES / NO
 * @return
 */
HRESULT WebContentsSender::prompt( HRESULT in_YesNo, char* in_pszReturn )
{

	__NotImplemented(); 
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc sender function called when sends result about alert
 * @param in_OK : HRESULT_NOK / HRESULT_OK
 * @return
 */
HRESULT WebContentsSender::alert( HRESULT in_OK )
{

	__NotImplemented(); 
	return HRESULT_NOTSUPPORT;
}




} // namespace WebFrame