/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics.
 */
/**
 * @file		CControl.cpp
 * @brief		implement interface IControl of WebControl
 * @author	Sangheon Kim (sangheon77.kim@samsung.com) @n
					bs0425.lee@samsung.com
 * @version	1.0
 * @date		2008.03.18	v1.0		Created							shkim @n
					2008.03					Touch feature added		bslee @n

 */
#include "config.h"
#include "WebControl.h"
#include "WebChromeClient.h"
#include "WebContextMenuClient.h"
#include "WebDragClient.h"
#include "WebEditorClient.h"
#include "WebInspectorClient.h"
#include "WebFrameLoaderClient.h"
#include "WebPageInfo.h"
#include "WebElementPropertyMap.h"
#include "WebContentsSender.h"
#include "WebDOMCore.h"

#include "HTMLImageElement.h" 
#include "MemoryManager.h"
#include "GCController.h"

#include "FrameLoader.h"
#include "SharedTimer.h"
#include "FrameView.h"
#include "kjs_proxy.h"
#include "cache.h"
#include "Settings.h"
#include "Cursor.h"
#include "CString.h"
#include "GraphicsContext.h"
#include "EventHandler.h"
#include "PlatformKeyboardEvent.h"
#include "NativeEditField.h"
#include "NativeComboField.h"
#include "HitTestResult.h"
#include "HitTestRequest.h"
#include "MouseEventWithHitTestResults.h"
#include "SubstituteData.h"
#include "CachedImage.h"
#include "WebCorePageData.h"
#include "RenderImage.h"
#include "RenderWidget.h"
#include "HTMLObjectElement.h"
#include "CachedResource.h"
#include "HTMLInputElement.h"
#include "HTMLTextAreaElement.h"

//#include "BrAppEventTypes.h"//sfb_re_eng
#include "WMLDocument.h"
#include "WMLDoElement.h"
#include "Tokenizer.h"
#include "SegmentedString.h"
#include "RsrcMgr.h" // Zafar ECOM
#include "DocumentType.h"
#include "ShpJasmine.h"
#include "NodeList.h"
#include "FrameTree.h"
#include "PlatformString.h"
#include "FocusController.h"
#include "selectionController.h"
#include "GWES.h"
#include "BrShpRsrcBitmap.h"
#include "RsrcBitmap.h"
#ifdef SFB_PROGRESSIVE_RENDERING
#include "WsRsrcCheck.h"
#endif // SFB_PROGRESSIVE_RENDERING

#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)
#include "NativeListbox.h"
#endif //#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)

//ancil - fix for JS exit
#include "GCController.h"
#include "TransactionManager.h"

#include "Navigation.h"

#include "PlatformScrollbar.h"
#include "SFbrowserRsrc.h"
#include "MIMETypeRegistry.h"
#ifdef HTTP_POST_UPLOAD_SUPPORT
#include "RenderFileUploadControl.h"
#endif // HTTP_POST_UPLOAD_SUPPORT

#ifdef SFB_PROGRESSIVE_RENDERING
#define SFB_RSRC_FILENAME "SFBrowserRsrc.rbm"
#endif // SFB_PROGRESSIVE_RENDERING

#if defined(_SFB_BACKINGSTORE)
#include "WebBackingStore.h"
#endif
#ifdef SUPPORT_AUTO_FORM_FILL
#include "AutoFormFill.h"
#endif

#ifdef SFB_SMARTLINK_FEATURE
#include "SmartLinkMap.h"
#endif

#include "kjs_window.h"

#define _SFB_JAVASCRIPT_ENABLE
#define SFB_IMPROVE_END_KEY		/* To Improve end Key performance */

#define ZOOMING_MAX 3.0

//#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
#define PAGE_PREVIEW_DIFF_SCROLLBAR	7
#define PAGE_PREVIEW_SCROLLSPEED	5
#define PAGE_PREVIEW_SCROLLPADDING 50 
#define PAGE_PREVIEW_MAX_WIDTH 960
#define PAGE_PREVIEW_MAX_HEIGHT 960
#define PAGE_SAVEWRKING_BUFFER kit(child->page())->m_docSize*5

#define DOUBLECLICK_MAX_THRESHOLD 250 //ms
#if defined(_SHP_SIMUL)
#define DOUBLECLICK_MIN_THRESHOLD 0 //ms
#else
#define DOUBLECLICK_MIN_THRESHOLD 120 //ms
#endif

//#endif

#define	_BROWSER_MAX_STRING				1024
#define	_BROWSER_MAX_URL				_BROWSER_MAX_STRING
#define	BRPL_EXTENSION_STRING_LEN_MAX			5

#define	BRPL_FILENAME_STR_LEN_MAX				46  /* Added by siso */
#define FILEPATH_LEN_MAX						71

// 2008.08.08 bs0425.lee modified for long press zoom. 
#define		ZOOM_BITMAP_PADDING		15
//temp compiler fix- to be removed later.
bool g_inFinalize = false;
// bs0425.lee modified for Page Preview.
bool gPreview= false; // TO DO : need to check 
bool result;
// bs0425.lee Test for scroll without multitab. 
bool bUseScroll;
static String mainMarkUpdata("");
static unsigned int icount;
static Vector<WebCorePageDataItem *> itemArray;
//static WebCorePageDataItem* dataItem = NULL;
static WTF::HashMap<String,int> fileNameMap;

extern "C" void ReInitDlState();

#ifdef MULTIPLE_INSTANCE_SUPPORT //Pramod
typedef struct
{
	WTF::HashMap<void*,int>* hWebControlInstanceMap;
	int iCount;

}WebControlContext;

static WebControlContext *gWebControlContext;
#endif //MULTIPLE_INSTANCE_SUPPORT

#ifdef SFB_GLOBAL_STATIC_HANDLING

void initPageSave(){
  mainMarkUpdata.initString("");
  icount = 0;
  //dataItem = NULL;
}
void deinitPageSave(){
   mainMarkUpdata.reInitString();
   icount = 0;
   itemArray.clear();
   //delete dataItem;
   //dataItem=NULL;
   fileNameMap.clear();
}
#endif


#ifdef SFB_GLOBAL_STATIC_HANDLING
extern void initTransactionManagerGlobals();

extern void initCursorShpGlobals();
extern void InitMemoryManager();
extern void DeInitMemoryManager();

extern void initNpRuntime();
extern void initObject();
  extern void InitMalloc();

extern "C"
{
    void Initdev_zero_fd();
    void initDtoa();
}
extern void initNPJSObject();
extern void InitRequestDispatcher();
extern void initDebugger();
extern void initWTFFastMalloc();
namespace WebCore
{
    void initGCController();
    void initArenaGlobals();
    void initTimerGlobals();
    void initDeprecatedStringDataGlobals();
    void initAtomicStringGlobals();
    void initFontCacheGlobals();
    void initTextEncodingRegistryGlobals();
    void initSharedTimerShpGlobals();
    void initMIMETypeRegistryGlobals();
    void initNativeEditFieldGlobals();
    void initNativeComboFieldGlobals();
#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)
	void initNativeListBoxGlobals();
#endif //#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)
    void initCacheMemCollectorGlobals();
    void initFrameLoaderGlobals();
	void InitRootInlineBox();
	void InitRenderReplaced();
	void InitRenderBox();
	void InitLayoutState();
#ifndef NDEBUG
    void initSubresourceLoaderGlobals();
#endif
#if USE(ICU_UNICODE)
    void initTextCodecICUGlobals();
#endif
	void initMediaQueryEvaluator();
	void initCssStyleSelector();
	void initEventTarget();
	void initQualifiedName();
	void initXmlTokenizer();
	void initStyledElement();
	void initRange();
	void initContainerNode();
	void initKJSEvents();
	void initHistoryItem();
	void initCachedPage();
    void initRenderBlock();
    void InitRenderBox();
    void InitRenderLayer();
    void InitRenderObject();
#if !PLATFORM(CF) && USE(ICU_UNICODE)
    void initSmartReplaceICU();
#endif
#if USE(SHP_UNICODE)
    void initTextBreakIteratorShp();
#endif
    void initCNavigationGlobals();

	void initCSSParser();
	void initCachedImage();
	void initCache();
	void initBidi();
	void initFont();
    void initFontPlatformData();
    void initGCController();
    void initReplaceSelectionCommand();
    namespace MediaFeatureNames
    {
        void initFnStatic();
    }
    void initNode();

    namespace EventNames
    {
	    void initEventNames();
    }

	void initEventHandler();
	void initDocument();
	void initFrameStatic();
	void initHtmlEditingStatic();
	void intitHTMLFormElementStatic();
	void initHTMLLabelElementStatic();
	void initTableElementStatic();
	void initIndentOutdentCommandStatic();
	void initHTMLInterchangeStatic();
    void initPage();
    void initPageCache();
    void initPasteboard();
    void initFontFamilyValue();
    void initJSEventTargetPrototype();
    void initjsHTMLInputElementBaseFunctionSetSelectionRange();
    void initDOMImplementation();
    void initStyleChange();
    void initSecurityOrigin();
    void initFontCacheShp();
    void initTextEncoding();
    void initPluginView();
    void InitInlineBox();
    void initRenderFlexibleBox();
    void initXMLHttpRequest();
    void initMarkUp();
    void initHTMLButtonElement();
    void initHTMLFieldSetElement();
    void initHTMLInputElement();
    void initHTMLKeygenElement();
    void initHTMLLegendElement();
    void initHTMLOptGroupElement();
    void initHTMLOptionElement();
    void initHTMLScriptElement();
    void initHTMLSelectElement();
    void initHTMLTextAreaElement();
    void initInspectorController();
    void initCSSSelector();
    void initEditorCommand();
    namespace XPath
    {
        void initXPATHFunctionsGlobals();
    }
#ifdef SUPPORT_AUTO_FORM_FILL
    void initAutoFormFill();
#endif
    void initPageGroupGlobals();
	void initFrameLoaderLoadEventStatus();

// DeInit Routines
    void deInitGCController();
    void DeInitRenderWidget();
    void DeInitSecurityOrigin();
	void deInitDOMImplementation();
    void deInitJSHTMLElementWrapperFactory();
    void deInitFrameStatic();
	void deInitHTMLParserStatic();
    void deInitCSSMutableStyleDeclaration();
    void deInitTextEncoding();
    void deInitRenderBlock();
    void deInitRenderCounter();
    void deInitStyleSurroundData();
    void deInitXMLHttpRequest();
    void deInitHtmlEditingStatic();
	void deInitHTMLElementStatic();
    void deInitTableElementStatic();
    void deInitIndentOutdentCommandStatic();
    void deInitBidi();
    void deInitFrameLoaderGlobals();
    void DeinitBidiRun();
    void deInitJSEventTargetPrototype();
    void deInitjsHTMLInputElementBaseFunctionSetSelectionRange();
    void deInitKJSEvents();
    void deInitCssStyleSelector();
    void deInitMediaQueryEvaluator();
    void deInitContainerNode();
    void deInitStyledElement();
    void deInitPage();
    void deInitTimerGlobals();
    void deInitFontCacheGlobals();
    void deInitGlyphPageTreeNodeStatic();
    void deInitAtomicStringGlobals();
    void deInitStreamMap();
    void deInitRenderBox();
    void deInitRenderFlexibleBox();
    void deInitRenderReplaced();
    void deInitRootInlineBox();
    void deInitFontCacheShp();
    void deInitStyleChange();
    void DeInitHTMLInterchangeStatic();
    void deInitMarkup();
    void deInitHTMLButtonElement();
    void deInitHTMLFieldSetElement();
    void deInitHTMLInputElement();
    void deInitHTMLKeygenElement();
    void deInitHTMLLegendElement();
    void deInitHTMLOptGroupElement();
    void deInitHTMLOptionElement();
    void deInitHTMLSelectElement();
    void deInitHTMLTextAreaElement();
    void deInitInspectorController();
    void deInitCSSSelector();
    void deInitEditorCommand();
    void deInitElement();
    void deInitQualifiedName();
    void deInitReplaceSelectionCommand();
    void deInitHTMLElementFactory();
    void deInitTextEncodingRegistryGlobals();
    void deInitPluginDatabaseShp();
    void deInitCachedImage();
	void deInitLibXmlParser();
	void deInitTextBreakIteratorShp();
    namespace XPath
    {
        void deInitXPATHFunctionsGlobals();
        void deInitXPathParser();
        void deInitEvaluationContext();
    }
    namespace HTMLNames
    {
        void deInit();
    }
    namespace XMLNames
    {
        void deInit();
    }
#ifdef SUPPORT_AUTO_FORM_FILL
	void deInitAutoFormFill();
#endif
    void deInitKURLGlobals();
    void deInitFontPlatformData();
}

namespace KJS
{
    void initObject();
    void initKJSWindow();
	void initUString();
namespace Bindings{
	void initRuntime();
}
	void initArrayInstance();
	void initMathObject();
	void initJSLock();
	void initPropertyMap();
	void initJsCoreCollector();
	void initInterpreter();
	void initIdentifier();

	void initCommonIdentifiers();
	void initJsGlobalObjectStatic();
    void initObjectNmSpace();
    void initParser();
    void initFunctionPrototype();
    void initObjectPrototype();
    void initLexer();
    void initRegExpPrototype();
    void initDateObjectImp();
    void initJSXMLHttpRequestPrototype();
    void InitArray_object();
	void initExecState() ;
	void initKjsNodes(); /*HQ modification*/
//  DeInit Routines
    void deInitKjsNodes();
	void deInitArray_object();
    void deInitCollector();
    void deInitExecState();
    void deInitFunctionPrototype();
    void deInitObjectPrototype();
    void deInitLexer();
    void deInitRegExpPrototype();
    void deInitDateObjectImp();
    void deInitKjs_Binding();
    void deInitKJSList();
    void deInitJSXMLHttpRequestPrototype();
    void deInitUString();
    void deInitPluginBaseStatic();
namespace Bindings{
    void deInitCClass();
    void deInitRuntineRoot();
}

}

namespace WTF
{
	void initHashTableStats();
}

#endif//SFB_GLOBAL_STATIC_HANDLING




#define TAG_IMG "img"
#define TAG_A "a"
#define TAG_INPUT "input"
#define TAG_SELECT "select"
#define TAG_FRAME "frame"
#define TAG_LABEL "label"
/*Sneha:Added textarea to fix bug id 1440*/

#define TAG_TEXTAREA "textarea"


// added by bs0425.lee  to check touch release offset.
#define INNER_INDEX		20

// bs0425.lee flick scroll dx,dy, max,min.
#define FLICK_MAX		250
#define FLICK_MIN		1
#define FLICK_RATE		1.5
#define FLICK_SCROLL_INTERVAL	1
#define FLICK_OFFSET		4
using namespace WebCore;

namespace WebCore {

extern bool bDoResetSSRDefaultStyle  ;
}

namespace WebControl {

extern "C" void WebControl_CloseWindow(HTimer hTimer, ULONG param)
{
	((WebControl* )(param)) ->closeWindowTimerFired() ;
	WmDeleteTimer( hTimer );
}
/**
* All WebControl instances will share one Cache/Cookie storage.
* So Cache/Cookie storage is created when first WebControl instance is created.
* and these storage will be destroyed when last WebControl instance is destroyed.
*/
static SFBHCache		g_hCacheStorage;
static SFBHCookie		g_hCookieStorage;
static SFBRmHClient		g_RmClient;

#ifdef _SHP_SIMUL
typedef stlport::vector<WebControl*>  WebControlVector;
#define FindWebControlVector stlport::find
#else
typedef std::vector<WebControl*>  WebControlVector;
#define FindWebControlVector std::find
#endif
WebControlVector* g_pWebControlVector = NULL;

static void touchZoomReleasedTimerFired(HTimer hTimer, ULONG param);

void receivePDPEvent(SFBRmHClient hClient, SFBRmPDPContextEventParams *pParams, SFBRmPDPContextCallbackResponse fpCallbackResponse);

WebCore::Frame* core( WebFrame* pWebFrame )
{
    return pWebFrame->GetFrame();
}

WebFrame* kit( WebCore::Frame* pFrame )
{
    WebFrameLoaderClient* pClient = static_cast<WebFrameLoaderClient*>(pFrame->loader()->client());
    return pClient->GetWebFrame();
}

WebCore::Page* core( WebControl* pWebControl )
{
    return pWebControl->GetPage();
}

WebControl* kit(WebCore::Page* pPage )
{
    WebChromeClient* pClient = static_cast<WebChromeClient*>(pPage->chrome()->client());
    return pClient->GetWebControl();
}

/**
* Initialize member variables
* In order to crete this instance, Client should use WebControl::New funtion instead of this constructor.
*/
WebControl::WebControl()
{
	m_openerWindowUrl = NULL; //for BUG ID 1738 Gnana
	m_pPage = NULL;
	m_pMainWebFrame = NULL;
	m_hGC = INVALID_HOBJ;
	m_hWin = INVALID_HOBJ;
	m_nGCType = BIT_16_GC; // 16 bit gc is default.  by nayoungkang
        bEnableZoom = false;
//#ifdef _SFB_PAGE_PREVIEW - Pramod 23/July/08
	// Added for Page Pilot
	m_Preview = false;
	m_SetToPagePilotAfterFirstRendering = false;
	m_ReSetPagePilot = false;
        m_pagePreview_X = 0;
	m_pagePreview_Y = 0;
//#endif

#ifdef SFB_PROGRESSIVE_RENDERING
	m_pbmPlaceHolder = NULL;
#endif

	m_pLoadGroup = NULL;

	m_pWebPreferences = NULL;

	m_lastFocusedNode = NULL;

	m_isLoadingDone = false;
	m_pContentListener = NULL;
	m_RenderingMode = STANDARD_RENDERING_MODE;
	m_pWindowListener = NULL;
	m_pDecisionListener = NULL;
	m_pDownloadListener = NULL;
	m_initialUrl = NULL ;
#if defined(_SFB_BACKINGSTORE)
	m_pBackingStore = NULL;
#endif
	m_cantOverrideDefaultPreferences = false ;
// bs0425.lee modified for one click touch. 
	m_bOneClickMode=false;
#if defined(JS_MULTIPLE_WINDOW_SUPPORT)/* Anjana - UI changes for JS New Window */
	m_bSecureMode=false;
#endif
	hTouchReleasedTimer = INVALID_HOBJ;
	m_bExitInstance = false;
        m_bEndKeyPressed = false;
	m_docSize = 0;

	jsConfirmResult=false ;

	if( g_pWebControlVector == NULL )
	{
		g_pWebControlVector = new WebControlVector();
		g_hCacheStorage = SFBCreateCache(WEBCONTROL_CACHE_STORAGE, NULL);
		g_hCookieStorage  = SFBInitCookie(WEBCONTROL_COOKIE_STORAGE, NULL);
		// create ILoadGroup
		// [todo]
		
		//g_RmClient = SFBRmRegisterClient( "Browser", (SFBRmPDPContextCallback)receivePDPEvent );
	}

	//m_pLoadGroup = (void*)g_RmClient;
		//if( (int)(m_pLoadGroup) == INVALID_HOBJ )
		//	return ; //HRESULT_NOK;
		//SFBRmSetCookieStorage((int)m_pLoadGroup, g_hCookieStorage);
		//SFBRmSetCacheStorage((int)m_pLoadGroup, g_hCacheStorage);

	g_pWebControlVector->push_back( this );
# ifdef SFB_WIDGET
	m_ExposeEventPresent = false;
#endif
}

WebControl::~WebControl()
{
	Destruct();

	WebControlVector::iterator pos;
	pos = FindWebControlVector( g_pWebControlVector->begin(), g_pWebControlVector->end(), this );
	if( pos != g_pWebControlVector->end() )
		g_pWebControlVector->erase( pos );

	if( g_pWebControlVector->empty() )
	{
		// destroy ILoadGroup
		//SFBRmUnregisterClient( (int)m_pLoadGroup );

		delete g_pWebControlVector;
		g_pWebControlVector = NULL;

		SFBExitCache(g_hCacheStorage);
		SFBExitCookie(g_hCookieStorage);
	}
	
	if( m_pWebPreferences )
	{
		m_pWebPreferences->removeListener( this );
	
		if( gWebControlContext == NULL )
		{
			delete m_pWebPreferences;
			m_pWebPreferences = NULL;
		}
	}
# ifdef SFB_WIDGET
	bool shouldOOMReSet = true;
	if(g_pWebControlVector == NULL)
		return;
	for(  int i = 0; i < g_pWebControlVector->size() ; i++ )
	{
		WebControl *pWebControl = g_pWebControlVector->at(i);
		if (pWebControl->m_strName == "Browser")
		{
			shouldOOMReSet = false;
			break;
}
	}
	if (shouldOOMReSet)
	{
		MemoryManager::ResetOOMState();
	}
	m_ExposeEventPresent = false;
# endif
}






HRESULT
WebControl::New( char* szName, HWIN hWin, int x, int y, int w, int h, WebControl **ppWebControl , GCType in_gcType )
{
	HRESULT hResult;
	WebControl *pWebControl = NULL;

	*ppWebControl = NULL;
	pWebControl = new WebControl();

	if( pWebControl == NULL )
		return HRESULT_NO_MEMORY;

	hResult =  pWebControl->Construct( szName, hWin, x, y, w, h, in_gcType );
	if( HRESULT_ISFAIL(hResult) )
	{
		delete pWebControl;
		return hResult;
	}
	else
	{
		pWebControl->pFuncHandler = NULL;     
		*ppWebControl = pWebControl;
		return hResult;
	}

}


void
WebControl::InitGlobals()
{
#ifdef SFB_GLOBAL_STATIC_HANDLING
	InitMalloc();
	KJS::initJsCoreCollector();
    InitMemoryManager();
    initWTFFastMalloc();
    initAtomicStringGlobals();
    initTimerGlobals();
    ///After initAtomicStringGlobals() call
	initBidi();

    // KJS::initIdentifier() has to be called before any other JS and
    // JS binding initialization. Thus placing it up into order
#ifdef _SFB_JAVASCRIPT_ENABLE
    KJS::initIdentifier();
#endif

    initGCController();
	initTransactionManagerGlobals();
    initCursorShpGlobals();
    initArenaGlobals();
    initDeprecatedStringDataGlobals();
    initFontCacheGlobals();
    initTextEncodingRegistryGlobals();
    initSharedTimerShpGlobals();
    initMIMETypeRegistryGlobals();
    initNativeEditFieldGlobals();
    initNativeComboFieldGlobals();

#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)
	initNativeListBoxGlobals();
#endif //#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)

    initCacheMemCollectorGlobals();
	InitRenderObject();
	InitRootInlineBox();
	InitRenderReplaced();
	InitRenderBox();
	InitLayoutState();
#ifndef NDEBUG
    initSubresourceLoaderGlobals();
#endif
#if USE(ICU_UNICODE)
    initTextCodecICUGlobals();
#endif
	initMediaQueryEvaluator();
	initEventTarget();
	initQualifiedName();
	initXmlTokenizer();
	initStyledElement();
	initRange();
	initContainerNode();
    initPage();
    initPageCache();
	initHistoryItem();
	initCachedPage();
	initDtoa();
#ifdef _SFB_JAVASCRIPT_ENABLE
	initNPJSObject();
#endif

    InitRenderBox();
    InitRenderLayer();
    InitRenderReplaced();
    InitRequestDispatcher();
#if !PLATFORM(CF) && USE(ICU_UNICODE)
    initSmartReplaceICU();
#endif
#if USE(SHP_UNICODE)
    initTextBreakIteratorShp();
#endif
    initCNavigationGlobals();
	initCSSParser();
	initCachedImage();
	initCache();
	initFontCacheGlobals();
	initFont();
    initFontPlatformData();
    WebCore::EventNames::initEventNames();
	initEventHandler();
	initDocument();
	initDebugger();
    WebCore::MediaFeatureNames::initFnStatic();
    initNode();
	initFrameStatic();
	initHtmlEditingStatic();
	intitHTMLFormElementStatic();
	initHTMLLabelElementStatic();
	initTableElementStatic();
	initIndentOutdentCommandStatic();
	initHTMLInterchangeStatic();
    initPasteboard();
    initFontFamilyValue();
#ifdef _SFB_JAVASCRIPT_ENABLE
    initJSEventTargetPrototype();
#endif

    initDOMImplementation();
#ifdef _SFB_JAVASCRIPT_ENABLE
    initjsHTMLInputElementBaseFunctionSetSelectionRange();
#endif
    initStyleChange();
    initSecurityOrigin();
    initFontCacheShp();
    initTextEncoding();
    initPluginView();
    InitInlineBox();
    initRenderFlexibleBox();
    initXMLHttpRequest();

/* 
As per SISO comments for SFB00001644 removing the #ifdef _SHP_SIMUL
 for initMarkUp().
*/
    initMarkUp();

    initHTMLButtonElement();
    initHTMLFieldSetElement();
    initHTMLInputElement();
    initHTMLKeygenElement();
    initHTMLLegendElement();
    initHTMLOptGroupElement();
    initHTMLOptionElement();
    initHTMLScriptElement();
    initHTMLSelectElement();
    initHTMLTextAreaElement();
    initInspectorController();
    initCSSSelector();
    initEditorCommand();
    initReplaceSelectionCommand();

    WebCore::XPath::initXPATHFunctionsGlobals();
#ifdef SUPPORT_AUTO_FORM_FILL
    initAutoFormFill();
#endif
#ifdef _SFB_JAVASCRIPT_ENABLE
	KJS::initExecState();
    KJS::initObject();
    KJS::InitArray_object();
    KJS::initCommonIdentifiers();
	KJS::initKJSWindow();
	KJS::initJsGlobalObjectStatic();

    KJS::initUString();
	KJS::Bindings::initRuntime();
	KJS::initArrayInstance();
	KJS::initMathObject();
	KJS::initJSLock();
	KJS::initPropertyMap();
	KJS::initInterpreter();
    KJS::initJSXMLHttpRequestPrototype();
    KJS::initObjectNmSpace();
    KJS::initParser();
    KJS::initFunctionPrototype();
    KJS::initObjectPrototype();
    KJS::initLexer();
    KJS::initRegExpPrototype();
    KJS::initDateObjectImp();
	KJS::initKjsNodes();
#endif

    //These two initilization routines need to be after KJS::initIdentifier(); thus tagging them together
	WTF::initHashTableStats();

#if !PLATFORM(DARWIN) || !defined(__LP64__)
    initNpRuntime();
#endif //!PLATFORM(DARWIN) || !defined(__LP64__)

    initPageSave();

    // initialize the kjsevents after all the kjs initialization.
#ifdef _SFB_JAVASCRIPT_ENABLE
    initKJSEvents();
#endif

    initPageGroupGlobals();

	initFrameLoaderLoadEventStatus();

#endif //SFB_GLOBAL_STATIC_HANDLING
}

void
WebControl::DeInitGlobals()
{
#ifdef SFB_GLOBAL_STATIC_HANDLING
    //call deInitGCController() first to make sure the cache is purged
    deInitGCController();
#ifdef SUPPORT_AUTO_FORM_FILL
		deInitAutoFormFill();
#endif
    DeInitRenderWidget();
    DeInitSecurityOrigin();
	deInitDOMImplementation();
#ifdef _SFB_JAVASCRIPT_ENABLE
    deInitJSHTMLElementWrapperFactory();
#endif
    deInitFrameStatic();
    deInitHTMLParserStatic();
    deInitCSSMutableStyleDeclaration();
    deInitTextEncoding();
    deInitRenderBlock();
    deInitRenderCounter();
    deInitStyleSurroundData();
    deInitXMLHttpRequest();
    deInitHtmlEditingStatic();
    deInitHTMLElementStatic();
    deInitTableElementStatic();
    deInitIndentOutdentCommandStatic();
    deInitBidi();
    deInitFrameLoaderGlobals();
    DeinitBidiRun();
#ifdef _SFB_JAVASCRIPT_ENABLE
    deInitJSEventTargetPrototype();
    deInitjsHTMLInputElementBaseFunctionSetSelectionRange();
    deInitKJSEvents();
#endif
    deInitCssStyleSelector();
    deInitMediaQueryEvaluator();
    deInitContainerNode();
    deInitStyledElement();
    deInitPage();
    deInitTimerGlobals();
    deInitFontCacheGlobals();
    deInitGlyphPageTreeNodeStatic();
    deInitStreamMap();
    deInitRenderBox();
    deInitRenderFlexibleBox();
    deInitRenderReplaced();
    deInitRootInlineBox();
    deInitFontCacheShp();
    deInitStyleChange();
    DeInitHTMLInterchangeStatic();
    deInitMarkup();
    deInitHTMLButtonElement();
    deInitHTMLFieldSetElement();
    deInitHTMLInputElement();
    deInitHTMLKeygenElement();
    deInitHTMLLegendElement();
    deInitHTMLOptGroupElement();
    deInitHTMLOptionElement();
    deInitHTMLSelectElement();
    deInitHTMLTextAreaElement();
    deInitInspectorController();
    deInitCSSSelector();
    deInitEditorCommand();
    deInitElement();
    deInitQualifiedName();
    deInitReplaceSelectionCommand();
    deInitHTMLElementFactory();
    deInitTextEncodingRegistryGlobals();
    deInitPluginDatabaseShp();
    deInitCachedImage();
    WebCore::XPath::deInitXPATHFunctionsGlobals();
    WebCore::XPath::deInitXPathParser();
    WebCore::XPath::deInitEvaluationContext();
    HTMLNames::deInit();
    XMLNames::deInit();
#ifdef _SFB_JAVASCRIPT_ENABLE
    KJS::deInitArray_object();
    KJS::deInitKjsNodes();
	KJS::deInitCollector();
    KJS::deInitExecState();
    KJS::deInitFunctionPrototype();
    KJS::deInitObjectPrototype();
    KJS::deInitLexer();
    KJS::deInitRegExpPrototype();
    KJS::deInitDateObjectImp();
    KJS::deInitKjs_Binding();
    KJS::deInitKJSList();
    KJS::deInitJSXMLHttpRequestPrototype();
    KJS::deInitUString();
    KJS::deInitPluginBaseStatic();
    KJS::Bindings::deInitCClass();
    KJS::Bindings::deInitRuntineRoot();
#endif
    deinitPageSave();
    deInitKURLGlobals();
    deInitFontPlatformData();
		deInitLibXmlParser();
		deInitTextBreakIteratorShp();
#endif //#ifdef SFB_GLOBAL_STATIC_HANDLING

}

HRESULT
WebControl::Construct( char* szName, HWIN hWin, int x, int y, int w, int h , GCType in_gcType)
{
	HRESULT hResult;

	m_strName = szName;
	m_hWin = hWin;
	m_x = x;
	m_y = y;
	m_w = w;
	m_h = h;
	m_nGCType = in_gcType;

	setWebControlCloseWindowFired(false) ;

	// create ILoadGroup
	// [todo]
	/*
	int hRmClient;
	hRmClient = SFBRmRegisterClient( szName, (SFBRmPDPContextCallback)receivePDPEvent );
	m_pLoadGroup = (void*)hRmClient;
	if( (int)(m_pLoadGroup) == INVALID_HOBJ )
		return HRESULT_NOK;
	*/

	// Fix for Widget Refresh 3G animation issue : Partho
//#ifdef SFB_WIDGET 
//	if(AcStricmp(szName, "Widget") == 0)
//	{
//		SFBRmSetCookieStorage((int)m_pLoadGroup, NULL);
//		SFBRmSetCacheStorage((int)m_pLoadGroup, NULL);
//	}
//#endif // SFB_WIDGET

#ifdef MULTIPLE_INSTANCE_SUPPORT //Pramod
	if(gWebControlContext->iCount == 0)
	{
#endif //MULTIPLE_INSTANCE_SUPPORT

	WebCore::bDisableJS = false ;

    // VDJ - Init all static and global variables
    InitGlobals();
		//initialize the navigation
		CNavigation::init();

		// Initialize the place-holder image
#ifdef SFB_PROGRESSIVE_RENDERING
		#pragma message ("DD - TODO - Need to get the place-holder image resource from SISO")
		m_pbmPlaceHolder = RsrcLoadBitmap( SFB_RSRC_FILENAME, IDB_CHECK_ON );
#endif // SFB_PROGRESSIVE_RENDERING

#ifdef MULTIPLE_INSTANCE_SUPPORT //Pramod
	}
#endif //MULTIPLE_INSTANCE_SUPPORT

	//Now Using OSTimer to destroy windows
	//m_pCloseWindowTimer = new WebCore::Timer<WebControl>( this, &WebControl::closeWindowTimerFired);
	hTouchReleasedTimer = WmCreateTimer( "Touchzoom");

	// create WebCore page
	m_pPage = new WebCore::Page( new WebChromeClient(this), new WebContextMenuClient(), new WebEditorClient(this), new WebDragClient(), new WebInspectorClient());
	if( m_pPage == NULL )
		return HRESULT_NO_MEMORY;

	// apply default settings
	WebPreferences *pWebPreferences = WebPreferences::GetDefaultPreferences();
	setISetting( pWebPreferences );

	CNavigation::setNavigationMode(NAVIGATION_MODE_SCROLL_ONLY);
	m_pPage->setNavigationInstance(CNavigation::createInstance());

	// create MainFrame
	hResult = WebFrame::New( this, &m_pMainWebFrame );
	if( HRESULT_ISFAIL(hResult))
		return hResult;
	m_pMainWebFrame->SetIsMainFrame(true);

#ifdef SFB_GLOBAL_STATIC_HANDLING

#ifdef MULTIPLE_INSTANCE_SUPPORT //Pramod
	if(gWebControlContext->iCount == 0)
	{
#endif //MULTIPLE_INSTANCE_SUPPORT

    //We need to initailize Css Style selector after the frame is created.
    initCssStyleSelector();

#ifdef MULTIPLE_INSTANCE_SUPPORT //Pramod
	}
#endif //MULTIPLE_INSTANCE_SUPPORT

#endif


	// initialize GC
	SetHWIN( hWin, x, y, w, h );

#if defined(_SFB_BACKINGSTORE)
	m_pBackingStore = new WebBackingStore( this, gWebControlContext->iCount );
#endif

#ifdef MULTIPLE_INSTANCE_SUPPORT //Pramod
	if(gWebControlContext->iCount == 0)
	{
		WTF::HashMap<void*,int>* hWebControlInstanceMap = new WTF::HashMap<void*,int>();
		gWebControlContext->hWebControlInstanceMap = hWebControlInstanceMap;
	}
	gWebControlContext->hWebControlInstanceMap->add((void*)this,gWebControlContext->iCount);
	gWebControlContext->iCount++;
#endif //MULTIPLE_INSTANCE_SUPPORT

m_stateChangeInfo.presentMainFrame = EState_HTTP;
	m_stateChangeInfo.m_stateChange = false;
	m_stateChangeInfo.m_prevMainFrame = EState_HTTP;
	return HRESULT_OK;
}

HRESULT
WebControl::Destruct()
{
	//free resources via GCController at page end.
	g_inFinalize = true;

	//StopLoading(); Exit time perf change -- Thariq

	if (m_initialUrl)
		SFBalMemFree(m_initialUrl);
         //Using WImTimer instead of Webcore timer
	//if( m_pCloseWindowTimer )
		//delete m_pCloseWindowTimer;

	if(hTouchReleasedTimer != INVALID_HOBJ)
	{
		WmDeleteTimer( hTouchReleasedTimer);
		hTouchReleasedTimer = INVALID_HOBJ;
	}

	WebCore::Frame *frame = core(m_pMainWebFrame);

#if ! defined(SFB_IMPROVE_END_KEY)
	// the JS objects are released only when we navigate to some other page. As we are leaving the browser
	// application we achieve the same purpose by initializing the frame. We may have come to the current
	// page by a back, forward or some other operation hence we explicitly need to set the frame load
	// type to standard.

	frame->document()->removeAllEventListenersFromAllNodes();
	frame->loader()->setLoadType(FrameLoadTypeStandard);
	frame->loader()->init();
#endif
	CNavigation::destroyInstance(m_pPage->navigationInstance());

	delete m_pPage;
#ifdef MULTIPLE_INSTANCE_SUPPORT
	if(gWebControlContext->iCount == 1)
	{
#endif
	//delete pageData->settings;
	//delete pageData->userAgent;


	// Cleanup the place-holder image
	#ifdef SFB_PROGRESSIVE_RENDERING
	if(m_pbmPlaceHolder)
	{
		__WsFreeBitmap(m_pbmPlaceHolder);
	}
	#endif // SFB_PROGRESSIVE_RENDERING

#ifdef MULTIPLE_INSTANCE_SUPPORT
	}
#endif
	//WebCore::NativeEditField::destroyNativeEditFieldInstance();//sneha
	WebCore::NativeEditField* pNativeEdit = NativeEditField::getNativeEditFieldInstance();
    WebAssert(pNativeEdit);
   // bs0425.lee 2008.04.28 chaned to fix edit field.
	if (pNativeEdit->isActivated() )
    {
		  WebCore::NativeEditField::destroyNativeEditFieldInstance();
 	}

	//for handling mouse event in case of combo box- Hema
	WebCore::NativeComboField* pNativeCombo = NativeComboField::getNativeComboFieldInstance();  
    WebAssert(pNativeCombo);
   	if ( pNativeCombo->isActivated() )
    {
		WebCore::NativeComboField::destroyNativeComboFieldInstance();
 	}


#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)
	WebCore::NativeListBox* pNativeList = NativeListBox::getNativeListBoxInstance();
	WebAssert(pNativeList);
	if( pNativeList->isActivated() )
    {
		   WebCore::NativeListBox::destroyNativeListBoxInstance();
 	}
#endif //#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)

	//cleanup
	if( WmVerifyGC(m_hGC) == TRUE )
	{
		WmDeleteGC(m_hGC);
		m_hGC = INVALID_HOBJ;
	}

#ifdef MULTIPLE_INSTANCE_SUPPORT
	if(gWebControlContext->iCount == 1)
	{
#endif
	//finalize the navigation
	CNavigation::finalize();

	//destroy cursor window
	WebCore::CPlatformCursorHelper::Finalize();

#ifdef MULTIPLE_INSTANCE_SUPPORT
	}
#endif

	//webkit_shp_frame_finalize(pageData->mainFrame);
	delete m_pMainWebFrame;

	//for bug id 1738 gnana
	if(m_openerWindowUrl != NULL)
	{
		delete m_openerWindowUrl;
		m_openerWindowUrl = NULL;
	}

	//delete page->privatePage;


	//shkim
	// finalize RmClient
	//SFBExitCache(SFBRmGetCacheStorage(page->m_hRmClient));
	//SFBExitCookie(SFBRmGetCookieStorage(page->m_hRmClient));
	//SFBRmUnregisterClient( page->m_hRmClient );


	//delete page;
#if defined(_SFB_BACKINGSTORE) // destroy backingstore
		delete m_pBackingStore;
		m_pBackingStore = NULL;
#endif

#ifdef MULTIPLE_INSTANCE_SUPPORT
	if(gWebControlContext->iCount == 1)
	{
#endif
	KJS::JSLock lock;


	KJS::Collector::collect();

	WebCore::Cache *pCache = WebCore::cache();
	pCache->removeDeadResources();
	pCache->removeImageResources(true);

#ifdef LOG_MEMORY_LEAKS
	_CrtDumpMemoryLeaks();
#endif // LOG_MEMORY_LEAKS

    // VDJ - Call before other DeInit routines
    gcController().stopTimer();

    // VDJ - DeInit static global variables
    DeInitGlobals();

	//WebCore::finalizeMIMETypeRegistry();
	TransactionManager::removeTransactionMgr();

#ifdef SFB_GLOBAL_STATIC_HANDLING
// SDG - finalize the automic string at the end.
    deInitAtomicStringGlobals();
#endif

    //vamsi - avoid large number of timer create/destroy
    WebCore::stopSharedTimer(true);

#ifdef USE_FAST_MALLOC
	ReInitDlState();
#endif

#ifdef MULTIPLE_INSTANCE_SUPPORT
	//Delete the instance from hashmap and deinit the hashmap
	gWebControlContext->hWebControlInstanceMap->remove((void*)this);
	delete(gWebControlContext->hWebControlInstanceMap);
#endif
	//Perform a forceSweep of the GC to release any zombie blocks if any.
	KJS::Collector::sweepOutVMIfNeeded() ;

#ifdef SFB_GLOBAL_STATIC_HANDLING
  //DeInitMemoryManager() should be last deinit routine.
    DeInitMemoryManager();
#endif

#ifdef MULTIPLE_INSTANCE_SUPPORT
	}
#endif


	// destroy ILoadGroup
	//SFBRmUnregisterClient( (int)m_pLoadGroup );


	//vamsi - crash fix at browser exit when page is still loading. Need to make
	//client based exit later.
#ifdef MULTIPLE_INSTANCE_SUPPORT
	if(gWebControlContext->iCount > 1)
	{
		void* pCurrentInstance = (void*)this;
		SFBalInstanceExitEvent(pCurrentInstance, m_pLoadGroup);
		
		gWebControlContext->hWebControlInstanceMap->remove((void*)this);
		gWebControlContext->iCount--;
	}
	else
	{
	SFBalExitEvent();

		gWebControlContext->iCount = 0;
		SFBalMemFree(gWebControlContext);
		gWebControlContext = NULL;
	}

#else
	SFBalExitEvent();
#endif

	return HRESULT_OK;
}

HRESULT
WebControl::BrowsingWindowHide( )
{
	return HRESULT_OK;
}

/*
 * Sets the string string result from the Confirm window
*/
void WebControl::setJSConfirmResult(bool bResult)
{
	jsConfirmResult = bResult ;
}
/*
 * Gets the result and returns to the ChromeClient which further passes it onto KJS
*/
bool  WebControl::getJSConfirmResult(void)
{
    return jsConfirmResult ;
}
/*
 * Sets the string string result from the prompt window
*/
void WebControl::setJSPromptResultString(const char *inStrData)
{
	jsPromptResult = WebCore::String(inStrData) ;
}
/*
 * Gets the string result and returns to the ChromeClient which further passes it onto KJS
*/
WebCore::String&  WebControl::getJSPromptResultString(void)
{
    return jsPromptResult ;
}


HRESULT
WebControl::SetHWIN( HWIN hWin, int x, int y, int w, int h )
{
	
	

	if( m_hGC != INVALID_HOBJ )
	{
		WmDeleteGC( m_hGC );
		m_hGC = INVALID_HOBJ;
	}

	//create cursor window
	//WebCore::CPlatformCursorHelper::Initialize(static_cast<int>(m_hWin));/*Commented to solve www.youtube.com media player issue*/

    m_hWin = hWin;
    //m_hGC = WmCreateClientGC(hWin);
	if ( GetGCType() == BIT_32_GC  )
	{
		WmSetWindowBgColor( hWin, BLACK );// alpha 
		m_hGC = WmCreateClientSubMemGC32( hWin, x, y, w, h );   
	}
	else
	{
		m_hGC = WmCreateClientSubGC( hWin, x, y, w, h );

		/* Initializes the  GC attributes */
		WmInitGCAttrib ( m_hGC );
	}


    WmRect          rect;
	WebCore::Frame* frame = core(m_pMainWebFrame);

    //WmGetWindowClientRect( hWin, &rect );
    frame->view()->resize( w, h );

    frame->view()->adjustViewSize();
    frame->sendResizeEvent();

	CNavigation* navigation = frame->page()->navigationInstance();
	if(navigation)
	{
		navigation->setGC(m_hGC);
		navigation->setPage(frame->page());
		navigation->setMainFrameView(frame->view());
		CNavigation::switchPage(navigation);
	}

	return HRESULT_OK;
}


HRESULT
WebControl::StopLoading()
{
	//Added To Reset the Memory Manager State
	MemoryManager::ResetOOMState();

	WebCore::FrameLoader* loader = m_pMainWebFrame->GetFrame()->loader();
    if (loader)
	{
        loader->stopAllLoaders();
		loader->client()->setLoadingComlete();//gnana added for bugid 1951
	}

	return HRESULT_OK;
}

HRESULT WebControl::setDecisionListener(IDecisionListener *in_pDecisionListener )
{
	m_pDecisionListener = in_pDecisionListener;

	return HRESULT_OK;
}

HRESULT WebControl::setDownloadListener( URILoader::IStreamListener *in_pIStreamListener )
{
	m_pDownloadListener = in_pIStreamListener;

	return HRESULT_OK;
}

/*
 * @desc registers with Content's listner to a Browser Engine.
 * @param in_pIContentsListener : pointer to contents listeners
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::setContentListener( IContentsListener* in_pIContentsListener )
{
	m_pContentListener = in_pIContentsListener;

	return HRESULT_OK;
}

HRESULT WebControl::getContentListener( IContentsListener** out_pIContentsListener )
{
	if( m_bExitInstance == true ) 
		*out_pIContentsListener = NULL;
	else
	*out_pIContentsListener = m_pContentListener;
	return HRESULT_OK;
}

/*
 * @desc registers with IME's listner to a Browser Engine.
 * @param in_pIIMEListener : pointer to a IME listeners
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::setIMEListener( IIMEListener* in_pIIMEListener )
{
	__NotRequired();
	return HRESULT_NOTSUPPORT;
}
/*
 * @desc registers with Window's listner to a Browser Engine.
 * @param in_pIWindowListner : pointer to a Window listeners
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::setWindowListener( IWindowListener* in_pIWindowListener )
{
	m_pWindowListener = in_pIWindowListener;
	return HRESULT_OK;
}

/*
 * @desc registers with PDPContext's listner to a Browser Engine.
 * @param in_pIPDPContextListener : pointer to a PDPContext listeners
 * @return HRESULT TRUE/FALSE
 */
//virtual HRESULT setPDPContextListner( IPDPContextListener* in_pIPDPContextListener ) = 0;

/*
 * @desc suspends a Browser Engine
 * @param
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::suspend( void )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc resumes a Browser Engine
 * @param
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::resume( void )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc aborts page request and page actions of a Browser Engine
 * @param
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::abort( void )
{
	if(NULL == m_pMainWebFrame)
		return HRESULT_OK;
	
	Frame* pFrame =	m_pMainWebFrame->GetFrame();	// Exit time perf change -- Thariq

	if(NULL == pFrame)
		return HRESULT_OK;

	if( isOneClickMode() )
		{
			// WebCore::stopSharedTimer(true);
			stopLingerTimerAfterAjaxRequest();
			return StopLoading();
		}
	if( !isOneClickMode() && m_isLoadingDone == FALSE )
	{
		// WebCore::stopSharedTimer(true);
	return StopLoading();
}

	SysDebug(( MID_EXCEPTION,"WebControl::abort: Not page loading. So, skip abort transaction. \n" ));
	return HRESULT_OK;
    
}

void WebControl::onOOMEvent( void )
{
	WebCore::Frame *frame = core(m_pMainWebFrame);

    //TODO : The clear script proxy function works only if bDisableJS is false. Need to revisit this
    WebCore::bDisableJS = false;

    if(frame)
    {
	    if (frame->renderer()) 
        {
		    frame->view()->layoutIfNeededRecursive();
        }

        for (Frame* child = frame->tree()->firstChild(); child; child = child->tree()->nextSibling())
        {
            child->clearScriptProxy();
            child->clearDOMWindow();
            child->clearTimers();
            child->clearScriptObjects();
        }

        frame->clearScriptProxy();
        frame->clearDOMWindow();
        frame->clearTimers();
        frame->clearScriptObjects();

#ifdef SFB_SMARTLINK_FEATURE
			frame->GetSmartLinkMap()->StopIndexingSmartLinks();		
#endif

    }

    WebCore::bDisableJS = true;

//    abort(); // This functionality is handled in BrCoreControl_OOMHandler() function
}

/*
 * @desc creates a thumbnail image for displayed page currently.
 * @param
 * @return IImage pointer to IImage
 */
HRESULT WebControl::createPageThumbnail( IImage** out_Image )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc gets an image informatioin on focused.
 * @param out_pImage : pointer to IImage
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::getCurrentImageInfo( IImage** out_pImage )
{
	HRESULT hr;
	IPropertyMap *pMap = NULL;
	hr = getElementByFocus( &pMap );
	if( HRESULT_ISFAIL(hr))
	{
		SysDebug(( MID_EXCEPTION, "WebControl::getCurrentImageInfo : getElementByFocus failed\n" ));
		return HRESULT_NOK;
	}

	hr = pMap->getValue( ELEMENT_PROPERTY_IMAGE_KEY, (void**)out_pImage );
	if( HRESULT_ISFAIL(hr))
	{
		SysDebug(( MID_EXCEPTION, "WebControl::getCurrentImageInfo : getValue( IMAGE_KEY) failed\n" ));
		return HRESULT_NOK;
	}

	pMap->Release();

	return HRESULT_OK;
}

/*
 * @desc gets an image informatioin with position.
 * @param out_pImage : pointer to IMAGEINFO
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::getCurrentImageInfoByPoint( IImage** out_pImage, int x, int y )
{
	HRESULT hr;
	IPropertyMap *pMap = NULL;
	hr = getElementByPoint( &pMap,  x,  y );
	if( HRESULT_ISFAIL(hr))
	{
		SysDebug(( MID_EXCEPTION, "WebControl::getCurrentImageInfoByPoint : getElementByPoint failed\n" ));
		return HRESULT_NOK;
	}

	hr = pMap->getValue( ELEMENT_PROPERTY_IMAGE_KEY, (void**)out_pImage );
	if( HRESULT_ISFAIL(hr))
	{
		SysDebug(( MID_EXCEPTION, "WebControl::getCurrentImageInfoByPoint : getValue( IMAGE_KEY) failed\n" ));
		return HRESULT_NOK;
	}

	pMap->Release();

	return HRESULT_OK;
}

#ifdef SFB_DRAG_DROP
    /*
	* @desc gets a current element rect informatioin.
	* @param height, width,actualX, actualY 
	* @return HRESULT TRUE/FALSE
	*/
	HRESULT WebControl::getCurrentDragElementRect(int*height, int*width,int* actualX,int* actualY, int x, int y , int* ctElement, int* elementType)
	{//hema
		Frame* frame = core(m_pMainWebFrame);
		WmRect preview_Rect;
		WebCore::IntRect ptRect;
		WebCore::IntPoint pTempPt;
 #if defined(_SFB_BACKINGSTORE)
		WebCore::IntPoint pBackStorePt;
 #endif   //(_SFB_BACKINGSTORE)

			WebCore::Element* element = NULL;
			WebCore::Node* curNode = NULL;
			RenderObject* r = NULL;

			//m_Preview = FALSE;
#if defined(_SFB_BACKINGSTORE)
			pBackStorePt = m_pBackingStore->WindowToBackingStore( WebCore::IntPoint(x, y) );
			WebCore::IntPoint ptContent = frame->view()->windowToContents(pBackStorePt);
#else
			WebCore::IntPoint ptContent = frame->view()->windowToContents(WebCore::IntPoint(x, y));
#endif //(_SFB_BACKINGSTORE)
			
			WebCore::Document* NewDocument=frame->documentAtPoint(ptContent);

			if( NewDocument )
			{
				//element = NewDocument->elementFromPoint(ptContent.x(), ptContent.y() );
				
				if(! NewDocument->renderer())
					return HRESULT_NOK;

				WebCore::HitTestRequest request(true, true);
				WebCore::HitTestResult result(IntPoint(ptContent.x(), ptContent.y()));
				NewDocument->renderer()->layer()->hitTest(request, result);
				WebCore::Node* node = result.innerNode();

				WebCore::Node* tempNode = node;
				
				while (tempNode && !tempNode->isElementNode())
					tempNode = tempNode->parentNode();
			    if (tempNode)
					tempNode = tempNode->shadowAncestorNode();

				element = static_cast< WebCore::Element* >(tempNode);

				if( node )
				{
					if( node->isLink()|| element->isLink() 
						|| equalIgnoringCase(element->tagName(), TAG_IMG)  )
					{
						if( equalIgnoringCase(element->tagName(), TAG_IMG) &&  element->isLink())
							*elementType = DRAG_DROP_LINK_IMAGE;
						else if(equalIgnoringCase(element->tagName(), TAG_IMG))
							*elementType = DRAG_DROP_IMAGE;
						else if(element->isLink() && !(element->textContent().isEmpty()) )
							*elementType = DRAG_DROP_LINK_TEXT;	
						else
							*elementType = DRAG_DROP_LINK;		

						*ctElement = (int)node;
						ptRect = element->getRect();

#if defined(_SFB_BACKINGSTORE)
						pTempPt = m_pBackingStore->BackingStoreToWindow( frame->view()->contentsToWindow(WebCore::IntPoint(ptRect.x(), ptRect.y())));
#else
                        pTempPt = frame->view()->contentsToWindow(WebCore::IntPoint(ptRect.x(), ptRect.y()));
#endif //(_SFB_BACKINGSTORE)

						*actualX = pTempPt.x();
						*actualY = pTempPt.y();

#if defined(_SFB_BACKINGSTORE)
						*height = ptRect.height()*m_pBackingStore->GetZoom();
						*width = ptRect.width()*m_pBackingStore->GetZoom();
#else
                        *height = ptRect.height();
						*width = ptRect.width();
#endif //(_SFB_BACKINGSTORE)

						if(*height <= 0 || *width <=0 )
							return HRESULT_NOK;

						return HRESULT_OK;
					}
					else if ( node->renderer()->isText() )
					{
						*elementType = DRAG_DROP_TEXT;
						*ctElement = (int)node;
						ptRect = node->getRect();
						WebCore:: InlineBox* tempInlineBox = node->renderer()->inlineBox();

#if defined(_SFB_BACKINGSTORE)
						pTempPt = m_pBackingStore->BackingStoreToWindow(frame->view()->contentsToWindow(WebCore::IntPoint(ptRect.x() + tempInlineBox->xPos() , ptRect.y()+ tempInlineBox->yPos())));
#else
                        pTempPt = frame->view()->contentsToWindow(WebCore::IntPoint(ptRect.x() + tempInlineBox->xPos() , ptRect.y()+ tempInlineBox->yPos()));
#endif  //(_SFB_BACKINGSTORE)

						*actualX = pTempPt.x();
						*actualY = pTempPt.y();

#if defined(_SFB_BACKINGSTORE)
						*height = ptRect.height()*m_pBackingStore->GetZoom();
						*width = ptRect.width()*m_pBackingStore->GetZoom();
#else
	                    *height = ptRect.height();
						*width = ptRect.width();
#endif  //(_SFB_BACKINGSTORE)

						if(*height <= 0 || *width <=0 )
							return HRESULT_NOK;

						return HRESULT_OK;
							
					}
				}
			}
			return HRESULT_NOK;
	}
	
	/*
	* @desc gets a current Drag element informatioin.
	* @param height, width,actualX, actualY 
	* @return HRESULT TRUE/FALSE
	*/
	HRESULT WebControl::getCurrentDragElementInfo(void* ctElement, char* srcUrl, char* hrefUrl, char** textString )
	{
		WebCore::Frame *frame = core(m_pMainWebFrame);
		WebCore::Node* node = static_cast< WebCore::Node* >(ctElement);

		WebCore::Node* tempNode = node;
		
		while (tempNode && !tempNode->isElementNode())
			tempNode = tempNode->parentNode();
		if (tempNode)
			tempNode = tempNode->shadowAncestorNode();
		

		WebCore::Element* element = static_cast< WebCore::Element* >(tempNode);

		if( equalIgnoringCase(element->tagName(), TAG_IMG) )
		{
			WebCore::String url = frame->document()->completeURL(element->getAttribute("src"));
			AcStrcpy( srcUrl, url.latin1().mutableData() );
		}
	
		WebCore::String tempString = node->textContent();
		if( !tempString.isEmpty() )
		{
			*textString = (char*)SFBalMemCalloc( 1, tempString.impl()->length() + 1);
			AcStrcpy( *textString, tempString.latin1().data() );
		}

	    while(tempNode)
		{
			if( tempNode->isLink() )
			{
				WebCore::String url = ( static_cast< WebCore::Element* > (tempNode) )->href();
				AcStrcpy( hrefUrl, url.latin1().mutableData() );
				break;
			}
			tempNode = tempNode->parentNode();
		}
		return HRESULT_OK;
	}
#endif

/*
 * @desc gets a current page informatioin.
 * @param out_pImageInfo : pointer to IPageInfo
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::getCurrentPageInfo( IPageInfo** out_pPageInfo )
{
	WebPageInfo *pPageInfo = NULL;
	//Sankalp -- CQ3372
	if(true == MemoryManager::IsOOMSent())
	{ 
		*out_pPageInfo = NULL;
		return HRESULT_NOK;
	}

	pPageInfo = new WebPageInfo( this );

	if( pPageInfo == NULL )
	{
		*out_pPageInfo = NULL;
		return HRESULT_NOK;
	}

		*out_pPageInfo = pPageInfo;
		return HRESULT_OK;

}

/*
 * @desc checks to exist a displayed contents currently.
 * @param
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::isExistDisplayedContents()
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc gets state whether plugin is enabled or not with given mime type.
 * @param in_PlugInType : plugin types
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::getPlugInEnable( char* in_pszPlugInMime )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc gets a position of virtual pointer
 * @param out_x, out_y : x, y
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::getVirtualPointerCurrentPos( int* out_x, int* out_y )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc loads URL with POST method
 * @param in_pszUrl : posted url
 * @param in_pszPostData : posted data
 * @param in_PostDataLen : length of the posted data
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::postURL( char* in_pszUrl, char* in_pszPostData, int in_PostDataLen )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}


/*
 * @desc loads URL
 * @param In_pszUrl : URL
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::loadURL(const char* in_pszUrl )
{
	MemoryManager::ResetOOMState();

    //Check if the shared timwe is valid. If not then initiaze the shared timer
    if(false == isSharedTimerValid())
        WebCore::setSharedTimerFireTime(0);

	DeprecatedString string = DeprecatedString::fromUtf8(in_pszUrl);
	core(m_pMainWebFrame)->loader()->load(ResourceRequest(KURL(string)));
	Frame* frame = m_pMainWebFrame->GetFrame();
	if (!frame->page()->backForwardList()->size())	{
		if (m_initialUrl)
			SFBalMemFree(m_initialUrl);
		int strLen = AcStrlen (in_pszUrl) + 1;
		m_initialUrl = (char *)SFBalMemAlloc(strLen*sizeof(char));
		if (m_initialUrl)
			AcStrcpy(m_initialUrl, in_pszUrl);
	}

	WebCore::bDoResetSSRDefaultStyle = true ;
	return HRESULT_OK;
}

/*
 * @desc gets current document URL
 * @param none
 * @return const char* docUrl
 */
const char* WebControl::getCurrentDocURL()
{
	Frame* frame = m_pMainWebFrame->GetFrame();
	if(frame && frame->document())
		return frame->document()->baseURL().ascii();

	return NULL;
}

/*
 * @desc check mode that a browser engine is image save mode or not
 * @param
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::isImageSaveMode( void )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc check image is on the point or not
 * @param
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::isPointOnImage( int x, int y )
{
	//WebCore::Frame* frame = m_pMainWebFrame->GetFrame();

	//WebCore::FrameView *frameview = frame->view();
	//WebCore::IntPoint ptContent = frameview->windowToContents(WebCore::IntPoint(x,y));
	//WebCore::Document* NewDocument=frame->documentAtPoint(ptContent);

	//WebCore::Element* element =NewDocument->elementFromPoint(ptContent.x(), ptContent.y());
	//
	//WebCore::String tagName =element->tagName();

	// if( equalIgnoringCase( tagName, TAG_IMG)  )
	// {
	//	 return HRESULT_TRUE;
	// }
	// else
	// {
	//	 return HRESULT_FALSE;
	// }

	WebCore::Settings* settings = m_pPage->settings();
	
	if( settings )
		if( settings->loadsImagesAutomatically() == false )
			return HRESULT_FALSE;

	HRESULT hr;
	IPropertyMap *pMap = NULL;
	hr = getElementByPoint( &pMap,  x,  y );
	if( HRESULT_ISFAIL(hr))
	{
		SysDebug(( MID_EXCEPTION, "WebControl::isPointOnImage : getElementByPoint failed\n" ));
		return HRESULT_NOK;
	}

	IDOMNode *pDOMNode = NULL;
	hr = pMap->getValue( ELEMENT_PROPERTY_DOM_NODE_KEY, (void**)&pDOMNode );
	if( HRESULT_ISFAIL(hr))
	{
		SysDebug(( MID_EXCEPTION, "WebControl::isPointOnImage : pMap->getValue failed\n" ));
		goto _FAILONERROR;
	}

	char tagName[20];
	hr = pDOMNode->getName( tagName, 20 );
	if( HRESULT_ISFAIL(hr))
	{
		SysDebug(( MID_EXCEPTION, "WebControl::isPointOnImage : pDOMNode->getName failed\n" ));
		goto _FAILONERROR;
	}

	if( AcStricmp( tagName, TAG_IMG ) == 0 )
	{
		WebCore::Node* pNode = ((DOMNode*)pDOMNode)->getWebCoreNode();

		if(NULL != pNode)
		{
			HTMLImageElement* imageNode = static_cast<HTMLImageElement *>(pNode);

			bool errorOccurred = imageNode->cachedImage() ? imageNode->cachedImage()->errorOccurred() : true;
			if(true == errorOccurred)
			{
				goto _FAILONERROR;
			}
		}
		else
		{
			goto _FAILONERROR;
		}
	if( pMap != NULL ) pMap->Release();
	if( pDOMNode != NULL ) pDOMNode->Release();

		return HRESULT_TRUE;
	}
_FAILONERROR:
	if( pMap != NULL ) pMap->Release();
	if( pDOMNode != NULL ) pDOMNode->Release();
	return FALSE;

}

/*
 * @desc check mode that a browser engine is search text mode or not
 * @param
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::isSearchTextMode( void )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc check mode that a browser engine is virtual pointer mode or not
 * @param
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::isVirtualPointerMode( void )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc passes key pressed event to browser engine
 * @param in_KeyValue : key value is based on platform
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::KeyPressed( int keyCode )
{

	return HandleKeyEvent( keyCode,  0 );
}

/*
 * @desc passes key released event to browser engine
 * @param in_KeyValue : key value is based on platform
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::KeyReleased( int keyCode )
{
	return HandleKeyEvent( keyCode, 1 );
}

/*
 * @desc loads contents with contents data
 * @param in_pszUrl : URL
 * @param in_pszData : contents
 * @param in_DataLen : length
 * @param in_pszMimeType : mimetype of contents
 * @param in_pszCharset : charset of contents
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::loadData( char* in_pszUrl, char* in_pszData, int in_DataLen, char* in_pszMimeType, char* in_pszCharset )
{
	MemoryManager::ResetOOMState();

    //Check if the shared timwe is valid. If not then initiaze the shared timer
    if(false == isSharedTimerValid())
        WebCore::setSharedTimerFireTime(0);

    //WebKitShpPagePrivate* pageData = page->privatePage;
    //WebKitShpFramePrivate* frameData = pageData->mainFrame->privateFrame;

	WebCore::KURL url( WebCore::DeprecatedString::fromUtf8(in_pszUrl) );

    //char* _content = (char *)malloc(in_DataLen);
    //AcStrcpy(_content, in_pszData);

	RefPtr<WebCore::SharedBuffer> sharedBuffer = new WebCore::SharedBuffer(in_pszData, in_DataLen );
	WebCore::SubstituteData substituteData(sharedBuffer.release(), WebCore::String(in_pszMimeType),
									WebCore::String(in_pszCharset), WebCore::KURL("about:blank"), url);

	m_pMainWebFrame->GetFrame()->loader()->load( WebCore::ResourceRequest(url), substituteData);

	return HRESULT_OK;
}

/*
 * @desc loads contents with file name
 * @param in_pszPathFileName : File name included path. ( without filescheme )
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::loadFile( char* in_pszPathFileName )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc loads contents with file name that is saved by a browser engine and is located in saved page directory.
 * @param in_pszPathFileName : File Name included path
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::loadSavedPage( char* in_pszPathFileName )
{
	MemoryManager::ResetOOMState();

    //Check if the shared timwe is valid. If not then initiaze the shared timer
    if(false == isSharedTimerValid())
        WebCore::setSharedTimerFireTime(0);

    WebCore::KURL url( WebCore::DeprecatedString::fromUtf8(in_pszPathFileName) );
	m_pMainWebFrame->GetFrame()->loader()->load( WebCore::ResourceRequest(url));
	return HRESULT_OK;
}

/*
 * @desc moves focus to next text
 * @param
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::nextSearchText( char* in_pszSearchText )
{
#ifdef _SFB_SEARCH_STRING_IN_PAGE //Jan-16 Vamsi/Pramod
	bool bstartInSelection = FALSE, bWrapFlag = TRUE, bCaseFlag = FALSE;
	bool bRet ;

	Frame* frame = core(m_pMainWebFrame);

	bRet = frame->findString(in_pszSearchText, true, bCaseFlag, bWrapFlag, bstartInSelection);
	return (HRESULT)bRet;

#else
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
#endif
}

/*
 * @desc gets a title of current page
 * @param out_pszTitle : pointer to title
 * @param in_TitleLen : length of title
 * @return HRESULT TRUE/FALSE
 */
// removed
//HRESULT WebControl::getPageInfoTitle( char* out_pszTitle, int in_TitleLen )
//{
//	__NotImplemented(); return HRESULT_NOTSUPPORT;
//}

/*
 * @desc gets a URL of current page
 * @param out_pszUrl : pointer to URL
 * @param in_UrlLen : length of URL
 * @return HRESULT TRUE/FALSE
 */
// removed
//HRESULT WebControl::getPageInfoURL( char* out_pszUrl, int in_UrlLen )
//{
//	__NotImplemented(); return HRESULT_NOTSUPPORT;
//}


// bs0425.lee modified for Page Preview.
//#ifdef _SFB_PAGE_PREVIEW

HRESULT WebControl::pagePreviewscrollPane(bool scroll_event, int rec_x, int rec_y )
{

	int pre_recX, pre_recY;
	int dx,dy;
	int m_Y,m2_Y,m3_Y,m_X,m2_X,m3_X;

	pre_recX=(min_X	+ max_X)/2;
	pre_recY=(min_Y + max_Y)/2;
	dx=rec_x-pre_recX;
	dy=rec_y-pre_recY;

	Frame* frame = core(m_pMainWebFrame);
	FrameView* view = frame->view();

	WmRect preview_Rect;
	IntRect viewRect = enclosingIntRect(frame->view()->visibleContentRectConsideringExternalScrollers());

	WmGetWindowClientRect( m_hWin , &preview_Rect );

		frame->view()->scrollBy(dx,dy);

	WmRect wmRect;
	wmRect.x = 0;
	wmRect.y = 0;
	wmRect.w = preview_Rect.w;
	wmRect.h = preview_Rect.h;

	drawScreen(wmRect.x,wmRect.y ,wmRect.w,wmRect.h);

	return HRESULT_OK;

}

//#endif




/*
 * @desc scrolls by given x, y
 * @param x,y :
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::scrollPane( int dx, int dy )
{
 //#if defined(_SFB_BACKINGSTORE)
  //2008.09.30  bs0425.lee modified for touch scrollbar. 
  	IntPoint mousePosition;
	IntRect track;
	IntRect extendtrack;
       int recX,recY;
	float Height,Width;
	float Height2,Width2;
	int dxBar,dyBar;
	int dxComp,dyComp;
	WmRect rc;
	float changedValue;

	SysDebug(( MID_EXCEPTION, "WebControl::scrollPane    dx : (%d)  ,dy : (%d))\n", dx,dy));

	WebCore::Frame* m_frame = m_pMainWebFrame->GetFrame();

	//some cases it comes to touchmove with out hitting touchpress
	WebCore::NativeEditField* pNativeEdit = NativeEditField::getNativeEditFieldInstance();
    WebAssert(pNativeEdit);
	if ( pNativeEdit->isActivated() )
    {
		   pNativeEdit->_sendTextToBrowserEngine();
		   pNativeEdit->unsetFocusedNode();
		   pNativeEdit->deactivate();
 	}

	WebCore::NativeComboField* pNativeCombo = NativeComboField::getNativeComboFieldInstance();
    WebAssert(pNativeCombo);
   	if ( pNativeCombo->isActivated() )
    {
		   pNativeCombo->SetSelectedIndex();
		   pNativeCombo->unsetFocusedNode();
		   pNativeCombo->deactivate();
 	}

#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)
	WebCore::NativeListBox* pNativeList = NativeListBox::getNativeListBoxInstance();
	WebAssert(pNativeList);

	if( pNativeList->isActivated() )
    {
		   pNativeList->unsetFocusedNode();
		   pNativeList->deactivate();
 	}
#endif //#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)

	//2008.08.07 bs0425.lee modified for long press scroll. 
	if (bEnableZoom==TRUE)
	{
		zoomPointX=zoomPointX+dx;
		zoomPointY=zoomPointY+dy;
		PressZoomMove(dx,dy);
		return HRESULT_OK;
	}
#if defined(_SFB_BACKINGSTORE)
	WmGetWindowClientRect ( GetHWIN(), &rc );

	if(scrollbarPressed)
        {
		recX=scrollBarPoint.x()-dx;
		recY=scrollBarPoint.y()-dy;
		scrollBarPoint.setX(recX);
		scrollBarPoint.setY(recY);

		IntRect track=m_pBackingStore->getBackingTrackRect();

		//extendtrack=touchScrollbar_ex(track);
	
		  changedValue=m_pBackingStore->GetZoom();
		if(innerTrack)  // modified -  if((track.contains(scrollBarPoint))&&(innerTrack))
		{
			Width = m_frame->view()->contentsWidth();
			Height = m_frame->view()->contentsHeight();
	
			Width=Width*changedValue;
			Height=Height*changedValue;

			Width2=rc.w;
			Height2=rc.h;

			(int)dxBar=(Width/Width2)*(int)dx;
			(int)dyBar=(Height/Height2)*(int)dy;

			// bs0425.lee modified to distinguish scrollbar scroll from plane scroll. 
		       if(dxBar<0)
		       	dxComp=(-dxBar);
		       else
		      		dxComp=dxBar;

		      	if(dyBar<0)
		      	   	dyComp=(-dyBar);
		       else
		      		dyComp=dyBar;
		    
		 #if defined(_SFB_BACKINGSTORE)
			
			  if( m_pBackingStore )
			  {
				  	if(track.width()>track.height())
				  	{	if(dxComp>=dyComp)
							m_pBackingStore->ScrollBy(-dxBar,0);
					else
							m_pBackingStore->ScrollBy(dx,dy);
					}
					else
					{	if(dyComp>=dxComp)
							m_pBackingStore->ScrollBy(0,-dyBar);
						else
							m_pBackingStore->ScrollBy(dx,dy);
					}
			  }
		#else
			if(track.width()>track.height())
			{	
				if(dxComp>=dyComp)
					m_frame->view()->scrollBy(-dxBar,0);
				else
					m_pBackingStore->ScrollBy(dx,dy);
			}		
			else
			{	if(dyComp>=dxComp)
					m_frame->view()->scrollBy(0,-dyBar);
			else
					m_pBackingStore->ScrollBy(dx,dy);
			}
	    #endif

		}
		else
		{
			innerTrack=FALSE;
		}
			
	}
	else
	{
     				  if( m_pBackingStore )
						m_pBackingStore->ScrollBy( dx, dy );

					  innerTrack=FALSE;

         }

#else  // _SFB_BACKINGSTORE
   	WebCore::Frame* frame = m_pMainWebFrame->GetFrame();

		frame->view()->scrollBy(dx, dy);
#endif // _SFB_BACKINGSTORE
// bs0425.lee 
bUseScroll=true;

	return HRESULT_OK;
}




/*
 * @desc flick scrolls by given x, y
 * @param x,y :
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::flickScrollPane( int dx, int dy )
{
	 float scrollX,scrollY;
	 int cntX=0;
	 int cntY=0;
	 int scrollRangeX;
	 int scrollRangeY;
	 int maxScrollValueX=0;
	 int maxScrollValueY=0;
	 int tempX=0;
	 int tempY=0;
	 int scCnt=0;
	 int scMax=0;
	 int scX=0;
	 int scY=0;

	 WebCore::Frame* frame = m_pMainWebFrame->GetFrame();
									
	scrollX=dx;
	scrollY=dy;

	  if(scrollX>FLICK_MAX)
			scrollX=FLICK_MAX ;
	  else if(scrollX<-FLICK_MAX)
			scrollX=-FLICK_MAX ;
	  
	  if(scrollY>FLICK_MAX)
		  scrollY=FLICK_MAX ;
	  else if(scrollY<-FLICK_MAX)
		  scrollY=-FLICK_MAX;

	  scrollRangeX=(float)scrollX*(float)FLICK_RATE;
	  scrollRangeY=(float)scrollY*(float)FLICK_RATE;

	  SysDebug(( MID_EXCEPTION, "WebControl::flickScrollPane    scrollRangeX : (%d)  ,scrollRangeY : (%d))\n", scrollRangeX,scrollRangeY));
	  if(scrollRangeX>0)
	  {  
		  tempX=1;  // initial scroll base . 
		 while(cntX<scrollRangeX)
		 {	
			  tempX=tempX+FLICK_OFFSET;
			  cntX=cntX+tempX;
			  maxScrollValueX=maxScrollValueX+FLICK_OFFSET;
		 }
	  }
	  else
	  {   
		  tempX=-1;
		   while(cntX>scrollRangeX)
		 {	
			  tempX=tempX-FLICK_OFFSET;
			  cntX=cntX+tempX;
			  maxScrollValueX=maxScrollValueX-FLICK_OFFSET;
		 }
	  }

	  if(scrollRangeY>0)
	  {
		  tempY=1;
		  while(cntY<scrollRangeY)
		  {	
			  tempY=tempY+FLICK_OFFSET;
			  cntY=cntY+tempY;
			  maxScrollValueY=maxScrollValueY+FLICK_OFFSET;
			}
	  }
	  else
	  {
		  tempY=-1;
		   while(cntY>scrollRangeY)
		 {	
			  tempY=tempY-FLICK_OFFSET;
			  cntY=cntY+tempY;
			  maxScrollValueY=maxScrollValueY-FLICK_OFFSET;
		 }
	  }

//	   SysDebug(( MID_EXCEPTION, "maxScrollValueX : (%d)  ,maxScrollValueY : (%d))\n", maxScrollValueX,maxScrollValueY));

	  if((scrollX>FLICK_MIN)||(scrollX<-FLICK_MIN))
	  { 
		  if((scrollY>FLICK_MIN)||(scrollY<-FLICK_MIN))
		  {   
			  //  dx and dy scroll... 
			  if((maxScrollValueX>0)&&(maxScrollValueY>0))
			  {
					if(maxScrollValueX>maxScrollValueY)
						  scMax=maxScrollValueX;
					  else
						{scMax=maxScrollValueY;
						}
						
					  scX=maxScrollValueX;
					  scY=maxScrollValueY;
					  
					  for(scCnt=scMax;scCnt>0;scCnt--){

						   if((scCnt<=(scMax-maxScrollValueX))||(scX<1))
							   scX=1;
						   if((scCnt<=(scMax-maxScrollValueY))||(scY<1))
							   scY=1;
										
							
							#if defined(_SFB_BACKINGSTORE)
				   			  if( m_pBackingStore )
									m_pBackingStore->ScrollBy( scX, scY );
							#else
										frame->view()->scrollBy(scX, scY);
							#endif // _SFB_BACKINGSTORE

						//	SysDebug(( MID_EXCEPTION, "scrollX and scrollY AFTER --scrollX : (%d) , b: (%d)\n", scX, scY));

							scX=scX-FLICK_OFFSET;
							scY=scY-FLICK_OFFSET;

							scCnt=scCnt-FLICK_OFFSET+1;

					}
			  }  //  ((maxScrollValueX>0)&&(maxScrollValueY>0))
			  else if(maxScrollValueX>0)
			  {		
					  if(maxScrollValueX>-maxScrollValueY)
						  scMax=maxScrollValueX;
					  else
						{scMax=-maxScrollValueY;
						}
					
					  scX=maxScrollValueX;
					  scY=maxScrollValueY;
					  
					  for(scCnt=scMax;scCnt>0;scCnt--){
	
						   if((scCnt<=(scMax-maxScrollValueX))||(scX<1))
							   scX=1;
						   if((scCnt<=(scMax+maxScrollValueY))||(scY>-1))
							   scY=-1;
										
							#if defined(_SFB_BACKINGSTORE)
				   			  if( m_pBackingStore )
									m_pBackingStore->ScrollBy(scX, scY );
							#else
										frame->view()->scrollBy(scX, scY);
							#endif // _SFB_BACKINGSTORE

					//		SysDebug(( MID_EXCEPTION, "scrollX and scrollY AFTER --scrollX : (%d) , b: (%d)\n",  scX, scY));
						
							scX=scX-FLICK_OFFSET;
							scY=scY+FLICK_OFFSET;

							scCnt=scCnt-FLICK_OFFSET+1;

					}
				}// maxScrollValueX>0
			   else if(maxScrollValueY>0)
			  {		
					  if(maxScrollValueY>-maxScrollValueX)
						  scMax=maxScrollValueY;
					  else
						{scMax=-maxScrollValueX;
						}
					
					  scX=maxScrollValueX;
					  scY=maxScrollValueY;

					  for(scCnt=scMax;scCnt>0;scCnt--){
																
						   if((scCnt<=(scMax+maxScrollValueX))||(scX>-1))
							   scX=-1;
						    if((scCnt<=(scMax-maxScrollValueY))||(scY<1))
							   scY=1;
										
							#if defined(_SFB_BACKINGSTORE)
				   			  if( m_pBackingStore )
									m_pBackingStore->ScrollBy( scX, scY);
							#else
										frame->view()->scrollBy(scX,scY);
							#endif // _SFB_BACKINGSTORE

						//	SysDebug(( MID_EXCEPTION, "scrollX and scrollY AFTER --scrollX : (%d) , b: (%d)\n",  scX, scY));

							scX=scX+FLICK_OFFSET;
							scY=scY-FLICK_OFFSET;

							scCnt=scCnt-FLICK_OFFSET+1;

					}
				}// maxScrollValueY>0
			  else
			  {		
					  if(maxScrollValueX>maxScrollValueY)
						  scMax=-maxScrollValueY;
					  else
						{scMax=-maxScrollValueX;
						}

					  scX=maxScrollValueX;
					  scY=maxScrollValueY;

					  for(scCnt=scMax;scCnt>0;scCnt--){

						   if((scCnt<=(scMax+maxScrollValueX))||(scX>-1))
							   scX=-1;
						   if((scCnt<=(scMax+maxScrollValueY))||(scY>-1))
							   scY=-1;
										
							#if defined(_SFB_BACKINGSTORE)
				   			  if( m_pBackingStore )
									m_pBackingStore->ScrollBy( scX, scY);
							#else
										frame->view()->scrollBy(scX, scY);
							#endif // _SFB_BACKINGSTORE

						//	SysDebug(( MID_EXCEPTION, "scrollX and scrollY AFTER --scrollX : (%d) , b: (%d)\n",  scX, scY));

							scX=scX+FLICK_OFFSET;
							scY=scY+FLICK_OFFSET;

							scCnt=scCnt-FLICK_OFFSET+1;
					}
				}// maxScrollValueX<0, maxScrollValueY<0
			} // dx and dy scroll 
			else 
			{
				// only dx scroll. 
				 if(maxScrollValueX>0)
					scMax=maxScrollValueX;
				 else
					scMax=-maxScrollValueX;
				 
				  scX=maxScrollValueX;

				 for(scCnt=scMax;scCnt>0;scCnt--){
					 if(maxScrollValueX>0)
					 {  
						 if(scX<1)
							 scX=1;
					 }
					 else
					 {	
						if(scX>-1)
							scX=-1;
					 }

					#if defined(_SFB_BACKINGSTORE)
				   	  if( m_pBackingStore )
							m_pBackingStore->ScrollBy( scX, 0 );
					#else
								frame->view()->scrollBy(scX, 0);
					#endif // _SFB_BACKINGSTORE

				//	SysDebug(( MID_EXCEPTION, "only scrollX  --scrollX : (%d))\n",  scX));

					 if(maxScrollValueX>0)
						  scX=scX-FLICK_OFFSET;
					 else
						 scX=scX+FLICK_OFFSET;

					scCnt=scCnt-FLICK_OFFSET+1;
				}
			}  // only dx scroll. 
	  } //  if((scrollX>FLICK_MIN)||(scrollX<-FLICK_MIN))
	  else if((scrollY>FLICK_MIN)||(scrollY<-FLICK_MIN))
	  {			// only dy scroll. 
				 if(maxScrollValueY>0)
					scMax=maxScrollValueY;
				 else
					scMax=-maxScrollValueY;
				 
				 scY=maxScrollValueY;

				 for(scCnt=scMax;scCnt>0;scCnt--){
					 if(maxScrollValueY>0)
					 {
						if(scY<1)
							scY=1;
					 }
					 else
					 {
						if(scY>-1)
							scY=-1;
					 }

					#if defined(_SFB_BACKINGSTORE)
				   	  if( m_pBackingStore )
							m_pBackingStore->ScrollBy( 0, scY);
					#else
								frame->view()->scrollBy(0, scY);
					#endif // _SFB_BACKINGSTORE

			//		SysDebug(( MID_EXCEPTION, "only scrollY  --scrollY : (%d))\n",  scY));

					if(maxScrollValueY>0)
						scMax=scY-FLICK_OFFSET;
					else
						scY=scY+FLICK_OFFSET;

					scCnt=scCnt-FLICK_OFFSET+1;
				}
		}  // only dy scroll. 

	 //  SysDebug(( MID_EXCEPTION, "======== end scroll ==================================="));

	return HRESULT_OK;
}






/*
 * @desc zoom level was changed by given x, y
 * @param x,y :
 * @return HRESULT TRUE/FALSE
 */
// bs0425.lee 2008.08.12 modified for long press zooml. 
HRESULT WebControl::PressZoomMove(int zoomX, int zoomY )
{
	if( zoomY == 0 )
			return HRESULT_OK;


	WebCore::Frame* frame = m_pMainWebFrame->GetFrame();

	WmRect rc;

	WmGetWindowClientRect ( GetHWIN(), &rc );

	//float multPerPicxel = 0;
	float multi;
	float offset;
	float factorValue;
	int reverseZoom;

	totalZoomY=totalZoomY+zoomY;
#if defined(_SFB_BACKINGSTORE)
	factorValue=m_pBackingStore->GetZoom();//frame->zoomFactor();
#else
	factorValue=m_pMainWebFrame->GetFrame()->zoomFactor();
#endif
	if(totalZoomY<0)
	{
		reverseZoom=-totalZoomY;
		offset=(2.0/(float)rc.h)*(float)reverseZoom;
#if defined(_SFB_BACKINGSTORE)
		zoomValue=m_pBackingStore->GetZoom()/(1+offset);
#else
		zoomValue=m_pMainWebFrame->GetFrame()->zoomFactor()/(1+offset);
#endif
	}
	else
	{	

		offset=(2.0/(float)rc.h)*(float)totalZoomY;
#if defined(_SFB_BACKINGSTORE)
		zoomValue=m_pBackingStore->GetZoom()*(1+offset);
#else
		zoomValue=m_pMainWebFrame->GetFrame()->zoomFactor()*(1+offset);
#endif
	}

#if defined(_SFB_BACKINGSTORE)
	m_pBackingStore->Zooming(zoomValue);
#endif

	return TRUE;
}

/*
 * @desc moves focus to previous text
 * @param
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::prevSearchText( char* in_pszSearchText )
{

#ifdef _SFB_SEARCH_STRING_IN_PAGE //Jan-16 Vamsi/Pramod
	bool bstartInSelection = FALSE, bWrapFlag = TRUE, bCaseFlag = FALSE;
	bool bRet ;

	Frame* frame = core(m_pMainWebFrame);

	bRet = frame->findString(in_pszSearchText, false, bCaseFlag, bWrapFlag, bstartInSelection);

	return (HRESULT)bRet;

#else
	__NotImplemented();
	return HRESULT_NOTSUPPORT;

#endif
}

/*
 * @desc reloads current contents
 * @param
 * @return HRESULT TRUE/FALSE
 */

HRESULT WebControl::reloadPage( void )
{
	char* szUrl = NULL;
	MemoryManager::ResetOOMState();

    //Check if the shared timwe is valid. If not then initiaze the shared timer
    if(false == isSharedTimerValid())
    WebCore::setSharedTimerFireTime(0);

	Frame* frame = m_pMainWebFrame->GetFrame();

	if (frame->page()->backForwardList()->size())
		core(m_pMainWebFrame)->loader()->reload();
	
    //CQ SFB00003324 
	else if(m_initialUrl)
    {
		int strLen = AcStrlen (m_initialUrl) + 1;
		szUrl = (char *)SFBalMemAlloc(strLen*sizeof(char));
		if (szUrl)
		{
			AcStrcpy(szUrl, m_initialUrl);
			loadURL(szUrl) ;
		}
		
    }
	WebCore::bDoResetSSRDefaultStyle = true ;
	if(szUrl)
	{
		SFBalMemFree(szUrl);
	}
	return HRESULT_OK;
}

typedef enum {
    WEBKIT_SHP_DIFF_RES,
	WEBKIT_SHP_SAME_NAME_RES_DIFF,
    WEBKIT_SHP_SAME_RES
} WEBKIT_SHP_RES_RESOLUTION;

// returns true if the image has been saved already else returns false
WEBKIT_SHP_RES_RESOLUTION is_already_saved(const Vector<WebCore::Node*>& imgDataList,const WebCore::Node* image)
{
	using WebCore::String;

	WEBKIT_SHP_RES_RESOLUTION result = WEBKIT_SHP_DIFF_RES;
	for(int i=0;i<imgDataList.size();i++)
	{
		String rPreOrgPath = imgDataList[i]->attributes()->getAttributeItem("src")->value();
		String rOrgPath = image->attributes()->getAttributeItem("src")->value();
		//if paths are same then do not compare the img attributes
		if(rPreOrgPath == rOrgPath)
		{
			result = WEBKIT_SHP_SAME_RES;
			break;
		}
		else
		{
	        int slashIndex = rPreOrgPath.reverseFind('/');
			String rPreFileName = ( -1 != slashIndex )?rPreOrgPath.substring(slashIndex + 1):rPreOrgPath;
			slashIndex = rOrgPath.reverseFind('/');
			String rFileName = ( -1 != slashIndex )?rOrgPath.substring(slashIndex + 1):rOrgPath;
			if(rPreFileName == rFileName)
			{
				Image* rPreImage = (static_cast<RenderImage *>(imgDataList[i]->renderer()))->cachedImage()->image();
				Image* rImage = static_cast<RenderImage *>(image->renderer())->cachedImage()->image();
				if(rPreImage->size() == rImage->size() &&
					rPreImage->width() == rImage->width() &&
					rPreImage->height() == rImage->height())
				{
					//path not same,just check that the img attributes are different.
					result = WEBKIT_SHP_SAME_RES;
					break;
				}
				else
				{
					result = WEBKIT_SHP_SAME_NAME_RES_DIFF;
					break;
				}
			}
		}
	}
	return result;
}

WebCore::String
validateFilename(WebCore::String orgPath,WebCore::String filesPath)
{
		using WebCore::String;
		String fileName("");
		String utf8FileName("");
		String fileExt("");
		String fileUnqName("");
		String newPath("");
		char  pTempName[BRPL_FILENAME_STR_LEN_MAX+1] ;   /*Added by siso BRPL_FILENAME_STR_LEN_MAX  46*/
		char szUrl[_BROWSER_MAX_URL+1];
		AcMemset( szUrl, 0x00, sizeof(szUrl) );
		AcMemset( pTempName, 0x00, sizeof(pTempName) ); 

		SFBalExtractNameFromURL((char*)orgPath.latin1().data(), szUrl, orgPath.impl()->length());
		fileName = szUrl;
		int slashIndex = orgPath.reverseFind('.');
		if( -1 != slashIndex )
			fileExt = orgPath.substring(slashIndex);

		slashIndex = fileExt.find('&');
		if(slashIndex == -1)
			fileName.append(fileExt);
		if( -1 != slashIndex && slashIndex <= BRPL_EXTENSION_STRING_LEN_MAX)
		{
			fileExt = fileExt.substring(0,slashIndex);
			fileName.append(fileExt);
		}

		SFBalGetValidFilename(szUrl,(char*)fileName.latin1().data(), fileName.impl()->length());
		/*Added by siso to fix crash in www.expage.com*/
		
		if (AcStrlen(szUrl) > BRPL_FILENAME_STR_LEN_MAX)
		{
			AcStrncpy(pTempName, szUrl, BRPL_FILENAME_STR_LEN_MAX);

			fileName = pTempName;
		}
		else
		{
		fileName = szUrl;
		}
		
		/*End of addition to fix crash in www.expage.com*/
		newPath = filesPath + fileName;

		if(SFBalIsFileExist((char*)newPath.deprecatedString().ascii()) == TRUE)
		{
			 int count = fileNameMap.get(fileName);
			 fileUnqName=fileUnqName.number(++count);
             fileUnqName.append("_");
			 fileNameMap.add(fileName,count);
			 fileUnqName.append(fileName);
        	 fileName = fileUnqName.copy();
		}
		utf8FileName = KURL::encode_string(fileName.deprecatedString());
		return fileName;
}


void WebControl::getLastViewedImageUrl( char *url )
{
#ifdef SFB_IVIEW_IMAGE  
	// TODO : Call the Document class function
	// Romit
	WebCore::String imgUrl((const char *)url);
	WebCore::Frame *frame = core( m_pMainWebFrame );
	frame->document()->setFocusedImage(imgUrl);
#else
		__NotImplemented();
#endif
}

HRESULT WebControl::getImagesInfo(bool bVisibleOnly, ImageSaveInfo*** imageList)
{
#ifdef SFB_IVIEW_IMAGE  
	Frame* frame = core( m_pMainWebFrame);
	PassRefPtr<HTMLCollection> imageCollection = frame->document()->images();
	FrameView* view = frame->view();
	int contentsx = view->contentsX();				// view port x position
	int contentsy = view->contentsY();				// view port y position
	int visibleW = view->visibleWidth();			// width of the display
	int visibleH = view->visibleHeight();			// height of the display

	IntRect viewRect(contentsx, contentsy, visibleW, visibleH);

	CachedImage* cachedImage;
    RenderImage *renderImage;

	String filesPath;
	WebCore::String fileExt;
	char	szUrl[_BROWSER_MAX_URL+1];
	AcMemset( szUrl, 0x00, sizeof(szUrl) );

	int count;
	getCountImages(&count);
	if(count <= 0)
		return HRESULT_NOK;
	// Allocate space for counted images
	// SFBalMalloc SFB00000680savePage

	/*Malloc commented by vijay and used SFBalMemCalloc*/

	//*(imageList) = (ImageSaveInfo**) SFBalMemAlloc(count * sizeof(ImageSaveInfo));
	*(imageList) = (ImageSaveInfo**) SFBalMemCalloc(count , sizeof(ImageSaveInfo));

	WebCore::String orgPath;
    WebCore::Node * tempNode;	
	int i=0, j=0;
    bool isRendered = false ;
    for (; i< imageCollection->length() ;i++)
    {
        tempNode = imageCollection->item(i);
        if(!tempNode)
            continue;
        renderImage = static_cast<RenderImage *>(tempNode->renderer());
        if(!renderImage)    {
            WebCore::HTMLImageElement *imageElement = static_cast<HTMLImageElement *>(tempNode) ;
            cachedImage = imageElement->cachedImage () ;
            if (!cachedImage)
            continue;
        }
        else
        cachedImage = renderImage->cachedImage();
		if(!cachedImage)
            continue;
        orgPath = tempNode->attributes()->getAttributeItem("src")->value();
        Image* image = cachedImage->image();

		int xPos = 0 , yPos = 0;
        IntRect imgRect ;
        isRendered = false ;
        if (renderImage)    {
		renderImage->absolutePosition(xPos, yPos, FALSE);
    		imgRect = IntRect(  xPos, yPos, renderImage->width(),renderImage->height() );
            isRendered = true ;
        }
		ImageSaveInfo* imageInfo;

		if (bVisibleOnly)
		{
			if(imgRect.intersects(viewRect) && isRendered)
			{
				(*(imageList))[j] = (ImageSaveInfo*) SFBalMemCalloc(1,sizeof(ImageSaveInfo));
				imageInfo = (*(imageList))[j];
				j++;
			}
			else 
				continue;
		}
		else
		{
			(*(imageList))[j] = (ImageSaveInfo*) SFBalMemCalloc(1, sizeof(ImageSaveInfo));
			imageInfo = (*(imageList))[j];
			j++;
		}
		CachedResource *cachedBuffer = static_cast<CachedResource *>(cachedImage) ;
		if(image->data() || cachedBuffer->data())
		{
			if (image->data())	{
			imageInfo->data = image->data()->data();
			imageInfo->size = image->data()->size();
			}
			else	{
				imageInfo->data = cachedBuffer->data()->data();
				imageInfo->size = cachedBuffer->data()->size();
			}
			
			WebCore::HTMLImageElement *imageElement = (WebCore::HTMLImageElement *)(tempNode);

			WebCore::String url = imageElement->src();
			//imageInfo->url  =  (char*) SFBalMemAlloc(sizeof(char) * (AcStrlen(url.latin1().data()) + 1));
			imageInfo->url  =  (char*) SFBalMemCalloc(1, sizeof(char) * (AcStrlen(url.latin1().data()) + 1));

			AcStrcpy(imageInfo->url, url.latin1().data());

			WebCore::String name;
			SFBalExtractNameFromURL((char*)orgPath.latin1().data(), szUrl, orgPath.impl()->length());
			name = szUrl;		

			const ResourceResponse response = cachedBuffer->response() ;
			const WebCore::String mimeType = response.mimeType() ;
			WebCore::String extn = MIMETypeRegistry::getMIExtensionForMimeType(mimeType) ;
			name.append(".") ;
			name.append(extn) ;
			if(extn.latin1().length() > 0)	{
				imageInfo->extn = (char*) SFBalMemCalloc(1, sizeof(char) * (AcStrlen(extn.latin1().data()) + 1));
				AcStrcpy(imageInfo->extn, extn.latin1().data());
			}

			SFBalGetValidFilename(szUrl,(char*)name.latin1().data(), name.latin1().length());
			name = szUrl;			
			if(name.latin1().length() > 0)
			{
				//	imageInfo->name = (char*) SFBalMemAlloc(sizeof(char) * (100)); /*Commented by vijay and using AcStrlen to get the length*/
				imageInfo->name = (char*) SFBalMemCalloc(1, sizeof(char) * (AcStrlen(name.latin1().data()) + 1));

				AcStrcpy(imageInfo->name, name.latin1().data());
			}
			
		}
	}
	return HRESULT_OK;
#else
		__NotImplemented();
	return HRESULT_NOTSUPPORT;
#endif
}



/*
 * @desc saves a current contents to given path
 * @param in_pszPath : path to indicate for saved page.
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::saveCurrentPage( char* in_pszPath )
{
	int itemTypeMask = 0x01 | 0x02 | 0x04;
	Frame* child = m_pMainWebFrame->GetFrame();
		OOM_PRE_CHECK(PAGE_SAVEWRKING_BUFFER, false)				
	if(false == MemoryManager::IsOOMSent())
	{
		child->getBridge()->GetPageData(itemTypeMask, child, in_pszPath);
		return HRESULT_OK;
	}
	OOM_POST_CHECK_FAILED(return HRESULT_NO_MEMORY;)
}

/*
 * @desc sets page rendering mode to browser engine
 * @param in_nRenderingMode : page rendering mode
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::setPageRenderingMode( int in_nRenderingMode )
{
	Frame* frame = m_pMainWebFrame->GetFrame();

    if(frame && frame->document() && frame->document()->styleSelector())
    {
       SSR_MODES mode = SSR_MODE_OFF;

	   if( in_nRenderingMode == STANDARD_RENDERING_MODE )
	   {
		   m_RenderingMode = STANDARD_RENDERING_MODE;

		   // Horizontal scroll-bar must be visible for standard rendering mode
			frame->view()->setHScrollbarMode(ScrollbarAuto);

		   mode = SSR_MODE_OFF;
	   }
	   else if( in_nRenderingMode == SMART_WIDTH_RENDERING_MODE )
	   {
			m_RenderingMode = SMART_WIDTH_RENDERING_MODE;

			// Horizontal scroll-bar must be hidden for SSR rendering mode
			frame->view()->setHScrollbarMode(ScrollbarAlwaysOff);

		   mode = SSR_MODE_ON;
	   }
	   else
		   SysNeverGetHere();

       frame->setSSRMode(mode);

       //update stats as page load is complete
       frame->page()->navigationInstance()->updateHorizontalStats();
       frame->page()->navigationInstance()->updateVerticalStats();
       frame->page()->navigationInstance()->setPointerLocation(0, 0);

       frame->view()->scroll(ScrollUp, ScrollByDocument);

#if( PLATFORM(SHP) && SFB_SMARTLINK_FEATURE )

		if( frame->loader()->client()->isLoadingComplete() && !isMapViewMode() )
		{
			Frame* pFrame = frame;

#ifdef MULTIPLE_INSTANCE_SUPPORT
			SFBalRemoveEventByID(BAL_APP_BROWSER,BAL_EVENT_BROWSER_SMARTLINK, m_pMainWebFrame->GetWebControl() );
#else
			SFBalRemoveEventByID(BAL_APP_BROWSER,BAL_EVENT_BROWSER_SMARTLINK, NULL);
#endif

			while(pFrame)
			{
#ifdef MULTIPLE_INSTANCE_SUPPORT
				SFBalSendEvent(BAL_APP_BROWSER,BAL_EVENT_BROWSER_SMARTLINK, NULL, (int)pFrame, NULL, m_pMainWebFrame->GetWebControl());
#else
				SFBalSendEvent(BAL_APP_BROWSER,BAL_EVENT_BROWSER_SMARTLINK, NULL, (int)pFrame, NULL);
#endif//MULTIPLE_INSTANCE_SUPPORT
				pFrame = pFrame->tree()->traverseNext();
			}
		}

#endif

	   return HRESULT_OK;
    }
	else
	{
		SysDebug(( MID_EXCEPTION, "WebControl::setPageRenderingMode : condition check failed\n" ));
		return HRESULT_NOK;
	}
}

/*
	 * @desc get page rendering mode to browser engine
	 * @param in_nRenderingMode : page rendering mode
	 * @return HRESULT TRUE/FALSE
	 */
HRESULT WebControl::getPageRenderingMode( int *out_nRenderingMode )
{
	*out_nRenderingMode = m_RenderingMode;
	return HRESULT_OK;
}

/*
 * @desc sets virtual pointer mode to browser engine
 * @param
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::setVirtualPointerMode( BOOL in_bOnOff )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc sets save image mode to browser engine
 * @param
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::setImageSaveMode( BOOL in_bOnOff )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc sets page Map View mode to browser engine
 * @param
 * @return HRESULT TRUE/FALSE
 */
/*Sneha:To solve the conflict in search option and flicker*/
HRESULT WebControl::isSearchMode( BOOL bMode )
{
	 result=bMode;
	return HRESULT_OK;
}

#if defined(JS_MULTIPLE_WINDOW_SUPPORT)/* Anjana - UI changes for JS New Window */
HRESULT WebControl::setSecureMode( BOOL bMode )
{
	 m_bSecureMode = bMode;
	return HRESULT_OK;
}

BOOL WebControl::getSecureMode(void)
{
	return m_bSecureMode;
}
#endif

/*
 * @desc sets OneClick Mode to browser engine
 * @param
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::setOneClickMode( BOOL bMode )
{
	m_bOneClickMode=bMode;
	return HRESULT_OK;
}	

BOOL WebControl::isOneClickMode()
{
	return m_bOneClickMode;
}


#if 0
HRESULT WebControl::setMapViewMode( BOOL in_bOnOff )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
#endif

/*
 * @desc sets zoom value of contents to browser engine
 * @param in_Zoom : Zoom Value ( 0 ~ 100 : 50 means 100% , 100 means 200% )
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::setZoom( unsigned int in_Zoom )
{
	Frame* pFrame =	m_pMainWebFrame->GetFrame();
#if !defined(_SFB_BACKINGSTORE)
		pFrame->setZoomFactor( ( (float)in_Zoom / 100.0 ) , false);
#else
	if( m_pBackingStore )
		m_pBackingStore->Zoom( ( (float)in_Zoom / 100.0 ) );
#endif

#if( PLATFORM(SHP) && SFB_SMARTLINK_FEATURE )
		if( pFrame->loader()->client()->isLoadingComplete() && !isMapViewMode() )
		{

#ifdef MULTIPLE_INSTANCE_SUPPORT		
			SFBalRemoveEventByID(BAL_APP_BROWSER,BAL_EVENT_BROWSER_SMARTLINK, m_pMainWebFrame->GetWebControl() );
#else
			SFBalRemoveEventByID(BAL_APP_BROWSER,BAL_EVENT_BROWSER_SMARTLINK, NULL);
#endif

			while(pFrame)
			{
#ifdef MULTIPLE_INSTANCE_SUPPORT
				SFBalSendEvent(BAL_APP_BROWSER,BAL_EVENT_BROWSER_SMARTLINK, NULL, (int)pFrame, NULL, m_pMainWebFrame->GetWebControl());
#else
				SFBalSendEvent(BAL_APP_BROWSER,BAL_EVENT_BROWSER_SMARTLINK, NULL, (int)pFrame, NULL);
#endif//MULTIPLE_INSTANCE_SUPPORT					
				pFrame = pFrame->tree()->traverseNext();

			}
		}
#endif

	return HRESULT_OK;
}


/*
 * @desc 
 * @param in_Zoom : Zoom Value ( 0 ~ 100 : 50 means 100% , 100 means 200% )
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::zooming( unsigned int in_Zoom )
{

	SysDebug(( MID_EXCEPTION, "WebControl::zooming : value(%d)\n", in_Zoom ));
#if defined(_SFB_BACKINGSTORE)
	if( m_pBackingStore )
	{
		m_pBackingStore->Zooming( ( (float)in_Zoom / 100.0 ) );
	}
#endif
	return HRESULT_OK;
}
/*
 * @desc gets zoom value of contents to browser engine
 * @param in_Zoom : Zoom Value ( 0 ~ 100 : 50 means 100% , 100 means 200% )
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::getZoom( unsigned int *out_Zoom )
{
#if !defined(_SFB_BACKINGSTORE)
	*out_Zoom = (m_pMainWebFrame->GetFrame()->zoomFactor())*100.0;
#else
	if( m_pBackingStore )
		*out_Zoom  = m_pBackingStore->GetZoom() * 100.0;
#endif
	return HRESULT_OK;
}

/*
HRESULT WebControl::TouchClick( int rectX, int rectY )
{
	if( m_touchReleasedTimer.isActive() )
	{
		m_touchReleasedTimer.stop();
#if defined(_SFB_BACKINGSTORE)
		if( m_pBackingStore )
			m_pBackingStore->SetPageOverview( WebCore::IntPoint( rectX, rectY ));
#endif
	}
	else
	{
		m_touchReleasedTimerPoint.setX( rectX );
		m_touchReleasedTimerPoint.setY( rectY );
		m_bTouchReleasedTimerIsClick = true;
		m_touchReleasedTimer.startOneShot( DOUBLECLICK_THRESHOLD * 0.001 );
	}

	return HRESULT_OK;
}
*/
/*
 * @desc passes click event by touch to browser engine
 * @param x, y  : position
 * @return HRESULT TRUE/FALSE
 */
 /*
HRESULT WebControl::__TouchClick( int x, int y )
{
#if defined(_SFB_BACKINGSTORE)
	WebCore::IntPoint p;
	if( m_pBackingStore )
		p = m_pBackingStore->WindowToBackingStore( WebCore::IntPoint( x, y ) );
	x = p.x();
	y = p.y();
#endif
		
	return HandleMouseEvent( x, y, WebCore::MouseEventPressed );
}
*/

/*
 * @desc passes focus move event by touch to browser engine
 * @param x, y  : position
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::TouchFocusMove( int x, int y )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc passes touch pressed event to browser engine
 * @param x, y  : position
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::TouchPressed( int rectX, int rectY )
{
#if defined(_SFB_BACKINGSTORE)
	//2008.07.22 bs0425.lee modified for touch scrollbar. 
	IntPoint mousePosition;
	IntRect newThumbRect;
	IntRect extendThumbRect;
	
	Frame* frame = core(m_pMainWebFrame);

// 2009.09.17 modified for touch scrollbar feature. 
#if defined(_SFB_TOUCH_SCROLLBAR)
	mousePosition.setX(rectX);
	mousePosition.setY(rectY);
// bs0425.lee block backingstroe

	//PlatformScrollbar* c_Bar=frame->view()->getScrollbarData(mousePosition);
	newThumbRect=m_pBackingStore->getBackingScrollbarData(mousePosition);

	//extendThumbRect=touchScrollbar_ex(newThumbRect);
	
	if(newThumbRect.contains(mousePosition))
	{
		innerTrack=TRUE;
		scrollbarPressed=TRUE;
		scrollBarPoint.setX(rectX);
		scrollBarPoint.setY(rectY);
	}
	else
	{

		scrollbarPressed=FALSE;
		scrollBarPoint.setX(0);
		scrollBarPoint.setY(0);

}
#else
			scrollbarPressed=FALSE;

#endif

	// for touch scrollbar. 



	WebCore::IntPoint p;
	if( m_pBackingStore )
		p = m_pBackingStore->WindowToBackingStore( WebCore::IntPoint( rectX, rectY ) );
	rectX = p.x();
	rectY = p.y();

#endif

	min_X	=  rectX-INNER_INDEX	;
	max_X	=  rectX+INNER_INDEX	;
	min_Y	=  rectY-INNER_INDEX	;
	max_Y	=  rectY+INNER_INDEX	;

	// bs0425.lee modified to fix long press zoom bug. 
	bUseScroll=false;
       SysDebug(( MID_EXCEPTION, "WebControl::TouchPressed   rectX : (%d)  ,rectY : (%d))\n", rectX,rectY));
	return HandleMouseEvent( rectX, rectY , WebCore::MouseEventMoved );

}

//OS timer for zoom
void WebControl::touchReleasedTimerFired()
{
	int rectX, rectY;
   
	rectX=m_touchReleasedTimerPoint.x();
	rectY=m_touchReleasedTimerPoint.y();
	
	if(linkFocusedByPoint(rectX,rectY)==TRUE)
			m_bFocused=TRUE;
	else
			m_bFocused=FALSE;

		__TouchReleased( m_touchReleasedTimerPoint.x(), m_touchReleasedTimerPoint.y() );

}
//OS timer for zoom
static void touchZoomReleasedTimerFired(HTimer hTimer, ULONG param)
{
	((WebControl*)param)->touchReleasedTimerFired();
}

HRESULT WebControl::TouchReleased( int rectX, int rectY, bool bHandleDoubleTab  )
{
	ULONG tickOffset;
	SysDebug(( MID_EXCEPTION, "WebControl::TouchReleased : bTimerActive(%d)\n", WmIsTimerActive(hTouchReleasedTimer)));
	
	 //#if defined(_SFB_BACKINGSTORE)
	 WmRect rc;
	 WmGetWindowClientRect ( GetHWIN(), &rc );

 // 2008.08.07  modified for long press zoom. 
	 if( bEnableZoom==TRUE)
	 {
		  if(pBmpWidget!=NULL)
			{
 				__WsFreeBitmap(pBmpWidget);
			}
		  if(pBmpWidget2!=NULL)
	 		{
	 			__WsFreeBitmap(pBmpWidget2);
			}
		 if(pBmpWidget1!=NULL)
	 		{
	 			__WsFreeBitmap(pBmpWidget1);
				pBmpWidget1 = NULL;
			}

			bEnableZoom=FALSE;

			if( zoomValue != 0 )
			{
 #if defined(_SFB_BACKINGSTORE)
				m_pBackingStore->Zoom(zoomValue);
#else
				if(zoomValue>5)
					zoomValue=5;
				else if(zoomValue<0.2)
					zoomValue=0.2;

		        setZoom(zoomValue*100);
#endif
			}
			else
			{
				//drawScreen(0,0,rc.w,rc.h);
				//showScreen(0,0,rc.w,rc.h);
				if(m_pContentListener)
				m_pContentListener->updateScreen(0,0,rc.w,rc.h);
			}

		   zoomValue=0;
		   totalZoomY = 0;

		   return HRESULT_TRUE;

	 }

	if( bHandleDoubleTab == false )
	{
		if(linkFocusedByPoint(rectX,rectY)==TRUE)
			m_bFocused=TRUE;
		else
			m_bFocused=FALSE;

		__TouchReleased( rectX, rectY );

		return HRESULT_OK;
	}
	else
	{

	  tickOffset = SFBSysGetTick() - m_prevReleasedTick;
	  m_prevReleasedTick = SFBSysGetTick();
	  
	  //OS timer for zoom
	  SysDebug(( MID_EXCEPTION, "WebControl::TouchReleased : bUseScroll(%d), tickOffset(%d)\n", bUseScroll, tickOffset));
	  if(bUseScroll==false)
	  {
		  if(WmIsTimerActive(hTouchReleasedTimer))
	  {
		  // double click
			  WmCancelTimer( hTouchReleasedTimer );		
  
  #if defined(_SFB_BACKINGSTORE)
			  if( m_pBackingStore && tickOffset > DOUBLECLICK_MIN_THRESHOLD )
			  m_pBackingStore->SetPageOverview( WebCore::IntPoint( rectX, rectY ));
  #endif
	  }
	  else
	  {
		  m_touchReleasedTimerPoint.setX( rectX );
		  m_touchReleasedTimerPoint.setY( rectY );
		  //m_bTouchReleasedTimerIsClick = false;
				  
				  WmStartTimer(hTouchReleasedTimer, DOUBLECLICK_MAX_THRESHOLD, touchZoomReleasedTimerFired, ULONG(this));
			  
	  }
	  }
	  else
	  {
		  WmCancelTimer( hTouchReleasedTimer );
	  }
		  
  
	  return HRESULT_OK;
	}
}


HRESULT WebControl::__TouchReleased( int rectX, int rectY )
{
	// bs0425.lee modified release function.
	 WebCore::Frame* frame = m_pMainWebFrame->GetFrame();
	 SysDebug(( MID_EXCEPTION, "WebControl::__TouchReleased   rectX : (%d)  ,rectY : (%d))\n", rectX,rectY));

  #if defined(_SFB_BACKINGSTORE)
	WebCore::IntPoint p;
	if( m_pBackingStore )
		p = m_pBackingStore->WindowToBackingStore( WebCore::IntPoint( rectX, rectY ) );
	rectX = p.x();
	rectY = p.y();
#endif

	WebCore::Node* curNode = 0;
	curNode = frame->page()->navigationInstance()->getHoveredNode();

	if( ( min_X < rectX ) && ( rectX  < max_X ) ) //why do we need this??
	{
		if( ( min_Y < rectY ) && ( rectY  < max_Y) ) //why do we need this??
		{
			if(curNode && curNode->isShadowNode())
				curNode = curNode->shadowParentNode();

			if(curNode)
			{
				if( (m_bFocused == TRUE )  ||(m_bOneClickMode==TRUE) )
				{
					HandleMouseEvent( rectX, rectY , WebCore::MouseEventPressed );
					return HRESULT_OK;
				}
				else if( equalIgnoringCase( curNode->nodeName(), TAG_INPUT ) ||equalIgnoringCase( curNode->nodeName(), TAG_SELECT )||equalIgnoringCase( curNode->nodeName(), TAG_TEXTAREA ))
					return HandleMouseEvent( rectX, rectY , WebCore::MouseEventPressed );	
				else if(equalIgnoringCase( curNode->nodeName(), TAG_LABEL ))
					return HandleMouseEvent( rectX, rectY , WebCore::MouseEventPressed );
				else
					return HandleMouseEvent( rectX, rectY , WebCore::MouseEventReleased );
			}
			else
				return HandleMouseEvent( rectX, rectY , WebCore::MouseEventReleased );
		}
	}

	return HRESULT_TRUE;
}


// 2008.08.07. bs0425.lee modified for long press zoom. 
/*
 * @desc passes touch LongPressed event to browser engine
 * @param x, y  : position
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::TouchLongPressed( int rectX, int rectY )
{	
	bEnableZoom=TRUE;
	// initialize long press zoom values. 
	zoomPointX=rectX;
	zoomPointY=rectY;
	totalZoomY = 0;
	zoomValue =0;
	pBmpWidget=NULL;
	pBmpWidget1=NULL;
	pBmpWidget2=NULL;

 //#if defined(_SFB_BACKINGSTORE)
	int hGC;
	getGC( &hGC );

	DrawZoomBitmap( hGC, rectX, rectY );
//#endif
       SysDebug(( MID_EXCEPTION, "WebControl::TouchLongPressed   rectX : (%d)  ,rectY : (%d))\n", rectX,rectY));
	return HRESULT_TRUE;

}

/*
 * @desc passes draw bitmap event to browser engine
 * @param x, y  : position
 * @return HRESULT TRUE/FALSE
 */

HRESULT WebControl::DrawZoomBitmap(int hGC, int positionX,int positionY)
{
	HGC	hScreenGC;
	BOOL result;
	int rsrcId;
    int recX,recY;
	int visibleWidth, visibleHeight;
	
	WebCore::Frame* frame = m_pMainWebFrame->GetFrame();

	WmRect rc;

	WmGetWindowClientRect ( GetHWIN(), &rc );
	visibleWidth = rc.w;
	visibleHeight = rc.h;

	//hScreenGC = hGC;
	hScreenGC = WmCreateClientGC( GetHWIN() );

	if( isPointOnImage( positionX, positionY ) == TRUE )
	{
		rsrcId = IDB_SFB_LONG_PRESS_ZOOM_POPUP;
		pBmpWidget=RsrcLoadBitmap( SFB_RSRC_FILENAME, rsrcId );

		if( pBmpWidget != NULL )
		{
			recX = (positionX - pBmpWidget->rectW) - ZOOM_BITMAP_PADDING*3;
			if(recX < ZOOM_BITMAP_PADDING)
			{
				recX = positionX + ZOOM_BITMAP_PADDING*3;
			}
			
			recY = (positionY - pBmpWidget->rectH) - ZOOM_BITMAP_PADDING/2;
			if(recY < ZOOM_BITMAP_PADDING)
			{
				recY = ZOOM_BITMAP_PADDING;
			}

			if(recY > (visibleHeight -(pBmpWidget->rectH) - ZOOM_BITMAP_PADDING))
			{
				recY = visibleHeight - (pBmpWidget->rectH) - ZOOM_BITMAP_PADDING;
			}
		}

	}
	else
	{
		rsrcId = IDB_SFB_LONG_PRESS_ZOOM;
		pBmpWidget = RsrcLoadBitmap( SFB_RSRC_FILENAME, rsrcId );
		
		if( pBmpWidget != NULL )
		{
			recX =(positionX - pBmpWidget->rectW) - ZOOM_BITMAP_PADDING*3;
			if(recX < ZOOM_BITMAP_PADDING)
			{
				recX = positionX + ZOOM_BITMAP_PADDING*3;
			}
			
			recY = (positionY - pBmpWidget->rectH) - ZOOM_BITMAP_PADDING/2;
			if(recY < ZOOM_BITMAP_PADDING)
			{
				recY = ZOOM_BITMAP_PADDING;
			}

			if(recY > (visibleHeight - (pBmpWidget->rectH) - ZOOM_BITMAP_PADDING))
			{
				recY = visibleHeight -(pBmpWidget->rectH) - ZOOM_BITMAP_PADDING;
			}
		}

	}

	if( pBmpWidget != NULL )
		result = WmDrawBitmap( hScreenGC, recX, recY, pBmpWidget );

	WmShowGC( hScreenGC );

	WmDeleteGC( hScreenGC );

	return TRUE;

}


/*
 * @desc gets total count of images in current contents
 * @param
 * @return int Count of images
 */
HRESULT WebControl::getCountImages( int *out_Count )
{
	//return HRESULT_OK;
		
	////////////////////////////////////////////////////////////////////////////////////
	Frame* frame = core( m_pMainWebFrame);

	PassRefPtr<HTMLCollection> imageCollection = frame->document()->images();

	FrameView* view = frame->view();
	
	int contentsx = view->contentsX();				// view port x position
	int contentsy = view->contentsY();				// view port y position
	int visibleW = view->visibleWidth();			// width of the display
	int visibleH = view->visibleHeight();			// height of the display

	IntRect viewRect(contentsx, contentsy, visibleW, visibleH);
	IntRect contentRect = IntRect(view->visibleContentRect());

	CachedImage* cachedImage;
    RenderImage *renderImage;

	String orgPath;

    WebCore::Node * tempNode;
	int count = 0;
    for (int i=0; i< imageCollection->length() ;i++)
    {
        tempNode = imageCollection->item(i);
        if(!tempNode)
            continue;
        renderImage = static_cast<RenderImage *>(tempNode->renderer());
        if(!renderImage)    {
            WebCore::HTMLImageElement *imageElement = static_cast<HTMLImageElement *>(tempNode) ;
            cachedImage = imageElement->cachedImage () ;
            if (!cachedImage)
            continue;
        }
        else
        cachedImage = renderImage->cachedImage();
        if(!cachedImage)
            continue;

	
		Image* image = cachedImage->image();
        CachedResource *cachedBuffer = static_cast<CachedResource *>(cachedImage) ;
		if((image && image->data()) || cachedBuffer->data())
				{
					count++;
				}
			}
	*out_Count = count;
	return HRESULT_OK;
	//////////////////////////////////////////////////////////////
}


/*
 * @desc searches text on current contents and if the text is found then enter the search mode
 * @param in_pszSearchText : pointer to text to search
 * @param out_pCount : pointer to the searched text if is found.
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::beginSearchText( char* in_pszSearchText, int* out_pCount )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}
/*
 * @desc finishes the search mode
 * @param
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::endSearchText( void )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc draws Browser Engin's screen
 * @param in_x, in_y, in_w, in_h : x, y, w, h
 * @return
 */
HRESULT WebControl::drawScreen( int in_x, int in_y, int in_w, int in_h )
{
#if !defined(_SFB_BACKINGSTORE)
	// shkim 2008.06.19 in Bresson should not be enale 
	// drawScreen can be used even though checkShowState return FALSE
	//if( m_pContentListener && m_pContentListener->checkShowState( in_x, in_y, in_w, in_h ) == HRESULT_FALSE )
	//{
	//	SysDebug(( MID_EXCEPTION, "WebControl::showScreen :  can't show now. WebControl might be hide state\n" ));
	//	return HRESULT_NOK;
	//}

	Frame* frame = core( m_pMainWebFrame );

	IntRect viewRect = frame->view()->frameGeometry();

	GraphicsContext ctx(&m_hGC,&m_hWin);

	// set the place-holder image
#ifdef SFB_PROGRESSIVE_RENDERING
	ctx.setPlaceholderImage(m_pbmPlaceHolder);
#endif // SFB_PROGRESSIVE_RENDERING

	if (frame->renderer()) {
		frame->view()->layoutIfNeededRecursive();
		IntRect rect(in_x, in_y, in_w, in_h );
		frame->view()->paint(&ctx, rect);
	}

#else

	if( m_pBackingStore )
		m_pBackingStore->PaintToWindow( in_x, in_y, in_w, in_h, true, false );

#endif


	return HRESULT_OK;
}

/*
 * @desc changes Browser screen's area
 * @param w, h : weight, height
 * @return
 */
HRESULT WebControl::changeScreenSize( int w, int h )
{
	m_w = w;
	m_h = h;
	return rotateScreen( m_x, m_y, m_w, m_h );
}

/*
 * @desc rotates Browser screen's area
 * @param in_x, in_y, in_w, in_h : x, y, w, h
 * @return
 */
HRESULT WebControl::rotateScreen( int in_x, int in_y, int in_w, int in_h )
{
	Frame* frame = m_pMainWebFrame->GetFrame();

	/* fix for 4371 START PART1*/
#if 0
	BOOL bNativeEditActive = false;
	WebCore::String editFieldStr;
	WebCore::Node* currentNode = NULL;
	WebCore::Document* tempDocument = NULL;
#endif
	/* fix for 4371 END PART1*/

	/*Remove the focus on rotate*/
	WebCore::NativeEditField* pNativeEdit = NativeEditField::getNativeEditFieldInstance();
    WebAssert(pNativeEdit);
	if ( pNativeEdit->isActivated() )
    {
		   //fix for 4371
#if 0
		   bNativeEditActive = true;
		   editFieldStr = pNativeEdit->getString(); 
		   currentNode = pNativeEdit->getDocument()->focusedNode();
#endif	
		pNativeEdit->_sendTextToBrowserEngine();
		pNativeEdit->unsetFocusedNode();
		   pNativeEdit->deactivate( false );
 	}

	WebCore::NativeComboField* pNativeCombo = NativeComboField::getNativeComboFieldInstance();
    WebAssert(pNativeCombo);
   	if ( pNativeCombo->isActivated() )
    {
//		   pNativeCombo->SetSelectedIndex();
//		   pNativeCombo->unsetFocusedNode();
		   pNativeCombo->deactivate( false );
 	}

#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)
	WebCore::NativeListBox* pNativeList = NativeListBox::getNativeListBoxInstance();
	WebAssert(pNativeList);

	if( pNativeList->isActivated() )
    {
		   pNativeList->unsetFocusedNode();
		   pNativeList->deactivate();
 	}
#endif //#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)


#if !defined( _SFB_BACKINGSTORE )
	if(frame->view()->width() == in_w && frame->view()->height() == in_h)
	{
		return HRESULT_OK;
	}
#endif

    m_x = in_x;
	m_y = in_y;
	m_w = in_w;
	m_h = in_h;

#if !defined( _SFB_BACKINGSTORE )
    if(INVALID_HOBJ != m_hGC)
    {
        WmDeleteGC( m_hGC );
        m_hGC = INVALID_HOBJ;
    }

	if ( GetGCType() == BIT_32_GC  )
		m_hGC = WmCreateClientSubMemGC32( m_hWin, in_x, in_y, in_w, in_h );   
	else
	m_hGC = WmCreateClientSubGC( m_hWin, in_x, in_y, in_w, in_h );

    //WmGetWindowClientRect( pageData->hWin , &wm_Rect );
    /* resizing of the frame to screen size */
    frame->view()->resize( in_w , in_h );
    frame->forceLayout();
    frame->view()->adjustViewSize();
    frame->sendResizeEvent();

 	//update the navigation instance and set properties
	CNavigation* navigation = frame->page()->navigationInstance();
	if(navigation)
	{
		navigation->setGC(m_hGC);
		navigation->setMainFrameView(frame->view());
		CNavigation::switchPage(navigation);
	}
	
#else
	if( m_pBackingStore )
		m_pBackingStore->RotateBackingStore( in_w, in_h );
#endif

/* fix for 4371 START PART2*/
#if 0
	if(bNativeEditActive)
	{
		WebCore::NativeWidgetAttrib attrib;
		unsigned int cursorPosition = 0;

		if( currentNode == NULL )
			return HRESULT_OK;

		if( WebCore::Node::ID_INPUT == currentNode->nodeId())
		{
			attrib.m_strValue = editFieldStr;
			//NATARAJ  - for bug SFB00001279
			attrib.m_inputFormat = static_cast<HTMLInputElement *>(currentNode)->getFormatValue();
		}

		if( WebCore::Node::ID_TEXTAREA == currentNode->nodeId())
		{
			attrib.m_strValue =  editFieldStr;
			
#if PLATFORM(SHP)
			//Fix for Widget issue - Partho
			attrib.m_inputFormat = static_cast<HTMLTextAreaElement *>(currentNode)->getFormatValue();
#endif //PLATFORM(SHP)
		}

		if(false == attrib.m_strValue.isEmpty())
		{
			cursorPosition = editFieldStr.length();
		}


		attrib.m_rect = currentNode->getRect();
		attrib.m_cursorOffset = cursorPosition;
		attrib.m_nodeID = currentNode->nodeId();

		ASSERT(currentNode);
		attrib.m_pStyle = currentNode->renderer()->style();

		if( WebCore::Node::ID_INPUT == currentNode->nodeId())
		{
			HTMLInputElement* inputElement = static_cast<HTMLInputElement*> (currentNode);

			if( WebCore::HTMLInputElement::PASSWORD == inputElement->inputType())
				attrib.m_isPassword = true;

			attrib.m_maxLength = ( MAX_EDIT_BOX_TXT_LEN > inputElement->maxLength() ) ? inputElement->maxLength() :  MAX_EDIT_BOX_TXT_LEN;
		}

		attrib.m_pDocument = currentNode->document();

		pNativeEdit->activate(attrib);

	}
#endif
/* fix for 4371 end PART2*/

//need to check with bresson model
//#if( PLATFORM(SHP) && SFB_SMARTLINK_FEATURE )
//
//		if( frame->loader()->client()->isLoadingComplete() && !isMapViewMode() )
//		{
//			Frame* pFrame = frame;
//			while(pFrame)
//			{
//#ifdef MULTIPLE_INSTANCE_SUPPORT
//				SFBalSendEvent(BAL_APP_BROWSER,BAL_EVENT_BROWSER_SMARTLINK, NULL, (int)pFrame, NULL, m_pMainWebFrame->GetWebControl());
//#else
//				SFBalSendEvent(BAL_APP_BROWSER,BAL_EVENT_BROWSER_SMARTLINK, NULL, (int)pFrame, NULL);
//#endif//MULTIPLE_INSTANCE_SUPPORT
//				pFrame = pFrame->tree()->traverseNext();
//			}
//		}
//
//#endif

	return HRESULT_OK;

}

/*
 * @desc shows Browser Engin's screen
 * @param in_x, in_y, in_w, in_h : x, y, w, h
 * @return
 */
HRESULT WebControl::showScreen( int in_x, int in_y, int in_w, int in_h )
{
	//if( !m_isLoadingDone )
	//	return HRESULT_OK;

	if( m_pContentListener && m_pContentListener->checkShowState( in_x, in_y, in_w, in_h ) == HRESULT_FALSE )
	{
		SysDebug(( MID_EXCEPTION, "WebControl::showScreen :  can't show now. WebControl might be hide state\n" ));
		return HRESULT_NOK;
	}

#if !defined( _SFB_BACKINGSTORE )
	Frame* frame = core( m_pMainWebFrame );

	IntRect viewRect = frame->view()->frameGeometry();

	GraphicsContext ctx(&m_hGC,&m_hWin);

	// set the place-holder image
#ifdef SFB_PROGRESSIVE_RENDERING
	ctx.setPlaceholderImage(m_pbmPlaceHolder);
#endif // SFB_PROGRESSIVE_RENDERING

	if (frame->renderer()) {
		frame->view()->layoutIfNeededRecursive();
		IntRect rect(in_x, in_y, in_w, in_h );
		frame->view()->paint(&ctx, rect);
	}

	HWIN hFocusedWnd = WmGetFocusWindow();

		if(WmGetParentWindow(m_hWin) != WsGetTopmostMainWindowByZOrder(SCREEN_MAIN))
		{
			WmShowWindow(WsGetTopmostMainWindowByZOrder(SCREEN_MAIN));
			return HRESULT_NOK;
		}
	if(hFocusedWnd == m_hWin)
			{
		WmShowGC( m_hGC );
		frame->page()->navigationInstance()->doPostPaintAction();
	}
else
	{
			int winType = 0;
			if(WmGetWindowType(hFocusedWnd,&winType))
			{
				if((WINDOW_EDITAREA == winType) || (WINDOW_EDITFIELD == winType)
					|| (WINDOW_MESSAGEBOX == winType) ||(WINDOW_COMBOBOX == winType))
				{
		if(m_hWin == WmGetParentWindow(hFocusedWnd))
		{
				//if(result)
			WmShowGC( m_hGC );
		}

		WmDrawWindow(hFocusedWnd, TRUE );
		WmShowWindow(hFocusedWnd);
	}
}
		}

#else

	// if the in_x, in_y, in_w, in_h is 0, the draw area is automatically calculated by the backingstore
	// otherwise only the specified area will be painted

	if( m_pBackingStore )
		m_pBackingStore->PaintToWindow( in_x, in_y, in_w, in_h, false, true );

	HWIN hFocusedWnd = WmGetFocusWindow();

	if(m_hWin == WmGetParentWindow(hFocusedWnd))
	{
		WmDrawWindow(hFocusedWnd, TRUE );
		WmShowWindow(hFocusedWnd);
	}

#endif

	return HRESULT_OK;
}

/*
 * @desc copy the content of screen to destination gc
 *
 */
HRESULT WebControl::copyScreen( int destGC, int destx, int desty, int srcx, int srcy, int srcw, int srch )
{

#if !defined( _SFB_BACKINGSTORE )
	if( drawScreen( srcx, srcy, srcw ,srch ) != HRESULT_OK )
		return HRESULT_NOK;

	WmCopyGC( destGC, destx, desty, m_hGC, srcx, srcy, srcw, srch );

	return HRESULT_OK;
#else
	if( m_pBackingStore )
		m_pBackingStore->CopyScreen( destGC, destx, desty,  srcx, srcy, srcw, srch );
	return HRESULT_OK;
#endif
}

/*
* @desc get the graphic context which WebControl is using
*/
HRESULT WebControl::getGC(int * out_gc)
{
#if !defined( _SFB_BACKINGSTORE )
	*out_gc=m_hGC;
	return HRESULT_OK;
#else

	*out_gc = INVALID_HOBJ;
	if( m_pBackingStore )
		*out_gc = m_pBackingStore->GetWindowGC();
	return HRESULT_OK;
#endif
}


/*
 * @desc updates page without reload.
 * @param
 * @return
 */
HRESULT WebControl::updatePage( void )
{
	if( m_pMainWebFrame == NULL )
		return HRESULT_NOK;

    Frame* coreFrame = m_pMainWebFrame->GetFrame();

    if (Document* document = coreFrame->document())
		document->recalcStyle(WebCore::Node::Force);

	return HRESULT_OK;
}

/*
 * @desc checks whether the focus is assigned on the point or not.
 * @param x, y : point
 * @return HRESULT TRUE/FALSE
 */
HRESULT	WebControl::isFocusedByPoint( int x, int y )
{
	static WebCore::Element* preElement;
	WebCore::Frame* frame = m_pMainWebFrame->GetFrame();
	WebCore::Element* element;

// bs0425.lee 208.05.14.  modified for Frame Tag. 
	while( frame )
	{
	WebCore::FrameView *frameview = frame->view();
	WebCore::IntPoint ptContent = frameview->windowToContents(WebCore::IntPoint(x, y));
	WebCore::Document* NewDocument=frame->documentAtPoint(ptContent);

 	if(NULL == NewDocument)
	{
		return FALSE;
	}

		element =NewDocument->elementFromPoint(ptContent.x(), ptContent.y());
		if(NULL == element)
			return FALSE;

		if( equalIgnoringCase( element->tagName(), TAG_FRAME ) )
		{
			WebCore::HTMLFrameOwnerElement *frameElement = (WebCore::HTMLFrameOwnerElement *)element;
			frame = frameElement->contentFrame();
			continue;
		}
		else
			break;
	}


	 if(preElement==element)
	 {
	 	preElement=element;

		if( equalIgnoringCase( element->nodeName(), TAG_INPUT ) ||equalIgnoringCase( element->nodeName(), TAG_SELECT ))	
			return FALSE;
		else if(equalIgnoringCase( element->nodeName(), TAG_LABEL ))
			return FALSE;
		else
			return TRUE;
			
	 }
	 else
	 {
		preElement=element;
		return HRESULT_FALSE;
	 }
}

/*
 * @desc Divide this function with UI API. checks whether the focus is assigned on the point or not.
 * @param x, y : point
 * @return HRESULT TRUE/FALSE
 */
HRESULT	WebControl::linkFocusedByPoint( int x, int y )
{
	HRESULT result = FALSE;

	WebCore::Node* node = 0;
	WebCore::Frame* frame = m_pMainWebFrame->GetFrame();
	node = frame->page()->navigationInstance()->getHoveredNode();

	if(node && (node == m_lastFocusedNode))
	{
		if(!equalIgnoringCase(node->nodeName(), TAG_INPUT) &&
			!equalIgnoringCase(node->nodeName(), TAG_SELECT) &&
			!equalIgnoringCase(node->nodeName(), TAG_LABEL))
		{
			result = TRUE;
		}
	}

	m_lastFocusedNode = node;

	return result;
}


/*
 * @desc checks whether the focus is possible on the point or not.
 * @param x, y : point
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::isFocusableByPoint( int x, int y )
{
	 bool isLink=FALSE;
	 WebCore::Frame* frame = m_pMainWebFrame->GetFrame();
	 WebCore::Element* element;


	 while( frame )
	{
	 WebCore::FrameView *frameview = frame->view();
	 WebCore::IntPoint ptContent = frameview->windowToContents(WebCore::IntPoint(x, y));
	 WebCore::Document* NewDocument=frame->documentAtPoint(ptContent);

 	 if(NULL == NewDocument)
	 {
	 	 return FALSE;
	 }

		element =NewDocument->elementFromPoint(ptContent.x(), ptContent.y());
		if(NULL == element)
			return FALSE;

		if( equalIgnoringCase( element->tagName(), TAG_FRAME ) )
		{
			WebCore::HTMLFrameOwnerElement *frameElement = (WebCore::HTMLFrameOwnerElement *)element;
			frame = frameElement->contentFrame();
			continue;
		}
		else
			break;
	}

	
	/* WebCore::FrameView *frameview = frame->view();
	 WebCore::IntPoint ptContent = frameview->windowToContents(WebCore::IntPoint(x, y));
	 WebCore::Document* NewDocument=frame->documentAtPoint(ptContent);

 	 if(NULL == NewDocument)
	 {
	 	 return FALSE;
	 }

	 WebCore::Element* element =NewDocument->elementFromPoint(ptContent.x(), ptContent.y());*/


	 // bs0425.lee 2008.05.14. modified for Frame TAG.
	if( equalIgnoringCase( element->nodeName(), TAG_INPUT ) ||equalIgnoringCase( element->nodeName(), TAG_SELECT ))
		return FALSE;
	else if(equalIgnoringCase( element->nodeName(), TAG_LABEL ))
		return FALSE;
	else
	 	return TRUE;
	 	
	
}

/*
 * @desc checks whether the anchor tag is on the point or not.
 * @param x, y : point
 * @return HRESULT TRUE/FALSE
 */
// removed
//HRESULT WebControl::isAnchorByPoint( int x, int y )
//{
//	__NotImplemented(); return HRESULT_NOTSUPPORT;
//}

/*
 * @desc gets DOM Node with point.
 * @param out_pIDomNode : pointer to IDOMNode class
 * @param in_x : x
 * @param in_y : y
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::getElementByPoint( IPropertyMap** out_propertyMap, int x, int y )
{
    if (!out_propertyMap)
	{
		SysDebug(( MID_EXCEPTION, "WebControl::getElementByPoint : out_propertyMap == NULL\n" ));
		return HRESULT_INVALID_PARAM;
    }

    *out_propertyMap = 0;

	WebCore::Frame* frame = m_pMainWebFrame->GetFrame();
    if (!frame)
	{
        SysDebug(( MID_EXCEPTION, "WebControl::getElementByPoint : m_pMainWebFrame->GetFrame() == NULL\n" ));
		return HRESULT_NOK;
	}

	//added for getting the correct values of x and y depending upon backing store: Hema
    #if defined(_SFB_BACKINGSTORE)
	WebCore::IntPoint pBackStorePt = m_pBackingStore->WindowToBackingStore( WebCore::IntPoint(x, y) );
	WebCore::IntPoint webCorePoint = frame->view()->windowToContents(pBackStorePt);
    #else
	WebCore::IntPoint webCorePoint = frame->view()->windowToContents(WebCore::IntPoint( x, y ));
     #endif//_SFB_BACKINGSTORE

	WebCore::HitTestResult result = WebCore::HitTestResult(webCorePoint);
    if ( frame->renderer() )
	{
        result = frame->eventHandler()->hitTestResultAtPoint( webCorePoint, true);
		*out_propertyMap = new WebElementPropertyMap( result );
	}
	else
	{
		SysDebug(( MID_EXCEPTION, "WebControl::getElementByPoint : frame->renderer dose not exist\n" ));
		return HRESULT_NOK;
	}
}

/*
 * @desc gets DOM Node with focus
 * @param out_pIDomNode : pointer to IDOMNode class
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::getElementByFocus( IPropertyMap** out_propertyMap )
{
	int x =0, y = 0;

    if (!out_propertyMap)
	{
		SysDebug(( MID_EXCEPTION, "WebControl::getElementByFocus : out_propertyMap == NULL\n" ));
		return HRESULT_INVALID_PARAM;
    }

    *out_propertyMap = 0;

	// [todo] in case of  4 way navigation or touch
	__NotImplemented();
	return HRESULT_NOTSUPPORT;


	WebCore::Frame* frame = m_pMainWebFrame->GetFrame();
    if (!frame)
	{
        SysDebug(( MID_EXCEPTION, "WebControl::getElementByFocus : m_pMainWebFrame->GetFrame() == NULL\n" ));
		return HRESULT_NOK;
	}

	WebCore::IntPoint webCorePoint = WebCore::IntPoint( x, y );	
	WebCore::HitTestResult result = WebCore::HitTestResult(webCorePoint);
    if ( frame->renderer() )
	{
        result = frame->eventHandler()->hitTestResultAtPoint( webCorePoint, false);
		*out_propertyMap = new WebElementPropertyMap( result );
	}
	else
	{
		SysDebug(( MID_EXCEPTION, "WebControl::getElementByFocus : frame->renderer dose not exist\n" ));
		return HRESULT_NOK;
	}

}


/*
 * @desc sets account for proxy
 * @param in_Account : account handle of Mocha
 * @return HRESULT TRUE / FALSE
 */
HRESULT WebControl::setAccessPoint( int in_Account )
{

	SFBRmSetPDPAccount( (int)m_pLoadGroup , in_Account  );


	return HRESULT_OK;
}


/*
 * @desc sets a request header property of Browser Engine.
 * @param in_pszHttpHeader : pointer to the HttpHeader
 * @param in_pszHeader : pointer to the HttpHeader Value
 * @return HRESULT TRUE / FALSE
 */
//HRESULT WebControl::setHttpHeaderProperty( char* in_pszHttpHeader, char* in_pszHeaderBody )
//{
//	__NotImplemented(); return HRESULT_NOTSUPPORT;
//}

	/*
	 * @desc goes to forward or backward history
	 * @param
	 * @return HRESULT TRUE / FALSE
	 */
HRESULT WebControl::goBackwardHistory( void )
{
	MemoryManager::ResetOOMState();

    //Check if the shared timwe is valid. If not then initiaze the shared timer
    if(false == isSharedTimerValid())
    WebCore::setSharedTimerFireTime(0);
	
	m_pPage->goBack() ;

	return HRESULT_OK;
}

#ifdef SFB_UPDATE_URL_BAR  //Pramod - Bresson Model - update Url ComboBox bar

/* - Pramod For updation of Urlbar with current url
 * @desc gets the current processing url
 * @param  : char* - Url string
 * @return : None
 */
void WebControl::getCurrentUrlFromHistoryList(char* szUrl)
{
	WebCore::KURL url = m_pMainWebFrame->GetFrame()->loader()->historyURL(0);
/*Fix for CQ 3886*/
	AcStrncpy(szUrl, url.string().latin1().data(),_BROWSER_MAX_URL);

	szUrl[_BROWSER_MAX_URL]='\0';

}
/* Fix:CQ 2813:Anjana, check for forward/backward url available */
void WebControl::getUrlFromHistoryList(int nDirect, char* szUrl)
{
	WebCore::KURL url = m_pMainWebFrame->GetFrame()->loader()->historyURL(nDirect);
/*Fix for CQ 3886*/
	AcStrncpy(szUrl, url.string().latin1().data(),_BROWSER_MAX_URL);

	szUrl[_BROWSER_MAX_URL]='\0';
}
#endif

	/*
	 * @desc goes to forward or backward history
	 * @param
	 * @return HRESULT TRUE / FALSE
	 */
HRESULT WebControl::goForwardHistory( void )
{
	MemoryManager::ResetOOMState();

    //Check if the shared timwe is valid. If not then initiaze the shared timer
    if(false == isSharedTimerValid())
    WebCore::setSharedTimerFireTime(0);

	m_pPage->goForward() ;

	return HRESULT_OK;
}

	/**
	* @desc check if gobackward is possible
	* @return HRESULT TRUE / FALSE
	*/
HRESULT WebControl::canGoBackwardHistory()
{
	return m_pMainWebFrame->GetFrame()->loader()->canGoBackOrForward(-1);
}

	/**
	* @desc check if goforward is possible
	* @return HRESULT TRUE / FALSE
	*/
HRESULT WebControl::canGoForwardHistory()
{
	return m_pMainWebFrame->GetFrame()->loader()->canGoBackOrForward(1);
}

/*
 * @desc clears history
 * @param
 * @return HRESULT TRUE / FALSE
 */
HRESULT WebControl::clearHistory( void )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc clears cache
 * @param
 * @return HRESULT TRUE / FALSE
 */
HRESULT WebControl::clearCache( void )
{
	SFBHCache hCache = SFBRmGetCacheStorage( (int)m_pLoadGroup );

	if( hCache == SFB_INVALID_CACHE_HANDLE )
	{
		SysDebug(( MID_EXCEPTION, "WebControl::clearCache : SFB_INVALID_CACHE_HANDLE\n" ));
		return HRESULT_NOK;
	}
	else
	{
		SFBClearAllCache( hCache );
		return HRESULT_OK;
	}
}

/*
 * @desc Init cache
 * @param
 * @return HRESULT TRUE / FALSE
 */
HRESULT WebControl::InitCache( void )
{
	SFBHCache hCache = SFBRmGetCacheStorage( (int)m_pLoadGroup );

	if( hCache == SFB_INVALID_CACHE_HANDLE )
	{
		SysDebug(( MID_EXCEPTION, "WebControl::clearCache : SFB_INVALID_CACHE_HANDLE\n" ));
		return HRESULT_NOK;
	}
	else
	{
		if( g_pWebControlVector->size() == 1 )
		SFBInitCache( hCache );
		return HRESULT_OK;
	}
}

HRESULT WebControl::getWmlDoCount( int *out_count )
{
	__FuncEntered();

	if( out_count == NULL )
		return HRESULT_INVALID_PARAM;

	Document *pDoc = m_pMainWebFrame->GetFrame()->document();
	*out_count = pDoc->WML_ItemCount();

	return HRESULT_OK;
}

HRESULT WebControl::getWmlDoName( int in_index, char *inout_buffer, int in_bufsize )
{
	*inout_buffer = '0';

	Document *pDoc = m_pMainWebFrame->GetFrame()->document();
	Document::HTMLMode mode = pDoc->htmlMode();

	if( mode == Document::Wml )
	{
		WMLDocument *pWMLDoc = static_cast<WMLDocument*>(pDoc);

		if( pWMLDoc == NULL )
			return HRESULT_NOK;

		WebCore::NodeMap *pNodeMap = pWMLDoc->getDoElementMap();
		if( pNodeMap == NULL )
			return HRESULT_NOK;

		WebCore::Node *node = pNodeMap->get( in_index + 1 );
		if( node == NULL )
			return HRESULT_NOK;
		WMLDoElement *element = static_cast<WMLDoElement *>(node);


		WebCore::String label = element->getDisplayString();
		if( label.length() > in_bufsize )
			return HRESULT_NO_MEMORY;
		else
			AcStrcpy( inout_buffer, label.latin1().data() );


		return HRESULT_OK;
	}
	else
	{
		return HRESULT_NOK;
	}

}

HRESULT WebControl::executeWmlDo( int in_index )
{
	Document *pDoc = m_pMainWebFrame->GetFrame()->document();

	pDoc->handleRequestedGoEvent(in_index+1);

	return HRESULT_OK;
}

/*
 * @desc emptiess cookie
 * @param
 * @return HRESULT TRUE / FALSE
 */
HRESULT WebControl::emptyCookie( void )
{
	SFBHCookie hCookie = SFBRmGetCookieStorage( (int)m_pLoadGroup );

	if( hCookie == SFB_INVALID_COOKIE_HANDLE )
	{
		SysDebug(( MID_EXCEPTION, "WebControl::emptyCookie : SFB_INVALID_COOKIE_HANDLE\n" ));
		return HRESULT_NOK;
	}
	else
	{
		SFBRemoveAllCookie( hCookie );
		return HRESULT_OK;
	}
}

/*
 * @desc gets IWMLDo
 * @param
 * @return pointer to the IWMLDo
 */
//HRESULT WebControl::getIWMLDo( IWMLDo** out_WMLDo )
//{
//	__NotImplemented(); return HRESULT_NOTSUPPORT;
//}

/*
 * @desc gets ICertificate
 * @param
 * @return pointer to the ICertificate
 */
HRESULT WebControl::getICertificate( ICertificate** out_Certificate )
{
	__NotImplemented();
	return HRESULT_NOTSUPPORT;
}

/*
 * @desc gets ISetting
 * If clients does not set any specific ISetting, the default ISetting will be returned.
 * The default ISetting is shared by all IControl instance.
 * @param
 * @return pointer to the ISetting
 */
HRESULT WebControl::getISetting( ISetting** out_Setting )
{
	*out_Setting = (ISetting*)m_pWebPreferences;
	return HRESULT_OK;
}

/*
 * @desc sets ISetting
 * @param
 * @return pointer to the ISetting
 */
HRESULT WebControl::setISetting( ISetting* in_Setting )
{
	if( in_Setting == m_pWebPreferences )
		return HRESULT_OK;

	if( m_pWebPreferences )
	{
		m_pWebPreferences->removeListener( this );

		delete m_pWebPreferences;
		m_pWebPreferences = NULL;
	}

	m_pWebPreferences = dynamic_cast<WebPreferences*>(in_Setting);

	ChangeSetting(  m_pWebPreferences );

	in_Setting->addListener( this );
}

/*
 *@param drawsBackround YES to cause the receiver to draw a default white background, NO otherwise.
*@desc Sets whether the receiver draws a default white background when the loaded page has no background specified.
*/
HRESULT WebControl::setDrawBackground( BOOL in_bDraw )
{
    BOOL drawsBackground = in_bDraw;
    Frame* coreFrame = m_pMainWebFrame->GetFrame();

    for (Frame* frame = coreFrame; frame; frame = frame->tree()->traverseNext(coreFrame)) {
        //WebFrameBridge *bridge = (WebFrameBridge *)frame->bridge();
        //WebFrame *webFrame = [bridge webFrame];
        // Never call setDrawsBackground:YES here on the scroll view or the background color will
        // flash between pages loads. setDrawsBackground:YES will be called in _frameLoadCompleted.
        //if (!drawsBackground)
           // [[[webFrame frameView] _scrollView] setDrawsBackground:NO];
        //[[[webFrame frameView] _scrollView] setBackgroundColor:backgroundColor];
        //id documentView = [[webFrame frameView] documentView];
        //if ([documentView respondsToSelector:@selector(setDrawsBackground:)])
        //    [documentView setDrawsBackground:drawsBackground];
        //if ([documentView respondsToSelector:@selector(setBackgroundColor:)])
        //    [documentView setBackgroundColor:backgroundColor];
        //[bridge setDrawsBackground:drawsBackground];
        //[bridge setBaseBackgroundColor:backgroundColor];

		if (frame->view())
			frame->view()->setTransparent(!drawsBackground);
    }

	return HRESULT_OK;
}


/*
* @desc Specifiy maximum zoom factor
*/
HRESULT WebControl::setMaxZoomFactor( float factor )
{
#if defined( _SFB_BACKINGSTORE)
	if( m_pBackingStore )
		m_pBackingStore->SetMaxZoomFactor( factor );
#endif
	return HRESULT_OK;
}

/*
* @desc Specifiy minimum zoom factor
*/
HRESULT WebControl::setMinZoomFactor( float factor )
{
#if defined( _SFB_BACKINGSTORE)
	if( m_pBackingStore )
		m_pBackingStore->SetMinZoomFactor( factor );
#endif
	return HRESULT_OK;
}


/*
* @desc minimize the size of backingstore
* backingstore means the memory gc used by WebControl
* WebControl use bigger memmory gc than actual window size, 
* So If you want to save memory when the webcontrol is hided or unfocused, 
* use this funtion to shrink the size of memorygc same as window size .
* The actual minimum backingstore size is calculated by WebContorl implementation.
*/
HRESULT WebControl::minimizeBackingStore()
{
#if defined( _SFB_BACKINGSTORE)
		if( m_pBackingStore )
		m_pBackingStore->MinimizeBackingStore();
#endif
	return HRESULT_OK;
}

/*
* @desc maximize the size of backingstore
* Use this funtion to restore the size of backingstore after calling minimizeBackingStore().
* The actual maximum backingstore size is calculated by WebControl implementation.
*/
HRESULT WebControl::maximizeBackingStore()
{
#if defined( _SFB_BACKINGSTORE)
		if( m_pBackingStore )
		m_pBackingStore->MaximizeBackingStore(false);
#endif
	return HRESULT_OK;
}



HRESULT WebControl::setInitialOverviewMode( bool bEnable )
{
#if defined( _SFB_BACKINGSTORE)
	if( m_pBackingStore )
		m_pBackingStore->SetInitialOverviewMode(bEnable);
#endif
	return HRESULT_OK;
}


/**
* called when the any values of pSettins is changed
* @param pSetting the setting whose property is changed
* @param in_PropertyID the property id whose value is changed
*/
void WebControl::OnSettingChanged( ISetting *pSetting, int in_PropertyID )
{
	ChangeSetting( pSetting );
}

void WebControl::setoverrideDefaultPreferences(bool set) 
{ 
	m_cantOverrideDefaultPreferences = set ;
}

HRESULT WebControl::ChangeSetting( ISetting* pSettings )
{
	if (m_cantOverrideDefaultPreferences ) //This will fix Poland CQ B0100145293 also
		return HRESULT_OK ;

	WebCore::Settings* settings = m_pPage->settings();
	int intValue;
	String	strValue;
	HRESULT hResult;

		abort();  // If setting is changed during loading, abort the curret page load

	hResult = pSettings->getProperty( LOADS_IMAGE_AUTOMATICALLY, &intValue );
	if( !HRESULT_ISFAIL(hResult) )
		    settings->setLoadsImagesAutomatically(intValue);

	hResult = pSettings->getProperty( MINIMUM_FONT_SIZE, &intValue );
	if( !HRESULT_ISFAIL(hResult) )
		    settings->setMinimumFontSize(intValue);

	hResult = pSettings->getProperty( ENABLE_BG_SOUND, &intValue );
	if( !HRESULT_ISFAIL(hResult) )
		    settings->setBgSoundEnabled(intValue);

	hResult = pSettings->getProperty( MINIMUM_LOGICAL_FONT_SIZE, &intValue );
	if( !HRESULT_ISFAIL(hResult) )
		    settings->setMinimumLogicalFontSize(intValue);


	hResult = pSettings->getProperty( SHOULD_PRINT_BACKGROUND, &intValue );
	if( !HRESULT_ISFAIL(hResult) )
		    settings->setShouldPrintBackgrounds(intValue);

	hResult = pSettings->getProperty( JAVASCRIPT_ENABLED, &intValue );
	if( !HRESULT_ISFAIL(hResult) )
		    settings->setJavaScriptEnabled(intValue);


	hResult = pSettings->getProperty( DEFAULT_FIXED_FONT_SIZE, &intValue );
	if( !HRESULT_ISFAIL(hResult) )
		    settings->setDefaultFixedFontSize(intValue);


	hResult = pSettings->getProperty( DEFAULT_FONT_SIZE, &intValue );
	if( !HRESULT_ISFAIL(hResult) )
		    settings->setDefaultFontSize(intValue);

	hResult = pSettings->getProperty( PLUGIN_ENABLED, &intValue );
	if( !HRESULT_ISFAIL(hResult) )
		    settings->setPluginsEnabled(intValue);

	hResult = pSettings->getProperty( PAGE_OVERVIEW_ENABLED, &intValue);
	if( !HRESULT_ISFAIL(hResult) )
		    settings->setPageOverViewEnabled(intValue);

	hResult = pSettings->getProperty( SANSSERIF_FONT_FAMILY, &strValue );
	if( !HRESULT_ISFAIL(hResult) )
		    settings->setSansSerifFontFamily(strValue.c_str());


	hResult = pSettings->getProperty( FIXED_FONT_FAMILY, &strValue );
	if( !HRESULT_ISFAIL(hResult) )
		    settings->setFixedFontFamily(strValue.c_str());

	hResult = pSettings->getProperty( STANDARD_FONT_FAMILY, &strValue );
	if( !HRESULT_ISFAIL(hResult) )
		    settings->setStandardFontFamily(strValue.c_str());

	hResult = pSettings->getProperty( STANDARD_FONT_FAMILY, &strValue );
	if( !HRESULT_ISFAIL(hResult) )
		    settings->setStandardFontFamily(strValue.c_str());

	// [todo] add more properties

	// rerendering ...
	updatePage();

	return HRESULT_OK;

}

//#ifdef _SFB_PAGE_PREVIEW


void WebControl::pagePreviewCancel(BOOL bRender)
{
	if(!m_Preview)
    {
		m_pagePreview_X = 0;
		m_pagePreview_Y = 0;
		return;
    }

	setZoom(100);

	/* Disabling the Page preview mode */
	m_Preview = FALSE;

	if(!bRender)
{
		m_pagePreview_X = 0;
		m_pagePreview_Y = 0;
		return;
	}

  Frame* frame = core(m_pMainWebFrame);

	WmRect preview_Rect;
	frame->view()->scrollBy(-frame->view()->contentsWidth(),-frame->view()->contentsHeight());
	frame->view()->setContentsPos(((m_pagePreview_X)), ((m_pagePreview_Y)));

	WmGetWindowClientRect( m_hWin , &preview_Rect );

	showScreen(0, 0, preview_Rect.w, preview_Rect.h);

	return;
}

bool WebControl::pagePilotScroll( int pos_X, int pos_Y)
{
	Frame* frame = core(m_pMainWebFrame);
	WmRect preview_Rect;

	if(m_Preview)
	{
			WebCore::Element* element = NULL;
			WebCore::Node* curNode = NULL;
			RenderObject* r = NULL;

			m_Preview = FALSE;
			
		WebCore::IntPoint ptContent = frame->view()->windowToContents(WebCore::IntPoint(pos_X, pos_Y));

		WebCore::Document* NewDocument=frame->documentAtPoint(ptContent);
		
			if( NewDocument )
				element = NewDocument->elementFromPoint(ptContent.x(), ptContent.y() );

		int Height = frame->view()->contentsHeight();
		int Width = frame->view()->contentsWidth();

		setZoom(100);

			if( element )
			{
				curNode = element;
				r  = curNode->renderer();
			}
		
		int x_NewPos = 0;
		int y_NewPos = 0;
		
		    int clientHeight, clientWidth;
		
		if( r != NULL )
		    {
		r->absolutePosition(x_NewPos, y_NewPos);

				if(element->hasTagName(WebCore::HTMLNames::anchorTag) || element->hasTagName(WebCore::HTMLNames::aTag) 
						|| element->hasTagName(WebCore::HTMLNames::baseTag))
				{
					clientWidth = r->parent()->clientWidth();
					clientHeight = r->parent()->clientHeight();
				}
				else
				{
					clientWidth = r->clientWidth();
					clientHeight = r->clientHeight();
				}
			}

		int x_CalcPos, y_CalcPos;
		x_CalcPos = (m_pagepreview_W * ptContent.x())/ Width;
		y_CalcPos = (m_pagepreview_H * ptContent.y())/ Height;
		
			if( x_NewPos == 0 || y_NewPos == 0 )
		{
			x_NewPos = x_CalcPos;
			y_NewPos = y_CalcPos;
		}
			else
			{
				if((((x_NewPos + clientWidth) > x_CalcPos) && (x_CalcPos > x_NewPos)) || curNode->isBlockFlowOrBlockTable())
					x_NewPos = x_CalcPos - (frame->view()->visibleWidth()/2);
				
				if(((((y_NewPos + clientHeight) > y_CalcPos) && (y_CalcPos > y_NewPos))) || curNode->isBlockFlowOrBlockTable())
					y_NewPos = y_CalcPos - (frame->view()->visibleWidth()/2);
			}

		frame->view()->scrollBy(-frame->view()->contentsWidth(),-frame->view()->contentsHeight());
		frame->view()->setContentsPos(x_NewPos, y_NewPos);

		WmGetWindowClientRect( m_hWin , &preview_Rect );

			//m_Preview = FALSE;

		showScreen(0, 0, preview_Rect.w, preview_Rect.h);

		return TRUE;
	}
	else
	{
			Element* element = NULL;
			WebCore::Node* node = NULL;

		WebCore::FrameView *frameview = frame->view();
		WebCore::IntPoint ptContent = frameview->windowToContents(WebCore::IntPoint(pos_X, pos_Y));
		m_pagePreview_X = ptContent.x();
		m_pagePreview_Y = ptContent.y();

		WebCore::Document* NewDocument = frame->documentAtPoint(WebCore::IntPoint( m_pagePreview_X, m_pagePreview_Y ));

			if( NewDocument )
				element = NewDocument->elementFromPoint(m_pagePreview_X, m_pagePreview_Y);

			if(element)
				node = element;

		while(node)
		{
			if(node->isLink() || equalIgnoringCase(element->tagName(), TAG_IMG)
					|| equalIgnoringCase(element->tagName(), TAG_INPUT) 
					|| element->hasTagName(WebCore::HTMLNames::anchorTag) 
					|| element->hasTagName(WebCore::HTMLNames::aTag) 
					|| element->hasTagName(WebCore::HTMLNames::baseTag))
				return FALSE;
			
			node = node->parentNode();
		}

		return TRUE;
	}
}
//#endif


// bs0425.lee 2008.04.29 modified for Page Preview.

/*
 * @desc check mode that a browser engine is page pilot mode or not
 * @param
 * @return HRESULT TRUE/FALSE
 */
#define PAGE_MOVE   50	// Border rect moves by this count

//#ifdef _SFB_PAGE_PREVIEW

HRESULT WebControl::isMapViewMode( void )
{
	// WebKitShpPagePrivate* pageData = page->privatePage;
//	WebCore::Page* m_pPage=WebCore::GetPage();
	 int bPreview;
	bPreview = m_Preview;
	return bPreview;
}

/*
 * @desc sets page Map View mode to browser engine
 * @param
 * @return HRESULT TRUE/FALSE
 */
HRESULT WebControl::setMapViewMode( BOOL in_bOnOff )
{

	if(in_bOnOff == FALSE)
	{
		pagePreviewCancel(TRUE);
		return HRESULT_OK;
	}

	if(m_Preview)
		return HRESULT_OK;

	Frame* frame = core(m_pMainWebFrame);

	HWIN m1_hWin=GetHWIN();

	WmRect preview_Rect;
	int Height,Width;

	// Implemented to avoid launching page pilot if Page width is less or equal to 240
	if( frame->view()->contentsWidth() <= 2 * frame->view()->visibleWidth() )
		return HRESULT_FALSE;

	m_Preview = true;

	WmGetWindowClientRect( m1_hWin , &preview_Rect );

	m_pagepreview_H = frame->view()->contentsHeight();
	m_pagepreview_W = frame->view()->contentsWidth();

	setZoom(30);
	
	Height = frame->view()->contentsHeight();
	Width = frame->view()->contentsWidth();

	frame->view()->scrollBy(-frame->view()->contentsWidth(),-frame->view()->contentsHeight());
	frame->view()->setContentsPos(((Width * (m_pagePreview_X)/ m_pagepreview_W)), ((Height * (m_pagePreview_Y))/ m_pagepreview_H));

		showScreen(0, 0, preview_Rect.w, preview_Rect.h);

	return HRESULT_OK;

}

#if 0 // Need ot check deleted in the latest code 
void WebControl::pagePreviewKeyEvent( int keyCode,  bool isKeyUp)
{
  	 Frame* frame = core(m_pMainWebFrame);
	FrameView* view = frame->view();
	int i, ii, ij;
	WmRect preview_Rect;
	IntRect viewRect = enclosingIntRect(frame->view()->visibleContentRectConsideringExternalScrollers());

	if(isKeyUp != false) return;

	WmGetWindowClientRect( m_hWin , &preview_Rect );

	switch(keyCode)
	{
		//case BR_APP_KEY_NAVI_UP: //14
		case VK_HP_UP:
			i = m_border_rect->y() - PAGE_MOVE;
			if(i > 0)
				m_border_rect->setY(i);
			else if(viewRect.y() == 0)
				m_border_rect->setY(0);
			else
				frame->view()->scroll(ScrollUp, ScrollByLine, 5);

			break;

		//case BR_APP_KEY_NAVI_DOWN: //15
		case VK_HP_DOWN:
			i = m_border_rect->y() + PAGE_MOVE;
			ii = i + m_border_rect->height();
			ij = viewRect.y() + viewRect.height();
			if(ii < preview_Rect.w)
				m_border_rect->setY(i);
			else if(ij == frame->view()->contentsHeight())
				m_border_rect->setY( (preview_Rect.h / 2));
			else
				frame->view()->scroll(ScrollDown, ScrollByLine, 5);

			break;

		//case BR_APP_KEY_NAVI_LEFT: //12
		case VK_HP_LEFT:
			i = m_border_rect->x() - PAGE_MOVE;
			if(i > 0)
				m_border_rect->setX(i);
			else if(viewRect.x() == 0)
				m_border_rect->setX(0);
			else
				frame->view()->scroll(ScrollLeft, ScrollByLine, 5);
			
			break;

		//case BR_APP_KEY_NAVI_RIGHT: //12
		case VK_HP_RIGHT:
			i = m_border_rect->x() + PAGE_MOVE;
			ii = i +m_border_rect->width();
			ij = viewRect.x() + viewRect.width();
			if (ii < preview_Rect.h)
				m_border_rect->setX(i);
			else if(ij >= (frame->view()->contentsWidth() ))
				m_border_rect->setX((preview_Rect.w / 2) );
			else
				frame->view()->scroll(ScrollRight, ScrollByLine, 5);

			break;

	}

	WmRect wmRect;
	wmRect.x = 0;
	wmRect.y = 0;
	wmRect.w = preview_Rect.w;
	wmRect.h = preview_Rect.h;

	drawScreen(wmRect.x,wmRect.y ,wmRect.w,wmRect.h);
}
# endif
//#endif

// bs0425.lee  2008.04.29.  modified to implement Page Pilot.
/////////////////////////////////////////////////////////////////////////////////////////////



HRESULT
WebControl::HandleKeyEvent( int keyCode,  int isKeyUp)
{
    Frame* frame = core(m_pMainWebFrame);

    HWIN hFocusedWnd = WmGetFocusWindow();
    if ( hFocusedWnd != m_hWin )
    {
        frame->eventHandler()->keyEvent(PlatformKeyboardEvent(keyCode,  isKeyUp));
        return HRESULT_OK;
    }

    frame->page()->navigationInstance()->handleNavigationEvent(keyCode, isKeyUp);

	return HRESULT_OK;
}


HRESULT WebControl::HandleMouseEvent( int recX, int recY, WebCore::MouseEventType in_MouseEventType)
{
	WebCore::Frame* frame = m_pMainWebFrame->GetFrame();
    HWIN hFocusedWnd = WmGetFocusWindow();

	SysDebug(( MID_EXCEPTION, "WebControl::HandleMouseEvent  recX : (%d) ,recY : (%d) , MouseEventType: (%d) \n", recX,recY,in_MouseEventType));

	// bs0425.lee 2008.04.30  modified for Page Preview.

	//#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview

	if((in_MouseEventType == MouseEventPressed) && gPreview)
	{
		int scroll_event=false;
		pagePreviewscrollPane(scroll_event, recX, recY);
		return HRESULT_OK	;
	}
	else if(gPreview)
	{
		return HRESULT_OK	;
	}

	//#endif


	WebCore::NativeEditField* pNativeEdit = NativeEditField::getNativeEditFieldInstance();
    WebAssert(pNativeEdit);
   // bs0425.lee 2008.04.28 chaned to fix edit field.
	if ( pNativeEdit->isActivated() )
    {
		   pNativeEdit->_sendTextToBrowserEngine();
		   pNativeEdit->unsetFocusedNode();
		   pNativeEdit->deactivate();
 	}

	//for handling mouse event in case of combo box- Hema
	WebCore::NativeComboField* pNativeCombo = NativeComboField::getNativeComboFieldInstance();  
    WebAssert(pNativeCombo);
   	if ( pNativeCombo->isActivated() )
    {
		   pNativeCombo->SetSelectedIndex();
		   pNativeCombo->unsetFocusedNode();
		   pNativeCombo->deactivate();
 	}
	//for handling mouse event in case of combo box- Hema

#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)
	WebCore::NativeListBox* pNativeList = NativeListBox::getNativeListBoxInstance();
	WebAssert(pNativeList);

	if( pNativeList->isActivated() )
    {
		   pNativeList->unsetFocusedNode();
		   pNativeList->deactivate();
 	}
#endif //#if defined(USE_NATIVE_WIDGET_SUPPORT) && defined(USE_LIST_BOX_SUPPORT)

#ifdef SUPPORT_SMART_TOUCH_FOCUSING
	if(frame->page()->navigationInstance()->isSmartTouchEnabled())
	{
	IntPoint modifiedPoint = frame->page()->navigationInstance()->getSmartTouchLocation(IntPoint(recX, recY));
	recX = modifiedPoint.x();
	recY = modifiedPoint.y();
	}
#endif

    switch(in_MouseEventType)
		{
			case WebCore::MouseEventMoved: // TouchPressed
			{
				 //for hovered
			/*
				WebCore::FrameView *frameview = frame->view();
				WebCore::IntPoint ptContent = frameview->windowToContents( WebCore::IntPoint(recX, recY));
				WebCore::PlatformMouseEvent pme( recX, recY,ptContent.x() , ptContent.y(), WebCore::NoButton, WebCore::MouseEventMoved,
							false, false, true, false, false, 0, 0);
				frameview->frame()->eventHandler()->mouseMoved(pme);


				const WebCore::HitTestRequest hitTest(false, true);
				WebCore::IntPoint documentPoint(ptContent.x(), ptContent.y());
				WebCore::MouseEventWithHitTestResults mev = frameview->frame()->document()->prepareMouseEvent(hitTest, documentPoint, pme);
			*/
#ifdef SUPPORT_TOUCH_SCREEN				
				frame->page()->navigationInstance()->touchMoved(WebCore::IntPoint(recX, recY));
#endif

      			break;
			}

			case WebCore::MouseEventPressed: // TouchClicked
			{
				WebCore::FrameView *frameview = frame->view();
				WebCore::MouseEventType met = WebCore::MouseEventPressed;

				// See : m_virtualCursor->sendVirtualMouseEvent(VIRTUAL_MOUSE_PRESSED);

				WebCore::IntPoint ptContent = frameview->windowToContents(WebCore::IntPoint(recX,recY));
				WebCore::PlatformMouseEvent pme(recX,recY,ptContent.x() , ptContent.y(), LeftButton, met,
						   true, false, false, false, false, 0, 0);

 				 frameview->frame()->eventHandler()->handleMousePressEvent(pme);

				// See :m_virtualCursor->sendVirtualMouseEvent(VIRTUAL_MOUSE_RELEASED);

				met = WebCore::MouseEventReleased;
				WebCore::PlatformMouseEvent pme_(recX, recY,ptContent.x() , ptContent.y(), LeftButton, met,
					   true, false, false, false, false, 0, 0);

				frameview->frame()->eventHandler()->handleMouseReleaseEvent(pme_);

#ifdef SUPPORT_TOUCH_SCREEN
				frame->page()->navigationInstance()->touchPressed(WebCore::IntPoint(recX, recY));
#endif
				break;
			}

			case WebCore::MouseEventReleased: // TouchReleased
			{
				 //for hovered
				WebCore::FrameView *frameview = frame->view();
				WebCore::IntPoint ptContent = frameview->windowToContents( WebCore::IntPoint(recX, recY));
				WebCore::PlatformMouseEvent pme( recX, recY,ptContent.x() , ptContent.y(), WebCore::NoButton, WebCore::MouseEventMoved,
							false, false, true, false, false, 0, 0);
				frameview->frame()->eventHandler()->mouseMoved(pme);


				const WebCore::HitTestRequest hitTest(false, true);
				WebCore::IntPoint documentPoint(ptContent.x(), ptContent.y());
				WebCore::MouseEventWithHitTestResults mev = frameview->frame()->document()->prepareMouseEvent(hitTest, documentPoint, pme);

#ifdef SUPPORT_TOUCH_SCREEN
				frame->page()->navigationInstance()->touchReleased(WebCore::IntPoint(recX, recY));
#endif

#ifdef SFB_SMARTLINK_FEATURE
				frameview->frame()->eventHandler()->handleMouseReleaseEvent(pme);
#endif
				break;
			}
		}

	return HRESULT_OK;
}






void WebControl::ReceivedAuthenticationChallenge(  SFBRmHClient hClient,
											SFBRmReqId reqID,
											int transID,
											char* szProposedUser,
											char* szProposedPass,
											char* szRealm,
											char* pszUrl,
											SFBRmAuthenticationCallbackResponse fpRespCallback )
{
	if( !m_pContentListener )
		return;

	WebContentsSender *pSender = new WebContentsSender( this );
	pSender->SetAuthenticationInfo( hClient, reqID, transID, fpRespCallback );
	SFBalDebug(BAL_COMMON, "ReceivedAuthenticationChallenge:RequestedUrl [%s] \n",pszUrl);
	m_pContentListener->auth( szRealm, pszUrl,szProposedUser, szProposedPass, pSender );
}

void WebControl::ReceivedCookieAcceptMethod ( SFBRmHClient hClient,
											SFBRmReqId reqID,
											char* szHostName,
											BOOL bCookieExist,
											char* szCookieString,
											void* pCookie,
											SFBRmCookieCallbackResponse fpCallbackResponse)
{
	if( !m_pContentListener )
		return;

	WebContentsSender *pSender = new WebContentsSender( this );

	pSender->SetCookieInfo( hClient, reqID, fpCallbackResponse );

	m_pContentListener->cookie( szHostName, szCookieString, pSender, pCookie );
}

	void WebControl::ReceivedPostInit()
	{
		if( !m_pContentListener )
			return;
		m_pContentListener->uploadInit(NULL);
	}


void WebControl::ReceivedPostStatus(SFBRmHClient hClient,
											SFBRmReqId reqID,
										//	int nLenPostedData,
											int nLenTotalPostedData,
											int nLenTotalPostData
											)
{

	if( !m_pContentListener )
		return;

		//WebContentsSender *pSender = new WebContentsSender( this );

		m_pContentListener->uploadState( nLenTotalPostData,  nLenTotalPostedData,  NULL);
	
}

void WebControl::DisplayUnsupportedFileError()
{
	if(m_pContentListener)
		m_pContentListener->DisplayUnsupportedFileError();
}

HRESULT WebControl::ReceivedRedirectionChallenge(  SFBRmHClient hClient,
											SFBRmReqId reqID,
											int transID,
											SFBRmRequestInfo* pRedirectedReq,
											char* pResHeaders,
											int nLenResHeaders,
											SFBRmRedirectCallbackResponse fpCallbackResponse )
{
	if( !m_pContentListener )
		return FALSE;

	WebContentsSender *pSender = new WebContentsSender( this );
	pSender->SetRedirectionInfo( hClient, reqID, pRedirectedReq, fpCallbackResponse );

	if(m_pContentListener->redirect( pRedirectedReq->szURL, pSender ) == FALSE)
		return FALSE;

	return TRUE;
}


/**
  * This function is used for SSL confirmation pop up
  * @param1 : hClient - Client Id
  * @param2 : reqID - request Id
  * @param3 : transID - transaction Id
  * @param4 : pCertData - certificate data to be shown
  * @param5 : fpRespCallback - SSL Confirmation callback response function
  */
void WebControl::GetSSLConfirmation(  SFBRmHClient hClient,
										SFBRmReqId reqID,
											int transID,
											char* pCertData,
											SFBalRmSSLCallbackResponse fpRespCallback )
{
      if( !m_pContentListener )
		return ;

	WebContentsSender *pSender = new WebContentsSender( this );
	pSender->SetSSLInfo( hClient, reqID, transID, fpRespCallback );

	m_pContentListener->confirm( CONFIRM_CERTIFICATE, NULL, pCertData, pSender );

	return ;
}



void WebControl::closeWindowSoon()
{
	setWebControlCloseWindowFired(true) ;
	HTimer hTimer = WmCreateTimer( "WebControlWindowClose" );
	ULONG pParam = (ULONG)this ;
	WmStartTimer( hTimer, 1, WebControl_CloseWindow, (ULONG)pParam );

    //m_pCloseWindowTimer->startOneShot(0);
}

//void WebControl::closeWindowTimerFired(WebCore::Timer<WebControl>*)
void WebControl::closeWindowTimerFired()
{
    closeWindow();
}

void WebControl::closeWindow()
{
	if( m_pContentListener )
		m_pContentListener->closeWebControl();
}


// For handling PDP Event - Sumdra
void WebControl::ReceivedHandlePDPevent(SFBRmHClient hClient, 
										SFBRmPDPContextEventParams *pParams, 
										SFBRmPDPContextCallbackResponse fpCallbackResponse)
{
	if( !m_pContentListener )
		return ;

	if(pParams->nEvent == SFB_PDPCTX_CONFIRM_DISCONECT_OTHER_PDP)
	{
		WebContentsSender *pSender = new WebContentsSender( this );
		pSender->SetPDPEventInfo(hClient, fpCallbackResponse );

		m_pContentListener->confirm(CONFIRM_PDP_OTHERUSE, NULL, NULL, pSender);
	}
	else if(pParams->nEvent == SFB_PDPCTX_TRANSACTION)
	{
		m_pContentListener->state(STATECHANGE_TRANSACTION, 0, NULL);
	}
	else if(pParams->nEvent == SFB_PDPCTX_NO_TRANSACTION)
	{
		m_pContentListener->state(STATECHANGE_NO_TRANSACTION, 0, NULL);
	}
	
}

void WebControl::AddURLToVisitedLinks(char* szURL)
{
    if (m_pPage)
        m_pPage->group().addVisitedLink(WebCore::KURL(szURL));
}


HRESULT
WebControl::getContentsSize( int *width, int *heigth)
{
	int nWidth, nHeight = 0;

	WebCore::Frame* frame = m_pMainWebFrame->GetFrame();
	nWidth = frame->view()->contentsWidth();
	nHeight = frame->view()->contentsHeight();

	*width = nWidth;
	*heigth = nHeight;

	return HRESULT_OK;
}

void WebControl::HandleOutOfMemory()
{
	// [todo] check if this WebControl is loading a page, otherwise return
	// if( !isloading() )
	//		return;

	if( m_pContentListener )
    {
		WebCore::WMLDocument *pWmlDoc;
        ErrorCode erroCode = ( MemoryManager::Status() & CMM_CHECKRESULT_CHECK_CMM ) ? ERROR_OUTOF_MEMORY : ERROR_COLLECTING_MEMORY;

		/*Currently we shall not display the pop-up related to images. Display only OOM pop-up*/
		if(ERROR_COLLECTING_MEMORY != erroCode)
        {
#ifdef _SHP_WIDGET
			/* 
			The Timer should be destroyed only when 
			destroying of last instance of webcontrol 
			*/
            WebCore::stopSharedTimer();
#else
			/* Keeping the default implementation */
            WebCore::stopSharedTimer(true);
#endif
		m_pContentListener->error( erroCode, NULL, NULL );
        }
#if defined(SHP_SUPPORT_WML)
		if(ERROR_OUTOF_MEMORY == erroCode)
		{
			WebCore::Frame *frame = core(m_pMainWebFrame);
			if(frame->document()->htmlMode() == WebCore::Document::Wml)
			{
				frame->setWMLOOMState() ;
			}
		}
#endif
    }
}

/**
 * @desc set the selected file path
 * @param      in_szFilePath  [in] The file path
 * @param      in_iPathLen  [in] The path length
 */	
HRESULT WebControl::setSelectedFilePath(const char* in_szFilePath, int in_iPathLen)
{
#ifdef HTTP_POST_UPLOAD_SUPPORT
	using WebCore::String;
	String filePath(in_szFilePath);

	Frame* frame = core(m_pMainWebFrame);

	//Rohit-Bug 2639
	if (frame)
	{
		WebCore::Node* curr = frame->page()->uploadControlNode();

		if(curr && curr->nodeId() == WebCore::Node::ID_INPUT)
		{
			//Fix for B0100137726
			if((NULL != in_szFilePath) &&(FALSE == SFBalIsValidPathName(in_szFilePath,BAL_FILE_TYPE)))
			{
				DisplayUnsupportedFileError();
				static_cast<RenderFileUploadControl*>(curr->renderer())->receiveSelectedFile(String());
			}
			else
				static_cast<RenderFileUploadControl*>(curr->renderer())->receiveSelectedFile(filePath);
		}
	}
#endif // HTTP_POST_UPLOAD_SUPPORT

	return HRESULT_OK;
}

URILoader::ILoadGroup*	WebControl::getLoadGroup(void)
{
	SysDebug(( MID_BROWSER, "WebControl::setLoadGroup : LoadGroup [%d]\n", (int)m_pLoadGroup ));

	return (URILoader::ILoadGroup*)m_pLoadGroup;
}

void	WebControl::setLoadGroup(URILoader::ILoadGroup* clinetID)
{
	SFBHCache	currentHCache;
	SFBHCookie	currentHCookie;

	m_pLoadGroup = (void*)clinetID;

	SysDebug(( MID_BROWSER, "WebControl::setLoadGroup : LoadGroup [%d]\n", (int)m_pLoadGroup ));

	currentHCache = SFBRmGetCacheStorage((SFBRmHClient)m_pLoadGroup);

	currentHCookie = SFBRmGetCookieStorage((SFBRmHClient)m_pLoadGroup);

	// Set cache
	if(currentHCache != NULL)
	{
		SFBRmSetCacheStorage((int)m_pLoadGroup, currentHCache);
	}
	else
	{
		if(AcStricmp(m_strName.c_str(), "Widget") == 0)
			SFBRmSetCacheStorage((int)m_pLoadGroup, NULL);
		else
			SFBRmSetCacheStorage((int)m_pLoadGroup, g_hCacheStorage);

	}

	//Set cookie
	
	if(currentHCookie != NULL)
	{
		SFBRmSetCookieStorage((int)m_pLoadGroup, currentHCookie);
	}
	else
	{
		if(AcStricmp(m_strName.c_str(), "Widget") == 0)
			SFBRmSetCookieStorage((int)m_pLoadGroup, NULL);

		else
			SFBRmSetCookieStorage((int)m_pLoadGroup, g_hCookieStorage);
	}
}

	void WebControl::updateActiveState()
	{
		Frame* frame = core(m_pMainWebFrame);
		frame->page()->focusController()->setActive(frame);
	}

	void WebControl::updateFocusedAndActiveState()
	{
		updateActiveState();
		
		Frame* frame = core(m_pMainWebFrame);

		bool active = frame->page()->focusController()->isActive();
		Frame* focusedFrame = frame->page()->focusController()->focusedOrMainFrame();

		frame->selectionController()->setFocused(active && frame == focusedFrame);
	}

        //gnana for end key performance SFB_IMPROVE_END_KEY
	void WebControl::InstanceExit()
	{
		SysDebug(( MID_EXCEPTION,"WebControl::InstanceExit: Start: BAL_EVENT_BROWSER_EXIT_ENGINE Event sent \n" ));

		// the JS objects are released only when we navigate to some other page. As we are leaving the browser
		// application we achieve the same purpose by initializing the frame. We may have come to the current
		// page by a back, forward or some other operation hence we explicitly need to set the frame load
		// type to standard.
		Frame* frame = core(m_pMainWebFrame);
		frame->document()->removeAllEventListenersFromAllNodes();
		frame->loader()->setLoadType(FrameLoadTypeStandard);
		frame->loader()->init();
		setExitInstanceStatus(true);
	
		SFBalSendEvent(BAL_APP_BROWSER,BAL_EVENT_BROWSER_EXIT_ENGINE, NULL, NULL, NULL, m_pMainWebFrame->GetWebControl());
		SysDebug(( MID_EXCEPTION,"WebControl::InstanceExit: End: BAL_EVENT_BROWSER_EXIT_ENGINE Event sent \n" ));
	}

	void WebControl::setEndKeyPressed(bool status)
	{
# if SFB_WIDGET
		if (m_strName == "Widget")
		{
			WebCore::Frame* pFrame = m_pMainWebFrame->GetFrame();
			WebFrameLoaderClient* pClient = static_cast<WebFrameLoaderClient*>(pFrame->loader()->client());
			return pClient->DeActivateNativeInstance();
			
		}	
# endif
		m_bEndKeyPressed = status ; /* default Implementation */
	}

	void WebControl::setSmartTouchEnabled(bool enabled)
	{
		WebCore::Frame* frame = m_pMainWebFrame->GetFrame();
		frame->page()->navigationInstance()->setSmartTouchEnabled(enabled);
	}

    bool WebControl::isSmartTouchEnabled() 
	{
		WebCore::Frame* frame = m_pMainWebFrame->GetFrame();
		return frame->page()->navigationInstance()->isSmartTouchEnabled();
	}

    void WebControl::setSmartTouchFocusRange(int range)
	{
		WebCore::Frame* frame = m_pMainWebFrame->GetFrame();
		frame->page()->navigationInstance()->setSmartTouchFocusRange(range);
	}

	void WebControl::setLcdDimmingMode(bool bIsDimming)
	{
		SysDebug(( MID_BROWSER, "WebControl::setLcdDimmingMode : Dimming Mode [%d]\n", bIsDimming ));
		SFBRmSetLcdDimmingMode((SFBRmHClient)m_pLoadGroup, bIsDimming);
	}


class WebControlOOMNotifier : public ICMMNotifier
{
public:
	//WebControlOOMNotifier();
	virtual gint32 Notify();
};

static WebControlOOMNotifier gWebControlOOMNotifier;


//WebControlOOMNotifier::WebControlOOMNotifier()
//{
//	MemoryManager::SetNotifier( this );
//}

gint32 WebControlOOMNotifier::Notify()
{
	// out of memory
	if( g_pWebControlVector == NULL )
		return 0;

	for(  int i = 0; i < g_pWebControlVector->size() ; i++ )
	{
		WebControl *pWebControl = g_pWebControlVector->at(i);
		pWebControl->HandleOutOfMemory();
	}
}


//[todo]
///////////////////////////////////////////////////////////////////////////////////////////////
// This code block will be removed soon. [start]
///////////////////////////////////////////////////////////////////////////////////////////////
//Pramod - Multiple instance support
#ifndef MULTIPLE_INSTANCE_SUPPORT
WebControl *gpWebControl = NULL ;
#endif


#ifdef MULTIPLE_INSTANCE_SUPPORT
#if !defined(_JASMINE_ECOM)
extern "C" void SFBRmEventHandler( SFBalEventDataList * evt );
#endif
extern "C" void __BrCoreControl_HandleUpdateContents( WmRect* wmRect, void* pWebControl )
{
	SysAssert(pWebControl != NULL);

	if(pWebControl)
	{
		WebControl *pTempWebControl = (WebControl*)pWebControl;
# ifdef  SFB_WIDGET
		// Fix B0100142435 amitg@samsung.com
		pTempWebControl->setExposeEvent(false);
# endif
#if !defined( _SFB_BACKINGSTORE )
	WmRect rect;
		WmGetWindowClientRect( ((WebControl*)pWebControl)->GetHWIN(), &rect );

		//HRESULT hr;

		//hr = ((WebControl*)pWebControl)->showScreen( rect.x, rect.y, rect.w, rect.h );
		IContentsListener* pContentListener = NULL;

		pContentListener = pTempWebControl->GetContentListener();
		if( pContentListener )
			pContentListener->updateScreen( rect.x, rect.y, rect.w, rect.h );
#else
		//IContentsListener* pContentListener = NULL;

		//// update backingstore
		//// The update area is automatically caculated by the BackingStore
		//pTempWebControl->GetBackingStore()->Paint();

		//pContentListener = pTempWebControl->GetContentListener();
		//if( pContentListener && !pTempWebControl->isZooming() )
		//	pContentListener->updateScreen(  0, 0, 0, 0 );
		if( pTempWebControl->isExitInstanceCalled() == false )
		pTempWebControl->GetBackingStore()->HandleUpdateContents( WebCore::IntRect( wmRect->x, wmRect->y, wmRect->w, wmRect->h ) );

#endif
	}
}
extern "C" void BrCoreControl_OOMHandler( void )
{
	WTF::HashMap<void*,int>::iterator endInstance = gWebControlContext->hWebControlInstanceMap->end();
	WTF::HashMap<void*,int>::iterator beginInstance = gWebControlContext->hWebControlInstanceMap->begin();

	 for (beginInstance; beginInstance != endInstance; ++beginInstance)
	 {
		void* pWebControlInstance = (beginInstance.get())->first;
		
		WebControl *pWebControl = (WebControl*)pWebControlInstance;				
		if(pWebControlInstance && ("Browser" == pWebControl->GetInstanceName()))
		{
			WebCore::FrameLoader* loader = ((WebControl*)pWebControlInstance)->GetMainWebFrame()->GetFrame()->loader();
			if (loader)
			{
				loader->stopAllLoaders();
				loader->client()->setLoadingComlete();//gnana added for bugid 1951
			}
			((WebControl*)pWebControlInstance)->onOOMEvent();
		}
	}
}

extern "C" void BrCoreControl_JSGCHandler( void )
{
    gcController().garbageCollectNow();
}

extern "C" void BrCoreControl_Relaod( void* pWebControl )
{
    ((WebControl*)pWebControl)->reloadPage();
}
    
extern "C" void BrCoreControl_TokenizerHandler(void* pWebControl)
{
	if( pWebControl == NULL )
		return;

	WebFrame*	pWebFrame = ((WebControl*)pWebControl)->GetMainWebFrame();
	Frame* frame = pWebFrame->GetFrame();

	if(frame && frame->document() && frame->document()->tokenizer())
    {
        frame->document()->tokenizer()->write(SegmentedString(), true);
        frame->view()->updateOnScroll();
    }
}

extern "C" void __BrCoreControl_FetchRequest (void* pWebControl)
{
	WebFrame*	pWebFrame = ((WebControl*)pWebControl)->GetMainWebFrame();

	pWebFrame->GetFrame()->loader()->loadRequestedUrl () ;
}

extern "C" BOOL __BrCoreControl_GoToURL( char*	in_url, void* pWebControl )
{
	((WebControl*)pWebControl)->loadURL( in_url );
	return TRUE;
}

#ifdef SFB_SMARTLINK_FEATURE
	extern "C" void BrCoreControl_SmartLinkIndexing( void* pWebControl, Frame* pFrame )
	{
		WebFrame*	pWebFrame = ((WebControl*)pWebControl)->GetMainWebFrame();

		Frame* TempFrame = pWebFrame->GetFrame();

		while(TempFrame)
		{
			if( (TempFrame == pFrame) && pFrame->loader()->client()->isLoadingComplete() && !(((WebControl*)pWebControl)->isMapViewMode()))
			pFrame->GetSmartLinkMap()->StartIndexingSmartLinks();
			
			TempFrame = TempFrame->tree()->traverseNext();

		}
	}
#endif

        //gnana for end key performance SFB_IMPROVE_END_KEY
	extern "C" void BrCoreControl_ExitEngine(void* pWebControl)
	{
		WebControl *pTempWebControl = (WebControl*)pWebControl;

		SysDebug(( MID_EXCEPTION,"BrCoreControl_ExitEngine: Start: BAL_EVENT_BROWSER_EXIT_ENGINE Event executing \n" ));
		pTempWebControl->Release();
		SysDebug(( MID_EXCEPTION,"BrCoreControl_ExitEngine: End: BAL_EVENT_BROWSER_EXIT_ENGINE Event Executed \n" ));
	}

#else

extern "C" void SFBRmEventHandler( SFBalEventDataList * evt );

extern "C" void __BrCoreControl_HandleUpdateContents( WmRect* wmRect )
{
	if(gpWebControl)
	{
	WmRect rect;
    WmGetWindowClientRect( gpWebControl->GetHWIN(), &rect );

	HRESULT hr;

	hr = gpWebControl->showScreen( rect.x, rect.y, rect.w, rect.h );

	}
}

extern "C" void BrCoreControl_OOMHandler( void )
{
	gpWebControl->onOOMEvent();
}

extern "C" void BrCoreControl_JSGCHandler( void )
{
    gcController().garbageCollectNow();
}

extern "C" void BrCoreControl_TokenizerHandler()
{
	WebFrame*	pWebFrame = gpWebControl->GetMainWebFrame();
	Frame* frame = pWebFrame->GetFrame();

	if(frame && frame->document() && frame->document()->tokenizer())
    {
        frame->document()->tokenizer()->write(SegmentedString(), true);
        frame->view()->updateOnScroll();
    }
}

extern "C" void __BrCoreControl_FetchRequest (void)
{
	WebFrame*	pWebFrame = gpWebControl->GetMainWebFrame();

	pWebFrame->GetFrame()->loader()->loadRequestedUrl () ;
}

extern "C" BOOL __BrCoreControl_GoToURL(char*	in_url)
{
	gpWebControl->loadURL( in_url );
	return TRUE;
}

#ifdef SFB_SMARTLINK_FEATURE
	extern "C" BOOL BrCoreControl_SmartLinkIndexing()
	{
		WebFrame*	pWebFrame = gpWebControl->GetMainWebFrame();

		Frame* pFrame = pWebFrame->GetFrame();

		while(pFrame)
		{
			pFrame->GetSmartLinkMap()->StartIndexingSmartLinks();
			pFrame = pFrame->tree()->traverseNext();

		}
	}
#endif
#endif //MULTIPLE_INSTANCE_SUPPORT

//#ifdef MULTIPLE_INSTANCE_SUPPORT
//static void __InitBAL( )
//#else
static void __InitBAL( WebControl *pWebControl )
//#endif
{
#ifndef	MULTIPLE_INSTANCE_SUPPORT
	gpWebControl = pWebControl;
#endif

	SFBalEventCallbacks callbacks;

	AcMemset( &callbacks, 0, sizeof(SFBalEventCallbacks) );

	callbacks.fpHandleUpdateContents = (SFBalHandleUpdateContents)__BrCoreControl_HandleUpdateContents;										   
	callbacks.fpOOMHandler = BrCoreControl_OOMHandler;
	callbacks.fpReload = BrCoreControl_Relaod;
	callbacks.fpGoToURL = (SFBalGoToURL)__BrCoreControl_GoToURL;
	callbacks.fpFetchRequest = __BrCoreControl_FetchRequest;
	callbacks.fpRmEventHandler = SFBRmEventHandler;
    callbacks.fpJSGCHandler = BrCoreControl_JSGCHandler;
    callbacks.fpEngineTokenizerHandler = BrCoreControl_TokenizerHandler;

#ifdef SFB_SMARTLINK_FEATURE
		callbacks.fpSmartLink = (SFBalSmartLink)BrCoreControl_SmartLinkIndexing;
#endif
        //gnana for end key performance SFB_IMPROVE_END_KEY
 	callbacks.fpExitEngine = (SFBalExitEngine)BrCoreControl_ExitEngine;


	SFBalSetEventCallbacks( &callbacks );
}


///////////////////////////////////////////////////////////////////////////////////////////////
// This code block will be removed soon. [end]
///////////////////////////////////////////////////////////////////////////////////////////////








/**
 * @desc creates factory function to create IControl
 * @param in_pszDomainName : pointer to Domain ( it means mocha application ) name
 * @param in_ppIControl
 * @return HRESULT TRUE / FALSE
*/
HRESULT CreateIControl( char* in_pszDomainName,
					   HWIN hWin,
					   int x, int y, int w, int h,
					   IControl** in_ppIControl, GCType gcType )
{
	WebControl *pWebControl = NULL;
	HRESULT hResult;

#ifdef MULTIPLE_INSTANCE_SUPPORT
	InitWebControlInstance();
#endif

	hResult =  WebControl::New( in_pszDomainName, hWin,
										x, y, w, h,
										&pWebControl, gcType );

	MemoryManager::SetNotifier( &gWebControlOOMNotifier );

	if( HRESULT_ISFAIL(hResult) )
	{
		*in_ppIControl = NULL;
	}
	else
	{
////////////////////////////////////////////////////
// This code will be removed soon [todo]
//#ifdef MULTIPLE_INSTANCE_SUPPORT
//	__InitBAL( );
//#else
		__InitBAL( pWebControl );
//#endif
////////////////////////////////////////////////////

		*in_ppIControl = pWebControl;
	}
	return hResult;
}

int GetWebControlCount()
{
	if (g_pWebControlVector != NULL && !g_pWebControlVector->empty() ) 
	{
		return (int)g_pWebControlVector->size();	
	}
	return 0;
	
}

URILoader::ILoadGroup*	CreateClientLoadGroup(char* appName)
{
	SFBRmHClient hProtoHClient;

	hProtoHClient = SFBRmRegisterClient( appName, (SFBRmPDPContextCallback)receivePDPEvent );

	if( (int)(hProtoHClient) == INVALID_HOBJ )
	{
		SysDebug(( MID_EXCEPTION, "CreateRmLoadGroup : SFBRmRegisterClient Fail! \n"));
		return NULL; //HRESULT_NOK;
	}

	//WebControl::setLoadGroup((URILoader::ILoadGroup*)hProtoHClient);

	SysDebug(( MID_BROWSER, "CreateRmLoadGroup : [%d]\n", hProtoHClient ));

	return (URILoader::ILoadGroup*)hProtoHClient;
}

void	DestroyClientLoadGroup(URILoader::ILoadGroup* clinetID)
{
	SysDebug(( MID_BROWSER, "DestroyRmLoadGroup : [%d]\n", clinetID ));

	SFBRmUnregisterClient( (SFBRmHClient)clinetID );

	//SFBExitCache(g_hCacheStorage);
	//SFBExitCookie(g_hCookieStorage);
}

//For handling PDP event - sumdra
void receivePDPEvent(SFBRmHClient hClient, SFBRmPDPContextEventParams *pParams, SFBRmPDPContextCallbackResponse fpCallbackResponse)
{
	if( g_pWebControlVector == NULL )
		return ;

	for(  int i = 0; i < g_pWebControlVector->size() ; i++ )
	{
		WebControl *pWebControl = g_pWebControlVector->at(i);

		if(pWebControl->GetLoadGroup() == (void*)hClient)
			pWebControl->ReceivedHandlePDPevent(hClient, pParams, fpCallbackResponse) ;
	}

	return;
}

#ifdef MULTIPLE_INSTANCE_SUPPORT
void
InitWebControlInstance()
{
	if(gWebControlContext == NULL)
	{
		WebControlContext *newContext = (WebControlContext*) SFBalMemAlloc(sizeof(WebControlContext));
		
		AcMemset(newContext,0x00,sizeof(WebControlContext));

	//	WTF::HashMap<void*,int>* hWebControlInstanceMap = new WTF::HashMap<void*,int>();
		newContext->hWebControlInstanceMap = NULL;
		newContext->iCount = 0;
		gWebControlContext = newContext;
	}
}
#endif

void WebControl::stopLingerTimerAfterAjaxRequest()
{
	if (!m_pContentListener)
	{
		return ;
	}

	m_pContentListener->state(STATECHANGE_NO_TRANSACTION, 0, NULL);
	SFBalSetLingerTimer((SFBRmHClient)m_pLoadGroup,10);
//	SFBRmCancelLingerTimer(g_RmClient);
	return;
}

void WebControl::cancelLingerTimer()
{
	SFBRmCancelLingerTimer((int)m_pLoadGroup);
	return;

}

} // namespace WebControl
