/*
 * Copyright (C) 2007 Holger Hans Peter Freyther
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer. 
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution. 
 * 3.  Neither the name of Apple, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

//#include "config.h"
#include "WebChromeClient.h"

#include "FloatRect.h"
#include "IntRect.h"
#include "PlatformString.h"
#include "CString.h"
#include "TextEncoding.h"
#include "FrameLoadRequest.h"
#include "FrameLoader.h"
#include "WindowFeatures.h"
#if defined(_SFB_BACKINGSTORE)
#include "WebBackingStore.h"
#endif
#include "SFBalMain.h"
#include "XMLHttpRequest.h"
using namespace WebCore;

//#define String WebCore::String;

namespace WebControl {

//using WebCore::String;

WebChromeClient::WebChromeClient( WebControl *pWebControl )
{
	m_pWebControl = pWebControl;
}

WebChromeClient::~WebChromeClient()
{
}


void WebChromeClient::chromeDestroyed()
{
    __NotImplemented();
}

FloatRect WebChromeClient::windowRect()
{
    __NotImplemented();
    return FloatRect();
}

void WebChromeClient::setWindowRect(const FloatRect& r)
{
    __NotImplemented();
}
#ifdef SFB_WIDGET

void WebChromeClient::setPreferenceForKeyInfo(WebCore::String& dataInfo,WebCore::String& keyInfo ) 
{
	int keyLen = keyInfo.length();
	char* key = (char*)MemAlloc(keyLen+1);
	int dataLen = dataInfo.length();
	UChar *pmcharData;;

	AcMemset(key,0x00,(keyLen+1));
	AcStrncpy(key,keyInfo.ascii().data(),keyLen);
	MCHAR * mVal = (MCHAR *) dataInfo.charactersWithNullTermination();		

	char	*pszText = NULL;
	int len = 3*dataLen +1;

	pszText = ( char *)MemCalloc(1 ,len*sizeof(char));
	if (pszText == NULL ) return;

	WmConvert2LCode( pszText, len, mVal );

	bool result = m_pWebControl->GetContentListener()->SetPreferenceForKey( pszText,key); 
	if(key)
		MemFree(key);
	if(pszText)
		MemFree(pszText);
	
		

}
char* WebChromeClient:: preferenceforkeyInfo(WebCore::String& prefKeyInfo) 
{
int len = prefKeyInfo.length();
	char* prefKey = (char*)MemAlloc(len+1);
	char* prefData;
	AcMemset(prefKey,0x00,(len+1));
	AcStrncpy( prefKey,prefKeyInfo.ascii().data(),len);
	bool result = m_pWebControl->GetContentListener()->PreferenceForKey(prefKey, &prefData);
	if(prefKey)
		MemFree(prefKey);
	return prefData;
}

bool WebChromeClient::setWidgetURL(WebCore::String& url) 
{
	int urlLen = url.length();
	char* urlInfo = (char*)MemAlloc(urlLen + 1);
	AcMemset(urlInfo,0x00,(urlLen+1));
	AcStrncpy( urlInfo,url.ascii().data(),urlLen);
	
	bool b = m_pWebControl->GetContentListener()->MaximizeWindow(urlInfo);

	if(urlInfo)
		MemFree(urlInfo);

	return b;
}

bool WebChromeClient::getIsNetworkAvailable()
{
	bool b = m_pWebControl->GetContentListener()->GetNetwork();
	return b;
}

const char* WebChromeClient::getLanguage()
{
	return m_pWebControl->GetContentListener()->GetLanguage();
}
void WebChromeClient::resizeWindow(int width, int height)
{
	m_pWebControl->GetContentListener()->ResizeWindow(width,height);
	return;
}

void WebChromeClient::openUrl(WebCore::String& destUrl)
{
	unsigned int urlLen = destUrl.length();
	char* url = (char*)MemAlloc(urlLen + 1);
	AcMemset(url,0x00,urlLen + 1);
	AcStrcpy(url,destUrl.ascii().data());
	m_pWebControl->GetContentListener()->OpenUrl(url);
	if(url)
		MemFree(url);
}

void WebChromeClient::errorNotify(int errType)
{
	if(errType == 1)
	{
		//cancel existing xmlHttpRequest 's of this document
		XMLHttpRequest::cancelRequests(m_pWebControl->GetMainWebFrame()->GetFrame()->document() );
		//notify finish to Stop 3G Animation
		if (m_pWebControl->GetContentListener())
			m_pWebControl->GetContentListener()->state( STATE_FINISH, 0, NULL );
		m_pWebControl->stopLingerTimerAfterAjaxRequest();
	}
	// For errType = 2 Currently using RM default timeout as we cannot set timeout in engine	
	return;
}

void WebChromeClient::setScroll(bool scroll)
{
	m_pWebControl->GetContentListener()->SetScroll(scroll);
	return;
}

const char* WebChromeClient::getMccMnc()
{
	const char* data = m_pWebControl->GetContentListener()->GetMccMnc();
	return data;
}


#endif
FloatRect WebChromeClient::pageRect()
{
    __NotImplemented();
    return FloatRect();
}

float WebChromeClient::scaleFactor()
{
    __NotImplemented();
    return 1.0;
}

void WebChromeClient::focus()
{
    __NotImplemented();
}

void WebChromeClient::unfocus()
{
    __NotImplemented();
}
    
Page* WebChromeClient::createWindow(Frame* frame, const FrameLoadRequest& request, const WebCore::WindowFeatures& features)
{
    if (features.dialog)
	{
        __NotImplemented();
        return 0;
    } 
	else 
	{
		int aSize = AcStrlen((char*)(frame->document()->url().latin1()));

#if defined( JS_MULTIPLE_WINDOW_SUPPORT )  
		if( m_pWebControl->GetContentListener() )
		{
			WebControl *pControl =  (WebControl*)m_pWebControl->GetContentListener()->createWebControl();
			//return pControl->GetPage();
			if( pControl )
			return pControl->GetPage();
			else
				return 0;
		}
		else
		{
			// by defaults, do nothings
			return 0; 
		}
#else		
		return 0;
		/*WebControl *pNewWebControl = (WebControl *)m_pWebControl->GetContentListener()->createJsWindow();
		pNewWebControl->m_openerWindowUrl = new char[aSize + 1];
		AcStrcpy(pNewWebControl->m_openerWindowUrl,(char*)frame->document()->url().latin1());
		WebFrame*	pWebFrame = pNewWebControl->GetMainWebFrame();
		if (!request.resourceRequest().url().isEmpty())
		{
			pWebFrame->GetFrame()->loader()->load(request, false, true, 0, 0, HashMap<WebCore::String, WebCore::String>());
		}
		return core(pNewWebControl);*/
		////For BUG ID 1738 gnana
		//if( m_pWebControl->m_openerWindowUrl != NULL )
		//{
		//	delete m_pWebControl->m_openerWindowUrl;
		//	m_pWebControl->m_openerWindowUrl = NULL;
		//}

		//m_pWebControl->m_openerWindowUrl = new char[aSize + 1];
		//AcStrcpy(m_pWebControl->m_openerWindowUrl,(char*)frame->document()->url().latin1());

		///* TODO: FrameLoadRequest is not used */
		//if (!request.resourceRequest().url().isEmpty())
		//{
		//	frame->loader()->load(request, false, true, 0, 0, HashMap<WebCore::String, WebCore::String>());
		//}

		//return core(m_pWebControl);
#endif 
	}
}

bool WebChromeClient::savePageFileContext (PlatformFileHandle &fileContext, const WebCore::String &filename, 
											const WebCore::String &filepath, 
											const char *buffer, int bufferLen, bool lastChunk)
{
	bool success = true ;
	if (!fileContext)
		//i need to open a file in this context
		openFile(filepath.utf8().data(), filename.utf8().data(),  
                    FILE_MODE_CREATE | FILE_MODE_WRITE , fileContext);
	if (fileContext)
		if (bufferLen)	{
			int writeLen = writeToFile (fileContext, buffer, bufferLen) ;
			success = (writeLen != bufferLen) ? false : true ;
		}
		//write into file here
	else
		return false ;
		//create file failed
	if (lastChunk || !success)	{
        if (fileContext)
		closeFile (fileContext) ;
		fileContext = 0 ;
	}
	return success ;
}

Page* WebChromeClient::createModalDialog(Frame*, const FrameLoadRequest&)
{
    __NotImplemented();
    return 0;
}

void WebChromeClient::show()
{
	if( m_pWebControl->GetContentListener() )
	{
		return m_pWebControl->GetContentListener()->showWebControl();
	}
}

bool WebChromeClient::canRunModal()
{
    __NotImplemented();
    return false;
}

void WebChromeClient::runModal()
{
    __NotImplemented();
}

void WebChromeClient::setToolbarsVisible(bool)
{
    __NotImplemented();
}

bool WebChromeClient::toolbarsVisible()
{
    __NotImplemented();
    return false;
}

void WebChromeClient::setStatusbarVisible(bool)
{
    __NotImplemented();
}

bool WebChromeClient::statusbarVisible()
{
    __NotImplemented();
    return false;
}

void WebChromeClient::setScrollbarsVisible(bool)
{
    __NotImplemented();
}

bool WebChromeClient::scrollbarsVisible() {
    __NotImplemented();
    return false;
}

void WebChromeClient::setMenubarVisible(bool)
{
    __NotImplemented();
}

bool WebChromeClient::menubarVisible()
{
    __NotImplemented();
    return false;
}

void WebChromeClient::setResizable(bool)
{
    __NotImplemented();
}

void WebChromeClient::closeWindowSoon()
{
#if defined( JS_MULTIPLE_WINDOW_SUPPORT )
  //  /*
  //  *   Fix for Bug SFB00001339
  //  *   If we try to close a window which is not opened using window.create() then 
  //  *   the value of m_openerWindowUrl will be NULL. 
  //  *   (As this is getting set inside WebChromeClient::createWindow() function)
  //  *   So we check if value of m_openerWindowUrl exists if not we currently do nothing.
  //  *   Mozilla does not do anything, whereas Opera opens a default page.
  //  */
  //  if(m_pWebControl->m_openerWindowUrl != NULL) // for BUG id 1738 Gnana
		//m_pWebControl->loadURL((char*)m_pWebControl->m_openerWindowUrl);		

	m_pWebControl->abort();
	m_pWebControl->closeWindowSoon();
#else
	__NotImplemented();
#endif //JS_MULTIPLE_WINDOW_SUPPORT
	}

bool WebChromeClient::canTakeFocus(FocusDirection)
{
    __NotImplemented();
    return true;
}

void WebChromeClient::takeFocus(FocusDirection)
{
    __NotImplemented();
}

bool WebChromeClient::canRunBeforeUnloadConfirmPanel()
{
    __NotImplemented();
    return false;
}

bool WebChromeClient::runBeforeUnloadConfirmPanel(const WebCore::String&, WebCore::Frame*)
{
    __NotImplemented();
    return false;
}

void WebChromeClient::addMessageToConsole(const WebCore::String& message, unsigned int lineNumber, const WebCore::String& sourceId)
{
    __NotImplemented();
}

void WebChromeClient::runJavaScriptAlert(Frame* frame, const WebCore::String& message)
{
#ifndef NDEBUG /*ascii fnc is defined under NDEBUG flag only*/
    //sfb_re_eng
	//BrAppCreateJsAlertMsgbox(message.ascii().data());
#endif
	if( m_pWebControl->pFuncHandler && m_pWebControl->pFuncHandler->getJsAlertFuncHandler()  )
		m_pWebControl->pFuncHandler->getJsAlertFuncHandler()(message.latin1().data());
}

bool WebChromeClient::runJavaScriptConfirm(Frame* frame, const WebCore::String& message)
{
#ifndef NDEBUG /*ascii fnc is defined under NDEBUG flag only*/
    //sfb_re_eng
	//return BrAppCreateJsConfirmPopup(message.ascii().data());
	//return true;
#endif
	//if( m_pWebControl->pFuncHandler &&  m_pWebControl->pFuncHandler->getJsConfirmFuncHandler() )
	//	return m_pWebControl->pFuncHandler->getJsConfirmFuncHandler()(message.ascii().data());
	//else
	//	return true;
	if( m_pWebControl->pFuncHandler &&  m_pWebControl->pFuncHandler->getJsConfirmFuncHandler()  )
	{
		bool retVal = m_pWebControl->pFuncHandler->getJsConfirmFuncHandler()(message.ascii().data());
		if (retVal)
		{
			//Get the resulting string from the webcontrol, set to the out Param and return success
			return m_pWebControl->getJSConfirmResult() ;
		}
		return retVal ;
	}
	else
	{
		if ( m_pWebControl->isOneClickMode() == TRUE )   // for widget
		{
			IContentsListener *pContentsListener = m_pWebControl->GetContentListener();
			bool retVal=  pContentsListener->confirm( 0 , NULL, message.ascii().data(), NULL, NULL );
			if (retVal)
			{
				//Get the resulting string from the webcontrol, set to the out Param and return success
				return m_pWebControl->getJSConfirmResult() ;
			}
			return retVal ;
		}
	}
	return false ;

}

bool WebChromeClient::runHtmlParseConfirm(Frame* frame)
{
    // When UI support is added we will that function from here.
    frame->page()->setXhtml(true);
    void* pWebControlInstance = frame->getWebControlInstance();
    SFBalSendEvent(BAL_APP_BROWSER, BAL_EVENT_RELOAD, 0, 0, 0, pWebControlInstance);
    return true;
}

bool WebChromeClient::runJavaScriptPrompt(Frame* frame, const WebCore::String& message, const WebCore::String& defaultValue, WebCore::String& result)
{
#ifndef NDEBUG /*ascii fnc is defined under NDEBUG flag only*/
	//sfb_re_eng
	//return BrAppCreateJsPromptPopup(message.ascii().data());
	//return true;
#endif
	if( m_pWebControl->pFuncHandler &&  m_pWebControl->pFuncHandler->getJsPromptFuncHandler()  )
	{
		bool retVal = m_pWebControl->pFuncHandler->getJsPromptFuncHandler()(message.ascii().data());
		if (retVal)
		{
			//Get the resulting string from the webcontrol, set to the out Param and return success
			if (m_pWebControl->getJSPromptResultString() != "" )
			{
				result = m_pWebControl->getJSPromptResultString() ;
			}
			else
			{
				result="" ;
			}			
		}
		return retVal ;
	}
	else
	{
		if ( m_pWebControl->isOneClickMode() == TRUE )   // for widget
		{
			IContentsListener *pContentsListener = m_pWebControl->GetContentListener();
			bool retVal =  pContentsListener->prompt( NULL , message.ascii().data(), NULL, NULL );
			if (retVal)
			{
				//Get the resulting string from the webcontrol, set to the out Param and return success
				if (m_pWebControl->getJSPromptResultString() != "" )
				{
					result = m_pWebControl->getJSPromptResultString() ;
				}
				else
				{
					result="" ;
				}	
			}
			return retVal ;
		}
	}
	return false ;
}

bool WebChromeClient::runWmlScriptAlert(const char * message)
{
	if( m_pWebControl->pFuncHandler->getWmlsAlertFuncHandler() )
		m_pWebControl->pFuncHandler->getWmlsAlertFuncHandler()(message);
	return true;
}

bool  WebChromeClient::runWmlScriptConfirm(const char * message,const char *okText,const char *cancelText)
{
	if( m_pWebControl->pFuncHandler->getWmlsConfirmFuncHandler() )
		return m_pWebControl->pFuncHandler->getWmlsConfirmFuncHandler()(message,okText,cancelText);
	return true;
}

bool  WebChromeClient::runWmlScriptPrompt(const char * message,const char *defInp)
{
	if( m_pWebControl->pFuncHandler->getWmlsPromptFuncHandler() )
		return m_pWebControl->pFuncHandler->getWmlsPromptFuncHandler()(message,defInp);
	return true;
}

void WebChromeClient::setStatusbarText(const WebCore::String& string)
{
 //   CString stringMessage = string.utf8();
	//if(m_webPage->m_statusbarTextChangedHandler)
	//	(m_webPage->m_statusbarTextChangedHandler)(stringMessage.data());
}

bool WebChromeClient::shouldInterruptJavaScript()
{
    return true;
}

bool WebChromeClient::tabsToLinks() const
{
    __NotImplemented();
    return false;
}

IntRect WebChromeClient::windowResizerRect() const
{
    __NotImplemented();
    return IntRect();
}

void WebChromeClient::addToDirtyRegion(const IntRect& rc)
{
	if (m_pWebControl->isExitInstanceCalled())
		return;
#if defined(_SFB_BACKINGSTORE)
	WebBackingStore *pBackingStore = m_pWebControl->GetBackingStore();

	if( pBackingStore )
		pBackingStore->AddToDirtyRegion( rc );
#endif

}

void WebChromeClient::scrollBackingStore(int dx, int dy, const IntRect& scrollViewRect, const IntRect& clipRect)
{
	if (m_pWebControl->isExitInstanceCalled())
		return;
#if defined(_SFB_BACKINGSTORE)
	WebBackingStore *pBackingStore = m_pWebControl->GetBackingStore();

	if( pBackingStore )
		pBackingStore->ScrollBackingStore( dx, dy, scrollViewRect, clipRect );
#endif
}

void WebChromeClient::updateBackingStore()
{
	if (m_pWebControl->isExitInstanceCalled())
		return;
#if defined(_SFB_BACKINGSTORE)
	WebBackingStore *pBackingStore = m_pWebControl->GetBackingStore();

	if( pBackingStore )
		pBackingStore->UpdateBackingStore( false  );
#endif
}

void WebChromeClient::mouseDidMoveOverElement(const HitTestResult&, unsigned modifierFlags)
{
    __NotImplemented();
}

void WebChromeClient::setToolTip(const WebCore::String&)
{
    __NotImplemented();
}

void WebChromeClient::print(Frame*)
{
    __NotImplemented();
}

void WebChromeClient::exceededDatabaseQuota(Frame*, const WebCore::String&)
{
    __NotImplemented();
}

void WebChromeClient::populateVisitedLinks()
{
    __NotImplemented();
}

void WebChromeClient::openFileStore()
{	
	IContentsListener *pContentsListener = m_pWebControl->GetContentListener();
	pContentsListener->openFileStore();
}

void WebChromeClient::displayErrorMessage()
{
	if(m_pWebControl)
	{
		IContentsListener *pContentsListener = m_pWebControl->GetContentListener();
		pContentsListener->displayErrorMessage();
	}
}

void WebChromeClient::getWindowHandles(HWIN* hWin, HGC* hGC)
{	
	*hWin	= int(m_pWebControl->GetHWIN());
	*hGC	= int(m_pWebControl->GetHGC());

#if defined(_SFB_BACKINGSTORE)
	WebBackingStore *pBackingStore = m_pWebControl->GetBackingStore();

	if( pBackingStore )
		*hGC = pBackingStore->GetBackingStoreGC();
	else
		*hGC = INVALID_HOBJ;
#endif

}

#if defined(_SFB_BACKINGSTORE)
WebCore::IntRect WebChromeClient::viewportRect() const
{
	WebBackingStore *pBackingStore = m_pWebControl->GetBackingStore();

	if( pBackingStore )
		return pBackingStore->GetViewRect();
		
	else
		return WebCore::IntRect();
}
WebCore::IntRect WebChromeClient::backingStoreRect() const
{
	WebBackingStore *pBackingStore = m_pWebControl->GetBackingStore();

	if( pBackingStore )
		return pBackingStore->GetBackingStoreRect();

	else
		return WebCore::IntRect();
}
float WebChromeClient::getZoomFactor() const
{
	if(m_pWebControl)
	{
		WebBackingStore *pBackingStore = m_pWebControl->GetBackingStore();

		if( pBackingStore )
			return pBackingStore->GetZoom();
	}
}

/*
* @param point left-upper position in contents coordinates
*/
bool WebChromeClient::scrollTo( IntPoint point )
{
	WebBackingStore *pBackingStore = m_pWebControl->GetBackingStore();
	if( !pBackingStore )
		return false;

	return pBackingStore->ScrollTo( point );
}

#endif //_SFB_BACKINGSTORE
// Platform SHP Flag Can be removed from
#if( PLATFORM(SHP) && SFB_SMARTLINK_FEATURE )
bool WebChromeClient::SmartLinkPopUp(const int aLinkType, char* pData) const 
{ 
    if( m_pWebControl )
    {
		IContentsListener *pContentsListener = m_pWebControl->GetContentListener();
	    if( pContentsListener )
        {
			if(aLinkType == 0)
	        pContentsListener->confirm(CONFIRM_SMART_LINK, "SMART LINK", "MAKE CALL?",NULL, pData);
			else
			pContentsListener->confirm(CONFIRM_SMART_LINK, "SMART LINK", "SEND MAIL?",NULL, pData );
        }

    }
    return TRUE;
}
#endif

bool WebChromeClient::isOneClickMode()
{
	return m_pWebControl->isOneClickMode();
}
}
