/*
 * Copyright (C) 2006 Zack Rusin <zack@kde.org>
 * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __WebFrameLoaderClient_Defined
#define __WebFrameLoaderClient_Defined

#include "IDecisionSender.h"


#include "FrameLoaderClient.h"
#include "ResourceResponse.h"
#include "CMMDefs.h"

#if defined(SHP_SUPPORT_WML)
	#include "StringHash.h"
	#include "PlatformString.h"
#endif

namespace WebCore{
	class PluginView;
}
namespace WebControl {

	class WebFrame;
	class WebFrameDecisionSender;




    class WebFrameLoaderClient : public WebCore::FrameLoaderClient 
    CMM_MODIFIED_ADD
    {
    public:
		WebFrameLoaderClient( WebFrame* pWebFrame );
		~WebFrameLoaderClient();

		WebFrame* GetWebFrame() { return m_pWebFrame; }
		WebFrameDecisionSender* setupDecisionSender( WebCore::FramePolicyFunction policyFuntion, bool bMimeTypeDecision );
		void DecidedPolicyForMIMEType( WebCore::PolicyAction action );
# if SFB_WIDGET
		void DeActivateNativeInstance();
# endif



	public:
		bool m_loadingComplete;
		////////////////////////////////////////////
		// virtual funtions in WebCore::FrameLoaderClient 
		////////////////////////////////////////////
        virtual void frameLoaderDestroyed();
		virtual void setLoadingComlete(){ m_loadingComplete = true;}
		virtual bool isLoadingComplete(){return m_loadingComplete; }
#ifdef PLATFORM(SHP)
		virtual int	getProtoAccountHandle ();
#endif
		virtual bool hasWebView() const;
        virtual bool hasFrameView() const;

        virtual bool privateBrowsingEnabled() const;

        virtual void makeDocumentView();
        virtual void makeRepresentation(WebCore::DocumentLoader*);
        virtual void setDocumentViewFromCachedPage(WebCore::CachedPage*);
        virtual void forceLayout();
        virtual void forceLayoutForNonHTML();

        virtual void setCopiesOnScroll();

        virtual void detachedFromParent1();
        virtual void detachedFromParent2();
        virtual void detachedFromParent3();
        virtual void detachedFromParent4();

        virtual void loadedFromCachedPage();

        virtual void assignIdentifierToInitialRequest(unsigned long identifier, WebCore::DocumentLoader*, const WebCore::ResourceRequest&);

        virtual void dispatchWillSendRequest(WebCore::DocumentLoader*, unsigned long  identifier, WebCore::ResourceRequest&, const WebCore::ResourceResponse& redirectResponse);
        virtual void dispatchDidReceiveAuthenticationChallenge(WebCore::DocumentLoader*, unsigned long identifier, const WebCore::AuthenticationChallenge&);
        virtual void dispatchDidCancelAuthenticationChallenge(WebCore::DocumentLoader*, unsigned long  identifier, const WebCore::AuthenticationChallenge&);
        virtual void dispatchDidReceiveResponse(WebCore::DocumentLoader*, unsigned long  identifier, const WebCore::ResourceResponse&);
        virtual void dispatchDidReceiveContentLength(WebCore::DocumentLoader*, unsigned long identifier, int lengthReceived);
        virtual void dispatchDidFinishLoading(WebCore::DocumentLoader*, unsigned long  identifier);
        virtual void dispatchDidFailLoading(WebCore::DocumentLoader*, unsigned long  identifier, const WebCore::ResourceError&);
        virtual bool dispatchDidLoadResourceFromMemoryCache(WebCore::DocumentLoader*, const WebCore::ResourceRequest&, const WebCore::ResourceResponse&, int length);

        virtual void dispatchDidHandleOnloadEvents();
        virtual void dispatchDidReceiveServerRedirectForProvisionalLoad();
        virtual void dispatchDidCancelClientRedirect();
        virtual void dispatchWillPerformClientRedirect(const WebCore::KURL&, double, double);
        virtual void dispatchDidChangeLocationWithinPage();
        virtual void dispatchWillClose();
        virtual void dispatchDidReceiveIcon();
        virtual void dispatchDidStartProvisionalLoad();
        virtual void dispatchDidReceiveTitle(const WebCore::String&);
        virtual void dispatchDidCommitLoad();
        virtual void dispatchDidFailProvisionalLoad(const WebCore::ResourceError&);
        virtual void dispatchDidFailLoad(const WebCore::ResourceError&);
        virtual void dispatchDidFinishDocumentLoad();
        virtual void dispatchDidFinishLoad();
        virtual void dispatchDidFirstLayout();

        virtual WebCore::Frame* dispatchCreatePage();
        virtual void dispatchShow();

        virtual void dispatchDecidePolicyForMIMEType(WebCore::FramePolicyFunction, const WebCore::String& MIMEType, const WebCore::ResourceRequest&);
        virtual void dispatchDecidePolicyForNewWindowAction(WebCore::FramePolicyFunction, const WebCore::NavigationAction&, const WebCore::ResourceRequest&, const WebCore::String& frameName);
        virtual void dispatchDecidePolicyForNavigationAction(WebCore::FramePolicyFunction, const WebCore::NavigationAction&, const WebCore::ResourceRequest&);
        virtual void cancelPolicyCheck();

        virtual void dispatchUnableToImplementPolicy(const WebCore::ResourceError&);

        virtual void dispatchWillSubmitForm(WebCore::FramePolicyFunction, WTF::PassRefPtr<WebCore::FormState>);

        virtual void dispatchDidLoadMainResource(WebCore::DocumentLoader*);
        virtual void revertToProvisionalState(WebCore::DocumentLoader*);
        virtual void setMainDocumentError(WebCore::DocumentLoader*, const WebCore::ResourceError&);
        virtual void clearUnarchivingState(WebCore::DocumentLoader*);

        virtual void postProgressStartedNotification();
        virtual void postProgressEstimateChangedNotification();
        virtual void postProgressFinishedNotification();

        virtual PassRefPtr<WebCore::Frame> createFrame(const WebCore::KURL& url, const WebCore::String& name, WebCore::HTMLFrameOwnerElement* ownerElement,
                                   const WebCore::String& referrer, bool allowsScrolling, int marginWidth, int marginHeight);
	virtual void deleteWebFrame();
        virtual WebCore::Widget* createPlugin(const WebCore::IntSize&, WebCore::Element*, const WebCore::KURL&, const WTF::Vector<WebCore::String>&, const WTF::Vector<WebCore::String>&, const WebCore::String&, bool);
        virtual void redirectDataToPlugin(WebCore::Widget* pluginWidget);
        virtual WebCore::Widget* createJavaAppletWidget(const WebCore::IntSize&, WebCore::Element*, const WebCore::KURL& baseURL, const WTF::Vector<WebCore::String>& paramNames, const WTF::Vector<WebCore::String>& paramValues);
        virtual WebCore::String overrideMediaType() const;

        virtual WebCore::ObjectContentType objectContentType(const WebCore::KURL& url, const WebCore::String& mimeType);

        virtual void setMainFrameDocumentReady(bool);

        virtual void startDownload(const WebCore::ResourceRequest&);

        virtual void willChangeTitle(WebCore::DocumentLoader*);
        virtual void didChangeTitle(WebCore::DocumentLoader*);

        virtual void committedLoad(WebCore::DocumentLoader*, const char*, int);
        virtual void finishedLoading(WebCore::DocumentLoader*);
        virtual void finalSetupForReplace(WebCore::DocumentLoader*);

        virtual void updateGlobalHistoryForStandardLoad(const WebCore::KURL&);
        virtual void updateGlobalHistoryForReload(const WebCore::KURL&);
        virtual bool shouldGoToHistoryItem(WebCore::HistoryItem*) const;

        virtual WebCore::ResourceError cancelledError(const WebCore::ResourceRequest&);
        virtual WebCore::ResourceError blockedError(const WebCore::ResourceRequest&);
        virtual WebCore::ResourceError cannotShowURLError(const WebCore::ResourceRequest&);
        virtual WebCore::ResourceError interruptForPolicyChangeError(const WebCore::ResourceRequest&);

        virtual WebCore::ResourceError cannotShowMIMETypeError(const WebCore::ResourceResponse&);
        virtual WebCore::ResourceError fileDoesNotExistError(const WebCore::ResourceResponse&);

        virtual bool shouldFallBack(const WebCore::ResourceError&);

        virtual void setDefersLoading(bool);

        virtual bool willUseArchive(WebCore::ResourceLoader*, const WebCore::ResourceRequest&, const WebCore::KURL& originalURL) const;
        virtual bool isArchiveLoadPending(WebCore::ResourceLoader*) const;
        virtual void cancelPendingArchiveLoad(WebCore::ResourceLoader*);
        virtual void clearArchivedResources();

        virtual bool canHandleRequest(const WebCore::ResourceRequest&) const;
        virtual bool canShowMIMEType(const WebCore::String&, const WebCore::KURL& m_Url) const;
        virtual bool representationExistsForURLScheme(const WebCore::String&) const;
        virtual WebCore::String generatedMIMETypeForURLScheme(const WebCore::String&) const;

        virtual void frameLoadCompleted();
        virtual void saveViewStateToItem(WebCore::HistoryItem*);
        virtual void restoreViewState();
        virtual void provisionalLoadStarted();
        virtual void didFinishLoad();
        virtual void prepareForDataSourceReplacement();

        virtual WTF::PassRefPtr<WebCore::DocumentLoader> createDocumentLoader(const WebCore::ResourceRequest&, const WebCore::SubstituteData&);
        virtual void setTitle(const WebCore::String& title, const WebCore::KURL&);

        virtual WebCore::String userAgent(const WebCore::KURL&);

        virtual void saveDocumentViewToCachedPage(WebCore::CachedPage*);

#if PLATFORM(SHP)
		virtual void stopLingerTimerAfterAJAX();
#endif

        virtual bool canCachePage() const;
#ifdef PLATFORM(SHP)
		virtual void download( IWebClientBridge*, WebCore::ResourceLoader*, const WebCore::ResourceRequest&, const WebCore::ResourceRequest&, const WebCore::ResourceResponse&);
#endif
        virtual void download(WebCore::ResourceHandle*, const WebCore::ResourceRequest&, const WebCore::ResourceRequest&, const WebCore::ResourceResponse&);


		virtual void savePlatformDataToCachedPage(WebCore::CachedPage*);
		virtual void transitionToCommittedFromCachedPage(WebCore::CachedPage*) ;
        virtual void transitionToCommittedForNewPage();

        virtual void windowObjectCleared();
        virtual void didPerformFirstNavigation() const; // "Navigation" here means a transition from one page to another that ends up in the back/forward list.
        virtual void registerForIconNotification(bool listen = true);

#if defined(SHP_SUPPORT_WML)
		virtual void showErrorMessageBox(const char *errMsg);
		virtual void delegateDoElementToUserMenu (int id, WebCore::String displayString) ;
		virtual void removeDoElementinUserMenu (int id);
		//NATARAJ SFB00001083 
		virtual int GetClientId();
		virtual void alertMsg (); 
		virtual void errorMsg();
		virtual void errorMsg(const char *errMsg);
#endif

# if PLATFORM(SHP)
# ifdef SFB_WIDGET
		virtual void SetExposeEvent(bool bSet );
		virtual bool GetExposeEvent();
# endif
# endif

#ifdef SFB_UPDATE_URL_BAR  //Pramod - Bresson Model - update Url ComboBox bar
		virtual WebCore::String setLinkUrl(const WebCore::KURL&);
#endif

#ifdef PLATFORM(SHP)
		virtual WebCore::String appName() const;
		virtual WebCore::String appVersion() const;
#endif

#if PLATFORM(SHP)
        // Don't know why is PLATFORM(SHP) added here!!!!!
        // Adding this block under PLATFORM(SHP) to maintain consistency
    private:
        bool isControlChar(const char c);
        bool isValidTextContent(const char* data, int length);
#endif // PLATFORM(SHP)

    private:
		WebFrame*				m_pWebFrame;
		WebCore::ResourceResponse m_response;
		// Points to the plugin view that data should be redirected to.
	    WebCore::PluginView* m_pluginView;	

		bool m_hasSentResponseToPlugin;
		bool m_bFrameLoaderMimeTypeDecision;
		WebCore::PolicyAction m_policyActionFormMimeType;
    };

} // namespace

#endif //__WebFrameLoaderClient_Defined
