/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		DOMHtml.cpp
 * @brief		implement DOM level2 HTML
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.18	v1.0		Created
 */

#include "WebDOMHtml.h"

#include "String.h"
#include "CString.h"
#include "HtmlDocument.h"


namespace WebControl {

DOMHtmlDocument::DOMHtmlDocument( WebCore::Document* pDoc )
: DOMDocument( pDoc )
{

}

HRESULT DOMHtmlDocument::getTitle( char* out_pszBuffer, int in_nBufferLen )
{
	WebCore::String title = m_pDoc->title();
    if( in_nBufferLen <= title.length())
    {
		return HRESULT_NO_MEMORY;    
    }
    else
    {
        AcStrcpy(out_pszBuffer, title.latin1().data());
    }

	return HRESULT_OK;
}

HRESULT DOMHtmlDocument::getURL( char* out_pszBuffer, int in_nBufferLen )
{
	WebCore::String url = m_pDoc->documentURI();
    if(in_nBufferLen <= url.length())
    {
		return HRESULT_NO_MEMORY;    
        
    }
    else
    {
        AcStrcpy(out_pszBuffer, url.latin1().data());
    }
	return HRESULT_OK;
}

HRESULT DOMHtmlDocument::getLastModified( char* out_pszBuffer, int in_nBufferLen )
{
	out_pszBuffer[0] = 0;
	WebCore::HTMLDocument *pHtmlDoc = dynamic_cast<WebCore::HTMLDocument*>(m_pDoc);

	if( pHtmlDoc == NULL )
		return HRESULT_NOK;

	WebCore::String time =  pHtmlDoc->lastModified();

    if(in_nBufferLen <= time.length())
    {
		return HRESULT_NO_MEMORY;        
    }
    else
    {
        AcStrcpy(out_pszBuffer, time.latin1().data());
    }

	return HRESULT_OK;
}



} // namespace WebFrame